/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file Af_ctrl_api.h
 *
 * @brief Include all AF's function and parameters
 *
 *****************************************************************************/

#ifndef __AFI_API_H__
#define __AFI_API_H__

#include "afi_lib_interface.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct AfiContext_s *AfiHandle_t;

/*****************************************************************************/
/**
 *          IsiAfLibInstanceConfig_t
 *
 * @brief   Config structure to create a new Af lib instance
 *
 */
/*****************************************************************************/
typedef struct AfiInstanceConfig_s
{
    uint8_t         ispId;
    AfiLib_t        *pAfiLib;           /**< Af Lib struct interface */
    AfiHandle_t     hAfiHandle;         /**< Af handle returned by AfiCreate */
} AfiInstanceConfig_t;

/*****************************************************************************/
/**
 *          AfiCreate
 *
 * @brief   register Af lib and get the Af lib's handle
 *
 * @param   handle      Handle to the Af lib intance
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
AAA_RESULT AfiCreate
( 
     AfiInstanceConfig_t  *pAfiConfig
);

/*****************************************************************************/
/**
 *          AfCtrlUnRegister
 *
 * @brief   unregister Af lib and release the Af lib's handle
 *
 * @param   handle       Handle to the Af lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AfiRelease
( 
    AfiHandle_t  handle,
    uint8_t      ispId
);

/*****************************************************************************/
/**
 *          AfiSetMode
 *
 * @brief   Set Af's working mode
 *
 * @param   handle            Handle to the Af lib intance
 * @param   hAfiModeSet    Working mode type
 * @param   modeSet           Working mode to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AfiSetMode
( 
    AfiHandle_t    handle,
    const AfMode_t *pAfMode
);

/*****************************************************************************/
/**
 *          AfCtrlGetMode
 *
 * @brief   Get Af's working mode
 *
 * @param   handle           Handle to the Af lib intance
 * @param   hAfiModeGet   Working mode type
 * @param   modeGet          The pointer to current Working mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AfiGetMode
( 
    AfiHandle_t  handle,
    AfMode_t     *pAfMode
);

/*****************************************************************************/
/**
 *          AfiSetConfig
 *
 * @brief   Set Af's calibration or tunning parameters
 *
 * @param   handle               Handle to the Af lib intance
 * @param   hAfiConfigSet   The pointer to tunning or calibraration parameters to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AfiSetConfig
( 
    AfiHandle_t        handle,
    const AfConfig_t  *pAfConfig
);

/*****************************************************************************/
/**
 *          AfCtrlGetConfig
 *
 * @brief   Get Af's calibration or tunning parameters
 *  
 * @param   handle              Handle to the Af lib intance
 * @param   hAfiConfigGet  The pointer to tunning or calibraration parameters to set
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AfiGetConfig
( 
    AfiHandle_t  handle,
    AfConfig_t  *pAfConfig
);


/*****************************************************************************/
/**
 *          AfiLibSetSensorAttributeParm
 *
 * @brief   return the specified runtime data and statistics
 *
 * @param   handle                       Handle to the Af lib intance
 * @param   hAfiSensorAtrributeParm   Type of runtime data to return
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AfiSetAttribute
( 
    AfiHandle_t         handle,
    const AfAttrParm_t  *pAfAttrParm
);


/*****************************************************************************/
/**
 *          AfiExecute
 *
 * @brief   running Af lib module to get the result
 *
 * @param   handle     Handle to the Af lib intance
 * @param   pAfInfo    Statistic value type
 * @param   pAfResult  The calculation result
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AfiExecute
( 
    AfiHandle_t  handle,
    AfInfo_t     *pAfInfo, 
    AfResult_t   *pAfResult     
);

/*****************************************************************************/
/**
 *          AfiReset
 *
 * @brief   Reset Af status to init status
 *
 * @param   handle     Handle to the Af lib intance
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AfiReset
( 
    AfiHandle_t  handle
);

#ifdef __cplusplus
}
#endif

#endif
