/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef DNR3V3_H
#define DNR3V3_H

/**
 * @file 3dnr.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup 3dnr module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include <oslayer/oslayer.h>
#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif


#define ISP_3DNR4_NOISEMODEL_A_MIN 0.1f
#define ISP_3DNR4_NOISEMODEL_A_MAX 100.0f

#define ISP_3DNR4_NOISEMODEL_B_MIN 0.1f
#define ISP_3DNR4_NOISEMODEL_B_MAX 10000.0f

#define ISP_3DNR_DIALTEH_MIN        3
#define ISP_3DNR_DIALTEH_MAX        7

#define ISP_3DNR_FILTER_LEN_MIN     0.0f
#define ISP_3DNR_FILTER_LEN_MAX     1024.0f

#define ISP_3DNR_NOISE_LEVEL_MIN    1.0f
#define ISP_3DNR_NOISE_LEVEL_MAX    1024.0f

#define ISP_3DNR_PREWEIGHT_MAX     32

#define ISP_3DNR_RANGE_MIN         3
#define ISP_3DNR_RANGE_MAX         7

#define ISP_3DNR_SAD_WEIGHT_MAX          16

#define ISP_3DNR_STRENGTH_MAX            128

#define ISP_3DNR_MOTION_SLOPE_MIN         1
#define ISP_3DNR_MOTION_SLOPE_MAX         1023

/*****************************************************************************/
/**
 *          A3Dnr3Handle_t
 *
 * @brief   3dnr Module instance handle
 *
 *****************************************************************************/
typedef void *A3Dnr3Handle_t;         /**< handle to 3dnr context */

/*****************************************************************************/
/**
 *          A3Dnr3InstanceConfig_t
 *
 * @brief   3dnr Module instance configuration structure
 *
 *****************************************************************************/
typedef struct A3Dnr3InstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */

    A3Dnr3Handle_t          hA3dnr;            /**< handle returned by A3Dnr3Init() */
} A3Dnr3InstanceConfig_t;

/*****************************************************************************/
/**
 *          A3Dnr3Config_t
 *
 * @brief   3dnr Module configuration structure
 *
 *****************************************************************************/
typedef struct A3Dnr3Config_s
{
    uint8_t dilateH;     /**<[3, 7]*/
    float32_t filterLen;      /**<[0, 1024]*/
    float32_t filterLen2;     /**<[0, 1024]*/
    float32_t noiseLevel;     /**<[0, 1024]*/

    uint8_t preWeight;    /**<[0, 32]*/
    uint8_t rangeH;       /**<[3, 7]*/
    uint8_t rangeV;       /**<[3, 7]*/
    uint8_t sadWeight;    /**<[0, 16]*/
    uint8_t strength;     /**<[0, 128]*/
    uint16_t thrMotionSlope; /**<[1, 1024]*/
} A3Dnr3Config_t;

typedef struct A3Dnr3LumaConfig_s
{
    float32_t pixelSlop;
    float32_t pixelSlopShift;
    float32_t pixelSlopMin;
    float32_t pixelValShift;
} A3Dnr3LumaConfig_t;

typedef struct A3Dnr3NlmConfig_s
{
    float32_t blendMotion;    /**<[0, 100]*/
    float32_t blendSlope;     /**<[0, 32]*/
    float32_t blendStatic;    /**<[0, 100]*/
    float32_t sigma;          /**<[0.1, 100]*/
} A3Dnr3NlmConfig_t;

typedef struct A3Dnr3NoiseModelConfig_s
{
    float32_t noisemodelA;   /**<[0, 1000]*/
    float32_t noisemodelB;   /**<[0, 1000]*/
    uint16_t fixCurveStart;
} A3Dnr3NoiseModelConfig_t;

typedef struct A3Dnr3Stats_s{
    uint32_t bgSum;
    uint32_t motionSum;
    uint32_t bgPixelCnt;
    uint32_t motionPixelCnt;
    uint32_t frameAvg;
} A3Dnr3Stats_t;

typedef struct A3Dnr3StatusConfig_s{
   bool_t preGammaEn;
   A3Dnr3Config_t  currentConfig;
   A3Dnr3NoiseModelConfig_t currenModel;
} A3Dnr3StatusConfig_t;


typedef struct A3Dnr3AutoParamTable_s
{
    uint8_t autoLevel;

    float32_t gains[AUTO_NUM];
    float32_t filterLen[AUTO_NUM];
    float32_t filterLen2[AUTO_NUM];
    float32_t noiseLevel[AUTO_NUM];
    uint16_t thrMotionSlope[AUTO_NUM];
    uint8_t sadWeight[AUTO_NUM];

    bool_t motionDilateEn[AUTO_NUM];
    bool_t motionErodeEn[AUTO_NUM];
    bool_t gammaFeBeEnable[AUTO_NUM];

    uint8_t strength[AUTO_NUM];
    uint8_t rangeH[AUTO_NUM];
    uint8_t rangeV[AUTO_NUM];
    uint8_t dilateH[AUTO_NUM];
    uint8_t preWeight[AUTO_NUM];
} A3Dnr3AutoParamTable_t;

/*****************************************************************************/
/**
 *          A3Dnr3Init()
 *
 * @brief   This function initializes the 3dnr
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT A3Dnr3Init
(
    A3Dnr3InstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          A3Dnr3Deinit()
 *
 * @brief   The function deinit the 3dnr
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3Deinit
(
    A3Dnr3Handle_t handle
);


/*****************************************************************************/
/**
 *          A3Dnr3SetMode()
 *
 * @brief   The function set the 3dnr mode
 *
 * @param   handle  Handle to 3dnr
 * @param   mode  3dnr work mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3SetMode
(
    A3Dnr3Handle_t     handle,
    const ARunMode_t	mode
);


/*****************************************************************************/
/**
 *          A3Dnr3GetMode()
 *
 * @brief   The function get the 3dnr mode
 *
 * @param   handle  Handle to 3dnr
 * @param   mode  3dnr work mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3GetMode
(
    A3Dnr3Handle_t handle,
    ARunMode_t  *pMode
);

/*****************************************************************************/
/**
 *          A3Dnr3SetConfigure()
 *
 * @brief   This function configures the 3dnr module
 *
 * @param   handle   Handle to 3dnr 
 * @param   pConfig  3dnr config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3SetConfigure
(
    A3Dnr3Handle_t handle,
    A3Dnr3Config_t *pConfig
);

/*****************************************************************************/
/**
 *          A3Dnr3GetConfigure()
 *
 * @brief   This function get the 3dnr module configures
 *
 * @param   handle  Handle to 3dnr
 * @param   pConfig 3dnr config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3GetConfigure
(
    A3Dnr3Handle_t handle,
    A3Dnr3Config_t *pConfig
);

/*****************************************************************************/
/**
 *          A3Dnr3LumaSetConfigure()
 *
 * @brief   This function configures the 3dnr module luma
 *
 * @param   handle   Handle to 3dnr 
 * @param   pConfig  3dnr luma config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3LumaSetConfigure
(
    A3Dnr3Handle_t handle,
    const A3Dnr3LumaConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          A3Dnr3LumaGetConfigure()
 *
 * @brief   This function get the 3dnr module luma configure
 *
 * @param   handle  Handle to 3dnr
 * @param   pConfig 3dnr luma config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3LumaGetConfigure
(
    A3Dnr3Handle_t handle,
    const A3Dnr3LumaConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          A3Dnr3LumaEnable()
 *
 * @brief   enable 3dnr luma
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3LumaEnable
(
    A3Dnr3Handle_t     handle
);

/*****************************************************************************/
/**
 *          A3Dnr3LumaDisable()
 *
 * @brief   disable 3dnr luma
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3LumaDisable
(
    A3Dnr3Handle_t     handle
);

/*****************************************************************************/
/**
 *          A3Dnr3GammaFeBeEnable()
 *
 * @brief   enable 3dnr pregamma fe be
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3GammaFeBeEnable
(
    A3Dnr3Handle_t     handle
);

/*****************************************************************************/
/**
 *          A3Dnr3GammaFeBeDisable()
 *
 * @brief   disable 3dnr pregamma fe be
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3GammaFeBeDisable
(
    A3Dnr3Handle_t     handle
);

/*****************************************************************************/
/**
 *          A3Dnr3NlmEnable()
 *
 * @brief   enable 3dnr nlm
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3NlmEnable
(
    A3Dnr3Handle_t     handle
);

/*****************************************************************************/
/**
 *          A3Dnr3NlmDisable()
 *
 * @brief   disable 3dnr nlm
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3NlmDisable
(
    A3Dnr3Handle_t     handle
);

/*****************************************************************************/
/**
 *          A3Dnr3NlmSetConfigure()
 *
 * @brief   This function configures the 3dnr nlm module
 *
 * @param   handle   Handle to 3dnr 
 * @param   pConfig  3dnr config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3NlmSetConfigure
(
    A3Dnr3Handle_t handle,
    A3Dnr3NlmConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          A3Dnr3NlmGetConfigure()
 *
 * @brief   This function get the 3dnr module nlm configures
 *
 * @param   handle  Handle to 3dnr
 * @param   pConfig 3dnr config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3NlmGetConfigure
(
    A3Dnr3Handle_t handle,
    A3Dnr3NlmConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          A3Dnr3Enable()
 *
 * @brief   enable 3dnr
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3Enable
(
    A3Dnr3Handle_t     handle
);

/*****************************************************************************/
/**
 *          A3Dnr3Disable()
 *
 * @brief   disable 3dnr
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3Disable
(
    A3Dnr3Handle_t     handle
);

/*****************************************************************************/
/**
 *          A3Dnr3SetNoiseModel()
 *
 * @brief   This function set the 3dnr noise curve calib data
 *
 * @param   handle   Handle to 3dnr 
 * @param   noiseModelA  noise model a
 * @param   noiseModelB  noise model b
 * @param   fixCurveStart
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3SetNoiseModel
(
    A3Dnr3Handle_t handle,
    float32_t      noiseModelA,
    float32_t      noiseModelB,
    uint16_t       fixCurveStart
);


/*****************************************************************************/
/**
 *          A3Dnr3GetNoiseModel()
 *
 * @brief    This function get the 3dnr noise curve calib data
 *
 * @param   handle  Handle to 3dnr
 * @param   pNoiseModelA  pointer to noise model a
 * @param   pNoiseModelB  pointer to noise model b
 * @param   pFixCurveStart
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3GetNoiseModel
(
    A3Dnr3Handle_t handle,
    float32_t *pNoiseModelA,
    float32_t *pNoiseModelB,
    uint16_t *pFixCurveStart
);

/*****************************************************************************/
/**
 * @brief   This function set 3Dnr3 motion config
 * stats.
 *
 * @param   hCamEngine          handle to the CamEngine instance
 * @param   motionDilateEn      enable/disable dilation op of motion mask
 * @param   motionErodeEn       enable/disable erosion op of motion mask
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT A3Dnr3MotionSetConfig
(
    A3Dnr3Handle_t      handle,
    bool_t              motionDilateEn,
    bool_t              motionErodeEn
);

/*****************************************************************************/
/**
 * @brief   This function get 3Dnr3 motion config
 * stats.
 *
 * @param   hCamEngine          handle to the CamEngine instance
 * @param   pMotionDilateEn         get the 3Dnr3 motion Dilation enable
 * @param   pMotionErodeEn          get the 3Dnr3 motion  Erosion
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT A3Dnr3MotionGetConfig
(
    A3Dnr3Handle_t      handle,
    bool_t              *pMotionDilateEn,
    bool_t              *pMotionErodeEn
);

/*****************************************************************************/
/**
 *          A3Dnr3GetStats()
 *
 * @brief   The function returns the statistics of the 3dnrV3
 *          module
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT     A3Dnr3GetStats
(
    A3Dnr3Handle_t     handle,
    A3Dnr3Stats_t      *pA3DnrStats
);

/*****************************************************************************/
/**
 *          A3Dnr3GetStatus()
 *
 * @brief   The function returns the status of the 3dnrV3
 *          module
 *
 * @param   handle  Handle to 3dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3GetStatus
(
    A3Dnr3Handle_t           handle,
    bool_t                   *pRunning,
    ARunMode_t               *pMode,
    ASensorMode_t            *pSensorMode,
    A3Dnr3StatusConfig_t     *pCurrentParam
);


/*****************************************************************************/
/**
 *          ADemosaic2SetAutoParamTable()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle         Handle to 3dnrV3
 * @param   pParamTable    auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3SetAutoParamTable
(
    A3Dnr3Handle_t handle,
    A3Dnr3AutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          A3Dnr3GetAutoParamTable()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle        Handle to 3dnrV3
 * @param   pParamTable   auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A3Dnr3GetAutoParamTable
(
    A3Dnr3Handle_t handle,
    A3Dnr3AutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          A3Dnr3ProcessFrame()
 *
 * @brief   3dnrV3 process frame
 *
 * @param   handle  Handle to 3dnrV3
 *          gain    current sensorgain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3ProcessFrame
(
    A3Dnr3Handle_t handle,
    const float32_t fGain
);

/*****************************************************************************/
/**
 *          A3Dnr3SwReset()
 *
 * @brief   3dnrV3 software reset
 *
 * @param   handle  Handle to 3dnrV3
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A3Dnr3SwReset
(
    A3Dnr3Handle_t handle
);

#ifdef __cplusplus
}
#endif


/* @} 3Dnr3 */


#endif /* DNR3V3_H*/
