/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

/**
 * Module    : Operating System Abstraction Layer
 *
 * Hierarchy :
 *
 * Purpose   : Encapsulates and abstracts services from different operating
 *             system, including user-mode as well as kernel-mode services.
 ******************************************************************************/
#ifdef FREERTOS

#include <stdlib.h>
#include "ebase/types.h"
#include "bsp/FreeRTOS.h"
#include "bsp/task.h"
#include "bsp/semphr.h"
#include "bsp/event_groups.h"
#include "bsp/FreeRTOS.h"
#include "bsp/ff.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define UCOSII_STACK_SIZE   1024
#define OSLAYER_ASSERT      DCT_ASSERT


typedef int32_t (*osThreadFunc)(void *);
typedef int32_t (*osIsrFunc)(void *);
typedef int32_t (*osDpcFunc)(void *);


#ifdef OSLAYER_EVENT
/*****************************************************************************/
/*  @brief  Event object (FreeRTOS Version) of OS Abstraction Layer */
typedef struct _osEvent
{
    EventGroupHandle_t event_flags;
    int32_t automatic;		/* Decides if flag to be consumed (xClearOnExit) */
    int32_t state;			/* Not used in Freertos */
    int32_t waitforall;		/* Added for future extension, need waiting for event bit */
    char *name_ptr;
} osEvent;
#endif /* OSLAYER_EVENT */


#ifdef OSLAYER_MUTEX
/*****************************************************************************/
/*  @brief  Mutex object (FreeRTOS Version) of OS Abstraction Layer */
typedef struct _osMutex
{
	SemaphoreHandle_t mutex;
    char *name_ptr;
} osMutex;
#endif /* OSLAYER_MUTEX */


#ifdef OSLAYER_SEMAPHORE
/*****************************************************************************/
/*  @brief  Semaphore object (FreeRTOS Version) of OS Abstraction Layer */
typedef struct _osSemaphore
{
	SemaphoreHandle_t sem;
    char *name_ptr;
} osSemaphore;
#endif /* OSLAYER_SEMAPHORE */


#ifdef OSLAYER_QUEUE
typedef struct _osQueue
{
	xQueueHandle	qHandle;
	UBaseType_t 	qlen;
	UBaseType_t 	qitemsize;
} osQueue;
#endif /*OSLAYER_QUEUE */


#ifdef OSLAYER_THREAD
/*****************************************************************************/
/*  @brief  Thread object (FreeRTOS Version) of OS Abstraction Layer */
typedef struct _osThread
{
    char *pcName;
	configSTACK_DEPTH_TYPE usStackDepth;
	UBaseType_t uxPriority;
	TaskHandle_t pxCreatedTask;
    char threadName[16];
} osThread;
#endif /* OSLAYER_THREAD */

#ifdef __cplusplus
}
#endif


#endif /* FREERTOS */
