/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_2DNR_API_H
#define CAMDEV_2DNR_API_H

#include "cam_device_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @cond DNR2_DUMMY
 * @defgroup cam_device_2dnr dummy CamDevice 2DNR Dummy Definitions
 * @{
 *
 *
 */

/******************************************************************************/
/**
 * @brief   CamDevice 2DNR configuration.
 *
 *****************************************************************************/
typedef struct CamDevice2DnrConfig_s {
    uint8_t nop;

}CamDevice2DnrConfig_t;


/******************************************************************************/
/**
 * @brief   CamDevice 2DNR status structure.
 *
 *****************************************************************************/
typedef struct CamDevice2DnrStatus_s {
    bool_t enable;                  /**< 2DNR enable status*/
}CamDevice2DnrStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets 2DNR configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   p2DnrCfg            This pointer of 2DNR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice2DnrSetConfig
(
    CamDeviceHandle_t      hCamDevice,
    CamDevice2DnrConfig_t *p2DnrCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets 2DNR configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   p2DnrCfg            This pointer of 2DNR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice2DnrGetConfig
(
    CamDeviceHandle_t      hCamDevice,
    CamDevice2DnrConfig_t *p2DnrCfg
);

/*****************************************************************************/
/**
 * @brief   This function enables 2DNR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice2DnrEnable
(
	CamDeviceHandle_t hCamDevice
);


/*****************************************************************************/
/**
 * @brief   This function disables 2DNR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice2DnrDisable
(
	CamDeviceHandle_t hCamDevice
);


/*****************************************************************************/
/**
 * @brief   This function gets 2DNR status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   p2DnrStatus         This pointer of 2DNR status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice2DnrGetStatus
(
	CamDeviceHandle_t      hCamDevice,
	CamDevice2DnrStatus_t *p2DnrStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets 2DNR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice2DnrReset
(
    CamDeviceHandle_t             hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets 2DNR version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            This pointer of 2DNR version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice2DnrGetVersion
(
	CamDeviceHandle_t hCamDevice,
	uint32_t         *pVersion
);

/* @} cam_device_2dnr dummy*/
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_2DNR_API_H
