/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_GE_API_H
#define CAMDEV_GE_API_H

#include "cam_device_common.h"

#ifdef __cplusplus
extern "C"
{
#endif


/**
 * @cond GE
 * 
 * @defgroup cam_device_ge CamDevice GE Definitions
 * @{
 *
 */

/******************************************************************************/
/**
 * @brief   CamDevice GE manual configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceGeManualConfig_s {
    float32_t threshold;                     /**< GE threshold */
}CamDeviceGeManualConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice GE configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceGeConfig_s {
    CamDeviceGeManualConfig_t manualCfg; /**< GE manual configuration */
}CamDeviceGeConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice GE status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceGeStatus_s {
    bool_t enable;                      /**< GE enable status */
    CamDeviceConfigMode_t currentMode;       /**< The run mode: 0--manual, 1--auto */
    CamDeviceGeManualConfig_t currentCfg; /**< GE current configuration */
}CamDeviceGeStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets GE configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pGeCfg              Pointer to GE configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGeSetConfig
(
    CamDeviceHandle_t hCamDevice,
    const CamDeviceGeConfig_t *pGeCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets GE configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pGeCfg              Pointer to GE configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGeGetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceGeConfig_t *pGeCfg
);

/*****************************************************************************/
/**
 * @brief   This function enables GE.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGeEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables GE.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGeDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets the GE status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pGeStatus           Pointer to GE status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGeGetStatus
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceGeStatus_t *pGeStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets GE.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGeReset
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets the GE version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to GE version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGeGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/* @} cam_device_ge */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_GE_API_H
