/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_ISP_SYSTEM_API_H
#define CAMDEV_ISP_SYSTEM_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include "cam_device_common.h"
#include <common/system_module.h>

/**
 * @defgroup cam_device_isp_system CamDevice ISP System Definitions
 * @{
 *
 *
 */

/*****************************************************************************/
/**
 * @brief   This function sets enqueue buffer callback.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   func                Enqueue buffer callback
 * @param   pUserContext        Pointer to user context
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
*****************************************************************************/
RESULT VsiCamDeviceSetEnqueueBufCb
(
    CamDeviceHandle_t     hCamDevice,
    IspSystemEnqueueBuf_t func,
    void                  *pUserContext
);

/*****************************************************************************/
/**
 * @brief   This function sets dequeue buffer callback.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   func                Dequeue buffer callback
 * @param   pUserContext        Pointer to user context
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
*****************************************************************************/
RESULT VsiCamDeviceSetDequeueBufCb
(
    CamDeviceHandle_t     hCamDevice,
    IspSystemDequeueBuf_t func,
    void                  *pUserContext
);

/*****************************************************************************/
/**
 * @brief   This function starts ISP streaming.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to start streaming configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
*****************************************************************************/
RESULT VsiCamDeviceStartStreaming
(
    CamDeviceHandle_t hCamDevice,
    void *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function stops ISP streaming.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to stop streaming configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
*****************************************************************************/
RESULT VsiCamDeviceStopStreaming
(
    CamDeviceHandle_t hCamDevice,
    void *pConfig
);


 /* @} cam_device_isp_system */

#endif    // __ISP_CAMDEV_ISP_SYSTEM_H__


