/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_RGBIR_API_H
#define CAMDEV_RGBIR_API_H


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @cond RGBIR_DUMMY
 *
 */

/*****************************************************************************/
/**
 * @brief   CamDevice RGBIR output Bayer pattern.
 *
 *****************************************************************************/
typedef unsigned char CamDeviceRgbirOutPat_t;

/******************************************************************************/
/**
 * @brief   CamDevice RGBIR configuration.
 *
 *****************************************************************************/
typedef unsigned char CamDeviceRgbirConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice RGBIR status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceRgbirStatus_s {
    bool_t enable;                /**< RGBIR enable status */
}CamDeviceRgbirStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets the RGBIR configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pRgbirCfg           This pointer of RGBIR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRgbirSetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceRgbirConfig_t *pRgbirCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets the RGBIR configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pRgbirCfg           This pointer of RGBIR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRgbirGetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceRgbirConfig_t *pRgbirCfg
);


/*****************************************************************************/
/**
 * @brief   This function sets RGBIR output pattern parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   outPattern          RGBIR output pattern parameters
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/\
RESULT VsiCamDeviceRgbirSetOutPattern
(
    CamDeviceHandle_t        hCamDevice,
    CamDeviceRgbirOutPat_t   outPattern
);

/*****************************************************************************/
/**
 * @brief   This function gets RGBIR output pattern parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pOutPattern         Pointer to RGBIR output pattern parameters
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/\
RESULT VsiCamDeviceRgbirGetOutPattern
(
    CamDeviceHandle_t        hCamDevice,
    CamDeviceRgbirOutPat_t  *pOutPattern
);

/*****************************************************************************/
/**
 * @brief   This function enables RGBIR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRgbirEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables RGBIR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRgbirDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets the RGBIR status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pRgbirStatus        This pointer of RGBIR status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRgbirGetStatus
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceRgbirStatus_t *pRgbirStatus
);


/*****************************************************************************/
/**
 * @brief   This function resets RGBIR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRgbirReset
(
    CamDeviceHandle_t             hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets the RGBIR version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            This pointer of RGBIR version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRgbirGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/* @} cam_device_rgbir */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_RGBIR_API_H
