/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    GE
 * 
 * @file    cam_engine_ge_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine GE Definitions
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_ge_api CamEngine GE Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_GE_API_H
#define CAM_ENGINE_GE_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_GE_THRESHOLD_MIN 0
#define CAM_ENGINE_GE_THRESHOLD_MAX (511.996)

/******************************************************************************/
/**
 * @brief   GE manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineGeConfig_s
{
	float threshold;    /**< GE threshold */
} CamEngineGeConfig_t;

/*****************************************************************************/
/**
 * @brief   Enables GE.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGeEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables GE.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGeDisable
(
    CamEngineHandle_t hCamEngine
);



/*****************************************************************************/
/**
 * @brief   Configures GE.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             GE configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineGeSetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineGeConfig_t    *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the GE status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             GE configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineGeGetConfig
(
    CamEngineHandle_t    hCamEngine,
    CamEngineGeConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets the GE status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The running status
 * @param   pCfg                GE configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGeGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    CamEngineGeConfig_t* pCfg
);


#ifdef __cplusplus
}
#endif


/* @} cam_engine_ge_api */


#endif /* CAM_ENGINE_GE_API_H */

