/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    MPCTL
 *
 * @file    cam_engine_mpctl.h
 *
 * @brief
 *
 *   Interface description of the CamEngine LUT3D API.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_mpctl CamEngine MPCTL API
 * @{
 *
 */

#ifndef CAM_ENGINE_MPCTL_H
#define CAM_ENGINE_MPCTL_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct CamEngineMpctlCfg_s
{
    bool_t outPixelNum;     /**<"At output port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"*/
    bool_t inPixelNum;      /**<"At input port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"*/
    uint16_t cropSize;      /**<    crop size for one line */
    uint16_t cropOffset;    /**<    crop offset for one line */
} CamEngineMpctlCfg_t;
/*****************************************************************************/
/**
 * @brief   This function enables the CamEngine ISP MPCTL Module.
 *
 * @param   hCamEngine              CamEngine driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamEngineIspMpctlEnable
(
    CamEngineHandle_t   hCamEngine
);


/*****************************************************************************/
/**
 * @brief   This function disables the CamEngine ISP MPCTL module (bypass the
 *          mpctrl module)
 *
 * @param   hCamEngine              CamEngine driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamEngineIspMpctlDisable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function enables the CamEngine ISP MPCTL Module.
 *
 * @param   hCamEngine              CamEngine driver handle
 * @param   pIsEnabled
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamEngineIspMpctlGetStatus
(
    CamEngineHandle_t   hCamEngine,
    bool_t              *pIsEnabled
);

/*****************************************************************************/
/**
 * @brief   This function set config of the CamEngine ISP MPCTL module.
 *
 * @param   hCamEngine              CamEngine driver handle
 * @param   pIsEnabled          Pointer to value to store current state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    pIsEnabled is a NULL pointer
 *
 *****************************************************************************/
RESULT CamEngineIspMpctlSetCfg
(
    CamEngineHandle_t        hCamEngine,
    const CamEngineMpctlCfg_t      *pMpctlCfg

);

/*****************************************************************************/
/**
 * @brief   This function get config of the CamEngine ISP MPCTL module.
 *
 * @param   hCamEngine              CamEngine driver handle
 * @param   pIsEnabled          Pointer to value to store current state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    pIsEnabled is a NULL pointer
 *
 *****************************************************************************/
RESULT CamEngineIspMpctlGetCfg
(
    CamEngineHandle_t       hCamEngine,
    CamEngineMpctlCfg_t     *pMpctlCfg

);
#ifdef __cplusplus
}
#endif

/* @} CamEngine_isp_mpctl */

/* @endcond */

#endif /* CAM_ENGINE_MPCTL_H */

