/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    RAW CROPPING
 *
 * @file    cam_engine_raw_crop_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine RAW CROPPING Algorithms.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_raw_crop_api CamEngine RAW CROPPING Algorithms API
 * @{
 *
 */

#ifndef CAM_ENGINE_raw_crop_api_H
#define CAM_ENGINE_raw_crop_api_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/ispcore_defs.h>

#ifdef __cplusplus
extern "C"
{
#endif

#include "cam_engine_common.h"

/******************************************************************************/
/**
 * @brief   Cam Engine RAW CROPPING manual configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineRawCropConfig_s
{
    CamEngineWindow_t rawCropWindows;
} CamEngineRawCropConfig_t;

/*****************************************************************************/
/**
 * @brief   This function enable the raw crop
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineRawCropEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the raw crop
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineRawCropDisable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function config the raw crop
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of raw crop
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineRawCropSetConfig
(
    CamEngineHandle_t      hCamEngine,
    const CamEngineRawCropConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function returns the raw crop config.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of raw crop
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineRawCropGetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineRawCropConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function get the status of raw crop
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            running status
 * @param   pCfg                raw crop config

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineRawCropGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    CamEngineRawCropConfig_t* pCfg
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_raw_crop_api */


#endif /* CAM_ENGINE_raw_crop_api_H */

