/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    cam_engine_tpg_api.h
 *
 * @file    cam_engine_tpg_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine ISP.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_tpg_api CamEngine Tpg Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_TPG_API_H
#define CAM_ENGINE_TPG_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include <cameric_drv/cameric_isp_tpg_drv_api.h>
#ifdef __cplusplus
extern "C"
{
#endif


RESULT CamEngineEnableTpg
(
    CamEngineHandle_t hCamEngine
);

RESULT CamEngineDisableTpg
(
    CamEngineHandle_t hCamEngine
);

typedef struct CamEngineTpgConfig_s
{
    CamerIcISPTpgImageType_t        imageType;
    CamerIcIspTpgBayerPattern_t     bayerPattern;
    CamerIcIspTpgColorDepth_t       colorDepth;
    CamerIcIspTpgResolution_t       resolution;

    uint16_t                        pixleGap;       // image 0 (3x3 block)
    uint16_t                        lineGap;        // image 0 (3x3 block)
    uint16_t                        gapStandard;    // image 1, 2, 3.
    uint32_t                        randomSeed;     // image 4
    uint32_t                        frameNum;

    CamerIcIspTpgUserDefineMode_t   userMode;

} CamEngineTpgConfig_t;

RESULT CamEngineTpgConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineTpgConfig_t    *pTpgConfig
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_tpg_api */


#endif /* CAM_ENGINE_TPG_API_H */

