/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMERIC_ISP_2DNR_DRV_API_H
#define CAMERIC_ISP_2DNR_DRV_API_H

/**
* @file    cameric_isp_2dnr_drv_api.h
*
* @brief   This file contains the CamerIC ISP 2DNR driver API definitions.
*
*****************************************************************************/
/**
* @cond cameric_isp_2dnr
*
* @defgroup cameric_isp_2dnr_drv_api CamerIC ISP 2DNR Driver API definitions
* @{
*
*/
#include <ebase/types.h>
#include <common/return_codes.h>
#ifdef __cplusplus
extern "C"
{
#endif

//parameter range check
#define CAMERIC_ISP_2DNR_STRENGTH_MAX           127
#define CAMERIC_ISP_2DNR_SIGMA_Y_LUT_MAX        4095
#define CAMERIC_ISP_2DNR_WEIGHT_MUL_FACTOR_MAX  2047
#define CAMERIC_ISP_2DNR_SIGMA_SQRT_MAX         4095

#define CAMERIC_ISP_2DNR_BLEND_STRENGTH_OFFSET_MAX         2047
#define CAMERIC_ISP_2DNR_BLEND_STRENGTH_MAX_MAX            2047
#define CAMERIC_ISP_2DNR_BLEND_STRENGTH_SLOPE_MAX          255

#define CAMERIC_2DNR_LUMA_CURVE_SIZE             12         //bin number of dnr_luma_curve
#define CAMERIC_ISP_2DNR_SIGMA_BIN               60

#define DENOISE2D_SIGMA_Y_NUM 60

typedef struct CamerIcIsp2DnrConfig_s {

    uint32_t  sigmaSqr;

    uint16_t weight;
    uint16_t strengthSlope;
    uint16_t strengthMax;
    uint16_t strengthOffset;
    uint16_t    sigmaYLut[CAMERIC_ISP_2DNR_SIGMA_BIN];          /**< denoise sigma Y */
} CamerIcIsp2DnrConfig_t;

/*****************************************************************************/
/**
*          CamerIcIsp2DnrEnable()
*
* @brief   Enable 2DNR Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp2DnrEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp2DnrDisable()
*
* @brief   Disable 2DNR Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp2DnrDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp2DnrSetConfig()
*
* @brief   Config 2DNR Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp2DnrSetConfig
(
    CamerIcDrvHandle_t         handle,
    CamerIcIsp2DnrConfig_t      *p2DnrCfg
);

/*****************************************************************************/
/**
*          CamerIcIsp2DnrGetConfig()
*
* @brief   Config 2DNR Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp2DnrGetConfig
(
    CamerIcDrvHandle_t         handle,
    CamerIcIsp2DnrConfig_t      *p2DnrCfg
);

#ifdef __cplusplus
}
#endif

/* @} cameric_isp_2dnr_drv_api */

/* @endcond */
#endif /* CAMERIC_ISP_2DNR_DRV_API_H */


