/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMERIC_ISP_3DNR3_DRV_API_H
#define CAMERIC_ISP_3DNR3_DRV_API_H

/**
* @file    cameric_isp_3dnr_drv_api.h
*
* @brief   This file contains the CamerIC ISP TDNR driver API definitions.
*
*****************************************************************************/
/**
* @cond cameric_isp_3dnr
*
* @defgroup cameric_isp_3dnr_drv_api CamerIC ISP TDNR Driver API definitions
* @{
*
*/
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif
typedef struct CamerIcIsp3Dnr3Config_s {

    uint16_t noiseLevel;   // [0, 100]  -> [0, 65535]

    uint16_t noiseMean;
    uint16_t motionMean;
    uint16_t noiseThreshold;

    uint16_t rangeH;
    uint16_t rangeV;
    uint16_t dilateRangeH;
    uint16_t dilateRangeV;

    uint8_t  diffShiftBit;
    uint8_t  valShiftBit;
    uint8_t  motionConvShift;
    uint16_t motionConvMax;

    uint32_t motionInvFactor;

    uint8_t sadWeight;
    uint8_t  preWeight;
    uint16_t motionSlope;

    uint32_t filterLen;
    uint32_t filterLen2;
} CamerIcIsp3Dnr3Config_t;


typedef struct CamerIcIsp3dnr3LumaConfig_s {
    uint8_t pixelSlopMin;
    uint8_t pixelSlopShift;
    uint8_t pixelSlop;
    uint8_t pixelValShift;

} CamerIcIsp3dnr3LumaConfig_t;


typedef struct CamerIcIsp3dnr3Stats_s{
    uint32_t bgSum;
    uint32_t motionSum;
    uint32_t bgPixelCnt;
    uint32_t motionPixelCnt;
    uint32_t frameAvg;
} CamerIcIsp3dnr3Stats_t;



/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3LumaSetConfig()
*
* @brief   set 3DNRv3 luma
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3LumaSetConfig
(
    CamerIcDrvHandle_t handle,
    const CamerIcIsp3dnr3LumaConfig_t *pLumaCfg
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3LumaGetConfig()
*
* @brief   get 3DNRv3 luma Module config
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3LumaGetConfig
(
    CamerIcDrvHandle_t handle,
    CamerIcIsp3dnr3LumaConfig_t *pLumaCfg

);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3LumaEnable()
*
* @brief   Enable 3DNRv3 luma Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3LumaEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3LumaDisable()
*
* @brief   disable 3DNRv3 luma Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3LumaDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3MotionErosionEnable()
*
* @brief   Enable 3DNRv3 motion erosion Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3MotionErosionEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3MotionErosionDisable()
*
* @brief   disable 3DNRv3 motion erosion Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3MotionErosionDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3MotionConverageEnable()
*
* @brief   Enable 3DNRv3 motion converage Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3MotionConverageEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3MotionConverageDisable()
*
* @brief   disable 3DNRv3 motion converage Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3MotionConverageDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3MotionDilationEnable()
*
* @brief   Enable 3DNRv3 motion dilation Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3MotionDilationEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3MotionDilationDisable()
*
* @brief   disable 3DNRv3 motion dilation Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3MotionDilationDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3NlmEnable()
*
* @brief   Enable 3DNRv3 Nlm Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3NlmEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3NlmDisable()
*
* @brief   disable 3DNRv3 Nlm Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3NlmDisable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3Enable()
*
* @brief   Enable 3DNRv3 Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3Enable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3Disable()
*
* @brief   disable 3DNRv3 Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3Disable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3SwReset()
*
* @brief  software reset 3DNRv3 Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3SwReset
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3SetUpdateFactors()
*
* @brief   update  3DNR Module threshold update & motion threshold update factor
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3SetUpdateFactor
(
    CamerIcDrvHandle_t  handle,
    uint16_t updateFactor,
    uint16_t motionUpdateFactor
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3GetUpdateFactors()
*
* @brief   get  3DNR Module threshold update & motion threshold update factor
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3GetUpdateFactor
(
    CamerIcDrvHandle_t  handle,
    uint16_t *pUpdateFactor,
    uint16_t *pMotionUpdateFactor
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3SetStrength()
*
* @brief   set 3DNR Module's denoiseStrength
*
* @param   handle          CamerIc driver handle
* @param   strength
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3SetStrength(
    CamerIcDrvHandle_t  handle,
    uint8_t strength
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3GetStrength()
*
* @brief   get 3DNR Module's denoiseStrength
*
* @param   handle          CamerIc driver handle
* @param   pStrength
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3GetStrength(
    CamerIcDrvHandle_t  handle,
    uint8_t *pStrength
);


/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3GetConfig()
*
* @brief   get 3DNR Module's config
*
* @param   handle          CamerIc driver handle
* @param   pStrength
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3GetConfig(
    CamerIcDrvHandle_t  handle,
    CamerIcIsp3Dnr3Config_t *pConfig
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3SetConfig()
*
* @brief   Set 3DNR Module's config
*
* @param   handle          CamerIc driver handle
* @param   pStrength
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3SetConfig(
    CamerIcDrvHandle_t  handle,
    const CamerIcIsp3Dnr3Config_t *pConfig
);

/*****************************************************************************/
/**
*          RESULT CamerIcIsp3Dnr3GetStats()
*
* @brief   get 3dnr Module's statistics
*
* @param   handle          CamerIc driver handle
* @param   denoiseStrength strength
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3GetStats(
    CamerIcDrvHandle_t  handle,
    CamerIcIsp3dnr3Stats_t *pStats
);


/*****************************************************************************/
/**
*          CamerIcIsp3DnrReset()
*
* @brief   set  3DNRv3 hardware reference reset bit
*
* @param   handle          CamerIc driver handle
*           reset          0:clear reset 1:reset
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
void CamerIcIsp3DnrReset(
    CamerIcDrvHandle_t handle,
    bool_t reset
);


/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3SetDummyBlank()
*
* @brief   set  3DNRv3 hardware DummyBlank
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3SetDummyBlank
(
    CamerIcDrvHandle_t handle,
    uint16_t dummy
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3GetDummyBlank()
*
* @brief   get  3DNRv3 hardware DummyBlank
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3GetDummyBlank
(
    CamerIcDrvHandle_t handle,
    uint16_t *pDummy
);


/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3SetGammaCurve()
*
* @brief   set  3DNRv3 pregamma and inv gamma curve
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3SetGammaCurve
(
    CamerIcDrvHandle_t handle,
    uint16_t        preGamma[33],
    uint16_t        invGamma[33]
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3GetGammaCurve
*
* @brief   get  3DNRv3 hardware pregamma and inv gamma curve
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3GetGammaCurve
(
    CamerIcDrvHandle_t handle,
    uint16_t*        pPreGamma,
    uint16_t*        pInvGamma
);
/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3InvGammaEnable()
*
* @brief   Enable 3DNRv3 inv gamma Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3InvGammaEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3InvGammaDisable()
*
* @brief   disable 3DNRv3 inv gamma Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3InvGammaDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3PreGammaEnable()
*
* @brief   Enable 3DNRv3 pre gamma Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3PreGammaEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIsp3Dnr3PreGammaDisable()
*
* @brief   disable 3DNRv3 pre gamma Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIsp3Dnr3PreGammaDisable
(
    CamerIcDrvHandle_t handle
);
#ifdef __cplusplus
}
#endif

/* @} cameric_isp_3dnr_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_3DNR3_DRV_API_H */

