/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CPROC_H
#define CPROC_H

/**
 * @file cproc.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup CPROC module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include <oslayer/oslayer.h>
#include <cameric_drv/cameric_drv_api.h>
#include <cameric_drv/cameric_isp_cproc_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define ACPROC_HUE_MAX            87.188f
#define ACPROC_HUE_MIN           -90.0f
#define ACPROC_CONTRAST_MAX      1.9921875f
#define ACPROC_CONTRAST_MIN       0.0f
#define ACPROC_BRIGHT_MAX         127
#define ACPROC_BRIGHT_MIN        -128
#define ACPROC_SATURATION_MAX     1.9921875f
#define ACPROC_SATURATION_MIN     0.0f

/*****************************************************************************/
/**
 *          ACprocHandle_t
 *
 * @brief   Cproc Module instance handle
 *
 *****************************************************************************/
typedef void *ACprocHandle_t;         /**< handle to Cproc context */

/*****************************************************************************/
/**
 *          ACprocInstanceConfig_t
 *
 * @brief   Cproc Module instance configuration structure
 *
 *****************************************************************************/
typedef struct ACprocInstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */
    ACprocHandle_t          hACproc;            /**< handle returned by ACprocInit() */
} ACprocInstanceConfig_t;


/*****************************************************************************/
/**
 *          ACprocConfig_t
 *
 * @brief   Cproc Module configuration structure
 *
 *****************************************************************************/
typedef struct ACprocConfig_s
{
    float32_t				bright;			/**< bright */
    float32_t				contrast;		/**< contrast */
    float32_t				hue;			/**< hue */
    float32_t				saturation;	    /**< saturation */
#ifdef ES_API
    bool_t              bCtModeEn;//0:contrast affect brightness
#endif	
} ACprocConfig_t;

/*****************************************************************************/
/**
 *          ACprocAutoParamTable_t
 *
 * @brief   Cproc Module parameters table structure
 *
 *****************************************************************************/
typedef struct ACprocAutoParamTable_s
{
    uint8_t autoLevel;
    float32_t gains[AUTO_NUM];
    float32_t contrast[AUTO_NUM];
    float32_t bright[AUTO_NUM];
    float32_t saturation[AUTO_NUM];
    float32_t hue[AUTO_NUM];
} ACprocAutoParamTable_t;


/*****************************************************************************/
/**
 *          ACprocInit()
 *
 * @brief   This function initializes the cproc
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ACprocInit
(
    ACprocInstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          ACprocDeinit()
 *
 * @brief   The function deinit the cproc
 *
 * @param   handle  Handle to Cproc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACprocDeinit
(
    ACprocHandle_t handle
);


/*****************************************************************************/
/**
 *          ACprocSetConfigure()
 *
 * @brief   This function configures the cproc module
 *
 * @param   handle   Handle to Cproc 
 * @param   pConfig  cproc config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ACprocSetConfigure
(
    ACprocHandle_t handle,
    ACprocConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ACprocGetConfigure()
 *
 * @brief   This function get the cproc module configures
 *
 * @param   handle  Handle to Cproc
 * @param   pConfig cproc config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ACprocGetConfigure
(
    ACprocHandle_t handle,
    ACprocConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ACprocSetAutoParamTable()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle         Handle to Cproc
 * @param   pParamTable    auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ACprocSetAutoParamTable
(
    ACprocHandle_t handle,
    ACprocAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          ACprocGetAutoParamTable()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle        Handle to Cproc
 * @param   pParamTable   auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ACprocGetAutoParamTable
(
    ACprocHandle_t handle,
    ACprocAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          ACprocSetMode()
 *
 * @brief   The function set cproc mode
 *
 * @param   handle  Handle to Cproc
 * @param   mode    Cproc mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACprocSetMode
(
    ACprocHandle_t 		handle,
    const ARunMode_t	mode
);

/*****************************************************************************/
/**
 *          ACprocGetMode()
 *
 * @brief   The function get cproc mode
 *
 * @param   handle  Handle to Cproc
 * @param   mode    Cproc mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACprocGetMode
(
    ACprocHandle_t 		handle,
    ARunMode_t	        *mode
);

/*****************************************************************************/
/**
 *          ACprocEnable()
 *
 * @brief   enable cproc
 *
 * @param   handle  Handle to Cproc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACprocEnable
(
    ACprocHandle_t 		handle
);

/*****************************************************************************/
/**
 *          ACprocDisable()
 *
 * @brief   disable cproc
 *
 * @param   handle  Handle to Cproc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACprocDisable
(
    ACprocHandle_t 		handle
);


/*****************************************************************************/
/**
 *          ACprocGetStatus()
 *
 * @brief   The function returns the status of the Auto wide dynamic range v3
 *          module
 *
 * @param   handle  Handle to Cproc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACprocGetStatus
(
    ACprocHandle_t 		handle,
    bool_t       		*pRunning,
    ARunMode_t	        *pMode,
    ACprocConfig_t 	    *pCurrentParam
);



/*****************************************************************************/
/**
 *          ACprocProcessFrame()
 *
 * @brief   cproc process frame
 *
 * @param   handle  Handle to Cproc
 *          gain    current sensor-gain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACprocProcessFrame
(
    ACprocHandle_t    	handle,
    const float32_t		fGain
);

#ifdef __cplusplus
}
#endif


/* @} Cproc */


#endif /* CPROC_H*/
