/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef DEMOSAIC2_H
#define DEMOSAIC2_H

/**
 * @file demosaic2.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 *  @ref module_name
 *
 * @defgroup DEMOSAIC2 module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define DEMOSAIC2_AUTO_NUM 20

#define DMSC2_DIRECTION_THRESHOLD_MAX 4095

#define DMSC2_DEMOIRE_THR_MAX 32
#define DMSC2_DEMOIRE_THR_MIN  0

#define DMSC2_DEMOIRE_SHRINK_MAX 32
#define DMSC2_DEMOIRE_SHRINK_MIN  0

#define DMSC2_DEMOIRE_R_MAX 256
#define DMSC2_DEMOIRE_R_MIN  0

#define DMSC2_DEMOIRE_T2_SHIFT_MAX 8
#define DMSC2_DEMOIRE_T2_SHIFT_MIN  0

#define DMSC2_DEMOIRE_EDGE_T1_MAX 256
#define DMSC2_DEMOIRE_EDGE_T1_MIN  0

#define DMSC2_SHARPEN_FACTOR_MAX 511
#define DMSC2_SHARPEN_FACTOR_MIN  0

#define DMSC2_SHARPEN_CLIP_MAX 2047
#define DMSC2_SHARPEN_CLIP_MIN  0

#define DMSC2_SHARPEN_T_MAX 2047
#define DMSC2_SHARPEN_T_MIN  0

#define DMSC2_SHARPEN_T_SHIFT_MAX 11
#define DMSC2_SHARPEN_T_SHIFT_MIN  0

#define DMSC2_SHARPEN_R_MAX 256
#define DMSC2_SHARPEN_R_MIN  0

#define DMSC2_SHARPEN_SIZE_MAX 16
#define DMSC2_SHARPEN_SIZE_MIN  0

#define DMSC2_DENOISE_STR_MAX 32
#define DMSC2_DENOISE_STR_MIN  0

#define DMSC2_FILTER_MAX  63
#define DMSC2_FILTER_MIN  0

#define DMSC2_SHARPEN_LINE_T_MAX 2047
#define DMSC2_SHARPEN_LINE_T_MIN  0

#define DMSC2_SHARPEN_LINE_T_SHIFT_MAX 10
#define DMSC2_SHARPEN_LINE_T_SHIFT_MIN  0

#define DMSC2_SHARPEN_LINE_R_MAX 256
#define DMSC2_SHARPEN_LINE_R_MIN  0

#define DMSC2_SHARPEN_LINE_STR_MAX 4095
#define DMSC2_SHARPEN_LINE_STR_MIN  0

#define DMSC2_SKIN_CBCR_MIN   -2047
#define DMSC2_SKIN_CBCR_MAX    2047

#define DMSC2_SKIN_Y_MAX  4095

#define DMSC2_DEPURPLE_SAT_SHRINK_MAX 8
#define DMSC2_DEPURPLE_SAT_SHRINK_MIN  0

#define DMSC2_DEPURPLE_THR_MIN  8

#define DMSC2_CAC_CHROMATIC_ABERRATION_MAX 15.9375f
#define DMSC2_CAC_CHROMATIC_ABERRATION_MIN -16.0000f

/*****************************************************************************/
/**
 * @brief
 *
 */
/*****************************************************************************/
typedef enum ADemosaic2RunMode_e
{
    DEMOSAIC2_MODE_INVALID                    = 0,        /**< initialization value */
    DEMOSAIC2_MODE_MANUAL                     = 1,        /**< run manual demosaic2*/
    DEMOSAIC2_MODE_AUTO                       = 2,        /**< run auto demosaic2 */
    DEMOSAIC2_MODE_MAX
} ADemosaic2Mode_t;


/*****************************************************************************/
/**
 *          ADemosaic2Handle_t
 *
 * @brief   Demosaic2 Module instance handle
 *
 *****************************************************************************/
typedef void *ADemosaic2Handle_t;         /**< handle to Demosaic2 context */

/*****************************************************************************/
/**
 *          ADemosaic2InstanceConfig_t
 *
 * @brief   Demosaic2 Module instance configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2InstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */

    ADemosaic2Handle_t      hADemosaic2;            /**< handle returned by ADemosaic2Init() */
} ADemosaic2InstanceConfig_t;


/*****************************************************************************/
/**
 *          ADemosaic2Config_t
 *
 * @brief   Demosaic2 Module configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2Config_s
{
    uint8_t        threshold;    	/**< Threshold for Bayer demosaicing texture detection.This value shifted 4bit is*/
                                            /**< compared width the different of the vertical and horizontal 12bit wide texture */
                                            /**<indicators, to decide if the vertical or horizontal texture flag must be set. */
                                            /**<0x00: maximum edge sensitivity*/
                                            /**<0xff: no texture detection */
    uint16_t    	dirThrMax;    /**< direction select threshold value in bright area, the interpolation operation */
                                        /**<between maximum threshold and minimal threshold */
    uint16_t    	dirThrMin;    /**< direction select threshold value in dark area, the interpolation operation*/
                                        /**<between maximum threshold and minimal threshold */
} ADemosaic2Config_t;

/*****************************************************************************/
/**
 *          ADemosaic2DemoireConfig_t
 *
 * @brief   demoire Module configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2DemoireConfig_s
{
    uint16_t dmscDemoireR1;
    uint16_t dmscDemoireR2;
    uint16_t demoireEdgeR1;
    uint16_t demoireEdgeR2;
    uint16_t demoireEdgeT1;
    uint8_t demoireEdgeT2Shift;
    uint8_t dmscDemoireAreaThr;
    uint8_t dmscDemoireSatShrink;
    uint8_t dmscDemoireT1;
    uint8_t dmscDemoireT2Shift;
} ADemosaic2DemoireConfig_t;


/*****************************************************************************/
/**
 * @brief  demosaic2 cac configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2CacConfig_s{
    float32_t aBlue; //calib
    float32_t aRed;  //calib
    float32_t bBlue; //calib
    float32_t bRed;  //calib
    float32_t cBlue; //calib
    float32_t cRed;  //calib
    int16_t centerHOffs; //calib
    int16_t centerVOffs; //calib    
} ADemosaic2CacConfig_t;



/*****************************************************************************/
/**
 * @brief  demosaic2 dmscSharpen configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2SharpenConfig_s {

    uint16_t dmscSharpenFactorBlack;                    /**<[0, 511] Sharpen strength*/
    uint16_t dmscSharpenFactorWhite;                    /**<[0, 511] Sharpen strength*/
    uint16_t dmscSharpenClipBlack;                  /**<[0, 2047] Max Sharpen strength*/
    uint16_t dmscSharpenClipWhite;                  /**<[0, 2047] Max Sharpen strength*/
    uint16_t dmscSharpenT4Shift;                    /**<[0, 11] T4 = t3 + 2^t4_shift*/
    uint16_t dmscSharpenT3;                 /**<[0, 2047]Linear interpolate in [r1, r2, r3, r4]*/
    uint8_t  dmscSharpenT2Shift;                 /**<[0, 11] T2 = t1 + 2^t2_shift*/
    uint16_t dmscSharpenT1;                 /**<[0, 2047] */
                                        /**<Threshold of texture value, control to dmscSharpen ratio*/
                                        /**< Use r1, if texture_val< t1*/
    uint16_t dmscSharpenR3;                 /**<[0, 511]*/
    uint16_t dmscSharpenR2;                 /**<[0, 511]*/
    uint16_t dmscSharpenR1;                 /**<[0, 511]*/
    uint8_t  dmscSharpenSize;                   /**<[0, 511]*/
    uint32_t dmscDenoise;                   /**<[0, 511]*/
} ADemosaic2SharpenConfig_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 sharpen line configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2SharpenLineConfig_s {
	//uint8_t sharpenLineShift2;  //not used
    uint8_t  dmscSharpenLineThrShift1;         /**<[0, 10]     */
    uint16_t dmscSharpenLineThr;             /**<[0, 2047]     */
    uint16_t dmscSharpenLineStrength;       /**<[0, 4095] Weight to merge sharpen_line image into output image     */
    uint16_t dmscSharpenLineR2;             /**< [0, 256] Linear interpolate in [r1, r2]    */
    uint16_t dmscSharpenLineR1;             /**<[0, 256]     */
} ADemosaic2SharpenLineConfig_t;


/*****************************************************************************/
/**
 * @brief  demosaic2 skin configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2SkinConfig_s {
    int16_t dmscSkinCbThrMax;    /**<[-2047, 2047]*/
    int16_t dmscSkinCbThrMin;    /**<[-2047, 2047]*/
    int16_t dmscSkinCrThrMax;    /**<[-2047, 2047]*/
    int16_t dmscSkinCrThrMin;    /**<[-2047, 2047]*/
    uint16_t dmscSkinYThrMax;     /**<[0, 4095]*/
    uint16_t dmscSkinYThrMin;     /**<[0, 4095]*/

} ADemosaic2SkinConfig_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 depurple cbcr mode enum
 *
 *****************************************************************************/
typedef enum ADmsc2DpulCbCrMode_e {
     ADMSC2_DEPURPLE_CB_CR_MODE_DISABLE = 0,
     ADMSC2_DEPURPLE_CR_CHANNEL,
     ADMSC2_DEPURPLE_CB_CHANNEL,
     ADMSC2_DEPURPLE_CB_CR_CHANNLE,
     ADMSC2_DEPURPLE_MAX,
}ADmsc2DpulCbCrMode_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 depurple configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2DepurpleConfig_s {
    ADmsc2DpulCbCrMode_t dmscDepurpleCbcrMode;  /**<{0, 1, 2, 3} Reduce saturation on cb/cr channel*/
    uint8_t dmscDepurpleSatShrink;          /**<[0, 8] Reduce saturation strength*/
    uint8_t dmscDepurpleThr;                /**<[0, 255]Threshold to judge purple aera*/

} ADemosaic2DepurpleConfig_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 status configuration structure
 *
 *****************************************************************************/
typedef struct ADemosaic2StatusConfig_s
{
    bool_t                     dmscv2Running;
    ADemosaic2Config_t         dmscv2Cfg;
    bool_t                     dmscv2CacRunning;
    ADemosaic2CacConfig_t      dmscv2CacCfg;

    bool_t                     dmscv2DemoireRunning;
    ADemosaic2DemoireConfig_t  dmscv2DemoireCfg;
    bool_t                     dmscv2SharpenRunning;
    ADemosaic2SharpenConfig_t  dmscv2SharpenCfg;
    bool_t                     dmscv2SharpenLineRunning;
    ADemosaic2SharpenLineConfig_t dmscv2SharpenLineCfg;
    bool_t                     dmscv2SkinRunning;
    ADemosaic2SkinConfig_t     dmscv2SkinCfg;
    bool_t                     dmscv2DepurpleRunning;
    ADemosaic2DepurpleConfig_t dmscv2DepurpleCfg;
} ADemosaic2StatusConfig_t;


/*****************************************************************************/
/**
 *          ADemosaic2AutoParamTable_t
 *
 * @brief   Demosaic2 Module parameters table structure
 *
 *****************************************************************************/
typedef struct ADemosaic2AutoParamTable_s
{
    uint8_t autoLevel;
    float32_t gains[AUTO_NUM];
    uint8_t dmscDenoiseStrength[AUTO_NUM];

    bool_t dmscSharpenEnable[AUTO_NUM];
    uint16_t dmscSharpenFactorBlack[AUTO_NUM];
    uint16_t dmscSharpenFactorWhite[AUTO_NUM];
    uint16_t dmscSharpenClipBlack[AUTO_NUM];
    uint16_t dmscSharpenClipWhite[AUTO_NUM];
    uint16_t dmscSharpenT1[AUTO_NUM];
    uint8_t dmscSharpenT2Shift[AUTO_NUM];
    uint16_t dmscSharpenT3[AUTO_NUM];
    uint8_t dmscSharpenT4Shift[AUTO_NUM];
    uint8_t dmscSharpenSize[AUTO_NUM];
    uint16_t dmscSharpenR1[AUTO_NUM];
    uint16_t dmscSharpenR2[AUTO_NUM];
    uint16_t dmscSharpenR3[AUTO_NUM];

    bool_t dmscDepurpleEnable[AUTO_NUM];
    uint8_t dmscDepurpleSatShrink[AUTO_NUM];
	uint8_t dmscDepurpleCbcrMode[AUTO_NUM];
    uint8_t dmscDepurpleThr[AUTO_NUM];
} ADemosaic2AutoParamTable_t;

/*****************************************************************************/
/**
 *          ADemosaic2Init()
 *
 * @brief   This function initializes the demosaic2
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADemosaic2Init
(
    ADemosaic2InstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          ADemosaic2Deinit()
 *
 * @brief   The function deinit the demosaic2
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2Deinit
(
    ADemosaic2Handle_t handle
);


/*****************************************************************************/
/**
 *          ADemosaic2SetConfigure()
 *
 * @brief   This function configures the demosaic2 module
 *
 * @param   handle   Handle to Demosaic2 
 * @param   pConfig  demosaic2 config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2SetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2Config_t *pConfig
);


/*****************************************************************************/
/**
 *          ADemosaic2GetConfigure()
 *
 * @brief   This function get the demosaic2 module configures
 *
 * @param   handle  Handle to Demosaic2
 * @param   pConfig demosaic2 config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2GetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2Config_t *pConfig
);


/*****************************************************************************/
/**
 *          ADemosaic2SetAutoParamTable()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle         Handle to Demosaic2
 * @param   pParamTable    auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2SetAutoParamTable
(
    ADemosaic2Handle_t handle,
    ADemosaic2AutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          ADemosaic2GetAutoParamTable()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle        Handle to Demosaic2
 * @param   pParamTable   auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2GetAutoParamTable
(
    ADemosaic2Handle_t handle,
    ADemosaic2AutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          ADemosaic2SetMode()
 *
 * @brief   The function set demosaic2 mode
 *
 * @param   handle  Handle to Demosaic2
 * @param   mode    Demosaic2 mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2SetMode
(
    ADemosaic2Handle_t     handle,
    const ADemosaic2Mode_t	mode
);

/*****************************************************************************/
/**
 *          ADemosaic2GetMode()
 *
 * @brief   The function get demosaic2 mode
 *
 * @param   handle  Handle to Demosaic2
 * @param   mode    Demosaic2 mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2GetMode
(
    ADemosaic2Handle_t     handle,
    ADemosaic2Mode_t	    *mode
);

/*****************************************************************************/
/**
 *          ADemosaic2Enable()
 *
 * @brief   enable demosaic2
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2Enable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2Disable()
 *
 * @brief   disable demosaic2
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2Disable
(
    ADemosaic2Handle_t     handle
);


/*****************************************************************************/
/**
 *          ADemosaic2DemoireSetConfigure()
 *
 * @brief   This function configures the demosaic2 demoire module
 *
 * @param   handle   Handle to Demosaic2 
 * @param   pConfig  demosaic2 demoire config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2DemoireSetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2DemoireConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ADemosaic2DemoireGetConfigure()
 *
 * @brief   This function get the demosaic2 demoire module configures
 *
 * @param   handle  Handle to Demosaic2
 * @param   pConfig demosaic2 demoire config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2DemoireGetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2DemoireConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ADemosaic2DemoireEnable()
 *
 * @brief   enable demosaic2 demoire
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2DemoireEnable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2DemoireDisable()
 *
 * @brief   disable demosaic2 demoire
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2DemoireDisable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2SharpenSetConfigure()
 *
 * @brief   This function configures the demosaic2 sharpen module
 *
 * @param   handle   Handle to Demosaic2 
 * @param   pConfig  demosaic2 sharpen config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2SharpenSetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2SharpenConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ADemosaic2SharpenGetConfigure()
 *
 * @brief   This function get the demosaic2 sharpen module configures
 *
 * @param   handle  Handle to Demosaic2
 * @param   pConfig demosaic2 sharpen config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2SharpenGetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2SharpenConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ADemosaic2SharpenEnable()
 *
 * @brief   enable demosaic2 sharpen
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2SharpenEnable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2SharpenDisable()
 *
 * @brief   disable demosaic2 sharpen
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2SharpenDisable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2SharpenLineSetConfigure()
 *
 * @brief   This function configures the demosaic2 sharpenLine module
 *
 * @param   handle   Handle to Demosaic2 
 * @param   pConfig  demosaic2 sharpenLine config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2SharpenLineSetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2SharpenLineConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ADemosaic2SharpenLineGetConfigure()
 *
 * @brief   This function get the demosaic2 sharpenLine module configures
 *
 * @param   handle  Handle to Demosaic2
 * @param   pConfig demosaic2 sharpenLine config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2SharpenLineGetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2SharpenLineConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ADemosaic2SharpenLineEnable()
 *
 * @brief   enable demosaic2 sharpenLine
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2SharpenLineEnable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2SharpenLineDisable()
 *
 * @brief   disable demosaic2 sharpenLine
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2SharpenLineDisable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2SkinSetConfigure()
 *
 * @brief   This function configures the demosaic2 Skin module
 *
 * @param   handle   Handle to Demosaic2 
 * @param   pConfig  demosaic2 Skin config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2SkinSetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2SkinConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ADemosaic2SkinGetConfigure()
 *
 * @brief   This function get the demosaic2 Skin module configures
 *
 * @param   handle  Handle to Demosaic2
 * @param   pConfig demosaic2 Skin config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2SkinGetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2SkinConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ADemosaic2SkinEnable()
 *
 * @brief   enable demosaic2 Skin
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2SkinEnable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2SkinDisable()
 *
 * @brief   disable demosaic2 Skin
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2SkinDisable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2DepurpleSetConfigure()
 *
 * @brief   This function configures the demosaic2 Depurple module
 *
 * @param   handle   Handle to Demosaic2 
 * @param   pConfig  demosaic2 Depurple config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2DepurpleSetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2DepurpleConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ADemosaic2DepurpleGetConfigure()
 *
 * @brief   This function get the demosaic2 Depurple module configures
 *
 * @param   handle  Handle to Demosaic2
 * @param   pConfig demosaic2 Depurple config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADemosaic2DepurpleGetConfigure
(
    ADemosaic2Handle_t handle,
    ADemosaic2DepurpleConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ADemosaic2DepurpleEnable()
 *
 * @brief   enable demosaic2 Depurple
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2DepurpleEnable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2DepurpleDisable()
 *
 * @brief   disable demosaic2 Depurple
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2DepurpleDisable
(
    ADemosaic2Handle_t     handle
);

/*****************************************************************************/
/**
 *          ADemosaic2GetStatus()
 *
 * @brief   The function returns the status of the Auto wide dynamic range v3
 *          module
 *
 * @param   handle  Handle to Demosaic2
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2GetStatus
(
    ADemosaic2Handle_t           handle,
    ADemosaic2Mode_t             *pMode,
    ADemosaic2StatusConfig_t     *pCurrentParam
);

/*****************************************************************************/
/**
 *          ADemosaic2ProcessFrame()
 *
 * @brief   demosaic2 process frame
 *
 * @param   handle  Handle to Demosaic2
 *          gain    current sensorgain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADemosaic2ProcessFrame
(
    ADemosaic2Handle_t      handle,
    const float32_t         fGain
);


/*****************************************************************************/
/**
 *          ADmscv2CacEnable()
 *
 * @brief   enable the demosaic2 cac
 *
 * @param   handle  Handle to Demosaic2      
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADmscv2CacEnable
(
    ADemosaic2Handle_t handle
);


/*****************************************************************************/
/**
 *          ADmscv2CacDisable()
 *
 * @brief   disable the demosaic2 cac
 *
 * @param   handle  Handle to Demosaic2      
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADmscv2CacDisable
(
    ADemosaic2Handle_t         handle
);

/*****************************************************************************/
/**
 *          ADmscv2CacSetConfigure()
 *
 * @brief   set config the demosaic2 cac
 *
 * @param   handle  Handle to Demosaic2
 * @param   Dmscv2CacConfig the demosaicv2 cac config      
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADmscv2CacSetConfigure
(
    ADemosaic2Handle_t         handle,
    ADemosaic2CacConfig_t         *pConfig
);


/*****************************************************************************/
/**
 *          ADmscv2CacGetConfigure()
 *
 * @brief   get config the demosaic2 cac
 *
 * @param   handle  Handle to Demosaic2
 * @param   Dmscv2CacConfig the demosaicv2 cac config      
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADmscv2CacGetConfigure
(
    ADemosaic2Handle_t         handle,
    ADemosaic2CacConfig_t         *pConfig
);

#ifdef __cplusplus
}
#endif


/* @} Demosaic2 */


#endif /* DEMOSAIC2_H*/
