/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef EECA_H
#define EECA_H

/**
 * @file ca.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup EECA module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"

#include <cameric_drv/cameric_drv_api.h>
#include "ee.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define CURVE_DATA_TABLE_LEN 65

#define CA_CURVE_LUT_TABLE_MAX 1024
/*****************************************************************************/
/**
 *          AEeCaConfig_t
 *
 * @brief   Ca Module configuration structure
 *
 *****************************************************************************/
typedef struct AEeCaConfig_s
{
    uint8_t caMode;
    uint16_t lutChromaTbl[CURVE_DATA_TABLE_LEN];
} AEeCaConfig_t;

typedef struct AEeCurveDx_s
{
    uint16_t lutXTbl[CURVE_DATA_TABLE_LEN];
    uint16_t lutShiftTbl[CURVE_DATA_TABLE_LEN];
} AEeCurveDx_t;

/*****************************************************************************/
/**
 *          AEeSetCurveDx()
 *
 * @brief   This function configures the ee module
 *
 * @param   handle   Handle to ee 
 * @param   pCurveDx  ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeSetCurveDx
(
    AEeHandle_t         handle,
    AEeCurveDx_t        *pCurveDx
);

/*****************************************************************************/
/**
 *          AEeCaSetConfigure()
 *
 * @brief   This function configures the ee module
 *
 * @param   handle   Handle to ee 
 * @param   pConfig  ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeCaSetConfigure
(
    AEeHandle_t handle,
    AEeCaConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          AEeCaGetConfigure()
 *
 * @brief   This function get the ee module configures
 *
 * @param   handle  Handle to ee
 * @param   pConfig ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeCaGetConfigure
(
    AEeHandle_t handle,
    AEeCaConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AEeCaSetCurrentConfigure()
 *
 * @brief   This function configures the ee module
 *
 * @param   handle   Handle to ee 
 * @param   pConfig  ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeCaSetCurrentConfigure
(
    AEeHandle_t handle,
    AEeCaConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AEeCaGetCurrentConfigure()
 *
 * @brief   This function get the ee module configures
 *
 * @param   handle  Handle to ee
 * @param   pConfig ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeCaGetCurrentConfigure
(
    AEeHandle_t handle,
    AEeCaConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AEeCaSetMode()
 *
 * @brief   The function set ee mode
 *
 * @param   handle  Handle to ee
 * @param   mode    ee mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeCaSetMode
(
    AEeHandle_t 		handle,
    const ARunMode_t	mode
);

/*****************************************************************************/
/**
 *          AEeCaGetMode()
 *
 * @brief   The function get ee mode
 *
 * @param   handle  Handle to ee
 * @param   mode    ee mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeCaGetMode
(
    AEeHandle_t 		handle,
    ARunMode_t	        *mode
);

/*****************************************************************************/
/**
 *          AEeCaEnable()
 *
 * @brief   enable ee
 *
 * @param   handle  Handle to ee
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeCaEnable
(
    AEeHandle_t 		handle
);

/*****************************************************************************/
/**
 *          AEeCaDisable()
 *
 * @brief   disable ee
 *
 * @param   handle  Handle to ee
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeCaDisable
(
    AEeHandle_t 		handle
);


/*****************************************************************************/
/**
 *          AEeCaGetStatus()
 *
 * @brief   The function returns the status of the edge enhancement module.
 *
 * @param   handle  Handle to ee
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeCaGetStatus
(
    AEeHandle_t handle,
    AEeCaConfig_t *pCaConfig,
    bool_t *pRunning
);



/*****************************************************************************/
/**
 *          AEeCaProcessFrame()
 *
 * @brief   ee process frame
 *
 * @param   handle  Handle to ee
 *          gain    current sensor-gain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeCaProcessFrame
(
    AEeHandle_t    	handle,
    const float32_t	fGain
);


#ifdef __cplusplus
}
#endif


/* @} ee */


#endif /* EECA_H*/
