/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef EEDCI_H
#define EEDCI_H

/**
 * @file ee.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup EEDCI module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include "ee.h"

#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CURVE_DATA_TABLE_LEN 65

#define DCI_CURVE_LUT_TABLE_MAX 1024

/*****************************************************************************/
/**
 *          AEeDciConfig_t
 *
 * @brief   ee dci Module configuration structure
 *
 *****************************************************************************/
typedef struct AEeDciConfig_s
{
    uint16_t lutLumaTbl[CURVE_DATA_TABLE_LEN];
} AEeDciConfig_t;

/*****************************************************************************/
/**
 *          AEeDciMode_t
 *
 * @brief   ee Module dci mode structure
 *
 *****************************************************************************/
typedef enum AEeDciMode_s
{
    AEEV2_DCI_MODE_DCI_CURVE = 0,
    AEEV2_DCI_MODE_CURVE_2SEC = 1,
    AEEV2_DCI_MODE_CURVE_3SEC = 2,
    AEEV2_DCI_MODE_CURVE_MAX
} AEeDciMode_t;

/*****************************************************************************/
/**
 *          AEeDciCurveConfig_t
 *
 * @brief   ee dci Module configuration structure
 *
 *****************************************************************************/
typedef struct AEeDciCurveConfig_s
{
    AEeDciMode_t dciMode;
    float32_t degamma;
    float32_t lowlumaIndex;
    float32_t middlelumaIndex;
    float32_t highlumaIndex;
    uint16_t inflctStartY;
    uint16_t inflct1X;
    uint16_t inflct1Y;
    uint16_t inflct2X;
    uint16_t inflct2Y;
    uint16_t inflctStopY;
} AEeDciCurveConfig_t;

/*****************************************************************************/
/**
 *          AEeDciStatusConfig_t
 *
 * @brief   ee dci Module status configuration structure
 *
 *****************************************************************************/
typedef struct AEeDciStatusConfig_s
{
    AEeDciConfig_t dciConfig;
    AEeDciCurveConfig_t dciCurveConfig;
} AEeDciStatusConfig_t;

/*****************************************************************************/
/**
 *          AEeDciSetConfigure()
 *
 * @brief   This function configures the ee module
 *
 * @param   handle   Handle to ee 
 * @param   pConfig  ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeDciSetConfigure
(
    AEeHandle_t handle,
    AEeDciConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          AEeDciGetConfigure()
 *
 * @brief   This function get the ee module configures
 *
 * @param   handle  Handle to ee
 * @param   pConfig ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeDciGetConfigure
(
    AEeHandle_t handle,
    AEeDciConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AEeDciSetDciCurveConfigure()
 *
 * @brief   This function configures the ee module
 *
 * @param   handle   Handle to ee 
 * @param   pConfig  ee dci curve config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeDciSetDciCurveConfigure
(
    AEeHandle_t handle,
    AEeDciCurveConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AEeDciGetDciCurveConfigure()
 *
 * @brief   This function configures the ee module
 *
 * @param   handle   Handle to ee 
 * @param   pConfig  ee dci curve config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeDciGetDciCurveConfigure
(
    AEeHandle_t handle,
    AEeDciCurveConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AEeDciSetCurrentConfigure()
 *
 * @brief   This function configures the ee module
 *
 * @param   handle   Handle to ee 
 * @param   pConfig  ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeDciSetCurrentConfigure
(
    AEeHandle_t handle,
    AEeDciConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AEeDciEnable()
 *
 * @brief   enable ee
 *
 * @param   handle  Handle to ee
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeDciEnable
(
    AEeHandle_t 		handle
);

/*****************************************************************************/
/**
 *          AEeDciDisable()
 *
 * @brief   disable ee
 *
 * @param   handle  Handle to ee
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeDciDisable
(
    AEeHandle_t 		handle
);


/*****************************************************************************/
/**
 *          AEeDciGetStatus()
 *
 * @brief   The function returns the status of the Auto wide dynamic range v3
 *          module
 *
 * @param   handle  Handle to ee
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeDciGetStatus
(
    AEeHandle_t handle,
    AEeDciStatusConfig_t *pDciConfig,
    bool_t *pRunning
);

#ifdef __cplusplus
}
#endif


/* @} ee */


#endif /* EEDCI_H*/
