/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @file hal_video_in_api.h
 *
 * @brief   Hardware video inAbstraction Layer
 *
 *          Encapsulates and abstracts services from different hardware platforms.
 *
 *****************************************************************************/

#ifndef __HAL_VIDEO_IN_API_H__
#define __HAL_VIDEO_IN_API_H__

#include <ebase/types.h>
#include <ebase/dct_assert.h>

#include <common/return_codes.h>
#include <oslayer/oslayer.h>

/******************************************************************************
 * inline implementations of API
 *****************************************************************************/
#include "hal_common.h"

/**
 * @defgroup HAL_VIDEO_IN API Hardware Abstraction Layer interface
 * @{
 * @brief  Encapsulates and abstracts services from different hardware platforms
 *         in a hardware platform independent way.
 *****************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif
typedef void* HalVideoInHandle_t;

/*****************************************************************************/
/**
 * @brief   open the low level driver
 * @return  handle of driver; NULL on failure
 *
 * @note    Only supports a single video in device instance.
 *****************************************************************************/
HalVideoInHandle_t HalVideoInOpen(uint8_t id);

/*****************************************************************************/
/**
 * @brief   close the low level driver
 * @param   HalMemHandle   Handle to HAL session as returned by @ref HalVideoInOpen.
 * @return  Result of operation.
 *
 * @note    Decrements internal ref count and closes low level driver if ref count reached zero.
 *****************************************************************************/
RESULT HalVideoInClose(HalVideoInHandle_t handle);



/*****************************************************************************/
/**
 * @brief   writes a value to the given address
 * @param   HalHandle   Handle to HAL session as returned by @ref HalOpen.
 * @param   reg_address Address of register to write.
 * @param   value       Value to write into register.
 * @return  none
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
 void HalVideoInWriteReg( HalHandle_t handle, uint32_t reg_address, uint32_t value );

/*****************************************************************************/
/**
 * @brief   reads a value from a given address
 * @param   HalHandle   Handle to HAL session as returned by @ref HalOpen.
 * @param   reg_address Address of register to read.
 * @return  Register value.
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
 uint32_t HalVideoInReadReg( HalHandle_t handle, uint32_t reg_address );


/******************************************************************************
 * HalVideoInConnectIrq()
 *****************************************************************************/
RESULT HalVideoInConnectIrq
(
    HalHandle_t HalHandle,
    HalIrqCtx_t *pIrqCtx,
    uint32_t    int_src,
    osIsrFunc   IsrFunction,
    osDpcFunc   DpcFunction,
    void*       pContext
);

/******************************************************************************
 * HalVideoInDisconnectIrq()
 *****************************************************************************/
RESULT HalVideoInDisconnectIrq
(
    HalIrqCtx_t *pIrqCtx
);
/******************************************************************************
 * stuff below here requires the inline API implementations being included
 * as it requires some more hal variant depended header files being loaded
 *****************************************************************************/

#ifdef __cplusplus
}
#endif

//!@} defgroup HAL_API

#endif /* __HAL_VIDEO_IN_API_H__ */
