/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef ISPCMDBUFFERDEV_H
#define ISPCMDBUFFERDEV_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/list.h>

#include <oslayer/oslayer.h>
#include "isp_cmd_buffer_api.h"
#include "hal_common.h"

#define REG_ADDR(x)  ((uint32_t)(uintptr_t)&all_regs->x)

typedef enum IspCmdBufferState_e {
    ISP_CMDBUFFER_STATE_INIT,
    ISP_CMDBUFFER_STATE_READY,
    ISP_CMDBUFFER_STATE_RUNNING,
    ISP_CMDBUFFER_STATE_WAITING,
    ISP_CMDBUFFER_STATE_EXIT
}IspCmdBufferState_t;

typedef enum IspCmdBufferCmdType_e {
    ISP_CMDBUFFER_CMD_REG_READ = 0, /**< unused */
    ISP_CMDBUFFER_CMD_REG_WRITE = 1,/**< Write immediate values into SW registers of IP */
    ISP_CMDBUFFER_CMD_END = 2,      /**< The last command in this command buffer */
    ISP_CMDBUFFER_CMD_NOP = 3,      /**< Add an empty command to */
    ISP_CMDBUFFER_CMD_LINK = 8,     /**< Link to next cmd. */
    ISP_CMDBUFFER_CMD_STALL = 9     /**< Wait frame end. */
}IspCmdBufferCmdType_t;


//CMD Write Register Format
union IspCmdBufferCmdWreg_u {
    struct CmdWreg_t {
        uint32_t startRegAddr:16; /**< [15:0] Specify the address of
                                    the first SW register to be writen*/
        uint32_t length:10;         /**< [25:16] Specify how many SW
                                    registers (32 bits for each) should
                                    be writen; 0:1024, 1:1, ?? , 1023:1023;
                                    length is fixed to 1 in current driver*/
        uint32_t rsv:1;             /**< [26] Reserved for future */
        uint32_t opCode:5;         /**< [31:27] opcode = 5??h01 */
    } cmd;
    uint32_t all;
};
//CMD Buffer Write Struct
typedef struct IspCmdBufferCmdWreg_s {
    union IspCmdBufferCmdWreg_u ICBCmd;
    uint32_t wData;
}IspCmdBufferCmdWreg_t;


//CMD End Format
union IspCmdBufferCmdEnd_u {
    struct cmd_end_t {
        uint32_t eventId:5;        /**< [4:0] ID indicator when this cmd is
                                    executed, set as 5'h1A*/
        uint32_t rsv0:3;            /**< [7:5] Reserved for future*/
        uint32_t eventEnable:1;    /**< [8] Event enable when this cmd is
                                    executed, set as 1'b1 */
        uint32_t rsv1:18;           /**< [26:9] Reserved for future*/
        uint32_t opCode:5;         /**< [31:27]opcode = 5??h02 */
    } cmd;
    uint32_t all;
};
//CMD Buffer End Struct
typedef struct IspCmdBufferCmdEnd_s {
    union IspCmdBufferCmdEnd_u ICBCmd;
    uint32_t rsv2;                  /**< Reserved for future */
}IspCmdBufferCmdEnd_t;

//CMD NOP Format
union IspCmdBufferCmdNop_u {
    struct cmd_nop_t {
        uint32_t rsv0:27;           /**< [26:0] Reserved for future*/
        uint32_t opCode:5;         /**< [31:27]opcode = 5??h03 */
    } cmd;
    uint32_t all;
};
//CMD Buffer NOP Struct
typedef struct IspCmdBufferCmdNop_s {
    union IspCmdBufferCmdNop_u ICBCmd;
    uint32_t rsv1;                  /**< Reserved for future */
}IspCmdBufferCmdNop_t;


//CMD Load State Format
union IspCmdBufferCmdLoadState_u {
    struct cmd_load_state_s {
        uint32_t word0;             /**< Need to be 0x08010e02 */
    } cmd;
    uint32_t all;
};
//CMD Buffer Load State Struct
typedef struct IspCmdBufferCmdLoadState_s {
    union IspCmdBufferCmdLoadState_u ICBCmd;
    uint32_t word1;                  /**< Need to be 0x30000701 */
}IspCmdBufferCmdLoadState_t;


//CMD STALL Format
union IspCmdBufferCmdStall_u {
    struct cmd_stall_s {
        uint32_t rsv0:27;          /**< [26:0] Reserved for future */
        uint32_t opCode:5;         /**< [31:27]opcode = 5??h03 */
    } cmd;
    uint32_t all;
};
//CMD Buffer STALL Struct
typedef struct IspCmdBufferCmdStall_s {
    union IspCmdBufferCmdStall_u ICBCmd;
    uint32_t word1;                  /**< Need to be 0x30000701 */
}IspCmdBufferCmdStall_t;


//CMD LINK Format
union IspCmdBufferCmdLink_u {
    struct cmd_link_s {
        uint32_t cmdNum:26;        /**< [25:0] Specify the link process command number. */
        uint32_t rsv0:1;           /**< [26] Reserved for future */
        uint32_t opCode:5;         /**< [31:27]opcode = 5??h03 */
    } cmd;
    uint32_t all;
};
//CMD Buffer LINK Struct
typedef struct IspCmdBufferCmdLink_s {
    union IspCmdBufferCmdLink_u ICBCmd;
    uint32_t linkCmdAddr;          /**< Next command address. */
}IspCmdBufferCmdLink_t;


//CMD Buffer
typedef union IspCmdBufferCmd {
    IspCmdBufferCmdWreg_t cmdWreg;
    IspCmdBufferCmdEnd_t cmdEnd;
    IspCmdBufferCmdNop_t cmdNop;
    IspCmdBufferCmdLoadState_t cmdLoadState;
    IspCmdBufferCmdLink_t cmdLink;
    IspCmdBufferCmdStall_t cmdStall;
}IspCmdBufferCmd_u;

// //TODO: ISP read ro register information
// typedef struct IspCmdBufferStatusRegs_s {
//     uint32_t regBase;
//     uint32_t regNum;
//     uint32_t regStep;  //4/8/12/...
// }IspCmdBufferStatusRegs_t;

// //TODO: IspCmdBuffer Register offset,value.
// typedef struct IspCmdBufferVirtReg_s {
//     uint8_t vdid;
//     uint32_t offset;
//     uint32_t val;
// }IspCmdBufferVirtReg_t;


/*
 *   CMD Buffer
 * +------------+
 * |            |
 * |            |
 * |  Register  |
 * |    Buffer  |
 * |            |
 * +------------+          +------------------+
 * |            |<---------+   tbl_cmd_buffer0|
 * |            |          +------------------+
 * |    LUT     |<---------+   tbl_cmd_buffer1|
 * |    Buffer  |          +------------------+
 * |            |<---------+       ...        |
 * |            |          +------------------+
 * |            |<---------+   tbl_cmd_bufferx|
 * +------------+          +------------------+
 */
typedef struct IspCmdBufferRegBuffer_s {
    uint32_t currCmdNum;
    //Todo: this Max number need to the high level params.
    uint32_t cmdNumMax;
    uint32_t genRegNum; /*The total number of cmd buffer register*/
    IspCmdBufferCmd_u *regCmdBuffer;
    //hw address
    uint32_t cmdDmaAddr;
}IspCmdBufferRegBuffer_t;

typedef struct IspCmdBufferCtrlBuffer_s {
    IspCmdBufferCmd_u *regCmdBuffer;
}IspCmdBufferCtrlBuffer_t;

typedef struct IspCmdBufferTblBuffer_s {
    uint16_t paramsNum;
    uint16_t realUsedNum;
    uint32_t tblRegOffset;
    uint16_t currIndex;
    IspCmdBufferCmdWreg_t *tblCmdBuffer;
}IspCmdBufferTblBuffer_t;

typedef struct CmdBufferDevRegVal_s
{
	uint32_t address;
	uint32_t value;
} CmdBufferDevRegVal_t;

typedef struct IspCmdBuffer_s {
    osMutex cmdBufferLock;
    uint8_t prevIndex; /* Speed up the search for LUT buffer index */

    uint8_t statusRegsNum;
    IspCmdBufferStatusRegs_t statusRegs[ISP_CMDBUFFER_STATUS_REGS_MAX];

    uint32_t feUpdate;
    uint32_t beUpdate;
    uint32_t storeGammaFeYdata[129];
    uint32_t storeGammaBeYdata[129];

    //cnr group holder
    uint32_t cnrUpdate;
    uint32_t storeGammaCnrTblFlt0[64];
    uint32_t storeGammaCnrTblFlt1[64];
    uint32_t storeGammaCnrTblFlt2[64];
    uint32_t storeGammaCnrTblFlt3[64];

    uint8_t tblRegAddrNum; /*number of register address for LUT */
    uint16_t tblTotalParamsNum;  /*the total number of params for LUT */
    uint16_t ctrlTotalParamsNum; /*The total number of ctrl register*/

    IspCmdBufferRegBuffer_t refreshFullRegs;
    IspCmdBufferTblBuffer_t refreshTblBuffer[ISP_CMDBUFFER_TBL_REG_MAX];
    IspCmdBufferCtrlBuffer_t refreshCtrlRegs;

    IspCmdBufferTblBuffer_t runningTblBuffer[ISP_CMDBUFFER_TBL_REG_MAX];
    IspCmdBufferRegBuffer_t runningFullRegs;
    IspCmdBufferCtrlBuffer_t runningCtrlRegs;
    IspCmdBufferCtrlBuffer_t runningRdcdRegs;

    IspCmdBufferTblBuffer_t runningTblBuffer2[ISP_CMDBUFFER_TBL_REG_MAX];
    IspCmdBufferRegBuffer_t runningFullRegs2;
    IspCmdBufferCtrlBuffer_t runningCtrlRegs2;
    IspCmdBufferCtrlBuffer_t runningRdcdRegs2;

    IspCmdBufferTblBuffer_t *pRunningTblBuffer;
    IspCmdBufferRegBuffer_t *pRunningFullRegs;
    IspCmdBufferCtrlBuffer_t *pRunningCtrlRegs;

    IspCmdBufferRegBuffer_t refreshBatchRegs[ISP_CMDBUFFER_BATCH_FRAME_COUNT_MAX - 1U];
    IspCmdBufferRegBuffer_t runningBatchRegs[ISP_CMDBUFFER_BATCH_FRAME_COUNT_MAX - 1U];
    IspCmdBufferRegBuffer_t runningBatchRegs2[ISP_CMDBUFFER_BATCH_FRAME_COUNT_MAX - 1U];
    IspCmdBufferRegBuffer_t *pRunningBatchRegs;

    IspCmdBufferRegBuffer_t refreshRdcdFullRegs; /* RDCD buffer */
    IspCmdBufferCtrlBuffer_t *pRunningRdcdRegs;

    uint32_t rdIndex;
    uint32_t fixedRegRdNum;  /*statics register which has the fixed offset */
    uint32_t *fixedRegBuffer; /* only store the value of statics register
                        which has the fixed offset */
}IspCmdBuffer_t;

typedef enum IspCmdBufferHashArea_e {
    ISP_CMDBUFFER_NOP_AREA = 0,  /**< No Used Register Area */
    ISP_CMDBUFFER_NOR_AREA = 1,  /**< Normal Register Area */
    ISP_CMDBUFFER_TBL_AREA = 2,  /**< Table Register Area */
    ISP_CMDBUFFER_CTRL_AREA = 3  /**< Ctrl Register Area */
}IspCmdBufferHashArea_t;


typedef struct IspCmdBufferHashMap_s{
    IspCmdBufferHashArea_t areaLocation;
    uint16_t offserIndex;
    uint32_t defaultVal;
}IspCmdBufferHashMap_t;

typedef struct IspCmdBufferContext_s {
    uint vidIndex;
    uint32_t frameCnt;
    bool enable;
    HalHandle_t hHalLayer;
    HalHandle_t hIspHalLayer;
    IspCmdBufferWorkMode_t ispCmdBufWorkMode;
    IspCmdBufferState_t state;
    IspCmdBufferReg_t generalCtrl;
    //This default value need to using a struct send into the Driver, no need to Read value form ISP HW.
    IspCmdBuffer_t *cmdBuff;
    IspCmdBufferHashMap_t *hashMapTbl; /*index table for mapping register offset
                            to the full command buffer index*/
    uint16_t *rdcdHashMapTbl;
    //used for cal the timer.
    uint64_t last_t_ns;
    uint8_t batchFrameCount;      /*frame count of a batch in batch mode */
}IspCmdBufferContext_t;
#endif /*_CMDBUFFERDEV_H_*/
