/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef ISPIC_DEV_H
#define ISPIC_DEV_H

#ifndef __KERNEL__
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <errno.h>

#define copy_from_user(a, b, c) isp_copy_data(a, b, c)
#define copy_to_user(a, b, c) isp_copy_data(a, b, c)

#include <isp_drv/isp_types.h>
#include <virtual_hal/virtual_hal_api.h>

#ifdef ISP_RDCD
#include "isp_rdcd.h"
#endif

void isp_ic_set_hal(VirtualHandle_t hal);
#endif

#define REG_ADDR(x)  ((uint32_t)(uintptr_t)&all_regs->x)

#ifdef ISP_MIV1
#define ISP_BUF_GAP             (1024)
#elif defined ISP_MIV2
#define ISP_BUF_GAP             (0)
#endif

#ifdef ISP_MI_BP
# define MI_PATH_NUM            (3U)
#else
# define MI_PATH_NUM            (2U)
#endif

typedef uint8_t u8;
typedef uint16_t u16;
typedef uint32_t u32;
typedef uint64_t u64;

#ifdef ISP_3DNR_V4
#define CLEAR_RST_FRAME_NUM 1U
#else
#define CLEAR_RST_FRAME_NUM 2U
#endif

struct ic_window {
    u16 x;
    u16 y;
    u16 width;
    u16 height;
};

struct isp_context {
    u32 mode;
    u32 sample_edge;
    bool_t hSyncLowPolarity, vSyncLowPolarity;
    u32 bayer_pattern;
    u32 sub_sampling;
    u32 seq_ccir;
    u32 field_selection;
    u32 input_selection;
    u32 latency_fifo;
    struct ic_window acqWindow;  /**< acquisition window */
    struct ic_window ofWindow;   /**< output formatter window */
    struct ic_window isWindow;   /**< image stabilization output window */
    u32 stitching_mode;
    bool_t  disable_nr_reloc;
    u8  statistis_3a_sel;
    bool_t dpf_raw_out;
};

#ifdef ISP_GWDR
#define ISP_GWDR_CURVE_SIZE 33

typedef enum isp_gwdr_mode 
{
    isp_gwdr_mode_0x0 = 0,   
    isp_gwdr_mode_0x87a  = 0x87a
} isp_gwdr_mode_t;

typedef struct isp_wdr_context
{
    bool_t            enable;

    isp_gwdr_mode_t   mode;
    float             gamma;
    u16               lumOffset;
    u16               rgbOffset;
    u16               curve[ISP_GWDR_CURVE_SIZE];

    bool_t            enableChanged;   //the wdr ctrl && reb shift does not have shandow
                              //register,need to change after frame end irq.
    bool_t            modeChanged;
    bool_t            offsetChanged;
} isp_wdr_context_t;
#endif

struct isp_digital_gain_cxt {
    /* data */
    bool_t enable;
    u16 gain_r;
    u16 gain_b;
    u16 gain_gr;
    u16 gain_gb;
    bool_t changed;
};

struct isp_dummy_hblank_cxt {
    u8 bp, fp, w, in_hsize;
};

typedef enum YcSplitterChannelMode_e
{
    ISP_YCSPLIT_CHMODE_INVALID          = 0,        /**< lower border (only for an internal evaluation) */
    ISP_YCSPLIT_CHMODE_OFF              = 1,        /**< disable Y/C splitter */
    ISP_YCSPLIT_CHMODE_MP               = 2,        /**< enable mainpath */
    ISP_YCSPLIT_CHMODE_SP               = 3,        /**< enable selfpath */
    ISP_YCSPLIT_CHMODE_SP2              = 4,        /**< enable selfpath2 */
    ISP_YCSPLIT_CHMODE_MAX                          /**< upper border (only for an internal evaluation) */
} YcSplitterChannelMode_t;

typedef enum MainPathMux_e
{
    ISP_MP_MUX_INVALID                  = 0,        /**< lower border (only for an internal evaluation) */
    ISP_MP_MUX_JPEG_DIRECT              = 1,        /**< data from DMA read port to JPEG encoder */
    ISP_MP_MUX_MI                       = 2,        /**< data from main resizer/scaler MRSZ to MI */
    ISP_MP_MUX_JPEG                     = 3,        /**< data from main resizer/scaler MRSZ to JPEG encoder */
    ISP_MP_MUX_MAX                                  /**< upper border (only for an internal evaluation) */
} MainPathMux_t;

typedef enum DmaReadPath_e
{
    ISP_DMA_READ_INVALID                = 0,        /**< lower border (only for an internal evaluation) */
    ISP_DMA_READ_SPMUX                  = 1,        /**< data feeding in selfpath */
    ISP_DMA_READ_SUPERIMPOSE            = 2,        /**< data feeding in superimpose module */
    ISP_DMA_READ_IMAGE_EFFECTS          = 3,        /**< data feeding in image effects module */
    ISP_DMA_READ_JPEG                   = 4,        /**< data feeding in jpeg encoder */
    ISP_DMA_READ_ISP                    = 5,        /**< data feeding in isp */
    ISP_DMA_READ_MAX                                /**< upper border (only for an internal evaluation) */
} DmaReadPath_t;

typedef enum InterfaceSelect_e
{
    ISP_ITF_SELECT_INVALID              = 0,        /**< lower border (only for an internal evaluation) */
    ISP_ITF_SELECT_PARALLEL             = 1,        /**< selects the parallel interface */
    ISP_ITF_SELECT_SMIA                 = 2,        /**< selects the SMIA interface */
    ISP_ITF_SELECT_MIPI                 = 3,        /**< selects the MIPI interface */
    ISP_ITF_SELECT_HDR                  = 4,        /**< selects the HDR interface */
    ISP_ITF_SELECT_MAX                              /**< upper border (only for an internal evaluation) */
} InterfaceSelect_t;

typedef enum MiPathId_e {
    ISP_MI_PATH_MP = 0,
    ISP_MI_PATH_SP,
    ISP_MI_PATH_SP2_BP,
    ISP_MI_PATH_MP_RAW,

#ifdef ISP_MI_HDR
    ISP_MI_PATH_HDR_RAW,
#endif

#ifdef ISP_MI_PP_WRITE
    ISP_MI_PATH_PP,
#endif

#ifdef ISP_MI_HDR
    ISP_MI_HDR_L,
    ISP_MI_HDR_S,
    ISP_MI_HDR_VS,
#ifdef ISP_HDR_STITCH_V3_1
    ISP_MI_HDR_XS,
#endif
#ifdef ISP_HDR_2_SENSORS
    ISP_MI_HDR1_L,
    ISP_MI_HDR1_S,
    ISP_MI_HDR1_VS,
#endif
#endif
    ISP_MI_PATH_ID_MAX
} MiPathId_t;

enum MIV2_SEPERATE_PATH_ID {
    ISP_MI_SEPERATE_PATH_INVALID  = -1,
    ISP_MI_SEPERATE_PATH_MP       = 1,
    ISP_MI_SEPERATE_PATH_SP       = 2,
    ISP_MI_SEPERATE_PATH_SP2_BP   = 4,
    ISP_MI_SEPERATE_PATH_MP_RAW   = 8,
    ISP_MI_SEPERATE_PATH_ID_MAX
};

#ifdef ISP_GC_V1

#define ISP_GAMMA_OUT_DATA_Y_NUM 17

typedef enum isp_gamma_out_curveMode_e
{
    ISP_GAMMAOUT_CURVE_MODE_INVALID  = -1,   /**< lower border (only for an internal evaluation) */
    ISP_GAMMAOUT_CURVE_MODE_LOG      = 0,    /**< logarithmic segmentation from 0 to 4095
                                                     (64,64,64,64,128,128,128,128,256,256,256,512,512,512,512,512) */
    ISP_GAMMAOUT_CURVE_MODE_EQU      = 1,    /**< equidistant segmentation from 0 to 4095
                                                     (256, 256, ... ) */
    ISP_GAMMAOUT_CURVE_MODE_MAX              /**< upper border (only for an internal evaluation) */
} isp_gamma_out_curveMode_t;

struct isp_gamma_out_context {
    bool_t enable;
    isp_gamma_out_curveMode_t curveMode;
    u16 curve[ISP_GAMMA_OUT_DATA_Y_NUM];

    bool_t statusChanged;
    bool_t curveModeChanged;
    bool_t curveChanged;
};

#endif

struct isp_misirq_regaddr {
    u32 isp_ctrl_offset;
    u32 isp_mis_offset;
    u32 isp_icr_offset;
    u32 isp_imsc_offset;

    u32 miv_mis_offset;
    u32 miv_icr_offset;
    u32 miv2_mis1_offset;
    u32 miv2_icr1_offset;
    u32 miv2_mis2_offset;
    u32 miv2_icr2_offset;
    u32 miv2_mis3_offset;
    u32 miv2_icr3_offset;
    u32 mi_mis_hdr1_offset;
    u32 mi_icr_hdr1_offset;
#ifdef ENABLE_IRQ
    u32 mi_status_offset;
#endif
};

struct isp_miv_info {
    bool_t miv2_open;
    bool_t mis2_open;
};

struct isp_buffer_context {
    PicBufType_t type;
    MiPathId_t path;
    u32 addr_y, addr_cb, addr_cr;
    u32 size_y, size_cb, size_cr;
};

struct isp_extmem_info {
    u64 addr;
    u64 size;
    u64 user_rmem_size;
};

struct isp_reg_mem_info {
    u64 phy_base;
    u32 reg_size;
};

struct isp_reg_t {
    u32 offset;
    u32 val;
};

typedef enum isp_swap_bits_control_e {
    ISP_MI_SWAP_INVALID = -1,       /**< lower border (only for an internal evaluation) */
    ISP_MI_NO_SWAP = 0,             /**< The value of no swap. */
    ISP_MI_SWAP_BYTES = 1,          /**< The value of swap bytes. */
    ISP_MI_SWAP_WORDS = 2,          /**< The value of swap words. */
    ISP_MI_SWAP_DOUBLE_WORDS = 4,   /**< The value of swap double words. */
    ISP_MI_SWAP_FOUR_WORDS = 8,     /**< The value of swap four words. */
    ISP_MI_SWAP_MAX,                /**< upper border (only for an internal evaluation) */
} isp_swap_bits_control_t;

typedef union isp_mi_swap_u {
    struct {
        isp_swap_bits_control_t y;
        isp_swap_bits_control_t u;
        isp_swap_bits_control_t v;
    } yuvSwap;
    isp_swap_bits_control_t rawSwap;
} isp_mi_swap_t;

struct isp_mi_data_path_context {
    bool_t enable;
    MiDataMode_t out_mode;              /**< output format */
    u32 in_mode;                        /**< input format */
    u32 data_layout;                    /**< layout of data */
    MiDataAlignMode_t data_alignMode;   /**< align mode of data */
    isp_mi_swap_t swap;                 /**< swap of data */
    u32 in_width;
    u32 in_height;
    u32 out_width;
    u32 out_height;
    bool_t hscale;
    bool_t vscale;
    int32_t pixelformat;
    MiPixelYuv_t yuv_bit;
    u32 tilemode_output_width;  //using for calculate llength,tilemode_output_width = isp0-crop-outwidth + isp1-crop-outwidth, isp0 and isp1 are same.
    int32_t mi_remainder;
                               //number of horizon pices
    u8  yuv_order;    // Specify the order of YUV or RGB channel. 
    u8  alpha;        /**< alpha value of ARGB format */

    bool_t pathEnableChanged;
    bool_t pathDisableChanged;
};

struct isp_mi_context {
    struct isp_mi_data_path_context path[ISP_MI_PATH_ID_MAX];
    u32 burst_len;
    bool_t tileMode;              //indicate whether if mi need to work in tile mode.

    uint8_t isp_mi_enable_path;
};

enum HDR_DMA_ID {
    ISP_MI_DMA_ID_MCM_PP = 0,
    ISP_MI_DMA_ID_HDR_L,
    ISP_MI_DMA_ID_HDR_S,
    ISP_MI_DMA_ID_HDR_VS,
#ifdef ISP_HDR_STITCH_V3_1
    ISP_MI_DMA_ID_HDR_XS,
#endif
#ifdef ISP_HDR_2_SENSORS
    ISP_MI_DMA_ID_HDR1_L,
    ISP_MI_DMA_ID_HDR1_S,
    ISP_MI_DMA_ID_HDR1_VS,
#endif
    ISP_MI_DMA_ID_MAX
};
typedef enum start_dma_path_e{
    ISP_MI_DMA_PATH_MCM_PP = 0,
    ISP_MI_DMA_PATH_HDR,
    ISP_MI_DMA_PATH_MAX,
} start_dma_path_t;

struct isp_bls_context {
    bool_t enabled;
    u32 mode;
    u16 a, b, c, d;
};

struct isp_tpg_userdefine_mode {
    u16 total, fp, sync, bp, act;
};

struct isp_tpg_context {
    bool_t enabled;
    u32 image_type;
    u32 frame_num;
    u32 bayer_pattern;
    u32 color_depth;
    u32 resolution;
    u16 pixleGap;
    u16 lineGap;
    u16 gapStandard;
    u32 randomSeed;
    struct isp_tpg_userdefine_mode user_mode_h, user_mode_v;
};

struct isp_mux_context {
    u32 mp_mux;             /**< main path muxer (vi_mp_mux) */
    u32 sp_mux;             /**< self path muxer (vi_dma_spmux) */
    u32 chan_mode;          /**< 1-mp, 2-sp, 4-sp2 */
    u32 ie_mux;             /**< image effects muxer (vi_dma_iemux) */
    u32 dma_read_switch;    /**< dma read switch (vi_dma_switch) */
    u32 if_select;          /**< interface selector (if_select) */
    u8  sp1_ir_sel;         /**< 0:self1 resize, 1:IR   */
    u8  sp1_y_only;         /**< 0:yuv, 1:only Y */
};

struct isp_awb_context {
    bool_t enable;
    u8 mode;
    u16 gain_r, gain_gr, gain_gb, gain_b;
    struct ic_window window;
    u16 refcb_max_b;
    u16 refcr_max_r;
    u16 max_y;
    u16 max_c_sum;
    u16 min_y_max_g;
    u16 min_c;
    bool_t changed;
};

struct isp_awb_mean {
    u32 r, g, b;
    u32 no_white_count;
};

struct isp_cnr_context {
    bool_t enable;
    u32 line_width;
    u32 threshold_1;
    u32 threshold_2;
};

#if !defined (ISP_CNR_V2_1) && !defined (ISP_CNR_V2_2)
#define CNR2_WEIGHT_TABLE_SIZE 128
#define MAX_CNR2_WEIGHT_TABLE 1023
#else
#define CNR2_WEIGHT_TABLE_SIZE 64
#define MAX_CNR2_WEIGHT_TABLE 255
#endif

#ifdef ISP_CNR_V2_2
#define CNR_SIGMA_LAYER_NUM 3
#else
#define CNR_SIGMA_LAYER_NUM 4
#endif

struct isp_cnr2_context {
    bool_t ynr_enable;
    bool_t cnr_enable;
    bool_t ynr_weight_table_change;
    bool_t cnr_weight_table_change;
    bool_t neighbor_pix_consider;
    float32_t c_sigma_layer[CNR_SIGMA_LAYER_NUM];
    float32_t y_sigma;
    u32 cnr_weight_table_filter[CNR_SIGMA_LAYER_NUM][CNR2_WEIGHT_TABLE_SIZE+1];
#if !defined (ISP_CNR_V2_1) && !defined (ISP_CNR_V2_2)
    float c_sigma_merge[3];
    u32 cnr_weight_table_merge[3][CNR2_WEIGHT_TABLE_SIZE+1];
#endif
    u32 ynr_weight_table_filter[CNR2_WEIGHT_TABLE_SIZE+1];
    u8 bit;
    u8 cnr_strength1;
    u8 cnr_strength2;
    u8 ynr_strength1;
    u8 ynr_strength2;
    u8 cnr_filter_shift;
    u32 cnr_filter_mask0;
    u32 cnr_filter_mask1;
    u8 texture_mask_select;
};

struct isp_cc_context {
    bool_t ccEnable;
    bool_t changed;
    u32 lCoeff[9];
    bool_t update_curve;
    bool_t conv_range_y_full, conv_range_c_full;
};

struct isp_xtalk_context {
    bool_t ccmEnable;
    u32 lCoeff[9];
    u32 r, g, b;
};

#define ISP_DRV_COLORING_TBL_SIZE 256U

struct isp_coloring_context {
    bool_t coloringEnable;
#if defined (ISP_COLORING_V1)
    u32 coloringTbl[ISP_DRV_COLORING_TBL_SIZE];
#endif
};

#if !defined(ISP_LSC_V3) && !defined(ISP_LSC_V4)

#define ISP_DRV_GRAD_TBL_SIZE 8U
#define ISP_DRV_DATA_TBL_SIZE 289U
#define ISP_DRV_MAX_LSC_SECTORS 16U
#define ISP_DRV_CURVE_DATA_TABLE_LEN 65U

struct isp_lsc_context {
    bool_t lsc_enable;
    bool_t tbl_size_changed;
    /**< correction values of R color part */
    u16 r[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of G (red lines) color part */
    u16 gr[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of G (blue lines) color part */
    u16 gb[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of B color part */
    u16 b[ISP_DRV_DATA_TBL_SIZE];
    /**< multiplication factors of x direction */
    u16 x_grad[ISP_DRV_GRAD_TBL_SIZE];
    /**< multiplication factors of y direction */
    u16 y_grad[ISP_DRV_GRAD_TBL_SIZE];
    /**< sector sizes of x direction */
    u16 x_size[ISP_DRV_GRAD_TBL_SIZE];
    /**< sector sizes of y direction */
    u16 y_size[ISP_DRV_GRAD_TBL_SIZE];
};

#else

#define ISP_DRV_GRAD_TBL_SIZE 32U
#define ISP_DRV_DATA_TBL_SIZE 1089U
#define ISP_DRV_MAX_LSC_SECTORS 32U
#define ISP_DRV_CURVE_DATA_TABLE_LEN 65U

#if defined(ISP_LSC_V3)
struct isp_lsc_context {
    bool_t lsc_enable;
    bool_t tbl_size_changed;
    /**< correction values of R color part */
    u16 r[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of G (red lines) color part */
    u16 gr[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of G (blue lines) color part */
    u16 gb[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of B color part */
    u16 b[ISP_DRV_DATA_TBL_SIZE];
    /**< multiplication factors of x direction */
    u16 x_grad[ISP_DRV_GRAD_TBL_SIZE];
    /**< multiplication factors of y direction */
    u16 y_grad[ISP_DRV_GRAD_TBL_SIZE/2];
    /**< sector sizes of x direction */
    u16 x_size[ISP_DRV_GRAD_TBL_SIZE];
    /**< sector sizes of y direction */
    u16 y_size[ISP_DRV_GRAD_TBL_SIZE/2];
};
#elif defined ISP_LSC_V4
struct isp_lsc_context {
    bool_t lsc_enable;
    bool_t tbl_size_changed;
    /**< correction values of R color part */
    u16 r[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of G (red lines) color part */
    u16 gr[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of G (blue lines) color part */
    u16 gb[ISP_DRV_DATA_TBL_SIZE];
    /**< correction values of B color part */
    u16 b[ISP_DRV_DATA_TBL_SIZE];
    /**< multiplication factors of x direction */
    u16 x_grad[ISP_DRV_GRAD_TBL_SIZE];
    /**< multiplication factors of y direction */
    u16 y_grad[ISP_DRV_GRAD_TBL_SIZE];
    /**< sector sizes of x direction */
    u16 x_size[ISP_DRV_GRAD_TBL_SIZE];
    /**< sector sizes of y direction */
    u16 y_size[ISP_DRV_GRAD_TBL_SIZE];
};
#endif
#endif

//demosaic
#ifdef ISP_DEMOSAIC1
struct isp_flt_context {
    bool_t enable;
    bool_t changed;
    uint8_t denoise;
    uint8_t sharpen;
    uint8_t chrVMode;
    uint8_t chrHMode;
};

struct isp_dmsc_context {
    bool_t bypass_mode;
    uint8_t threshold;
    struct isp_flt_context flt;
};

#elif defined (ISP_DEMOSAIC2)
struct isp_dmoi_context {
    bool_t enable;
    uint8_t demoire_area_thr;
    uint8_t demoire_sat_shrink;
    uint16_t demoire_r2;
    uint16_t demoire_r1;
    uint8_t demoire_t2_shift;
    uint8_t demoire_t1;
    uint16_t demoire_edge_r2;
    uint16_t demoire_edge_r1;
    uint8_t demoire_edge_t2_shift;
    uint16_t demoire_edge_t1;
};

struct isp_shap_context {
    bool_t enable;
    uint16_t sharpen_factor_black;
    uint16_t sharpen_factor_white;
    uint16_t sharpen_clip_black;
    uint16_t sharpen_clip_white;
    uint16_t sharpen_t4_shift;
    uint16_t sharpen_t3;
    uint8_t sharpen_t2_shift;
    uint16_t sharpen_t1;
    uint16_t sharpen_r3;
    uint16_t sharpen_r2;
    uint16_t sharpen_r1;
    uint8_t denoise_stren;
    uint8_t sharpen_size;

};

struct isp_shap_line_context {
    bool_t enable;
    uint8_t sharpen_line_shift1;
    uint16_t sharpen_line_t1;
    uint16_t sharpen_line_strength;
    uint16_t sharpen_line_r2;
    uint16_t sharpen_line_r1;
};

struct isp_gfilter_context {
    uint8_t hf_filt_00;
    uint8_t hf_filt_01;
    uint8_t hf_filt_02;
    uint8_t hf_filt_10;
    uint8_t hf_filt_11;
    uint8_t hf_filt_12;
    uint8_t hf_filt_20;
    uint8_t hf_filt_21;
    uint8_t hf_filt_22;
};

struct isp_skin_context {
    bool_t enable;
    uint16_t cb_thr_max_2047;
    uint16_t cb_thr_min_2047;
    uint16_t cr_thr_max_2047;
    uint16_t cr_thr_min_2047;
    uint16_t y_thr_max_2047;
    uint16_t y_thr_min_2047;
};

struct isp_depurple_context {
    bool_t enable;
    bool_t red_sat, blue_sat;
    uint8_t depurple_sat_shrink;
    uint8_t depurple_thr;
};

struct isp_intp_thr_cxt {
    uint16_t intp_dir_thr_min;
    uint16_t intp_dir_thr_max;
    uint8_t demosaic_thr;
    bool_t enable;
};

struct isp_dmsc_context {
    struct isp_intp_thr_cxt intp;
    struct isp_gfilter_context gFilter;
    struct isp_dmoi_context demoire;
    struct isp_shap_context sharpen;
    struct isp_shap_line_context sharpenLine;
    struct isp_skin_context skin;
    struct isp_depurple_context depurple;
};
#elif defined (ISP_DEMOSAIC3)
#define DMSC3_STRNGTH_LEVEL_NUM 4
#define DMSC3_INTERP_LARGE_STR 2
struct isp_intp2_context {    
    uint8_t diffColor_coef;  //8bit  [0, 128]
    uint8_t adpt_coef_mode;  //2bit  [0, 3]
    uint8_t adpt_coef_th_low;  //8bit  [0, 255]
    uint8_t adpt_coef_th_shift;  //4bit  [0, 8]
    uint8_t adpt_coef_min;  //8bit  [0, 128]
    uint8_t adpt_coef_max;    //8bit    [0, 128]

    uint16_t grad_th_min;    //12bit  [0, 4095]
    uint16_t grad_th_max;  //12bit  [0, 4095]

    uint16_t highlight_control_th;    //12bit  [0, 4095]
    uint8_t highlight_control_str;  //1bit  [0, 1]
    uint8_t deburst_mode;  //bit 0: for g   //bit 1: for rb  //2bit    [0, 3]

    uint8_t pure_G_weight;  //5bit  [0, 16]
    uint8_t pure_RB_weight;  //5bit  [0, 16]
    uint8_t pure_G_weight_ISO;  //5bit  [0, 16]
    uint8_t pure_RB_weight_ISO;  //5bit  [0, 16]
};

struct isp_interpCorner_context
{
    bool_t enable;  //1bit  [0, 1]
    uint8_t interp_dir_strength_level[DMSC3_STRNGTH_LEVEL_NUM];    //8bit    [0, 255]
    uint8_t interp_large_str[DMSC3_INTERP_LARGE_STR];    //5bit    [0, 16]
    uint8_t dir0_interp_type;  //2bit   [0, 3]
};

struct isp_decline_context
{
    bool_t enable;  //1bit  [0, 1]
    uint16_t corner_low_th;    //12bit  [0, 4095]
    uint8_t corner_shift;  //4bit  [0, 12]
};

struct isp_demoire_context {
    bool_t enable;  //1bit  [0, 1]
    uint16_t refine_g_th;  //12bit  [0, 4095]
};
    
struct isp_defalse_context {
    bool_t enable; //1bit  [0, 1]
    uint8_t color_str;  //8bit  [0, 128]
    uint8_t cbcr_class_th;  //3bit  [0, 7]
    uint8_t gray_protect_str;  //8bit  [0, 128]
};

#define DMSC3_DEPURPLE_NUM 2
struct isp_depurple2_context {
	bool_t enable;
    uint8_t thr;  //8bit  [0, 255]
    uint8_t cbcr_mode;  //2bit  [0, 3]
    uint8_t color_range_mode;  //2bit  [0, 3]
    
    uint8_t grad_control; //1bit  [0, 1]
    uint8_t protect_th_low;   //8bit  [0,255]
    uint8_t protect_shift;  //4bit  [0, 8]
    uint8_t sat_shrink;    //4bit    [0, 8]

    uint8_t cb_lowth[DMSC3_DEPURPLE_NUM];
    uint8_t cb_highth[DMSC3_DEPURPLE_NUM];
    uint8_t cr_lowth[DMSC3_DEPURPLE_NUM];
    uint8_t cr_highth[DMSC3_DEPURPLE_NUM];
};

#define DMSC3_SHARPEN_LAYER_NUMBER 3
struct isp_sharpen_layer_context {
    uint16_t white;
    uint16_t black;
};

struct isp_sharpen_curve_context {
    uint16_t layer_t1;  //11bit    [0, 2047]
    uint8_t layer_t2_shift;    //4bit    [0, 11]
    uint16_t layer_t3;    //11bit  [0, 2047]
    uint8_t layer_t4_shift;    //4bit    [0, 11]
    uint16_t layer_r1;  //9bit  [0, 256]
    uint16_t layer_r2;    //9bit    [0, 256]
    uint16_t layer_r3;    //9bit    [0, 256]
};

struct isp_sharpen_line_context {
    bool_t enable; //1bit  [0, 1]
    uint16_t strength;  //12bit  [0, 4095]
    uint16_t t1;  //11bit  [0, 2047]
    uint8_t shift1;    //4bit    [0, 11]
    uint16_t r1;    //9bit    [0, 256]
    uint16_t r2;  //9bit  [0, 256]
};
struct isp_sharpen_context {
    bool_t enable; //1bit  [0, 1]
    uint8_t denoise_strength;  //6bit  [0, 32]
    struct isp_sharpen_layer_context factor[DMSC3_SHARPEN_LAYER_NUMBER];
    struct isp_sharpen_layer_context clip[DMSC3_SHARPEN_LAYER_NUMBER];
    struct isp_sharpen_curve_context curve[DMSC3_SHARPEN_LAYER_NUMBER];
    struct isp_sharpen_line_context sharpen_line;
};

struct isp_dmsc3_context {
    bool_t enable;
    struct isp_intp2_context intp;
    struct isp_interpCorner_context interpCorner;
    struct isp_decline_context decline;
    struct isp_demoire_context demoire;
    struct isp_defalse_context defalse;
    struct isp_depurple2_context depurple;
    struct isp_sharpen_context sharpen;
};
#endif //dmsc3

struct isp_ge_context {
    bool_t enable;
    u16 threshold;
    u16 h_dummy;
};

struct isp_ca_context {
    bool_t enable;
    uint8_t mode;
    uint16_t lut_chroma[ISP_DRV_CURVE_DATA_TABLE_LEN];
};

struct isp_dci_context {
    bool_t enable;
    float degamma;
    uint16_t lut_luma[ISP_DRV_CURVE_DATA_TABLE_LEN];
};

struct isp_curve_context {
    bool_t enable;
    uint16_t lut_x[ISP_DRV_CURVE_DATA_TABLE_LEN];
    uint16_t lut_shift[ISP_DRV_CURVE_DATA_TABLE_LEN];
    struct isp_ca_context ca;
    struct isp_dci_context dci;
};

struct isp_bp_buffer_context {
    u32 addr_r;
    u32 addr_gr;
    u32 addr_gb;
    u32 addr_b;
};

struct isp_dma_context {
    PicBufType_t type;
    u32 base;
    u32 width;
    u32 height;
    u32 burst_y;
    u32 burst_c;
    u8  align;
    u8 id;
#if (defined ISP_MI_HDR) || (defined ISP8000NANO_V2302)
    u8 rd_wr_str;
#endif
    bool_t tileMode;
    u32 tileWidth;
    u32 tileHeight;
    u8 swap;
};

struct isp_dpf_context {
    bool_t enable;
    u32 filter_type;
    u32 gain_usage;
    u32 strength_r;
    u32 strength_g;
    u32 strength_b;
    u8 weight_g[6];
    u8 weight_rb[6];
    u16 denoise_talbe[17];
    u32 x_scale;
    u32 nf_gain_r;
    u32 nf_gain_gr;
    u32 nf_gain_gb;
    u32 nf_gain_b;
    bool_t filter_r_off;
    bool_t filter_gr_off;
    bool_t filter_gb_off;
    bool_t filter_b_off;
};

struct isp_is_context {
    bool_t enable;
    bool_t update;
    struct ic_window window;
    u32 recenter;
    u32 max_dx, max_dy;
    u32 displace_x, displace_y;
};

struct isp_ee_context {
    bool_t enable;
    u8 src_strength;
    u8 strength;
    u8 input_sel;
    u32 y_gain, uv_gain, edge_gain;
};

struct isp_ee3_context {
	bool_t enable;
	bool_t yuv_domain;
	uint16_t source_strength;
	uint16_t source_strength_skin;
    //edge detail
	uint8_t denoise_level;
	uint8_t scaler_narrow;
	uint8_t edge_use_auxi_dir;
	uint8_t detail_lvl;
	uint8_t scaler_detail;
	uint16_t detail_pre_enhance_str;
    //skin detection
	uint8_t skin_proc_enable;
	uint8_t skin_detect_str;
	uint16_t skin_hth_y_min;
	uint16_t skin_hth_y_max;
	int16_t skin_hth_cr_min;
	int16_t skin_hth_cr_max;
	int16_t skin_hth_cb_min;
	int16_t skin_hth_cb_max;
    //edge&detail merge
	uint16_t edge_str_t0;
	uint8_t edge_str_t1_shift;
	uint16_t edge_str_t2;
	uint8_t edge_str_t3_shift;
	uint16_t edge_skin_str_t0;
	uint8_t edge_skin_str_t1_shift;
	uint16_t edge_skin_str_t2;
	uint8_t edge_skin_str_t3_shift;
    //edge &detail noise cut off
	uint16_t grad_th_edge;
	uint16_t grad_th_detail;
	uint16_t grad_th_edge_skin;
	uint16_t grad_th_detail_skin;
    //edge&detail enhancement
    uint8_t strength;
	uint8_t sharp_curve_lvl;
	uint8_t sharp_skin_sharpe_lvl;
    uint16_t up_gain_other;
    uint16_t down_gain_other;
    uint16_t up_gain_skin;
    uint16_t down_gain_skin;
    uint8_t uv_gain;
    bool_t max_change_enable;
    uint8_t max_change_type;
    uint16_t max_change_up;
    uint16_t max_change_down;
    uint16_t max_change_up_skin;
    uint16_t max_change_down_skin;
    //depurple
	bool_t depurple_enable;
	uint8_t depf_detect_range;
	uint16_t depf_limit_down_u;
	uint16_t depf_limit_up_u;
	uint16_t depf_limit_down_v;
	uint16_t depf_limit_up_v;
	uint16_t depf_detect_luma_th;
	uint16_t depf_detect_luma_diff;
	uint16_t depf_comp_luma_diff;
	uint16_t depf_fix_str;
	uint16_t defp_sat_str;
};

struct isp_exp_context {
    bool_t enable;
    u32 mode;
    struct ic_window window;
};

struct isp_hist_context {
    bool_t enable;
    u32 mode;
    u32 step_size;
    struct ic_window window;
    u8 weight[25];
};

struct isp_hist64_context {
    bool_t enable;
    u8 channel;
    u32 mode;
    u8 vStepSize;
    u32 hStepInc;
    u8 r_coeff, g_coeff, b_coeff;
    u8 sample_shift;
    u16  sample_offset;
    u16 forced_upd_start_line;
    bool_t forced_upd;
    struct ic_window window;
    u8 weight[25];
};

struct isp_gtm_weight {
    u8 lightness_weight;
    u8 weight0;
    u8 weight1;
    u8 weight2;
};

struct isp_gtm_sta {
    u16 luma[128];
	u32	min;
    u32  max;
};

struct isp_gtm_context {
    bool_t enable;
    bool_t bm_cor_enable;
    u8 hstepSize;
    u8 vstepSize;
    u8 r_coeff, g_coeff, b_coeff;
    u32 base;
    u32 slope;
    struct isp_gtm_weight weight;
    u32 pa;
    u32 curve[129];
    struct isp_gtm_sta stat;
};

typedef struct bad_pixel_pos_s{
    u16 v_addr;     //bad pixel vertical address (pixel position)
    u16 h_addr;     //bad pixel horizontal address (pixel position)
} bad_pixel_pos_t;

#define ISP_DRV_MAX_BAD_PIXEL_CNT 2048 //sensor have a relatively small amount of defect pixels.
struct isp_dpcc_bpt {
	// u16 addr;	    //start address if a certain part of the table shall be write or read.
    bad_pixel_pos_t bad_pixel_tbl[ISP_DRV_MAX_BAD_PIXEL_CNT]; //pointer to the bad pixel table
    u16 bp_number;  //number of the bad pixels contained in the list
    u8 replace;  //replacement method
    bool_t enabled;

    /* Status flag for updating in vblank. */
    bool_t bptChanged;
};

struct isp_dpcc_params {
    u32 line_thresh;
    u32 line_mad_fac;
    u32 pg_fac;
    u32 rnd_thresh;
    u32 rg_fac;
};

struct isp_dpcc_context {
    bool_t enable;
    u32 mode;
    u32 outmode;
    u32 set_use;
    u32 methods_set[3];
    struct isp_dpcc_params params[3];
    u32 ro_limits;
    u32 rnd_offs;

    /* Status flag for updating in vblank. */
    bool_t cfgChanged;
};

struct isp_cac_context {
    bool_t enable;
    u32 hmode, vmode;
    u32 ab, ar, bb, br, cb, cr;
    u32 xns, xnf, yns, ynf;
    u16 hstart, vstart;
    u32 width, height;
    u32 hoffset, voffset;
};

/* degamma */
struct isp_deg_context {
    bool_t enable;
    u8 segment[16];
    u16 r[17];
    u16 g[17];
    u16 b[17];
};

struct isp_ie_context {
    bool_t enable;
    u32 mode;
    u32 color_sel;
    u32 color_thresh;
    u32 sharpen_factor;
    u32 sharpen_thresh;
    int32_t m[9];
    u32 tint_cr;
    u32 tint_cb;
    bool_t full_range;
};

#ifdef ISP_AFM_V1_V1

struct isp_afm_result {
    u32 sum_a_0, sum_b_0, sum_c_0;
    u32 lum_a_0, lum_b_0, lum_c_0;
    u32 sum_a_1, sum_b_1, sum_c_1;
    u32 lum_a_1, lum_b_1, lum_c_1;
    u32 sum_a_2, sum_b_2, sum_c_2;
    u32 lum_a_2, lum_b_2, lum_c_2;
};

struct isp_afm_context {
    bool_t enable;
    u32 thresh[3];
    struct ic_window window[3][3];
    u32 pixCnt[3][3];
    bool_t enableAfmInstance[3];
    u32 lum_shift[3];
    u32 afm_shift[3];
    u32 max_pix_cnt[3];
};
#else

struct isp_afm_result {
    u32 sum_a, sum_b, sum_c;
    u32 lum_a, lum_b, lum_c;
};

struct isp_afm_context {
    bool_t enable;
    u32 thresh;
    struct ic_window window[3];
    u32 pixCnt[3];
    // bool_t enableWinId[3];
    u32 lum_shift;
    u32 afm_shift;
    u32 max_pix_cnt;
};

#endif

#ifdef ISP_AFM_V3

struct isp_afmv3_result {
    struct {
        u32 highpass[15*15];
        u32 lowpass[15*15];
    } sharpness;

    struct {
        u32 high_data[15*15];
        u32 low_data[15*15];
    } hist;
};

struct isp_afmv3_context {
    bool_t enable;
    u16 bls; // black level on 12bit raw
    u8 hist_threshold;

    struct {
        u16 start_x;
        u16 start_y;
        u16 width;
        u16 height;
    } block;

    struct {
        u8 data_shift;
        u8 data_shift_v;
        struct {
            u8 filter_weight;
            u16 filter1_coeff[4];
            u16 filter2_coeff[4];
            u16 filter3_coeff[4];
        } highpass, lowpass;
    } iir;

    struct {
        u8 data_shift;
        u8 data_shift_v;
        struct {
            u8 filter_weight;
            u8 data_shift_x;
            u8 data_shift_y;
            u8 filter_coeff[4];
        } highpass;

        struct {
            u8 filter_weight;
            u8 data_shift_x;
            u8 data_shift_y;
            u8 filter_coeff_hor[4];
            u8 filter_coeff_ver[4];
        } lowpass;
    } fir;

    struct {
        bool_t enable;
        float value;
        u8 px[64];
        u32 y_data[64];
        u32 x_data[64];
    } gamma;

    struct {
        struct {
            u8 highpass_y[3];
            u8 highpass_x[4];
            u8 lowpass_y[3];
            u8 lowpass_x[4];
        } iir, fir;
    } ldg;

    struct {
        struct {
            u8 highpass_slope;
            u8 highpass_x;
            u8 highpass_y;
            u8 lowpass_slope;
            u8 lowpass_x;
            u8 lowpass_y;
        } iir, fir;
    } coring;
};

#endif

struct isp_vsm_result {
    u32 x, y;
};

struct isp_vsm_context {
    bool_t enable;
    struct ic_window window;
    u32 h_seg, v_seg;
};

#ifndef WDR3_BIN
#define WDR3_BIN 14
#endif
struct isp_wdr3_context {
    bool_t enable;
    bool_t changed;
    bool_t inited;
    u32 strength;
    u32 max_gain;
    u32 global_strength;
    u32 histogram[WDR3_BIN];
    u32 shift[WDR3_BIN];
    u32 invert_linear[WDR3_BIN];
    u32 invert_curve[WDR3_BIN];
    u32 gamma_pre[WDR3_BIN];
    u32 gamma_up[WDR3_BIN];
    u32 gamma_down[WDR3_BIN];
    u32 entropy[WDR3_BIN];
    u32 distance_weight[WDR3_BIN];
    u32 difference_weight[WDR3_BIN];
};

#ifdef ISP_WDR_V4
#ifndef WDR4_BIN
#define WDR4_BIN 20
#endif

struct isp_wdr4_context {
    bool_t enable;
    bool_t changed;
    bool_t inited;
    u8 ratioLSVS, ratioLS;
    u8 drc_bayer_ratio, drc_bayer_ratioLSVS;
    u32 strength;
    u32 high_strength;
    u32 low_strength;
    u32 low_gain;
    u32 global_strength;
    u32 contrast;
    u32 flat_strength;
    u32 flat_threshold;
    u32 entropy_slope;
    u32 entropy_base;
    u32 histogram[WDR4_BIN];
    u32 shift[WDR4_BIN];
    u32 shift0[WDR4_BIN];
    u32 invert_linear[WDR4_BIN];
    u32 invert_curve[WDR4_BIN];
    u32 gamma_pre[WDR4_BIN];
    u32 gamma_up[WDR4_BIN];
    u32 gamma_down[WDR4_BIN];
    u32 entropy[WDR4_BIN];
    u32 distance_weight[WDR4_BIN];
    u32 difference_weight[WDR4_BIN];
    u32 smooth_invert[WDR4_BIN];
};
#endif

#if defined(ISP_WDR_V5) || defined(ISP_WDR_V5_1) || defined(ISP_WDR_V5_2) || defined(ISP_WDR_V5_2_1) || defined(ISP_WDR_V5_3)
struct isp_wdr5_context{
    bool_t inited;
    u16 dummy_blk;
    bool_t enable;
    u16 block_height;
    u16 block_width;
    u32 block_area_inverse;
    u8 value_weight_3;
    u8 value_weight_2;
    u8 value_weight_1;
    u8 value_weight_0;
    u8 strength;
    u16 low_strength;
    u8 high_strength;
    u16 drc_bayer_ratio;
    u16 contrast;
    u16 pixel_base_entropy;
    u16 pixel_slope_entropy;
    u32 biliteral_height_sigma;
    u32 biliteral_width_sigma;
    u32 biliteral_value_sigma;
    u16 block_row_flag;
    u16 block_col_flag;
    u8 pixel_base_adjust;
    u8 pixel_slope_adjust;
    bool_t local_flat_mode;
    u8 local_flat_level;
    bool_t global_curve_mode;
    u8 global_flat_level;
    u16 global_fixed_weight;
    u32 frame_average;
    u32 frame_std;

    u8 lut_histogram_addr;
    u32 lut_histogram_write_data[65];
    u8 lut_shift_addr;
    u32 lut_shift_write_data[65];
    u8 lut_shift0_addr;
    u32 lut_shift0_write_data[65];
    u8 lut_gammapre_addr;
    u32 lut_gammapre_write_data[65];
    u8 lut_gammaup_addr;
	u32 lut_gammaup_write_data[65]	;
    u8 lut_gammadown_addr;
    u32 lut_gammadown_write_data[65];
    u8 lut_entropy_addr;
    u32 lut_entropy_write_data[65];
    u8 lut_distance_weight_addr;
    u32 lut_distance_weight_write_data[65];
    u8 lut_difference_weight_addr;
    u32 lut_difference_weight_write_data[65];
    u8 lut_weight_raw_addr;
    u32 lut_weight_raw_write_data[65];
    u8 lut_maxgain_addr;
	u32 lut_maxgain_write_data[65]	;
    u8 lut_mingain_addr;
    u32 lut_mingain_write_data[65];
    u8 lut_flat_factor_addr;
    u32 lut_flat_factor_write_data[272];
    u8 lut_flat_level_addr;
    u8 lut_flat_level_write_data[68];
    u8 lut_sat_x_addr;
    u32 lut_sat_x_write_data[18];
    u8 lut_sat_shift_addr;
    u32 lut_sat_shift_write_data[18];
    u8 lut_sat_addr;
    u32 lut_sat_write_data[18];
    u8 lut_degamma_addr;
	u32 lut_degamma_write_data[65]	;

    // u8 ratioLSVS;
    // u8 ratioLS;
    u32 buffer_log[65];

    bool_t wdr5_histogram_w_data_changed;
    bool_t wdr5_lut_shift_w_data_changed;
    bool_t wdr5_gammapre_w_data_changed;
    bool_t wdr5_gammadown_w_data_changed;
    bool_t wdr5_entropy_w_data_changed;
    bool_t wdr5_lut_shift0_w_data_changed;
    bool_t wdr5_lut_distance_weight_w_data_changed;
    bool_t wdr5_difference_weight_w_data_changed;
    bool_t wdr5_flat_factor_w_data_changed;
    bool_t wdr5_flat_level_w_data_changed;
    bool_t wdr5_sat_shift_w_data_changed;

#if defined(ISP_WDR_V5_2) || defined(ISP_WDR_V5_2_1) || defined(ISP_WDR_V5_3)

    // u8 ratioLSVS_E3_0
    u8 rgb_coef_0;
    u8 rgb_coef_1;
    u8 rgb_coef_2;
    u8 color_weight_0;
    u8 color_weight_1;
    u8 color_weight_2;
    u8 lightness_weight;

    bool_t light_enable;
    u8  light_sat_lothr;
    u32 light_sat_hithr_factor;

    u32 red_thr_0;
    u32 red_thr_1;
    u32 red_thr_2;
    u32 red_thr_3;
    u32 red_factor_01;
    u32 red_factor_23;

    u32 yellow_thr_0;
    u32 yellow_thr_1;
    u32 yellow_thr_2;
    u32 yellow_thr_3;
    u32 yellow_factor_01;
    u32 yellow_factor_23;

    u32 green_thr_0;
    u32 green_thr_1;
    u32 green_thr_2;
    u32 green_thr_3;
    u32 green_factor_01;
    u32 green_factor_23;
    
    u32 cyan_thr_0;
    u32 cyan_thr_1;
    u32 cyan_thr_2;
    u32 cyan_thr_3;
    u32 cyan_factor_01;
    u32 cyan_factor_23;
    
    u32 blue_thr_0;
    u32 blue_thr_1;
    u32 blue_thr_2;
    u32 blue_thr_3;
    u32 blue_factor_01;
    u32 blue_factor_23;
    
    u32 magenta_thr_0;
    u32 magenta_thr_1;
    u32 magenta_thr_2;
    u32 magenta_thr_3;
    u32 magenta_factor_01;
    u32 magenta_factor_23;

    u8 hist_range_thr;
    u32 hlc_base;
    u8 hlc_slope;

    u8 damp_mode;
    u8 damp_filter_size;
    u8 damp_curve_coef;
    u8 damp_avg_coef;
    u8 damp_coef_inc_limit;
    u32 damp_lothr;
    u8 damp_coef_dec_limit;
    u16 damp_thr_factor;
    u8 damp_avg_min;
    u8 damp_avg_max;
    u8 damp_curve_min;
    u8 damp_curve_max;
#endif
};
#endif

struct isp_exp2_context {
    bool_t enable;
    struct ic_window window;
    /* weight; */
    u8 r, gr, gb, b;
    u8 inputSelect;   //00: degamma output, 01: awb_gain output  10: WDR3 output
    /* write 4096/8192 EXPV2 mean value to dma by MI MP-JDP path. */
    /* physical address, alloacte by user */
    u64 pa;
};

struct isp_expv3_context {
    bool_t enable;
    u8   measure_mode;
    u8   inputSelect;
    bool_t windowEnable[25];
    u16  v_offset[25];
    u16  h_offset[25];
    u16  v_size[25];
    u16  h_size[25];
};

#define ISP_2DNR_SIGMA_BIN 60
#ifdef ISP_2DNR_V3
struct isp_2dnr_context {
    bool_t enable;
    u32 pre_gamma;
    u32 strength;
    u16 sigmaYLut[ISP_2DNR_SIGMA_BIN];
    u32 sigma_sqr;
    u32 weight;
};
#elif defined(ISP_2DNR_V5_V1) 
struct isp_2dnr_context {
    bool_t enable;
    bool_t luma_en;
    u32 pre_gamma;
    u32 strength;
    u16 sigmaYLut[ISP_2DNR_SIGMA_BIN];
    u32 sigma_sqr;
    u32 weight;
    u16 sigma_luma_px[12];
    u16 sigma_luma_y[12];
    u16 str_off;
    u16 str_max;
    u8 str_slope;
};
#elif defined(ISP_2DNR_V5_V2) 
struct isp_2dnr_context {
    bool_t enable;
    bool_t luma_en;
    bool_t pre_gamma;
    u32 strength;
    u16 sigmaYLut[ISP_2DNR_SIGMA_BIN];
    u32 sigma_sqr;
    u32 weight;
    u16 sigma_luma_px[12];
    u16 sigma_luma_y[12];

    u16 str_off;
    u16 str_max;
    u8 str_slope;
};
#elif defined(ISP_2DNR_V5)
struct isp_2dnr_context {
    bool_t enable;
    u32 pre_gamma;
    u32 strength;
    u16 sigmaYLut[ISP_2DNR_SIGMA_BIN];
    u32 sigma_sqr;
    u32 weight;

    u16 str_off;
    u16 str_max;
    u8 str_slope;
};
#elif defined(ISP_2DNR_V6)
struct isp_2dnr_context {
    bool_t enable;
    bool_t luma_ctrl_en;
    bool_t lsc_gain_comp_en;
    bool_t motion_ctrl_en;

    u16 sigmaYLut[ISP_2DNR_SIGMA_BIN];
    u32 sigma_sqr;

    u16 str_off;
    u16 str_max;
    u8 str_slope;

    u16 sigma_factor_motion_min;
    u16 sigma_factor_motion_max;
    u16 sigma_factor_motion_max_mul;

    u16 static_detail_thresh[2][3];
    u16 static_detail_boost_thresh[2][3];
    u16 static_detail_boost[2][3];
    u16 static_detail_clip_thresh[2][3];
    u16 moving_detail_thresh[2][3];
    u16 moving_detail_boost_thresh[2][3];
    u16 moving_detail_boost[2][3];
    u16 moving_detail_clip_thresh[2][3];
    u16 sigma_factor_mul[3];
    u16 static_factor[3];
    u16     motion_anchor[2];

    u16 motion_lut_px[12];
    u16 motion_lut_y[12];
    u16 motion_lut_x[12];
    u16 comp_lut_px[12];
    u16 comp_lut_y[12];
    u16 comp_lut_x[12];

    u16 sigma_luma_px[12];
    u16 sigma_luma_y[12];
    u16 sigma_luma_x[12];
    u16 sigma_scale[3];

};

#else
struct isp_2dnr_context {
    bool_t enable;
    u32 pre_gamma;
    u32 strength;
    u16 sigmaYLut[ISP_2DNR_SIGMA_BIN];
    u32 sigma_sqr;
    u32 weight;
};
#endif

#ifdef ISP_3DNR_V2_4

struct isp_3dnr_motion_detect_roi{
    u32 motion_detect_en_0_31;   // Denoise3D motion detection enable control, ROI 0~31
    u32 motion_detect_en_32_63;  // Denoise3D motion detection enable control, ROI 32~63

    struct ic_window roi_windows[64]; // 64 motion detetc ROI windows 
};

//motion detect diffrence value for each ROI
struct isp_3dnr_motion_roi_diff {
    uint32_t roiDiff[64];    
} ;

#endif

struct isp_3dnr_compress_context {
    u8 weight_up_y[2];
    u8 weight_down[4];
    u8 weight_up[8];
};
struct isp_3dnr_context {
    bool_t enable;
    bool_t update_bin;
    bool_t enable_h, enable_v;
    bool_t enable_temperal;
    bool_t enable_dilate;
    bool_t init;
    u32 spacial_curve[17];
    u16 temporal_curve[17];
    u32 strength;
    u32 motion_factor;
    u16 delta_factor;
    /* write full denoise3d reference raw image to dma by MI SP2. */
    /* physical address, alloacte by user */
    u64 pa;
    u32 size;
    struct isp_3dnr_compress_context compress;

    u16 update_temporal;
    u8 strength_curve_temporal;
    u8 range_t_h;
    u8 range_t_v;
    u32 range_t_inv;

#ifdef ISP_3DNR_V2_4
    struct isp_3dnr_motion_detect_roi motion_roi;
#endif
};

struct isp_3dnr_update {
    u32 thr_edge_h_inv;
    u32 thr_edge_v_inv;
    u32 thr_motion_inv;
    u32 thr_range_s_inv;
    u32 range_t_h;
    u32 range_t_v;
    u32 range_d;
    u32 thr_range_t_inv;
    u32 thr_delta_h_inv;
    u32 thr_delta_v_inv;
    u32 thr_delta_t_inv;
};

#define DENOISE3D_GAMMA_BIN 33U
#define NOISE_CURVE_MAX 1048575U
#define NOISE_CURVE_INPUT_BIT 20U
#define NOISE_CURVE_SAMPLE_NUM 129U

struct isp_3dnr_stats {
    u32 bg_sum;
    u32 motion_sum;
    u32 bg_pixel_cnt;
    u32 motion_pixel_cnt;
    u32 frame_avg;
};
struct isp_3dnr3_curve {
    u16 preGamma_y[DENOISE3D_GAMMA_BIN];
    u16 invGamma_y[DENOISE3D_GAMMA_BIN];
};

#ifdef ISP_GAMMA_FEBE
struct isp_gamma_febe {
    u32 compress[NOISE_CURVE_SAMPLE_NUM];
    u32 expand[NOISE_CURVE_SAMPLE_NUM];
    u32  pre_blsA ;
    u32  pre_blsB ;
    u32  pre_blsC ;
    u32  pre_blsD ;

    u32  blsA ;
    u32  blsB ;
    u32  blsC ;
    u32  blsD ;
    bool_t enable_fe;
    bool_t enable_be;
    bool_t enable_bls;

    /* Status flag for updating in vblank. */
    bool_t curve_changed;
};
#endif

#ifdef ISP_3DNR_V3_1

struct isp_3dnr_buffer {
    /* write full denoise3d reference raw image to dma by MI SP2. */
    /* physical address, alloacte by user */
    u32 pa_refer;
    u32 size_refer;
    u32 pa_motion;
    u32 size_motion;
};

struct isp_3dnrV3_context {
    bool_t enable;

    bool_t enable_motion_erosion;
    bool_t enable_motion_converage;
    bool_t enable_motion_dilation;
    bool_t enable_tnr;
    bool_t enable_2dnr;
    u8  strength;
    u16 noise_level;
    u16 noise_mean;
    u16 noise_threshold;
    u16 motion_mean;
    u8  range_h;
    u8  range_v;
    u8  dilate_range_h;
    u8  dilate_range_v;
    u32 motion_inv_factor;
    u16 update_factor;
    u16 motion_update_factor;
    u16 pre_motion_weight;
    u16 motion_slope;
    u16 motion_converage_shift;
    u16 motion_converage_max;
    u16 tnr_val_shift_bit;
    u16 tnr_diff_shift_bit;
    u16 sad_weight;
    u16 luma_pixel_val_shift;
    u16 luma_pixel_slope;
    u16 luma_pixel_slope_shift;
    u16 luma_pixel_slope_min;
    u16 dmy_hblank;
    u32 fn;

    struct isp_3dnr_buffer buf;
 
    u8 frames;  //skip the first frame before  clear reset.
                //keep the 3dnr enable in register to avoid scrolling screen when repeat enable

    bool_t enable_bls;
   u32 filterLen;      //[0,1024] 20
   u32 filterLen2;     //[0,1024] 10
      u8 dilateH;         //[3,7] 3
   u8 noisemodelTransStart;    //0
   u8 fixCurveStart;       //0
   u8 lumaCompEn;          

    struct isp_3dnr3_curve curve;

    bool_t enable_inv_gamma;
    bool_t enable_pre_gamma;
};
#endif

#ifdef ISP_3DNR_V3


struct isp_3dnr_buffer {
    /* write full denoise3d reference raw image to dma by MI SP2. */
    /* physical address, alloacte by user */
    u32 pa_refer;
    u32 size_refer;
    u32 pa_motion;
    u32 size_motion;
};

struct isp_3dnrV3_context {
    bool_t enable;

    bool_t enable_motion_erosion;
    bool_t enable_motion_converage;
    bool_t enable_motion_dilation;
    bool_t enable_tnr;
    bool_t enable_2dnr;
    u8  strength;
    u16 noise_level;
    u16 noise_mean;
    u16 noise_threshold;
    u16 motion_mean;
    u8  range_h;
    u8  range_v;
    u8  dilate_range_h;
    u8  dilate_range_v;
    u32 motion_inv_factor;
    u16 update_factor;
    u16 motion_update_factor;
    u16 pre_motion_weight;
    u16 motion_slope;
    u16 motion_converage_shift;
    u16 motion_converage_max;
    u16 tnr_val_shift_bit;
    u16 tnr_diff_shift_bit;
    u16 sad_weight;
    u16 luma_pixel_val_shift;
    u16 luma_pixel_slope;
    u16 luma_pixel_slope_shift;
    u16 luma_pixel_slope_min;
    u16 dmy_hblank;
    u32 fn;
    struct isp_3dnr3_curve curve;
    struct isp_3dnr_buffer buf;
 
    u8 frames;  //skip the first frame before  clear reset.
                //keep the 3dnr enable in register to avoid scrolling screen when repeat enable
    u32 filterLen;      //[0,1024] 20
    u32 filterLen2;     //[0,1024] 10
    bool_t enable_inv_gamma;
    bool_t enable_pre_gamma;
};
#endif

#ifdef ISP_3DNR_V4
#define ISP_3DNR4_DECOMPOSE_KERNEL 25
#define ISP_3DNR4_MOTION_GAUSS_KERNEL 5
#define ISP_3DNR4_THR_LUMA_CURVE_NUM 12
#define ISP_3DNR4_BOOST_C_NUM 3

struct isp_3dnr_buffer {
    /* write full denoise3d reference raw image to dma by MI SP2. */
    /* physical address, alloacte by user */
    u32 pa_refer;
    u32 size_refer;
    u32 pa_motion;
    u32 size_motion;
};

struct isp_3dnrV4_context {
    bool_t enable;

    bool_t enable_motion_converage;
    bool_t enable_motion_dilation;
    bool_t enable_tnr;
    bool_t enable_2dnr;
    bool_t enable_dither;
    bool_t enable_gauss_blur;
    u8  strength;
    u8  strength2;
    u16 noise_level;
    u16 noise_mean;
    u16 noise_threshold;
    u16 motion_mean;
    u8  range_h;
    u8  dilate_range_h;
    u32 motion_inv_factor;
    u16 update_factor;
    u16 motion_update_factor;
    u16 pre_motion_weight;

    u16 sad_weight;
    u16 luma_pixel_val_shift;
    u16 luma_pixel_slope;
    u16 luma_pixel_slope_shift;
    u16 luma_pixel_slope_min;
    u16 dmy_hblank;
    u32 fn;

    struct isp_3dnr_buffer buf;
 
    u8 frames;  //skip the first frame before  clear reset.
                //keep the 3dnr enable in register to avoid scrolling screen when repeat enable

    u32 filterLen;      //[0,1024] 20
    u32 filterLen2;     //[0,1024] 10
    u8 dilateH;         //[3,7] 3
    u8 noisemodelTransStart;    //0
    u8 fixCurveStart;       //0
    u8 lumaCompEn;          

    u8 motion_smooth_lvl;
    bool_t motion_diff_type;
    u8 motion_diff_shift_bit;
    u8 motion_dither_fac;
    u16 inv_sigma_l0;
    u16 inv_sigma_l1;
    u16 offset_l0;
    u16 offset_l1;
    u8 decompose_kernel[ISP_3DNR4_DECOMPOSE_KERNEL];
    u8 curve_x[ISP_3DNR4_THR_LUMA_CURVE_NUM];
    u16 curve_y[ISP_3DNR4_THR_LUMA_CURVE_NUM];
    u16 motion_slope_y[ISP_3DNR4_THR_LUMA_CURVE_NUM];
    u16 boost_c_t1[ISP_3DNR4_BOOST_C_NUM];
    u16 boost_c_t2[ISP_3DNR4_BOOST_C_NUM];
    u16 boost_fac[ISP_3DNR4_BOOST_C_NUM];
    u16 boost_cip_val[ISP_3DNR4_BOOST_C_NUM];
    u8 motion_gauss_kernel[ISP_3DNR4_MOTION_GAUSS_KERNEL];

    struct isp_3dnr3_curve curve;

    bool_t enable_inv_gamma;
    bool_t enable_pre_gamma;
};
#endif

#ifdef ISP_3D_LUT

#define LUT3D_TBL_NUM (u32)(17*17*17)
struct isp_lut3d_context{
    bool_t changed;
    bool_t enable;
    u32 lut3d_r[LUT3D_TBL_NUM];
    u32 lut3d_g[LUT3D_TBL_NUM];
    u32 lut3d_b[LUT3D_TBL_NUM];
    bool_t ahb_raw_sel;
    bool_t lut3d_w_data_changed;
};

#endif

struct isp_mi_hdr_context {
    u8 ppc;
};

struct isp_hdr_context {
    bool_t enable;
    bool_t dg_enable;
    bool_t wb_enable;
    bool_t regs_inform_enable;
    /* hdr bls */
    u16 width;
    u16 height;
    u8 weightR, weightG, weightB;
    u8 hSyncPol, vSyncPol;
    u8 bypass_select, hdr_mode;
    u8 bayer_pattern;
    u16 start_linear, norm_factor_mul_linear;
    u16 start_nonlinear, norm_factor_mul_nonlinear;
    u16 dummy_s_hblank, dummy_vs_hblank;
    u16 out_hblank, out_vblank;
    u16 short_exp;
    /* hdr bls */
    u8 bls_flag;
    u16 bls_a0, bls_b0, bls_c0, bls_d0;
    u16 bls_a1, bls_b1, bls_c1, bls_d1;
    u16 bls_a2, bls_b2, bls_c2, bls_d2;
    /* hdr dgain */
    u8 dg_flag;
    u16 dg_r0, dg_g0, dg_gr0, dg_gb0;
    u16 dg_r1, dg_g1, dg_gr1, dg_gb1;
    u16 dg_r2, dg_g2, dg_gr2, dg_gb2;
    /* hdr awb */
    u8 awb_flag;
    u32 awb_gain_r0, awb_gain_gr0, awb_gain_gb0, awb_gain_b0;
    u32 awb_gain_r1, awb_gain_gr1, awb_gain_gb1, awb_gain_b1;
    u32 awb_gain_r2, awb_gain_gr2, awb_gain_gb2, awb_gain_b2;
    /* long short, very short */
    u32 ratio_ls0, ratio_ls1, ratio_vss0, ratio_vss1;

    u8 combine_mode, baseFrame;
    u32 ls_ext_bit, vs_ext_bit;
    u32 vs_valid_thresh;
    u32 vs_valid_offset_val;
    u32 long_sat_thresh;
    u32 long_combine_weight;
};

struct isp_hdrv3_context {
    bool_t enable;
    bool_t dg_enable;
    bool_t wb_enable;
    bool_t regs_inform_enable;
    u16 width;
    u16 height;
    u8 weightR, weightG, weightB;
    u8 hSyncPol, vSyncPol;
    u8 bypass_select, hdr_mode;
    u8 bayer_pattern;
    u16 start_linear, norm_factor_mul_linear;
    u16 start_nonlinear, norm_factor_mul_nonlinear;
    u16 s_start_linear, s_norm_factor;
    u16 vs_start_linear, vs_norm_factor;
    u16 dummy_s_hblank, dummy_vs_hblank;
    u16 out_hblank, out_vblank;
    /* hdr bls */
    u8 bls_flag;
    u16 bls_a0, bls_b0, bls_c0, bls_d0;
    u16 bls_a1, bls_b1, bls_c1, bls_d1;
    u16 bls_a2, bls_b2, bls_c2, bls_d2;
    /* hdr dgain */
    u8 dg_flag;
    u16 dg_r0, dg_g0, dg_gr0, dg_gb0;
    u16 dg_r1, dg_g1, dg_gr1, dg_gb1;
    u16 dg_r2, dg_g2, dg_gr2, dg_gb2;
    /* hdr awb */
    u8 awb_flag;
    u32 awb_gain_r0, awb_gain_gr0, awb_gain_gb0, awb_gain_b0;
    u32 awb_gain_r1, awb_gain_gr1, awb_gain_gb1, awb_gain_b1;
    u32 awb_gain_r2, awb_gain_gr2, awb_gain_gb2, awb_gain_b2;
    /* long short, very short */
    u32 ratio_ls0, ratio_ls1, ratio_vss0, ratio_vss1;

    u8 combine_mode, baseFrame;
    u32 ls_ext_bit, vs_ext_bit;
    u32 vs_valid_thresh;
    u32 vs_valid_offset_val;
    u32 long_sat_thresh;
    u32 long_combine_weight;
    u32	compress_lut_start_idx;
    u32	compress_lut[15];
    u32 hdr_dma_start_mode;
    u32 hdr_dma_start_lines;
};

#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V2_1) || (defined ISP_HDR_STITCH_V2_1_2DOL)
struct isp_hdrv2_context {
    bool_t enable;
    bool_t dg_enable;
    bool_t wb_enable;
    bool_t regs_inform_enable;
    u16 width;
    u16 height;
    u8 weightR, weightG, weightB;
    u8 hSyncPol, vSyncPol;
    u8 bypass_select, hdr_mode;
    u8 bayer_pattern;
    u16 start_linear, norm_factor_mul_linear;
    u16 start_nonlinear, norm_factor_mul_nonlinear;
    u16 s_start_linear, s_norm_factor;
    u16 vs_start_linear, vs_norm_factor;
    u16 dummy_s_hblank, dummy_vs_hblank;
    u16 out_hblank, out_vblank;
    /* hdr bls */
    u8 bls_flag;
    u16 bls_a0, bls_b0, bls_c0, bls_d0;
    u16 bls_a1, bls_b1, bls_c1, bls_d1;
    u16 bls_a2, bls_b2, bls_c2, bls_d2;
    /* hdr dgain */
    u8 dg_flag;
    u16 dg_r0, dg_g0, dg_gr0, dg_gb0;
    u16 dg_r1, dg_g1, dg_gr1, dg_gb1;
    u16 dg_r2, dg_g2, dg_gr2, dg_gb2;
    /* hdr awb */
    u8 awb_flag;
    u32 awb_gain_r0, awb_gain_gr0, awb_gain_gb0, awb_gain_b0;
    u32 awb_gain_r1, awb_gain_gr1, awb_gain_gb1, awb_gain_b1;
    u32 awb_gain_r2, awb_gain_gr2, awb_gain_gb2, awb_gain_b2;
    /* long short, very short */
    u32 ratio_ls0, ratio_ls1, ratio_vss0, ratio_vss1;

    u8 combine_mode, baseFrame;
    u32 ls_ext_bit, vs_ext_bit;
    u32 vs_valid_thresh;
    u32 vs_valid_offset_val;
    u32 long_sat_thresh;
    u32 long_combine_weight;
	u32	compress_lut_start_idx;
	u32	compress_lut[15];
};
#endif

#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V3)
struct isp_hdr_dpf_context {
    bool_t                       dpf_enable;
	u32	                       gain_r, gain_gr, gain_gb, gain_b;
    u32                        invR;
    u32                        invG;
    u32                        invB;
    u32                        weight_g_1234;
    u32                        weight_g_56;
    u32                        weight_rb_1234;
    u32                        weight_rb_56;
    u16                        noise_curve[17];
};
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V3)  || (defined ISP_HDR_STITCH_V3_2DOL)
struct isp_hdr_deghost_context {
    bool_t                       motion_enable[2];
    u16                        motion_weight[2];
    u32                        motion_weight_shorter;
    u32                        motion_sat_threshold;
    u32                        motion_weight_update_threshold;
    u16                        motion_lower_threshold_ls[4];
    u16                        motion_factor_threshold_ls[4];
    u16                        motion_lower_threshold_lsvs[4];
    u16                        motion_factor_threshold_lsvs[4];
    u16                        dark_lower_threshold_ls[4];
    u16                        dark_factor_threshold_ls[4];
    u16                        dark_lower_threshold_lsvs[4];
    u16                        dark_factor_threshold_lsvs[4];
};
#endif

struct isp_hdrv31_context {
    bool_t enable;
    bool_t dg_enable;
    bool_t wb_enable;
    bool_t regs_inform_enable;
    /* hdr bls */
    u16 width;
    u16 height;
    u8 weightR, weightG, weightB;
    u8 hSyncPol, vSyncPol;
    u8 bypass_select, hdr_mode;
    u8 bayer_pattern;
    u16 start_linear, norm_factor_mul_linear;
    u16 start_nonlinear, norm_factor_mul_nonlinear;
    u16 s_start_linear, s_norm_factor;
    u16 vs_start_linear, vs_norm_factor;
    u16 lsvs_start_linear, lsvs_norm_factor;
    u16 e3_start_linear, e3_norm_factor;
    u16 dummy_s_hblank, dummy_vs_hblank;
    u16 out_hblank, out_vblank;
    /* hdr bls */
    u8 bls_flag;
    u16 bls_a0, bls_b0, bls_c0, bls_d0;
    u16 bls_a1, bls_b1, bls_c1, bls_d1;
    u16 bls_a2, bls_b2, bls_c2, bls_d2;
    u16 bls_a3, bls_b3, bls_c3, bls_d3;
    u16 bls_out_a, bls_out_b, bls_out_c, bls_out_d;
    /* hdr dgain */
    u16 dg_r0, dg_g0, dg_gr0, dg_gb0;
    u16 dg_r1, dg_g1, dg_gr1, dg_gb1;
    u16 dg_r2, dg_g2, dg_gr2, dg_gb2;
    u16 dg_r3, dg_g3, dg_gr3, dg_gb3;
    /* hdr awb */
    u8 awb_flag;
    u32 awb_gain_r0, awb_gain_gr0, awb_gain_gb0, awb_gain_b0;
    u32 awb_gain_r1, awb_gain_gr1, awb_gain_gb1, awb_gain_b1;
    u32 awb_gain_r2, awb_gain_gr2, awb_gain_gb2, awb_gain_b2;
    u32 awb_gain_r3, awb_gain_gr3, awb_gain_gb3, awb_gain_b3;
    /* long short, very short */
    u32 ratio_ls0, ratio_ls1, ratio_vss0, ratio_vss1, ratio_e3s0;
};

struct isp_simp_context {
    bool_t enable;
    u32 x, y;
    u32 r, g, b;
    u32 transparency_mode;
    u32 ref_image;
};

struct isp_compand_curve_context {
    bool_t     enable; 
    bool_t     update_curve;
    uint8_t  in_bit;
    uint8_t  out_bit;
    uint8_t  px[64];
    uint32_t x_data[64];
    uint32_t y_data[64];
};

struct isp_compand_bls_context {
    uint8_t  bit_width;
    uint8_t  in_bit;
    bool_t     enable;
    uint32_t a;
    uint32_t b;
    uint32_t c;
    uint32_t d;
};

/* COMPAND */
struct isp_comp_context {
    struct isp_compand_curve_context expand;
    struct isp_compand_bls_context   bls;
    struct isp_compand_curve_context compress;
    bool_t comp_w_data_changed;
};

struct isp_cproc_context {
    bool_t enable;
    bool_t changed;
    u32 contrast;
    u32 brightness;
    u32 saturation;
    u32 hue;
    bool_t y_out_full;
    bool_t c_out_full;
    bool_t y_in_full;
};

struct elawb_ellipse_info {
	u32 x, y;		/* ellipse center */
	u32 a1, a2, a3, a4;	/* ellipse axis */
	u32 r_max_sqr;
};

struct isp_elawb_context {
	bool_t enable;
	u32 id;			/* ellipse id,  1-8,  0 means update all. */
	struct elawb_ellipse_info info[8];
	struct ic_window window;
	u32 r, gr, gb, b;	/* gain */
};

struct isp_gcmono_data {
	u8 basePara[1024];
	u32 px[64];
	u32 dataX[63];
	u32 dataY[64];
};

struct isp_gcmono_context {
	u32 enable;
	u32 mode;
    bool_t gcmono_w_data_changed;
};

struct isp_rgbgamma_data {
	u32 rgbgc_r_px[64];
	u32 rgbgc_r_datax[63];
	u32 rgbgc_r_datay[64];
	u32 rgbgc_g_px[64];
	u32 rgbgc_g_datax[63];
	u32 rgbgc_g_datay[64];
	u32 rgbgc_b_px[64];
	u32 rgbgc_b_datax[63];
	u32 rgbgc_b_datay[64];
};

struct isp_rgbgamma_context {
	bool_t enable,changed,rgbgamma_data_changed;
	struct isp_rgbgamma_data rgbgamma_data;
};

struct isp_rgbir_bls_context {
	u32 a;
	u32 b;
	u32 c;
	u32 d;
};

struct isp_rgbir_rgb_gain_context {
	u16 r;
	u16 g;
	u16 b;
};

struct isp_rgbir_dpcc_context {
	u16 median_thr[4];
	u16 avg_thr[4];
};

struct isp_rgbir_cc_context {
	u16 mtx[3][4];
};

struct isp_rgbir_des_context {
	u32 ir_px1, ir_py[16];
	u32 ir_pd[16], l_pd[16];
	u32 l_px1, l_py[16];
};

struct isp_rgbir_sharpen_context {
	u8 sharpen_lvl, middle;
	u16 thresh_bl0, thresh_bl1;
	u16 thresh_sh0, thresh_sh1;
	u8 lum_weight_min, lum_weight_kink, lum_weight_gain;
};

struct isp_rgbir_ir_dnr_context {
	u16 ir_sigmaS;
	u8 winweight[6];
	u16 width, height;
};
struct isp_rgbir_context{
	bool_t prefilt_enable, green_filt_mode, green_filt_enable;
	u8 rgbirPattern, out_rgb_pattern, green_filt_stage1_select;
	u8 demosaic_threshold;
	bool_t part1_enable, part2_enable;
    bool_t enable_ir_raw_out;
	bool_t rccc_enable;
	uint32_t ir_threshold, l_threshold;

	struct isp_rgbir_bls_context bls;
	struct isp_rgbir_rgb_gain_context rgb_gain;
	struct isp_rgbir_dpcc_context dpcc;
	struct isp_rgbir_cc_context cc;
	struct isp_rgbir_des_context des;
	struct isp_rgbir_sharpen_context sharpen;
	struct isp_rgbir_ir_dnr_context ir_dnr;
};

struct isp_crop_context {
	bool_t enabled;
	struct ic_window window;
};

struct isp_pdaf_context {
	bool_t enable;
	bool_t correct_en;
	u8 bayer_pattern;
	u8 sensor_type;
	bool_t ocl2xl_shield;
	u32 image_h, image_v;
	u8 pix_00_shift_hw[2];
	u16	dummy_hblank;
	u8 correct_threshold;
	struct ic_window wcorrect_rect;
	u16 pd_area[4];
	u8 num_per_area[2];
	u8 shift_mark[32];
	u8 shift_lr[2];
	struct ic_window roi;
	u8 correct_r_channel, correct_l_channel;
	u8 correct_r_limit_base, correct_l_limit_base;
	u8 correct_r_change_down, correct_l_change_down;
	u8 correct_r_change_up, correct_l_change_up;
	/* physical address, alloacte by user */
	u32 pa;
};

struct isp_pp_path_context {
	u32 mi_pp_ctrl;
	u32 mi_pp_fmt;
	u32 mi_pp_data_swap;
	u32 mi_pp_y_base_ad_init;
	u32 mi_pp_y_size_init;
	u32 mi_pp_y_offs_cnt_init;
	u32 mi_pp_y_llength;
	u32 mi_pp_y_pic_width;    
	u32 mi_pp_y_pic_height;
	u32 mi_pp_y_pic_size;
	u32 mi_pp_y_lval_bytes;
};


#ifdef ISP_MI_PP_READ
typedef struct pp_dma_line_entry_s {
	u16 entry_line_num;
	u16 buf_line_num;

} pp_dma_line_entry_t;

#endif

#if defined(ISP_MPCTL)
struct isp_mp_ctrl_context {
    bool_t enabled;		//	"mp ctrl enable:0: bypass, output direct through input 1: eable the mp ctrl function"

    bool_t vsync_pol;		//	"Vertical sync polarity 0 : high active;1 : low active "
    bool_t hsync_pol;		//"Horizontal sync polarity 0 : high active; 1: low active"
    bool_t out_pixel_num;	//"At output port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"
    bool_t in_pixel_num;	//"At input port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"
    u16 crop_size; 		//	crop size for one line
    u16 crop_offset; 	//	crop offset for one line
    bool_t buf_full; 		//	buf full indicator  for line SRAM

};
#elif defined(ISP_MPCTL_V1_1)
struct isp_mp_ctrl_context {
    bool_t enabled;		//	"mp ctrl enable:0: bypass, output direct through input 1: eable the mp ctrl function"
    bool_t enabled1;      //	"mp ctrl1 enable:0: bypass, output direct through input 1: eable the mp ctrl function"
    bool_t sel_in;        // at input port 1'b0: select sensor input 1'b1: select bus_in input

    bool_t vsync_pol;		//	"Vertical sync polarity 0 : high active;1 : low active "
    bool_t hsync_pol;		//"Horizontal sync polarity 0 : high active; 1: low active"
    bool_t out_pixel_num;	//"At output port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"

    bool_t out_pixel_num1;	//"At output port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"
    bool_t in_pixel_num;	//"At input port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"
    u16 crop_size; 		//	crop size for one line
    u16 crop_offset; 	//	crop offset for one line

    u16 crop1_size; 		//	crop size for one line
    u16 crop1_offset; 	//	crop offset for one line

    bool_t buf_full; 		//	buf full indicator  for line SRAM

};

#endif
#ifdef ISP_STA

#ifdef ISP_STA_V1

#define AEV2_DMA_OFFSET 0U
#define AEV2_DMA_SIZE (4096U*2U)

#define HDR0_DMA_SIZE 168U
#define HDR0_DMA_OFFSET (4096U*2U) // 8192

#define HDR1_DMA_SIZE 168U
#define HDR1_DMA_OFFSET (4096U*2U+168U)  //8360

#define HDR2_DMA_SIZE 168U
#define HDR2_DMA_OFFSET (4096U*2U+168U+168U) //8528

#define AF_DMA_SIZE 24U
#define AF_DMA_OFFSET (4096U*2U+168U+168U+168U) //8696

#define HIST256_DMA_SIZE 1024U
#define HIST256_DMA_OFFSET (4096U*2U+168U+168U+168U+24U) //8720

#define HIST64_DMA_SIZE 128U
#define HIST64_DMA_OFFSET (4096U*2U+168U+168U+168U+24U+1024U) //9744

#define BASE_F_CNT_SIZE 4U
#define BASE_F_CNT_OFFSET (4096U*2U+168U+168U+168U+24U+1024U+128U)  //9872

#define ISP_STA_DMA_TOTAL_SIZE 9876U
#define ISP_STA_DMA_SIZE_MALLOC 9888U //(ISP_STA_DMA_TOTAL_SIZE+15)/16*16


#elif defined ISP_STA_V2

#ifdef ISP_AEV2_V4
#define AEV2_DMA_OFFSET 0U
#define AEV2_DMA_SIZE ((u32)(4096 *4))

#define HDR0_DMA_SIZE 168U
#define HDR0_DMA_OFFSET ((u32)(4096*4)) // 16384

#define HDR1_DMA_SIZE 168U
#define HDR1_DMA_OFFSET ((u32)(4096*4+168))  //16552

#define HDR2_DMA_SIZE 168U
#define HDR2_DMA_OFFSET ((u32)(4096*4+168+168)) //16720

#define HDR3_DMA_SIZE 168U
#define HDR3_DMA_OFFSET ((u32)(4096*4+168+168+168))//16888

#define AF_DMA_SIZE 24U
#define AF_DMA_OFFSET ((u32)(4096*4+168+168+168+168)) //17056

#define HIST256_DMA_SIZE 1024U
#define HIST256_DMA_OFFSET ((u32)(4096*4+168+168+168+168+24)) //17080

#define HIST64_DMA_SIZE 128U
#define HIST64_DMA_OFFSET ((u32)(4096*4+168+168+168+168+24+1024)) //18104

#define GTM_DMA_SIZE 128U*4U
#define GTM_DMA_OFFSET ((u32)(4096*4+168+168+168+168+24+1024+128))  //18232

#define BASE_F_CNT_SIZE 4U
#define BASE_F_CNT_OFFSET ((u32)(4096*4+168+168+168+168+24+1024+128+128*4))  //18744

#define ISP_STA_DMA_TOTAL_SIZE 18748U
#define ISP_STA_DMA_SIZE_MALLOC 18752U //(ISP_STA_DMA_TOTAL_SIZE+15)/16*16
#endif //ISP_AEV2_V4

#ifdef ISP_AEV2_V2

#ifdef ISP_AFM_V3

#define AEV2_DMA_OFFSET 0U
#define AEV2_DMA_SIZE ((u32)(4096 *2))

#define HDR0_DMA_SIZE 168U
#define HDR0_DMA_OFFSET ((u32)(4096*2) )

#define HDR1_DMA_SIZE 168U
#define HDR1_DMA_OFFSET ((u32)(4096*2+168)  )

#define HDR2_DMA_SIZE 168U
#define HDR2_DMA_OFFSET ((u32)(4096*2+168+168) )

#define AF_DMA_SIZE 24U
#define AF_DMA_OFFSET ((u32)(4096*2+168+168+168)) 

#define HIST256_DMA_SIZE 1024U
#define HIST256_DMA_OFFSET ((u32)(4096*2+168+168+168+24)) 

#define HIST64_DMA_SIZE 128U
#define HIST64_DMA_OFFSET ((u32)(4096*2+168+168+168+24+1024)) 

#define AFIIR_DMA_SIZE 3600U
#define AFIIR_DMA_OFFSET ((u32)(4096*2+168+168+168+24+1024+128))

#define BASE_F_CNT_SIZE 4U
#define BASE_F_CNT_OFFSET ((u32)(4096*2+168+168+168+24+1024+128+3600))  //13472

#define ISP_STA_DMA_TOTAL_SIZE 13476U
#define ISP_STA_DMA_SIZE_MALLOC 13488U //(ISP_STA_DMA_TOTAL_SIZE+15)/16*16

#else //ISP_AFM_V3

#define AEV2_DMA_OFFSET 0U
#define AEV2_DMA_SIZE ((u32)(4096 *2))

#define HDR0_DMA_SIZE 168U
#define HDR0_DMA_OFFSET ((u32)(4096*2)) 

#define HDR1_DMA_SIZE 168U
#define HDR1_DMA_OFFSET ((u32)(4096*2+168))  

#define HDR2_DMA_SIZE 168U
#define HDR2_DMA_OFFSET ((u32)(4096*2+168+168)) 

#define HDR3_DMA_SIZE 168U
#define HDR3_DMA_OFFSET ((u32)(4096*2+168+168+168)) 

#define AF_DMA_SIZE 24U
#define AF_DMA_OFFSET ((u32)(4096*2+168+168+168+168)) 

#define HIST256_DMA_SIZE 1024U
#define HIST256_DMA_OFFSET ((u32)(4096*2+168+168+168+168+24)) 

#define HIST64_DMA_SIZE 128U
#define HIST64_DMA_OFFSET ((u32)(4096*2+168+168+168+168+24+1024)) 

#define GTM_DMA_SIZE 128U*4U
#define GTM_DMA_OFFSET ((u32)(4096*2+168+168+168+168+24+1024+128))  

#define BASE_F_CNT_SIZE 4U
#define BASE_F_CNT_OFFSET ((u32)(4096*2+168+168+168+168+24+1024+128+128*4))  //10522

#define ISP_STA_DMA_TOTAL_SIZE 10556U
#define ISP_STA_DMA_SIZE_MALLOC 10560U //(ISP_STA_DMA_TOTAL_SIZE+15)/16*16

#endif //ISP_AFM_V3

#endif //ISP_AEV2_V2

#endif //ISP_STA_V2

#else

#ifdef ISP_AEV2_V4
#define AEV2_DMA_SIZE ((u32)(4096 *4))
#elif defined ISP_AEV2_V2
#define AEV2_DMA_SIZE ((u32)(4096 *2))
#else
#define AEV2_DMA_SIZE 4096U
#endif

#endif

struct isp_sta_context {
    bool_t isp_sta_enable;
    bool_t stitch_sta_enable;
    u64 pa;
};

struct isp_fusa_rdcd_context {
    bool_t eccEn;
    bool_t pixLossEn;
    uint8_t crcInRevEn;
    bool_t crcOutRevEn;
    bool_t crcXorEn;
    bool_t crcEn;
    bool_t dupEn;
    bool_t ecc2bitEn;
    bool_t ecc1bitEn;
    bool_t fifoEn;
};
struct isp_fusa_context {
    u32 crcLevel;
    bool_t eccEn;
    bool_t pixCountEn;
    bool_t timeOutEn;
    bool_t bistEn;
    bool_t bistPowerUpEn;
    uint8_t crcInRevEn;
    bool_t crcOutRevEn;
    bool_t crcXorEn;
    bool_t crcEn;
    bool_t ahbTimeOutEn;
    bool_t dupEn;
    bool_t ecc2bitEn;
    bool_t ecc1bitEn;

    u32 crcMpRoiH;
    u32 crcMpRoiV;
    u32 crcSp1RoiH;
    u32 crcSp1RoiV;
    u32 crcSp2RoiH;
    u32 crcSp2RoiV;

    u32 pixCountMrszOutH;
    u32 pixCountMrszOutV;
    u32 pixCountSrsz1OutH;
    u32 pixCountSrsz1OutV;
    u32 pixCountSrsz2OutH;
    u32 pixCountSrsz2OutV;
    u32 pixCountFiltOutH;
    u32 pixCountFiltOutV;

    u32 tpgBistImgHblank0;
    u32 tpgBistImgVblank0;
    u32 tpgBistImgHsize0;
    u32 tpgBistImgVsize0;
    u32 tpgBistSeed0;
    u32 tpgBistImgHblank1;
    u32 tpgBistImgVblank1;
    u32 tpgBistImgHsize1;
    u32 tpgBistImgVsize1;
    u32 tpgBistSeed1;
    u32 tpgBistImgHblank2;
    u32 tpgBistImgVblank2;
    u32 tpgBistImgHsize2;
    u32 tpgBistImgVsize2;
    u32 tpgBistSeed2;

    u32 bist0Frame0Crc;
    u32 bist0Frame1Crc;
    u32 bist0Frame2Crc;
    u32 bist1Frame0Crc;
    u32 bist1Frame1Crc;
    u32 bist1Frame2Crc;

    u32 bist2MpYFrame0Crc;
    u32 bist2MpCbFrame0Crc;
    u32 bist2MpCrFrame0Crc;
    u32 bist2MpYFrame1Crc;
    u32 bist2MpCbFrame1Crc;
    u32 bist2MpCrFrame1Crc;
    u32 bist2MpYFrame2Crc;
    u32 bist2MpCbFrame2Crc;
    u32 bist2MpCrFrame2Crc;
    
    u32 bist2Sp1YFrame0Crc;
    u32 bist2Sp1CbFrame0Crc;
    u32 bist2Sp1CrFrame0Crc;
    u32 bist2Sp1YFrame1Crc;
    u32 bist2Sp1CbFrame1Crc;
    u32 bist2Sp1CrFrame1Crc;
    u32 bist2Sp1YFrame2Crc;
    u32 bist2Sp1CbFrame2Crc;
    u32 bist2Sp1CrFrame2Crc;

    u32 bist2Sp2YFrame0Crc;
    u32 bist2Sp2CbFrame0Crc;
    u32 bist2Sp2CrFrame0Crc;
    u32 bist2Sp2YFrame1Crc;
    u32 bist2Sp2CbFrame1Crc;
    u32 bist2Sp2CrFrame1Crc;
    u32 bist2Sp2YFrame2Crc;
    u32 bist2Sp2CbFrame2Crc;
    u32 bist2Sp2CrFrame2Crc;
};

struct isp_ic_dev {
    u8 id;
    VirtualHandle_t virtualHalHandle;
    HalMssHandle_t hMss;
    struct isp_context ctx;
    struct isp_digital_gain_cxt dgain;
    struct isp_bls_context bls;
    struct isp_tpg_context tpg;
    struct isp_mux_context mux;
    struct isp_awb_context awb;
    struct isp_lsc_context lsc;
#ifdef ISP_GC_V1
    struct isp_gamma_out_context gamma_out;
#endif
    struct isp_xtalk_context xtalk;
    struct isp_cc_context cc;
    struct isp_cnr_context cnr;
    struct isp_cnr2_context cnr2;
    struct isp_coloring_context coloring;
    struct isp_is_context is;
    struct isp_is_context rawis;
    struct isp_mi_context mi;
    struct isp_dpf_context dpf;
    struct isp_ee_context ee;
    struct isp_ee3_context ee3;
    struct isp_exp_context exp;
    struct isp_hist_context hist;
    struct isp_fusa_context fusaMechanism;
    struct isp_fusa_rdcd_context fusaRdcd;
#ifdef ISP_HIST64
    struct isp_hist64_context hist64;
#endif
    struct isp_dpcc_context dpcc;
    struct isp_cac_context cac;
    struct isp_deg_context deg;
    struct isp_ie_context ie;
    struct isp_vsm_context vsm;
    struct isp_afm_context afm;
#ifdef ISP_AFM_V3
    struct isp_afmv3_context afmv3;
#endif
    struct isp_wdr3_context wdr3;
    struct isp_exp2_context exp2;
	struct isp_expv3_context	expv3;
#ifdef ISP_HDR_STITCH_V1_3
    struct isp_hdr_context hdr;
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V2_1) || (defined ISP_HDR_STITCH_V2_1_2DOL)
    struct isp_hdrv2_context hdr;
#endif
#if (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_2DOL)
	struct isp_hdrv3_context hdr;
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V3)
    struct isp_hdr_dpf_context hdr_dpf[3];
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_2DOL)
    struct isp_hdr_deghost_context hdr_deghost;
#endif
#ifdef ISP_HDR_STITCH_V3_1
    struct isp_hdrv31_context hdr;
#endif
    struct isp_mi_hdr_context miHdr;
#ifdef ISP_HDR_2_SENSORS
    u8 hdr_id;
#endif
    u32 input_type;
    u32 sensor_mode;

    struct isp_exp_context hdrexp;
    struct isp_hist_context hdrhist;
    struct isp_2dnr_context dnr2;
    struct isp_3dnr_context dnr3;

    struct isp_comp_context comp;
    struct isp_simp_context simp;
    struct isp_cproc_context cproc;
    struct isp_elawb_context elawb;
    struct isp_gcmono_context gcmono;
    struct isp_rgbgamma_context rgbgamma;
#if defined (ISP_DEMOSAIC1) || defined (ISP_DEMOSAIC2)
    struct isp_dmsc_context demosaic;
#endif
#ifdef ISP_DEMOSAIC3
    struct isp_dmsc3_context demosaic3;
#endif
    struct isp_ge_context ge;
    struct isp_curve_context curve;
    struct isp_dummy_hblank_cxt hblank;
    struct isp_misirq_regaddr irqaddr;
#ifdef ISP_GWDR
    isp_wdr_context_t wdr;
#endif
    bool_t streaming;
    bool_t update_gamma_en;
#ifdef ISP_WDR_V4
    struct isp_wdr4_context wdr4;
#endif
#if defined(ISP_3DNR_V3) || defined(ISP_3DNR_V3_1)
    struct isp_3dnrV3_context tdnr;
#endif

#ifdef ISP_3DNR_V4
    struct isp_3dnrV4_context tdnr;
#endif

#ifdef ISP_3D_LUT
    struct isp_lut3d_context lut3d;
#endif

    struct isp_rgbir_context rgbir;
    u32   isp_mis;
#ifdef ISP_MI_PP_READ
    pp_dma_line_entry_t pp_dma_line_entry;
#endif
    struct isp_miv_info miv_info;
    struct isp_crop_context  crop[(u32)ISP_MI_PATH_SP2_BP + 1U], cropBackEnd[(u32)ISP_MI_PATH_SP2_BP + 1U];
    struct isp_crop_context  raw_crop;
    struct isp_dpcc_bpt dpcc_bpt; // dpcc bad pixel table
    struct isp_pdaf_context pdaf; 
    struct isp_pp_path_context pp_path;

#if defined(ISP_WDR_V5) || defined(ISP_WDR_V5_1) || defined(ISP_WDR_V5_2) || defined(ISP_WDR_V5_2_1) || defined(ISP_WDR_V5_3)
    struct isp_wdr5_context wdr5;
#endif 

#if defined(ISP_MPCTL) || defined(ISP_MPCTL_V1_1)
    struct isp_mp_ctrl_context mp_ctrl;
#endif

//#ifdef ISP_STA
    struct isp_sta_context isp_sta;
//#endif
    struct isp_gtm_context gtm;
#ifdef ISP_GAMMA_FEBE
    struct isp_gamma_febe gamma_fe_be;
#endif
};


void isp_write_reg(struct isp_ic_dev *dev, u32 offset, u32 val);
u32 isp_read_reg(struct isp_ic_dev *dev, u32 offset);
void isp_write_tbl(struct isp_ic_dev *dev, u32 offset, u32 val);
u32 isp_read_tbl(struct isp_ic_dev *dev, u32 offset);
int32_t isp_drv_read_reg(struct isp_ic_dev *dev, void *args);
int32_t isp_drv_write_reg(struct isp_ic_dev *dev, void *args);

void mss_write_reg(struct isp_ic_dev *dev, u32 offset, u32 val);
u32 mss_read_reg(struct isp_ic_dev *dev, u32 offset);

void isp_rdc_write_reg(struct isp_ic_dev *dev, u32 offset, u32 val);
u32 isp_rdc_read_reg(struct isp_ic_dev *dev, u32 offset);

#endif /* _ISPIC_DEV_H_ */
