/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef ISP_TYPES_H
#define ISP_TYPES_H

#include <common/picture_buffer.h>

typedef enum MiDataMode_e {
	IC_MI_DATAMODE_INVALID = 0,		/**< lower border (only for an internal evaluation) */
	IC_MI_DATAMODE_DISABLED = 1,		/**< disables the path */
	IC_MI_DATAMODE_JPEG = 2,			/**< data output format is JPEG (only valid for mainpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_YUV444 = 3,		/**< data output format is YUV444 */
	IC_MI_DATAMODE_YUV422 = 4,		/**< data output format is YUV422 */
	IC_MI_DATAMODE_YUV420 = 5,		/**< data output format is YUV420 */
	IC_MI_DATAMODE_YUV400 = 6,		/**< data output format is YUV400 */
	IC_MI_DATAMODE_RGB888 = 7,		/**< data output format is RGB888 (only valid for selpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RGB666 = 8,		/**< data output format is RGB666 (only valid for selpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RGB565 = 9,		/**< data output format is RGB565 (only valid for selpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RAW8 = 10,			/**< data output format is RAW8 (only valid for mainpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RAW12 = 11,		   /**< data output format is RAW12 (only valid for mainpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_DPCC = 12,			/**< path dumps out the current measured defect pixel table */
	IC_MI_DATAMODE_RAW10 = 13,		   /**< data output format is RAW10 (only valid for mainpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RAW14 = 14,		   /**< data output format is RAW14 (only valid for mainpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RAW16 = 15,		   /**< data output format is RAW16 (only valid for mainpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RAW20 = 16,		   /**< data output format is RAW20 (only valid for mainpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RAW20_COMPRESS = 17,/**< for compile, not surpport */
	IC_MI_DATAMODE_RAW24 = 18,		   /**< data output format is RAW24 (only valid for mainpath @ref CamerIcMiPath_e) */
	IC_MI_DATAMODE_RAW24_COMPRESS = 19,/**< for compile, not surpport */
	IC_MI_DATAMODE_MAX					 /**< upper border (only for an internal evaluation) */
} MiDataMode_t;

enum {
	IC_MI_DATASTORAGE_INVALID = 0,		/**< lower border (only for an internal evaluation) */
	IC_MI_DATASTORAGE_PLANAR = 1,		/**< YUV values are packed together as: YYYY......, UVUVUVUV...... */
	IC_MI_DATASTORAGE_SEMIPLANAR = 2,	/**< YUV values are packed together as: YUV, YUV, YUV, ...... */
	IC_MI_DATASTORAGE_INTERLEAVED = 3,	/**< Y values for all pixels are put together, as well as U and V,
													 like: YYYYYY......, UUUUUUU......., VVVVVV...... */
	IC_MI_DATASTORAGE_MAX				  /**< upper border (only for an internal evaluation) */
};

typedef enum MiPixelYuv_e {
	IC_MI_PIXEL_YUV_BIT_INVALID = -1, /**< lower border (only for an internal evaluation) */
	IC_MI_PIXEL_YUV_8_BIT = 0,	       /**< The value of yuv pixel is 8 bit. */
	IC_MI_PIXEL_YUV_10_BIT = 1,       /**< The value of yuv pixel is 10 bit.*/
	IC_MI_PIXEL_YUV_12_BIT = 2,       /**< The value of yuv pixel is 12 bit.*/
	IC_MI_PIXEL_YUV_BIT_MAX	       /**< upper border (only for an internal evaluation) */
} MiPixelYuv_t;

enum {
	IC_DPF_RB_FILTERSIZE_INVALID = 0,	/**< lower border (only for an internal evaluation) */
	IC_DPF_RB_FILTERSIZE_9x9 = 1,		/**< red and blue filter kernel size 9x9 (means 5x5 active pixel) */
	IC_DPF_RB_FILTERSIZE_13x9 = 2,		/**< red and blue filter kernel size 13x9 (means 7x5 active pixel) */
	IC_DPF_RB_FILTERSIZE_MAX			   /**< upper border (only for an internal evaluation) */
};

typedef enum MiDataAlignMode_e {
	ISP_MI_DATA_ALIGN_MODE_INVALID = -1,
	ISP_MI_DATA_UNALIGN_MODE = 0,	   /* pixel data not aligned. */
	ISP_MI_DATA_ALIGN_128BIT_MODE = 1, /* pixel data  aligned with 128 bit. */
	ISP_MI_DATA_ALIGN_DOUBLE_WORD = 1, /* yuv pixel data double word aligned. */
	ISP_MI_DATA_ALIGN_WORD = 2,	   /**< The value of yuv pixel is  word align.*/
	ISP_MI_DATA_ALIGN_16BIT_MODE = 2, /* raw pixel data  aligned with 16 bit. */

	ISP_MI_DATA_ALIGN_MODE_MAX
} MiDataAlignMode_t;

enum {
	IC_DPF_GAIN_USAGE_INVALID = 0,		  /**< lower border (only for an internal evaluation) */
	IC_DPF_GAIN_USAGE_DISABLED = 1,		 /**< don't use any gains in preprocessing stage */
	IC_DPF_GAIN_USAGE_NF_GAINS = 2,		 /**< use only the noise function gains  from registers DPF_NF_GAIN_R, ... */
	IC_DPF_GAIN_USAGE_LSC_GAINS = 3,	/**< use only the gains from LSC module */
	IC_DPF_GAIN_USAGE_NF_LSC_GAINS = 4,    /**< use the moise function gains and the gains from LSC module */
	IC_DPF_GAIN_USAGE_AWB_GAINS = 5,	/**< use only the gains from AWB module */
	IC_DPF_GAIN_USAGE_AWB_LSC_GAINS = 6,   /**< use the gains from AWB and LSC module */
	IC_DPF_GAIN_USAGE_MAX				  /**< upper border (only for an internal evaluation) */
};

enum {
	IC_NLL_SCALE_INVALID = 0,		/**< lower border (only for an internal evaluation) */
	IC_NLL_SCALE_LINEAR = 1,			/**< use a linear scaling */
	IC_NLL_SCALE_LOGARITHMIC = 2,		/**< use a logarithmic scaling */
	IC_NLL_SCALE_MAX					   /**< upper border (only for an internal evaluation) */
};

#endif /* _ISP_TYPES_H_ */
