/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* Powered by Yang, Yong since 2024 */

#pragma once

#include "common_base.h"

#include "common_stdio.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef uintptr_t TCommonTracerHandle_t;

typedef enum {
  TCommonTracerLevel_Debug,
  TCommonTracerLevel_Info,
  TCommonTracerLevel_Warning,
  TCommonTracerLevel_Error,

  TCommonTracerLevel_Sum,
} TCommonTracerLevel_t;

typedef struct {
  TCommonTracerLevel_t availableLevel;
  bool isColor;
  bool isLevel;
  bool isTimestamp;
  TCommonStdioFile_t *pTracer;
} TCommonTracerOpenParam_t;

typedef TCommonTracerOpenParam_t TCommonTracerConfig_t;

int TCommonTracer_open(TCommonTracerOpenParam_t *pParam,
                       TCommonTracerHandle_t *pHandle);

int TCommonTracer_close(TCommonTracerHandle_t *pHandle);

int TCommonTracer_configGet(TCommonTracerHandle_t handle,
                            TCommonTracerConfig_t *pParam);

int TCommonTracer_configSet(TCommonTracerHandle_t handle,
                            TCommonTracerConfig_t const *pParam);

int TCommonTracer_print(TCommonTracerHandle_t handle,
                        TCommonTracerLevel_t level, char const *pFormat, ...);

char *TCommonTracer_errorMessage(char *pBuffer, size_t size);

char *TCommonTracer_errorMessage2(int error, char *pBuffer, size_t size);

TCommonStdioFile_t *TCommonTracer_tracer(TCommonTracerHandle_t handle);

extern TCommonTracerHandle_t TCommonTracerHandle;

#if defined(ASSERT)
#undef ASSERT
#endif // ASSERT

#define ASSERT(exp)                                                            \
  do {                                                                         \
    if (!(exp)) {                                                              \
      char buffer[256] = {};                                                   \
      TCommonTracer_errorMessage(buffer, sizeof buffer);                       \
      TCT_ERROR2("Assertion failed: '%s'; error: '%s'\n", #exp, buffer);       \
      abort();                                                                 \
    }                                                                          \
  } while (0)

#if defined(TCT_DEBUG)
#undef TCT_DEBUG
#endif // TCT_DEBUG

#define TCT_DEBUG(format, ...)                                                 \
  TCommonTracer_print(TCommonTracerHandle, TCommonTracerLevel_Debug, format,   \
                      ##__VA_ARGS__)

#if defined(TCT_DEBUG2)
#undef TCT_DEBUG2
#endif // TCT_DEBUG2

#define TCT_DEBUG2(format, ...)                                                \
  TCommonTracer_print(TCommonTracerHandle, TCommonTracerLevel_Debug,           \
                      "%s:%d: %s(): " format, __FILE__, __LINE__,              \
                      __FUNCTION__, ##__VA_ARGS__)

#if defined(TCT_INFO)
#undef TCT_INFO
#endif // TCT_INFO

#define TCT_INFO(format, ...)                                                  \
  TCommonTracer_print(TCommonTracerHandle, TCommonTracerLevel_Info, format,    \
                      ##__VA_ARGS__)

#if defined(TCT_WARN)
#undef TCT_WARN
#endif // TCT_WARN

#define TCT_WARN(format, ...)                                                  \
  TCommonTracer_print(TCommonTracerHandle, TCommonTracerLevel_Warning, format, \
                      ##__VA_ARGS__)

#if defined(TCT_WARN2)
#undef TCT_WARN2
#endif // TCT_WARN2

#define TCT_WARN2(format, ...)                                                 \
  TCommonTracer_print(TCommonTracerHandle, TCommonTracerLevel_Warning,         \
                      "%s:%d: %s(): " format, __FILE__, __LINE__,              \
                      __FUNCTION__, ##__VA_ARGS__)

#if defined(TCT_ERROR)
#undef TCT_ERROR
#endif // TCT_ERROR

#define TCT_ERROR(format, ...)                                                 \
  TCommonTracer_print(TCommonTracerHandle, TCommonTracerLevel_Error, format,   \
                      ##__VA_ARGS__)

#if defined(TCT_ERROR2)
#undef TCT_ERROR2
#endif // TCT_ERROR2

#define TCT_ERROR2(format, ...)                                                \
  TCommonTracer_print(TCommonTracerHandle, TCommonTracerLevel_Error,           \
                      "%s:%d: %s(): " format, __FILE__, __LINE__,              \
                      __FUNCTION__, ##__VA_ARGS__)

#if defined(DEBUG)
#undef DEBUG
#endif // DEBUG

#define DEBUG(ret)                                                             \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      TCT_DEBUG2("%s(%d)\n", TCommonError_text(ret), ret);                     \
      ret = RET_SUCCESS;                                                       \
    }                                                                          \
  } while (0)

#if defined(JUMP)
#undef JUMP
#endif // JUMP

#define JUMP(ret)                                                              \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      TCT_ERROR2("%s(%d)\n", TCommonError_text(ret), ret);                     \
      goto error;                                                              \
    }                                                                          \
  } while (0)

#if defined(REPORT)
#undef REPORT
#endif // REPORT

#define REPORT(ret)                                                            \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      TCT_ERROR2("%s(%d)\n", TCommonError_text(ret), ret);                     \
      return ret;                                                              \
    }                                                                          \
  } while (0)

#if defined(REPORT2)
#undef REPORT2
#endif // REPORT2

#define REPORT2(ret, lambda)                                                   \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      lambda;                                                                  \
      TCT_ERROR2("%s(%d)\n", TCommonError_text(ret), ret);                     \
      return ret;                                                              \
    }                                                                          \
  } while (0)

#if defined(REPORT3)
#undef REPORT3
#endif // REPORT3

#define REPORT3(ret, code)                                                     \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      TCT_ERROR2("%s(%d)\n", TCommonError_text(ret), ret);                     \
      return code;                                                             \
    }                                                                          \
  } while (0)

#if defined(REPORT_IGNORE)
#undef REPORT_IGNORE
#endif // REPORT_IGNORE

#define REPORT_IGNORE(ret)                                                     \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      TCT_DEBUG2("%s(%d)\n", TCommonError_text(ret), ret);                     \
      return RET_SUCCESS;                                                      \
    }                                                                          \
  } while (0)

#if defined(REPORT_IGNORE2)
#undef REPORT_IGNORE2
#endif // REPORT_IGNORE2

#define REPORT_IGNORE2(ret, lambda)                                            \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      lambda;                                                                  \
      TCT_DEBUG2("%s(%d)\n", TCommonError_text(ret), ret);                     \
      return RET_SUCCESS;                                                      \
    }                                                                          \
  } while (0)

#if defined(REPORT_QUIET)
#undef REPORT_QUIET
#endif // REPORT_QUIET

#define REPORT_QUIET(ret)                                                      \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      return ret;                                                              \
    }                                                                          \
  } while (0)

#if defined(REPORT_QUIET2)
#undef REPORT_QUIET2
#endif // REPORT_QUIET2

#define REPORT_QUIET2(ret, lambda)                                             \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      lambda;                                                                  \
      return ret;                                                              \
    }                                                                          \
  } while (0)

#if defined(REPORT_ERRNO)
#undef REPORT_ERRNO
#endif // REPORT_ERRNO

#define REPORT_ERRNO(ret)                                                      \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      char buffer[256] = {};                                                   \
      TCommonTracer_errorMessage(buffer, sizeof buffer);                       \
      TCT_ERROR2("%s(%d); error: '%s'\n", TCommonError_text(ret), ret,         \
                 buffer);                                                      \
      return ret;                                                              \
    }                                                                          \
  } while (0)

#if defined(WARN)
#undef WARN
#endif // WARN

#define WARN(ret)                                                              \
  do {                                                                         \
    if (ret != RET_SUCCESS) {                                                  \
      TCT_WARN2("%s(%d)\n", TCommonError_text(ret), ret);                      \
      ret = RET_SUCCESS;                                                       \
    }                                                                          \
  } while (0)

#ifdef __cplusplus
}
#endif
