/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* Powered by Yang, Yong since 2022 */

#pragma once

#include "t_database_c_config.h"

#include "database_attribute.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef uintptr_t TDatabaseHandle_t;

#define T_DATABASE_CLASS_NAME_LENGTH 20
#define T_DATABASE_AUTO_TABLE_SUM 20

#define T_DATABASE_AUTO_TABLE_CLEAR(c)                                         \
  do {                                                                         \
    FREE(c->pTables);                                                          \
    c->tableNumber = 0;                                                        \
  } while (0)

#define T_DATABASE_AUTO_TABLE_INDEX(c, i)                                      \
  do {                                                                         \
    if (i < c->tableNumber) {                                                  \
      break;                                                                   \
    }                                                                          \
    if (c->pTables) {                                                          \
      c->pTables =                                                             \
          TCommonStdlib_realloc(c->pTables, (i + 1) * sizeof(*c->pTables));    \
      if (i + 1 > c->tableNumber) {                                            \
        memset(c->pTables + c->tableNumber, 0,                                 \
               (i + 1 - c->tableNumber) * sizeof(*c->pTables));                \
      }                                                                        \
    } else {                                                                   \
      c->pTables = TCommonStdlib_calloc(i + 1, sizeof(*c->pTables));           \
    }                                                                          \
    c->tableNumber = i + 1;                                                    \
  } while (0)

typedef enum {
  TDatabasePeriod_Long,
  TDatabasePeriod_Short,
  TDatabasePeriod_VeryShort,

  TDatabasePeriod_Sum,
} TDatabasePeriod_t;

typedef struct {
  float blue;
  float greenBlue;
  float greenRed;
  float red;
} TDatabaseGain_t;

typedef struct {
  int32_t x;
  int32_t y;
} TDatabasePoint_t;

typedef struct {
  float max;
  float min;
} TDatabaseRange_t;

typedef struct {
  uint32_t height;
  TDatabasePoint_t offset;
  uint32_t width;
} TDatabaseRectangle_t;

typedef enum {
  TDatabaseState_Off,
  TDatabaseState_Manual,
  TDatabaseState_Auto,
} TDatabaseState_t;

typedef struct {
  uint16_t hOffset;
  uint16_t vOffset;
  uint16_t width;
  uint16_t height;
} TDatabaseWindow_t;

typedef struct {
  TCommonLoggerHandle_t hLogger;
} TDatabaseOpenParam_t;

int TDatabaseGain_parse(TJsonValue_t *pValue, TDatabaseGain_t *pConfig);

int TDatabaseGain_serialize(TDatabaseGain_t *pConfig, TJsonValue_t *pValue);

int TDatabasePoint_parse(TJsonValue_t *pValue, TDatabasePoint_t *pConfig);

int TDatabasePoint_serialize(TDatabasePoint_t *pConfig, TJsonValue_t *pValue);

int TDatabaseRange_parse(TJsonValue_t *pValue, TDatabaseRange_t *pConfig);

int TDatabaseRange_serialize(TDatabaseRange_t *pConfig, TJsonValue_t *pValue);

int TDatabaseRectangle_parse(TJsonValue_t *pValue,
                             TDatabaseRectangle_t *pConfig);

int TDatabaseRectangle_serialize(TDatabaseRectangle_t *pConfig,
                                 TJsonValue_t *pValue);

int TDatabaseWindow_parse(TJsonValue_t *pValue, TDatabaseWindow_t *pConfig);

int TDatabaseWindow_serialize(TDatabaseWindow_t *pConfig, TJsonValue_t *pValue);

#ifdef __cplusplus
}
#endif
