/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* Powered by Yang, Yong since 2022 */

#pragma once

#include "database_base.h"

#ifdef __cplusplus
extern "C" {
#endif

#define DESCRIPTOR_LENGTH 50

typedef struct {
  char name[DESCRIPTOR_LENGTH];
  float fps;
} TDatabaseCalibrationHeaderResolutionFramerate_t;

typedef struct {
  char name[DESCRIPTOR_LENGTH];
  char id[DESCRIPTOR_LENGTH];
  uint16_t width;
  uint16_t height;

  TDatabaseCalibrationHeaderResolutionFramerate_t *pFramerate;
  size_t framerateNumber;
} TDatabaseCalibrationHeaderResolution_t;

typedef struct {
  char date[DESCRIPTOR_LENGTH];
  char creator[DESCRIPTOR_LENGTH];
  char sensorName[DESCRIPTOR_LENGTH];
  char sampleName[DESCRIPTOR_LENGTH];
  char generatorVersion[DESCRIPTOR_LENGTH];

  TDatabaseCalibrationHeaderResolution_t *pResolutions;
  size_t resolutionNumber;
} TDatabaseCalibrationHeader_t;

typedef struct {
  char name[DESCRIPTOR_LENGTH];
  char resolution[DESCRIPTOR_LENGTH];
  float svdMeanValue[3];
  float pcaMatrix[6];
  float centerLine[3];
  float afRg2[16];
  float afMaxDist2[16];
  float afRg1[16];
  float afMaxDist1[16];
  float afGlobalFade2[16];
  float afGlobalGainDistance2[16];
  float afGlobalFade1[16];
  float afGlobalGainDistance1[16];
  float fRgProjIndoorMin;
  float fRgProjMax;
  float fRgProjMaxSky;
  float fRgProjOutdoorMin;
  char awbClipOutDoor[DESCRIPTOR_LENGTH];
  float kFactor;
  float afFade2[6];
  int32_t afCbMinRegionMax[6];
  int32_t afCrMinRegionMax[6];
  int32_t afMaxCSumRegionMax[6];
  int32_t afCbMinRegionMin[6];
  int32_t afCrMinRegionMin[6];
  int32_t afMaxCSumRegionMin[6];
  int32_t regionSize;
  float regionSizeInc;
  float regionSizeDec;

  struct Iir {
    float dampCoefAdd;
    float dampCoefSub;
    float dampFilterThreshold;
    float dampingCoefMin;
    float dampingCoefMax;
    float dampingCoefInit;
    int32_t expPriorFilterSizeMax;
    int32_t expPriorFilterSizeMin;
    float expPriorMiddle;
  } iir;
} TDatabaseCalibrationSensorAwbGlobal_t;

typedef struct {
  char name[DESCRIPTOR_LENGTH];
  char doorType[DESCRIPTOR_LENGTH];

  struct Gmm {
    float invCovMatrix[4];
    float gaussianScalingFactor;
    float tau[2];
    float gaussianMeanValue[2];
  } gmm;

  struct Alsc {
    char resolution[DESCRIPTOR_LENGTH];
    char profile[DESCRIPTOR_LENGTH * 4];
  } alsc;

  float manualWb[4];
  float manualCcMatrix[9];
  float manualCcOffset[3];

  char awbType[DESCRIPTOR_LENGTH];

  struct SatCt {
    int32_t gain[4];
    int32_t saturation[4];
  } satCt;

  struct VigCt {
    int32_t gain[4];
    int32_t vignetting[4];
  } vigCt;

  struct Acc {
    char profile[DESCRIPTOR_LENGTH * 4];
  } acc;
} TDatabaseCalibrationSensorAwbIllumination_t;

typedef struct {
  float sensorGains;

  TDatabaseCalibrationSensorAwbGlobal_t *pGlobals;
  size_t globalNumber;

  TDatabaseCalibrationSensorAwbIllumination_t *pIlluminations;
  size_t illuminationNumber;
} TDatabaseCalibrationSensorAwb_t;

typedef struct {
  float kA;
  float pB;
} TDatabaseCalibrationSensorBnr_t;

typedef struct {
  float sensorGains;
  char name[DESCRIPTOR_LENGTH];
  char resolution[DESCRIPTOR_LENGTH];
  char illumination[DESCRIPTOR_LENGTH];
  int32_t sectors;
  int32_t no;
  int32_t xo;
  int32_t yo;
  int32_t sectSizeX[32];
  int32_t sectSizeY[32];
  int32_t vignetting;

  int32_t sampleRed[33][33];
  int32_t sampleGreenR[33][33];
  int32_t sampleGreenB[33][33];
  int32_t sampleBlue[33][33];
} TDatabaseCalibrationSensorLsc_t;

typedef struct {
  char name[DESCRIPTOR_LENGTH];
  float saturation;
  float ccMatrix[9];
  float ccOffset[3];
  float wb[4];
  float sensorGains;
} TDatabaseCalibrationSensorCc_t;

typedef struct {
  int32_t tbd;
} TDatabaseCalibrationSensorAf_t;

typedef struct {
  char name[DESCRIPTOR_LENGTH];
  char resolution[DESCRIPTOR_LENGTH];
  int32_t data[4];
  float sensorGains;
} TDatabaseCalibrationSensorBls_t;

typedef struct {
  char name[DESCRIPTOR_LENGTH];
  char resolution[DESCRIPTOR_LENGTH];
  float xNormalShift;
  float xNormalFactor;
  float yNormalShift;
  float yNormalFactor;
  float xOffset;
  float yOffset;
  float redParameters[3];
  float blueParameters[3];
} TDatabaseCalibrationSensorCac_t;

typedef struct {
  char name[DESCRIPTOR_LENGTH];
  char resolution[DESCRIPTOR_LENGTH];
  float nllSegmentation;
  int32_t nllCoeffN[17];
  float sensorGains;
  float nma;
  float nmb;
  float sigmaGreen;
  float sigmaRedBlue;
  float gradient;
  float offset;
  float nlGains[4];
} TDatabaseCalibrationSensorDpf_t;

typedef struct {
  TDatabaseCalibrationSensorAf_t af;

  TDatabaseCalibrationSensorAwb_t awb;

  TDatabaseCalibrationSensorBls_t *pBls;
  size_t blsNumber;

  TDatabaseCalibrationSensorBnr_t *pBnr;
  size_t bnrNumber;

  TDatabaseCalibrationSensorCac_t *pCac;
  size_t cacNumber;

  TDatabaseCalibrationSensorCc_t *pCc;
  size_t ccNumber;

  TDatabaseCalibrationSensorDpf_t *pDpf;
  size_t dpfNumber;

  TDatabaseCalibrationSensorLsc_t *pLsc;
  size_t lscNumber;
} TDatabaseCalibrationSensor_t;

typedef struct System {
  char afpsDefault[DESCRIPTOR_LENGTH];
} TDatabaseCalibrationSystem_t;

typedef struct {
  TDatabaseCalibrationHeader_t header;
  TDatabaseCalibrationSensor_t sensor;
  TDatabaseCalibrationSystem_t system;

  bool isLoaded;
} TDatabaseCalibration_t;

#ifdef __cplusplus
}
#endif
