/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* Powered by Yang, Yong since 2022 */

#pragma once

#include "database_base.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  float aBlue;
  float aRed;
  float bBlue;
  float bRed;
  float cBlue;
  float cRed;
  int16_t centerHOffs;
  int16_t centerVOffs;
} TDatabaseDeMosaic_3_Cac_t;

typedef struct {
  uint16_t cornerLowTh;
  uint8_t cornerShift;
} TDatabaseDeMosaic_3_DeclineConfig_t;

typedef struct {
  uint8_t interpDirStrength[4];
  uint8_t interpLargeStr[2];
  uint16_t dir0InterpType;
} TDatabaseDeMosaic_3_InterpCornerConfig_t;

typedef struct {
  uint16_t layerT1;
  uint8_t layerT2Shift;
  uint16_t layer0T3;
  uint8_t layerT4Shift;
  uint16_t layerR1;
  uint16_t layerR2;
  uint16_t layerR3;
} TDatabaseDeMosaic_3_SharpenCurve_t;

typedef struct {
  uint16_t strength;
  uint16_t t1;
  uint8_t shift1;
  uint16_t r1;
  uint16_t r2;
} TDatabaseDeMosaic_3_SharpenLineCurve_t;

typedef struct {
  bool enable;
  bool dmscCacEnable;
  bool declineEn;
  bool dmscDeFalseColorEnable;
  bool dmscDeMoireEnable;
  bool dmscDepurpleEnable;
  bool dmscInterpCornerEnable;
  bool dmscSharpenEnable;
  bool dmscSharpenLineEnable;

  uint16_t gradThMin;
  uint16_t gradThMax;
  uint8_t diffColorCoef;
  uint16_t adptCoefMode;
  uint8_t adptCoefThLow;
  uint8_t adptCoefThShift;
  uint8_t adptCoefMin;
  uint8_t adptCoefMax;

  uint16_t highlightControlTh;
  uint8_t highlightControlStr;

  uint16_t deburstMode;
  uint8_t pureGWeight;
  uint8_t pureRBWeight;
  uint8_t pureGWeightISO;
  uint8_t pureRBWeightISO;

  TDatabaseDeMosaic_3_Cac_t cac;
  uint16_t refineGTh;

  TDatabaseDeMosaic_3_SharpenLineCurve_t sharpenLineCurves[1];

  uint8_t sharpenCfgDeNoiseStrength;
  uint16_t dmscSharpenFactor[6];
  uint16_t dmscSharpenClip[6];
  TDatabaseDeMosaic_3_SharpenCurve_t sharpenCfgCurve[3];

  uint8_t defalseCfgColorStr;
  uint8_t defalseCfgCbCrClassTh;
  uint8_t defalseCfgGrayProtectStr;

  uint8_t dmscDepurpleCfgGradControl;
  uint8_t dmscDepurpleCfgProtectThLow;
  uint8_t dmscDepurpleCfgProtectShift;
  uint8_t dmscDepurpleThr;
  uint16_t dmscDepurpleCfgCbCrMode;
  uint8_t dmscDepurpleSatShrink;
  uint16_t dmscDepurpleCfgColorRangeMode;
  uint16_t dmscDepurpleCbCrRange[8];

  TDatabaseDeMosaic_3_InterpCornerConfig_t interpCornerCfg;
  TDatabaseDeMosaic_3_DeclineConfig_t declineCfg;

  bool overrideCaliEn;

  DEPRECATED_MSG("use adptCoefMax instead") uint8_t adaptCoefficientMax;
  DEPRECATED_MSG("use adptCoefMin instead") uint8_t adaptCoefficientMin;
  DEPRECATED_MSG("use adptCoefMode instead") uint16_t adaptCoefficientMode;
  DEPRECATED_MSG("use adptCoefThLow instead")
  uint8_t adaptCoefficientThrLow;
  DEPRECATED_MSG("use adptCoefThShift instead")
  uint8_t adaptCoefficientThrShift;
  DEPRECATED_MSG("use deburstMode instead") uint16_t deBurstMode;
  DEPRECATED_MSG("use diffColorCoef instead") uint8_t diffColorCoefficient;
  DEPRECATED_MSG("use gradThMax instead") uint16_t gradThrMax;
  DEPRECATED_MSG("use gradThMin instead") uint16_t gradThrMin;
  DEPRECATED_MSG("use highlightControlStr instead")
  uint8_t highlightControlShift;
  DEPRECATED_MSG("use highlightControlTh instead")
  uint16_t highlightControlThr;
  DEPRECATED_MSG("use pureGWeight instead") uint8_t pureGreenWeight;
  DEPRECATED_MSG("use pureGWeightISO instead") uint8_t pureGreenWeightIso;
  DEPRECATED_MSG("use pureRBWeight instead") uint8_t pureRedBlueWeight;
  DEPRECATED_MSG("use pureRBWeightISO instead")
  uint8_t pureRedBlueWeightIso;
  DEPRECATED_MSG("use defalseCfgColorStr instead") uint8_t colorStrength;
  DEPRECATED_MSG("use defalseCfgCbCrClassTh instead") uint8_t cbCrClassThr;
  DEPRECATED_MSG("use dmscDepurpleCfgProtectThLow instead")
  uint8_t protectThrLow;
  DEPRECATED_MSG("use dmscDepurpleSatShrink instead") uint8_t saturationShrink;
  DEPRECATED_MSG("use defalseCfgGrayProtectStr instead")
  uint8_t grayProtectStrength;
  DEPRECATED_MSG("use refineGTh instead") uint16_t refineGreenThr;

  RESERVED uint16_t sharpenCurveIdx;
  RESERVED uint16_t sharpenLineCurveIdx;

  // temp save config value when get current value
  RESERVED bool dmscDepurpleEnTemp;
  RESERVED bool dmscSharpenEnTemp;
  RESERVED bool dmscDeclineEnTemp;

  LEGACY TDatabaseState_t state;

  // shell
  RESERVED bool isStatistics;

  // simulator section
  SIMULATOR char class_[T_DATABASE_CLASS_NAME_LENGTH];
} TDatabaseDeMosaic_3_t;

int TDatabaseDeMosaic_3_parse(TJsonValue_t *pValue,
                              TDatabaseDeMosaic_3_t *pConfig);

int TDatabaseDeMosaic_3_serialize(TDatabaseDeMosaic_3_t *pConfig,
                                  TJsonValue_t *pValue);

int TDatabaseDeMosaic_3_reset(TDatabaseDeMosaic_3_t *pConfig);

#ifdef __cplusplus
}
#endif
