/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* Powered by Yang, Yong since 2022 */

#pragma once

#include "json_base.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
  TJsonValueType_Null,

  TJsonValueType_Array,
  TJsonValueType_Boolean,
  TJsonValueType_Float,
  TJsonValueType_Integer,
  TJsonValueType_Object,
  TJsonValueType_String,
} TJsonValueType_t;

typedef struct TJsonValueArray {
  size_t number;
  struct TJsonValue *pValues;
} TJsonValueArray_t;

typedef struct TJsonValuePair {
  char *pKey;
  struct TJsonValue *pValue;
} TJsonValuePair_t;

typedef struct TJsonValueObject {
  size_t number;
  TJsonValuePair_t *pPairs;
} TJsonValueObject_t;

typedef struct TJsonValue {
  union Holder {
    bool _bool;

    double _float;

    long integer;

    TJsonValueArray_t *pArray;

    char *pChar;

    TJsonValueObject_t *pObject;
  } holder;

  char *pComment;
  char *pComment2;

  TJsonValueType_t type;
} TJsonValue_t;

int TJsonValue_clean(TJsonValue_t *pValue);

int TJsonValue_create(TJsonValueType_t type, TJsonValue_t **ppValue);

int TJsonValue_create2(TJsonValueType_t type, TJsonValue_t **ppValue);

int TJsonValue_clone(TJsonValue_t *pValue, TJsonValue_t **ppValue);

int TJsonValue_createNull(TJsonValue_t **ppValue);

int TJsonValue_createBoolean(bool value, TJsonValue_t **ppValue);

int TJsonValue_createFloat(double value, TJsonValue_t **ppValue);

int TJsonValue_createInteger(long value, TJsonValue_t **ppValue);

int TJsonValue_createString(char const *pValue, TJsonValue_t **ppValue);

int TJsonValue_delete(TJsonValue_t **ppValue);

bool TJsonValue_asBoolean(TJsonValue_t *pValue);

double TJsonValue_asFloat(TJsonValue_t *pValue);

long TJsonValue_asInteger(TJsonValue_t *pValue);

char *TJsonValue_asString(TJsonValue_t *pValue);

bool TJsonValue_isEmpty(TJsonValue_t *pValue);

size_t TJsonValue_size(TJsonValue_t *pValue);

int TJsonValue_swap(TJsonValue_t *pValue, TJsonValue_t *pValue2);

int TJsonValue_to(TJsonValue_t *pValue, TJsonValueType_t type);

int TJsonValue_assign(TJsonValue_t *pTarget, TJsonValue_t **ppSource);

int TJsonValue_assignAsBoolean(TJsonValue_t *pValue, bool value);

int TJsonValue_assignAsFloat(TJsonValue_t *pValue, double value);

int TJsonValue_assignAsInteger(TJsonValue_t *pValue, long value);

int TJsonValue_assignAsString(TJsonValue_t *pValue, char const *pChar);

int TJsonValue_addBooleanToObject(TJsonValue_t *pValue, char const *pKey,
                                  bool value);

int TJsonValue_addFloatToObject(TJsonValue_t *pValue, char const *pKey,
                                double value);

int TJsonValue_addIntegerToObject(TJsonValue_t *pValue, char const *pKey,
                                  long value);

int TJsonValue_addStringToObject(TJsonValue_t *pValue, char const *pKey,
                                 char const *pChar);

int TJsonValue_addValueToObject(TJsonValue_t *pValue, char const *pKey,
                                TJsonValue_t **ppValue2);

int TJsonValue_addBooleanToArray(TJsonValue_t *pValue, bool value);

int TJsonValue_addFloatToArray(TJsonValue_t *pValue, double value);

int TJsonValue_addIntegerToArray(TJsonValue_t *pValue, long value);

int TJsonValue_addStringToArray(TJsonValue_t *pValue, char const *pChar);

int TJsonValue_addValueToArray(TJsonValue_t *pValue, TJsonValue_t **ppItem);

int TJsonValue_removeMemberFromObject(TJsonValue_t *pValue, char const *pKey);

int TJsonValue_removeItemFromArray(TJsonValue_t *pValue, size_t index);

TJsonValue_t *TJsonValue_member(TJsonValue_t *pValue, char const *pKey);

TJsonValue_t *TJsonValue_member2(TJsonValue_t *pValue, char const *pKey);

char const *TJsonValue_memberKey(TJsonValue_t *pValue, size_t index);

TJsonValue_t *TJsonValue_item(TJsonValue_t *pValue, size_t index);

bool TJsonValue_itemAsBoolean(TJsonValue_t *pValue, size_t index);

double TJsonValue_itemAsFloat(TJsonValue_t *pValue, size_t index);

long TJsonValue_itemAsInteger(TJsonValue_t *pValue, size_t index);

char *TJsonValue_itemAsString(TJsonValue_t *pValue, size_t index);

#ifdef __cplusplus
}
#endif
