/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef DNR2_H
#define DNR2_H

/**
 * @file 2dnr.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup 2DNR module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include <oslayer/oslayer.h>
#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define ISP_2DNR_AUTO_NUM                  20
#define ISP_2DNR_SIGMA_BIN                 60
#define ISP_2DNR_LUMA_CURVE_SIZE           12//bin number of dnr_luma_curve

#define ISP_2DNR_BLEND_MOTION_MIN         0.0f
#define ISP_2DNR_BLEND_MOTION_MAX         100.0f

#define ISP_2DNR_BLEND_SLOPE_MIN         0.0f
#define ISP_2DNR_BLEND_SLOPE_MAX         32.0f

#define ISP_2DNR_BLEND_STATIC_MIN         0.0f
#define ISP_2DNR_BLEND_STATIC_MAX         100.0f

#define ISP_2DNR_SIGMA_MIN         0.1f
#define ISP_2DNR_SIGMA_MAX         100.0f

/*****************************************************************************/
/**
 *          A2DnrHandle_t
 *
 * @brief   2Dnr Module instance handle
 *
 *****************************************************************************/
typedef void *A2DnrHandle_t;         /**< handle to 2Dnr context */

/*****************************************************************************/
/**
 *          A2DnrInstanceConfig_t
 *
 * @brief   2Dnr Module instance configuration structure
 *
 *****************************************************************************/
typedef struct A2DnrInstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */

    A2DnrHandle_t           hA2Dnr;            /**< handle returned by A2DnrInit() */
} A2DnrInstanceConfig_t;


/*****************************************************************************/
/**
 *          A2DnrConfig_t
 *
 * @brief   2Dnr Module configuration structure
 *
 *****************************************************************************/
typedef struct A2DnrConfig_s
{
    float32_t blendMotion;    /**<[0, 100]*/
    float32_t blendSlope;     /**<[0, 32]*/
    float32_t blendStatic;    /**<[0, 100]*/
    float32_t sigma;
} A2DnrConfig_t;

/*****************************************************************************/
/**
 *          A2DnrAutoParamTable_t
 *
 * @brief   2Dnr Module parameters table structure
 *
 *****************************************************************************/
typedef struct A2DnrAutoParamTable_s
{
    uint8_t autoLevel;
    float32_t gains[ISP_2DNR_AUTO_NUM];
    float32_t sigma[AUTO_NUM];
    float32_t blendStatic[AUTO_NUM];
    float32_t blendMotion[AUTO_NUM];
    float32_t blendSlope[AUTO_NUM];
} A2DnrAutoParamTable_t;


/*****************************************************************************/
/**
 *          A2DnrInit()
 *
 * @brief   This function initializes the 2dnr
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT A2DnrInit
(
    A2DnrInstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          A2DnrDeinit()
 *
 * @brief   The function deinit the 2dnr
 *
 * @param   handle  Handle to 2Dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A2DnrDeinit
(
    A2DnrHandle_t handle
);


/*****************************************************************************/
/**
 *          A2DnrSetConfigure()
 *
 * @brief   This function configures the 2dnr module
 *
 * @param   handle   Handle to 2Dnr 
 * @param   pConfig  2dnr config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A2DnrSetConfigure
(
    A2DnrHandle_t handle,
    A2DnrConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          A2DnrGetConfigure()
 *
 * @brief   This function get the 2dnr module configures
 *
 * @param   handle  Handle to 2Dnr
 * @param   pConfig 2dnr config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A2DnrGetConfigure
(
    A2DnrHandle_t handle,
    A2DnrConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          A2DnrGetCurrentConfigure()
 *
 * @brief   This function get the 2dnr module configures
 *
 * @param   handle  Handle to 2Dnr
 * @param   pConfig 2dnr config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/

RESULT A2DnrGetCurrentConfigure
(
    A2DnrHandle_t handle,
    A2DnrConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          A2DnrSetAutoParamTable()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle         Handle to 2Dnr
 * @param   pParamTable    auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A2DnrSetAutoParamTable
(
    A2DnrHandle_t handle,
    A2DnrAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          A2DnrGetAutoParamTable()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle        Handle to 2Dnr
 * @param   pParamTable   auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT A2DnrGetAutoParamTable
(
    A2DnrHandle_t handle,
    A2DnrAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          A2DnrSetMode()
 *
 * @brief   The function set 2dnr mode
 *
 * @param   handle  Handle to 2Dnr
 * @param   mode    2Dnr mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A2DnrSetMode
(
    A2DnrHandle_t         handle,
    const ARunMode_t      mode
);

/*****************************************************************************/
/**
 *          A2DnrGetMode()
 *
 * @brief   The function get 2dnr mode
 *
 * @param   handle  Handle to 2Dnr
 * @param   mode    2Dnr mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A2DnrGetMode
(
    A2DnrHandle_t         handle,
    ARunMode_t            *mode
);

/*****************************************************************************/
/**
 *          A2DnrEnable()
 *
 * @brief   enable 2dnr
 *
 * @param   handle  Handle to 2Dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A2DnrEnable
(
    A2DnrHandle_t         handle
);

/*****************************************************************************/
/**
 *          A2DnrDisable()
 *
 * @brief   disable 2dnr
 *
 * @param   handle  Handle to 2Dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A2DnrDisable
(
    A2DnrHandle_t         handle
);

/*****************************************************************************/
/**
 *          A2DnrGetStatus()
 *
 * @brief   The function returns the status of the Auto wide dynamic range v3
 *          module
 *
 * @param   handle  Handle to 2Dnr
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A2DnrGetStatus
(
    A2DnrHandle_t           handle,
    bool_t                  *pRunning,
    ARunMode_t              *pMode,
    A2DnrConfig_t           *pCurrentParam
);



/*****************************************************************************/
/**
 *          A2DnrProcessFrame()
 *
 * @brief   2dnr process frame
 *
 * @param   handle  Handle to 2Dnr
 *          gain    current sensor-gain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT A2DnrProcessFrame
(
    A2DnrHandle_t           handle,
    const float32_t         fGain
);


#ifdef __cplusplus
}
#endif


/* @} 2Dnr */


#endif /* DNR2_H*/
