/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file aaa_common.h
 *
 * @brief
 *   This header files contains general AAA_RETurn codes .
 *
 *****************************************************************************/
#ifndef __AAA_COMMON_H__
#define __AAA_COMMON_H__

#include <stdbool.h>
#include <stdint.h>

#define  ISPINDEMIN  0
#define  ISPINDEMAX  32

// typedef signed int      int32_t;
typedef float           float32_t;
typedef int             AAA_RESULT;

#define ROI_WINDOWS_MAX 25

#define AEI_EXPV2_GRID_NUM 32
#define AEI_EXPV2_PIXEL_CHANNEL 4

#define AEI_EXPV3_GRID_NUM 25
#define AEI_EXPV3_PIXEL_CHANNEL 2

enum AAA_RESULT_e
{
    AAA_RET_SUCCESS             = 0,    //!< this has to be 0, if clauses rely on it
    AAA_RET_FAILURE             = 1,    //!< general failure
    AAA_RET_NOTSUPP             = 2,    //!< feature not supported
    AAA_RET_BUSY                = 3,    //!< there's already something going on...
    AAA_RET_CANCELED            = 4,    //!< operation canceled
    AAA_RET_OUTOFMEM            = 5,    //!< out of memory
    AAA_RET_OUTOFRANGE          = 6,    //!< parameter/value out of range
    AAA_RET_IDLE                = 7,    //!< feature/subsystem is in idle state
    AAA_RET_WRONG_HANDLE        = 8,    //!< handle is wrong
    AAA_RET_NULL_POINTER        = 9,    //!< the/one/all parameter(s) is a(are) NULL pointer(s)
    AAA_RET_NOTAVAILABLE        = 10,   //!< profile not available
    AAA_RET_DIVISION_BY_ZERO    = 11,   //!< a divisor equals ZERO
    AAA_RET_WRONG_STATE         = 12,   //!< state machine in wrong state
    AAA_RET_INVALID_PARM        = 13,   //!< invalid parameter
    AAA_RET_PENDING             = 14,   //!< command pending
    AAA_RET_WRONG_CONFIG        = 15,   //!< given configuration is invalid
    AAA_RET_TIME_OUT            = 16,   //!< wait time out
    AAA_RET_UNSUPPORT_ID        = 0xFFFF //!< unsupport index number

};


/*****************************************************************************/
/**
 *          PerformanceiOptiMode_t
 *
 * @brief   AWB performance optimization mode choose
 *
 */
/*****************************************************************************/
typedef enum PerformanceiOptiMode_e
{
    PERFORMANCEI_NO_OPTIMIZATION       = 1,          
    PERFORMANCEI_GENERAL_OPTIMIZATION  = 2,
    PERFORMANCEI_FAST_OPTIMIZATION     = 4,
} PerformanceiOptiMode_t;


/*****************************************************************************/
/**
 *          AaaRoiWindow_t
 *
 * @brief   I3AI  Roi window parameters
 *
 */
/*****************************************************************************/
typedef struct AaaRoiWindow_s { 
    uint16_t fx;    
    uint16_t fy;    
    uint16_t fw;   
    uint16_t fh;  
    float    weight;
} AaaRoiWindow_t;

/*****************************************************************************/
/**
 *          AaaRoi_t
 *
 * @brief   AAA Roi set and get function's parameters
 *
 */
/*****************************************************************************/
typedef struct AaaRoi_s { 
    uint8_t         roiNum;
    float           roiWeight;
    AaaRoiWindow_t  roiWindow[ROI_WINDOWS_MAX];
} AaaRoi_t;

/*****************************************************************************/
/**
 *          I3aiAeRect_t
 *
 * @brief   I3AI ae rect area
 *
 */
/*****************************************************************************/
typedef struct AaaRect_s { 
    uint16_t x;    
    uint16_t y;   
    uint16_t width; 
    uint16_t height;  
} AaaRect_t;

/*****************************************************************************/
/**
 *          AeStatistic_t
 *
 * @brief   AE ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AeStatistic_s {
#ifdef ISP_AEV2_V4
    uint32_t statisticExpv2[AEI_EXPV2_GRID_NUM * AEI_EXPV2_GRID_NUM * AEI_EXPV2_PIXEL_CHANNEL];
#elif ISP_AEV2_V2
    uint16_t statisticExpv2[AEI_EXPV2_GRID_NUM * AEI_EXPV2_GRID_NUM * AEI_EXPV2_PIXEL_CHANNEL];
#else
    uint8_t statisticExpv2[AEI_EXPV2_GRID_NUM * AEI_EXPV2_GRID_NUM * AEI_EXPV2_PIXEL_CHANNEL];
#endif
} AeStatistic_t;

#endif

