/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file ae_ctrl_api.h
 *
 * @brief Include all aaa_ctrl AE's function and parameters
 *
 *****************************************************************************/

#ifndef __AEI_API_H__
#define __AEI_API_H__

#include "aei_lib_interface.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct AeiContext_s *AeiHandle_t;

/*****************************************************************************/
/**
 *          IsiAeLibInstanceConfig_t
 *
 * @brief   Config structure to create a new ae lib instance
 *
 */
/*****************************************************************************/
typedef struct AeiInstanceConfig_s
{
    uint8_t         ispId;
#ifndef ES_3A	
    AeiLib_t        *pAeiLib;           /**< Ae Lib struct interface */
#endif
	AeiHandle_t     hAeiHandle;         /**< Ae handle returned by AeiCreate */
} AeiInstanceConfig_t;

/*****************************************************************************/
/**
 *          AeiCreate
 *
 * @brief   register Ae lib and get the ae lib's handle
 *
 * @param   handle      Handle to the AE lib intance
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
AAA_RESULT AeiCreate
( 
     AeiInstanceConfig_t  *pAeiConfig
);

#ifdef ES_3A
AAA_RESULT AeiInitEs
(
    AeiHandle_t  handle,
    const V_ISP_AE_PARAM_S   *params//xiaoy todo:init params
);
AAA_RESULT AeiExitEs
(
    AeiHandle_t  handle
);
#endif

/*****************************************************************************/
/**
 *          AeCtrlUnRegister
 *
 * @brief   unregister Ae lib and release the ae lib's handle
 *
 * @param   handle       Handle to the AE lib intance
 * @param   ispId        isp id
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiRelease
( 
    AeiHandle_t  handle,
    uint8_t      ispId
);

/*****************************************************************************/
/**
 *          AeiSetRoi
 *
 * @brief   Set the Ae calculation's ROI area
 *
 * @param   handle    Handle to the AE lib intance
 * @param   pRoi      The pointer roi area size and number
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiSetRoi
( 
    AeiHandle_t     handle,
    const AaaRoi_t  *pRoi 
);

/*****************************************************************************/
/**
 *          AeiGetRoi
 *
 * @brief   Get the Ae calculation's ROI area
 *
 * @param   handle    Handle to the AE lib intance
 * @param   pRoi      The pointer roi area size and number
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiGetRoi
( 
    AeiHandle_t  handle,
    AaaRoi_t     *pRoi   
);

/*****************************************************************************/
/**
 *          AeiSetMode
 *
 * @brief   Set AE's working mode
 *
 * @param   handle     Handle to the AE lib intance
 * @param   pAeMode    The pointer to Working mode to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiSetMode
( 
    AeiHandle_t    handle,
    const AeMode_t *pAeMode
);

/*****************************************************************************/
/**
 *          AeCtrlGetMode
 *
 * @brief   Get AE's working mode
 *
 * @param   handle      Handle to the AE lib intance
 * @param   pAeMode     The pointer to Working mode to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiGetMode
( 
    AeiHandle_t  handle,
    AeMode_t     *pAeMode
);

/*****************************************************************************/
/**
 *          AeiSetConfig
 *
 * @brief   Set AE's calibration or tunning parameters
 *
 * @param   handle       Handle to the AE lib intance
 * @param   pAeConfig    The pointer to tunning or calibraration parameters to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiSetConfig
( 
    AeiHandle_t        handle,
    const AeConfig_t  *pAeConfig
);

/*****************************************************************************/
/**
 *          AeCtrlGetConfig
 *
 * @brief   Get AE's calibration or tunning parameters
 *  
 * @param   handle        Handle to the AE lib intance
 * @param   pAeConfig     The pointer to tunning or calibraration parameters to set
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiGetConfig
( 
    AeiHandle_t  handle,
    AeConfig_t  *pAeConfig
);


/*****************************************************************************/
/**
 *          AeiSetAttribute
 *
 * @brief   Set basic atrribution parameters to ae lib to calcualte 
 *
 * @param   handle        Handle to the AE lib intance
 * @param   pAeAttrParm   The ae attribution parameters to send
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiSetAttribute
( 
    AeiHandle_t         handle,
    const AeAttrParm_t  *pAeAttrParm
);


/*****************************************************************************/
/**
 *          AeiMetaDataExecute
 *
 * @brief   AE meta data running function
 * 
 * @param   handle              Handle to the AE lib intance
 * @param   pAeMetaDataInfo     The pointer to ae metadata input value
 * @param   pAeMetaDataResult   The AE metadata calculation result
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiMetaDataExecute
( 
    AeiHandle_t         handle,
    AeMetaDataInfo_t    *pAeMetaDataInfo, 
    AeMetaDataResult_t  *pAeMetaDataResult     
);


/*****************************************************************************/
/**
 *          AeiExecute
 *
 * @brief   AE execute function to run ae code
 *
 * @param   handle      Handle to the AE lib intance
 * @param   pAeInfo     The AE calculation input value
 * @param   pAeResult   The calculation result
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AeiExecute
( 
    AeiHandle_t  handle,
    AeInfo_t     *pAeInfo, 
    AeResult_t   *pAeResult     
);

#ifdef __cplusplus
}
#endif

#endif
