/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file Awb_ctrl_api.h
 *
 * @brief Include all aaa_ctrl Awb's function and parameters
 *
 *****************************************************************************/

#ifndef __AWBI_API_H__
#define __AWBI_API_H__

#include "awbi_lib_interface.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct AwbiContext_s *AwbiHandle_t;
/*****************************************************************************/
/**
 *          AwbiInstanceConfig_t
 *
 * @brief   Config structure to create a new Awbi lib instance
 *
 */
/*****************************************************************************/
typedef struct AwbiInstanceConfig_s
{
    uint8_t       ispId;
#ifndef ES_3A
	AwbiLib_t	  *pAwbiLib;		   /**< Awbi Lib struct interface */
#else
	V_ISP_AWB_EXP_FUNC_S *pAwbiLib;
#endif

    AwbiHandle_t  hAwbiHandle;         /**< Awbi handle returned by AwbiiCreate */
} AwbiInstanceConfig_t;


/*****************************************************************************/
/**
 *          AwbiCreate
 *
 * @brief   register Awb lib and get the Awb lib's handle
 *
 * @param   handle      Handle to the Awb lib intance
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
AAA_RESULT AwbiCreate
( 
     AwbiInstanceConfig_t  *pAwbConfig
);


/*****************************************************************************/
/**
 *          AwbCtrlUnRegister
 *
 * @brief   unregister Awb lib and release the Awb lib's handle
 *
 * @param   handle       Handle to the Awb lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiRelease
( 
    AwbiHandle_t  handle,
    uint8_t       ispId
);


/*****************************************************************************/
/**
 *          AwbiSetRoi
 *
 * @brief   Set the Awb calculation's ROI area
 *
 * @param   handle         Handle to the Awb lib intance
 * @param   pRoi           Pointer Awb regions of interest
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiSetRoi
( 
    AwbiHandle_t     handle,
    const AaaRoi_t   *pRoi 
);


/*****************************************************************************/
/**
 *          AwbiGetRoi
 *
 * @brief   Get the Awb calculation's ROI area
 *
 * @param   handle         Handle to the Awb lib intance
 * @param   pRoi           Pointer to the number of regions of interest
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiGetRoi
( 
    AwbiHandle_t  handle,
    AaaRoi_t      *pRoi   
);


/*****************************************************************************/
/**
 *          AwbiSetMode
 *
 * @brief   Set Awb's working mode
 *
 * @param   handle        Handle to the Awb lib intance
 * @param   pAwbMode      Working mode to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiSetMode
( 
    AwbiHandle_t     handle,
    const AwbiMode_t *pAwbMode
);


/*****************************************************************************/
/**
 *          AwbCtrlGetMode
 *
 * @brief   Get Awb's working mode
 *
 * @param   handle         Handle to the Awb lib intance
 * @param   pAwbMode       The pointer to Working mode type
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiGetMode
( 
    AwbiHandle_t  handle,
    AwbiMode_t    *pAwbMode
);


/*****************************************************************************/
/**
 *          AwbiSetConfig
 *
 * @brief   Set Awb's calibration or tunning parameters
 *
 * @param   handle       Handle to the Awb lib intance
 * @param   pAwbConfig   The pointer to tunning or calibraration parameters to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiSetConfig
( 
    AwbiHandle_t        handle,
    const AwbiConfig_t  *pAwbConfig
);


/*****************************************************************************/
/**
 *          AwbCtrlGetConfig
 *
 * @brief   Get Awb's calibration or tunning parameters
 *  
 * @param   handle      Handle to the Awb lib intance
 * @param   pAwbConfig  The pointer to tunning or calibraration parameters to set
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiGetConfig
( 
    AwbiHandle_t  handle,
    AwbiConfig_t  *pAwbConfig
);


/*****************************************************************************/
/**
 *          AwbiLibSetSensorAttributeParm
 *
 * @brief   return the specified runtime data and statistics
 *
 * @param   handle         Handle to the Awb lib intance
 * @param   pAwbAttrParm   The awb attribution parameters to send
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiSetAttribute
( 
    AwbiHandle_t         handle,
    const AwbAttrParm_t  *pAwbAttrParm
);


/*****************************************************************************/
/**
 *          AwbiSetCalibData
 *
 * @brief   running Awb lib module to get the result
 *
 * @param   handle            Handle to the Awb lib intance
 * @param   pAwbCalibData     The pointer to calibration data

 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiSetCalibData
( 
    AwbiHandle_t          handle,
    const AwbCalibData_t  *pAwbCalibData
);


/*****************************************************************************/
/**
 *          AeiMetaDataExecute
 *
 * @brief   AE meta data running function
 * 
 * @param   handle              Handle to the AE lib intance
 * @param   pAeMetaDataInfo     The pointer to ae metadata input value
 * @param   pAeMetaDataResult   The AE metadata calculation result
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiMetaDataExecute
( 
    AwbiHandle_t         handle,
    AwbMetaDataInfo_t    *pAwbMetaDataInfo, 
    AwbMetaDataResult_t  *pAwbMetaDataResult    
);


/*****************************************************************************/
/**
 *          AwbiExecute
 *
 * @brief   running Awb lib module to get the result
 *
 * @param   handle        Handle to the Awb lib intance
 * @param   pAwbInfo      The pointer to awb input value
 * @param   pAwbResult    The calculation result
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
AAA_RESULT AwbiExecute
( 
    AwbiHandle_t  handle,
    AwbInfo_t     *pAwbInfo, 
    AwbResult_t   *pAwbResult  
);

#ifdef __cplusplus
}
#endif

#endif
