/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @file awb_lib_interface.h
 *
 * @brief
 *
 *****************************************************************************/
 

#ifndef __AWBI_LIB_INTERFACE_H__
#define __AWBI_LIB_INTERFACE_H__

#include "aaa_common.h"


#ifdef __cplusplus
extern "C"
{
#endif

typedef void *AwbLibHandle_t;

#define AWBI_CCM_MATRIX_NUM           9
#define AWBI_CCM_OFFSET_NUM           3
#define AWBI_COLOR_TEM_WEIGHT_NUM     7
#define AWBI_CONFIG_FADE_NUM          16
#define AWBI_CONFIG_GAINDIST_NUM      16    
#define AWBI_CONFIG_FILTER_NUM        50
#define AWBI_CONFIG_COMPONENTGAIN_NUM 4
#define AWBI_CONFIG_PROFILE_NAME      20
#define AWBI_CONFIG_ILLUPROFILE_NUM   10
#define AWBI_WB_GAIN_NUM              4

/*****************************************************************************/
/**
 *          AwbLibInstanceConfig_t
 *
 * @brief   Config structure to create a new awb lib instance
 *
 */
/*****************************************************************************/
typedef struct AwbLibInstanceConfig_s
{
    AwbLibHandle_t  hAwbLibHandle;           /**< AWB Lib handle returned by AwbiCreate */
} AwbLibInstanceConfig_t;

/*****************************************************************************/
/**
 *          AwbisMode_t
 *
 * @brief   Ctrl Awb Mode set and get function's parameters
 *
 */
/*****************************************************************************/
typedef enum AwbiMode_s {
    AWBI_MODE          = 0,
    AWBI_METEDATA_MODE = 1,
} AwbiMode_t;


/*****************************************************************************/
/**
 *          AwbOtp_t
 *
 * @brief   awb OTP basic parameters
 */
/*****************************************************************************/
typedef struct AwbOtp_s {
    uint16_t r;
    uint16_t gr;
    uint16_t gb;
    uint16_t b;
    uint16_t rgRatio;
    uint16_t bgRatio;
} AwbOtp_t;

/*****************************************************************************/
/**
 *          AwbiOTPData_t
 *
 * @brief   awb otp data configuration parameters
 */
/*****************************************************************************/
typedef struct AwbiOTPData_s {
   uint32_t otpVersion;
   bool     otpAwbEnable;
   AwbOtp_t awbCTL;
   AwbOtp_t awbCTM;
   AwbOtp_t awbCTH; 
   AwbOtp_t awbGoldenCTL;
   AwbOtp_t awbGoldenCTM;
   AwbOtp_t awbGoldenCTH;
} AwbiOTPData_t;


/*****************************************************************************/
/**
 *          AwbiSettings_t
 *
 * @brief   Awb lib configuration which lib calculation need
 */
/*****************************************************************************/
typedef struct AwbiConfig_s {
    bool                       useDamping;
     
    //performance optimization
    PerformanceiOptiMode_t     performanceOptiMode;   

    //awb lock paramerters
    float32_t                  awbEnterLockThreshold;
    float32_t                  awbEnterUnlockThreshold;      
} AwbiConfig_t;


/*****************************************************************************/
/**
 *          I3aiAwbAtrributeParm_t
 *
 * @brief   Awb ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbAttrParm_s {
    uint8_t       ispBitwidth;
    AaaRect_t     hAaaRect;
    uint16_t      expv2Grid;
    AwbiOTPData_t otpAwb;
    float32_t     wbGainMin;
    float32_t     wbGainMax;   
} AwbAttrParm_t;


/*****************************************************************************/
/**
 *          AwbCenterLine_t
 *
 * @brief   Awb ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCenterLine_s{
    float32_t rg;
    float32_t bg;
    float32_t d;
} AwbCenterLine_t;


/*****************************************************************************/
/**
 *          AwbCtCurve_t
 *
 * @brief   Awb ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCtCurve_s
{
    float32_t cta;
    float32_t ctb;
    float32_t ctc;
} AwbCtCurve_t;


/*****************************************************************************/
/**
 *          AwbFadeParam_t
 *
 * @brief   Awb ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbFadeParam_s
{
    float32_t fade[AWBI_CONFIG_FADE_NUM];
    float32_t gainDist[AWBI_CONFIG_GAINDIST_NUM];
    int32_t   fadePoint;
} AwbFadeParam_t;


/*****************************************************************************/
/**
 *          AwbIlluProfile_t
 *
 * @brief   Awb ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbIlluProfile_s{
    char       name[AWBI_CONFIG_PROFILE_NAME];
    uint8_t    id;
    int32_t    doorType;
    int32_t    awbType;
    float32_t  ccmCoeff[AWBI_CCM_MATRIX_NUM];
    float32_t  ccmOffset[AWBI_CCM_OFFSET_NUM];
    float32_t  componentGain[AWBI_CONFIG_COMPONENTGAIN_NUM];
} AwbIlluProfile_t;


/*****************************************************************************/
/**
 *          AwbExpPrior_t
 *
 * @brief   Awb ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbExpPrior_s {
    float32_t  iirAdd;
    float32_t  iirSub;
    float32_t  iirThreshold;
    float32_t  iirMin;
    float32_t  iirMax;
    float32_t  iirCoefInit;
    int32_t    filterItemNumber;
} AwbExpPrior_t;


/*****************************************************************************/
/**
 *          I3aiAwbAtrributeParm_t
 *
 * @brief   Awb ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCalibData_s {
    float32_t         rgProjIndoorMin;
    float32_t         rgProjMax;
    float32_t         kFactor;
    AwbCenterLine_t   centerLine;
    AwbCtCurve_t      ctcurve;
    AwbFadeParam_t    fade1, fade2;
    AwbIlluProfile_t  illuProfiles[AWBI_CONFIG_ILLUPROFILE_NUM];
    AwbExpPrior_t     iir;
    int32_t           illuProfileNumber;
} AwbCalibData_t; 


/*****************************************************************************/
/**
 *          AwbResult_t
 *
 * @brief   I3ai Awb Lib every Frame need to config parameters
 *
 */
/*****************************************************************************/
typedef struct AwbInfo_s {
    float32_t         sensorGain;  //totalgain
    float32_t         sensorIntergramTime;
    float32_t         awbGain[AWBI_WB_GAIN_NUM];
    float32_t         awbBls;
    AeStatistic_t     hAeStatistic;
} AwbInfo_t;


/*****************************************************************************/
/**
 *          AwbResult_t
 *
 * @brief   Awb lib calculation result
 *
 */
/*****************************************************************************/
typedef struct AwbResult_s {
    float32_t  awbGain[AWBI_WB_GAIN_NUM];
    float32_t  awbColorTempWeight[AWBI_COLOR_TEM_WEIGHT_NUM];
    int32_t    AwbLibStatus;
} AwbResult_t;


/*****************************************************************************/
/**
 *          AwbMetaDataInfo_t
 *
 * @brief   AWB MetaData input value
 *
 */
/*****************************************************************************/
typedef struct AwbMetaDataInfo_s {
    float32_t    metaDataCCt;
    float32_t    metaDataWbGain[AWBI_CONFIG_COMPONENTGAIN_NUM];
} AwbMetaDataInfo_t;


/*****************************************************************************/
/**
 *          AwbMetaDataResult_t
 *
 * @brief   Awb meta data result
 *
 */
/*****************************************************************************/
typedef struct AwbMetaDataResult_s {
    float32_t    metaDataCcMatrix[AWBI_CCM_MATRIX_NUM];
    float32_t    metaDataCcOffset[AWBI_CCM_OFFSET_NUM];
    float32_t    metaDataColorTemWeight[AWBI_CONFIG_ILLUPROFILE_NUM];
} AwbMetaDataResult_t;


typedef AAA_RESULT (*AwbiCreate_t)             (AwbLibInstanceConfig_t *pAwbLibConfig, uint8_t ispId);
typedef AAA_RESULT (*AwbiRelease_t)            (AwbLibHandle_t handle, uint8_t ispId);
typedef AAA_RESULT (*AwbiSetRoi_t)             (AwbLibHandle_t handle, const AaaRoi_t *pRoi);
typedef AAA_RESULT (*AwbiGetRoi_t)             (AwbLibHandle_t handle, AaaRoi_t *pRoi);
typedef AAA_RESULT (*AwbiSetMode_t)            (AwbLibHandle_t handle, const AwbiMode_t *pAwbMode);
typedef AAA_RESULT (*AwbiGetMode_t)            (AwbLibHandle_t handle, AwbiMode_t *pAwbMode);
typedef AAA_RESULT (*AwbiSetConfig_t)          (AwbLibHandle_t handle, const AwbiConfig_t *pAwbConfig);
typedef AAA_RESULT (*AwbiGetConfig_t)          (AwbLibHandle_t handle, AwbiConfig_t *pAwbConfig);
typedef AAA_RESULT (*AwbiSetAttribute_t)       (AwbLibHandle_t handle, const AwbAttrParm_t *pAwbAttrParm);
typedef AAA_RESULT (*AwbiSetCalibData_t)       (AwbLibHandle_t handle, const AwbCalibData_t *pAwbCalibData);
typedef AAA_RESULT (*AwbiExecute_t)            (AwbLibHandle_t handle, AwbInfo_t *pAwbinfo, AwbResult_t *pAwbResult);
typedef AAA_RESULT (*AwbiMetaDataExecute_t)    (AwbLibHandle_t handle, AwbMetaDataInfo_t *pAwbMetaDatainfo, AwbMetaDataResult_t *pAwbMetaDataResult);

/*****************************************************************************/
/**
 *          AwbiLib_t
 *
 * @brief   Config structure to create a new sensor instance
 *
 */
/*****************************************************************************/
typedef struct AwbiLib_s
{
    const char                   *pAwbLibName;
    AwbiCreate_t                  pAwbiCreate;                
    AwbiRelease_t                 pAwbiRelease;             
    AwbiSetRoi_t                  pAwbiSetRoi;
    AwbiGetRoi_t                  pAwbiGetRoi;
    AwbiSetMode_t                 pAwbiSetMode;
    AwbiGetMode_t                 pAwbiGetMode;
    AwbiSetConfig_t               pAwbiSetConfig;
    AwbiGetConfig_t               pAwbiGetConfig;
    AwbiSetAttribute_t            pAwbiSetAttribute;
    AwbiSetCalibData_t            pAwbiSetCalibData;
    AwbiExecute_t                 pAwbiExecute;
    AwbiMetaDataExecute_t         pAwbiMetaDataExecute;
} AwbiLib_t;

#ifdef ES_3A
	/**AWB**/
	typedef enum VENDOR_AWB_CTRL_CMD_E {
		V_AWB_ROI_SET,
		V_AWB_ROI_GET,
		V_AWB_MODE_SET,
		V_AWB_MODE_GET,
		V_AWB_CONFIG_SET,
		V_AWB_CONFIG_GET,
		V_AWB_ATTR_SET,
		V_AWB_CALIB_SET,
	} V_AWB_CTRL_CMD_E;
	
	typedef struct VENDOR_ISP_AWB_PARAM_S {//todo
		uint32_t			 SensorId;
	} V_ISP_AWB_PARAM_S;
	
	typedef struct VENDOR_ISP_AWB_EXP_FUNC_S {
		int32_t (*pfn_awb_init)(int32_t s32Handle, const V_ISP_AWB_PARAM_S *pstAwbParam, AwbResult_t *pstAwbResult);
		int32_t (*pfn_awb_run)(int32_t s32Handle, const AwbInfo_t *pstAwbInfo, AwbResult_t *pstAwbResult, int32_t s32Rsv);
		int32_t (*pfn_awb_ctrl)(int32_t s32Handle, uint32_t u32Cmd, void *pValue);
		int32_t (*pfn_awb_exit)(int32_t s32Handle);
	} V_ISP_AWB_EXP_FUNC_S;
	
	typedef struct VENDOR_ISP_AWB_REGISTER_S {
		V_ISP_AWB_EXP_FUNC_S stAwbExpFunc;
	} V_ISP_AWB_REGISTER_S;
#endif

#ifdef __cplusplus
}
#endif

#endif 

