/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file ae_ctrl_api.h
 *
 * @brief Include all aaa_ctrl AE's function and parameters
 *
 *****************************************************************************/

#ifndef __AAA_CTRL_COMMON_H__
#define __AAA_CTRL_COMMON_H__

#include <stdbool.h>
#include <stdint.h>
#include <math.h>
#include <ebase/types.h>
#include <ebase/dct_assert.h>
#include <common/return_codes.h>
#include <ebase/trace.h>

#include "isi/isi.h"
#include "isi/isi_iss.h"
#include "cameric_drv/cameric_drv_api.h"

#include "cameric_drv/cameric_isp_expv2_drv_api.h"
#include "cameric_drv/cameric_isp_wb_drv_api.h"
#ifdef ISP_DG
#include "cameric_drv/cameric_isp_dg_drv_api.h"
#endif

#if defined ISP_HDR_STITCH_V1_3
#include <cameric_drv/cameric_isp_stitching_drv_api.h>
#endif

#if defined ISP_HDR_STITCH_V2
#include <cameric_drv/cameric_isp_stitching2_drv_api.h>
#endif

#if defined ISP_HDR_STITCH_V2_1
#include <cameric_drv/cameric_isp_stitching2_v1_drv_api.h>
#endif

#if defined ISP_HDR_STITCH_V3
#include <cameric_drv/cameric_isp_stitching3_drv_api.h>
#endif

#if defined ISP_HDR_STITCH_V3_1
#include <cameric_drv/cameric_isp_stitching3_v1_drv_api.h>
#endif

#ifdef __cplusplus
extern "C"
{
#endif

typedef unsigned short ushort;
typedef unsigned int   uint;
// typedef signed int     int32_t;
typedef float          float32_t;

#define CTRL_MAX_CHANNEL_NUM 4
#define CTRL_ROI_WINDOWS_MAX 25    /**< Maximum ROI windows num*/
#define CTRL_ROI_WEIGHT_MAX 255    /**< Maximum ROI windows weight*/


#if defined(ISP_AEV2_V4) || defined(ISP_AEV2_V2) || defined(ISP_AEV2)
#define AE_CTRL_EXPV2_GRID_SIZE 32
#define AE_CTRL_EXPV2_PIXEL_CHANNEL 4
#endif

#if defined ISP_EXP_V3
#define AE_CTRL_EXPV3_GRID_SIZE 25
#define AE_CTRL_EXPV3_PIXEL_CHANNEL 2
#endif

/*****************************************************************************/
/**
 *          CtrlPerformanceOptiMode_t
 *
 * @brief   AE and AWB performance optimization mode choose
 *
 */
/*****************************************************************************/
typedef enum CtrlPerformanceOptiMode_e
{
    CTRL_PERFORMANCE_NO_OPTIMIZATION         = 1,          
    CTRL_PERFORMANCE_GENERAL_OPTIMIZATION    = 2,
    CTRL_PERFORMANCE_FAST_OPTIMIZATION       = 4,
} CtrlPerformanceOptiMode_t;


/*****************************************************************************/
/**
 *          AaaCtrlRoiWindow_t
 *
 * @brief   Aaa  Roi window parameters
 *
 */
/*****************************************************************************/
typedef struct AaaCtrlRoiWindow_s { 
    uint16_t fx;    
    uint16_t fy;    
    uint16_t fw;   
    uint16_t fh;  
    float    weight;
} AaaCtrlRoiWindow_t;

/*****************************************************************************/
/**
 *          AaaRoi_t
 *
 * @brief   Aaa  Roi set and get function's parameters
 *
 */
/*****************************************************************************/
typedef struct AaaCtrlRoi_s { 
    uint8_t             roiNum;
    float               roiWeight;
    AaaCtrlRoiWindow_t  roiWindow[CTRL_ROI_WINDOWS_MAX];
} AaaCtrlRoi_t;

#ifdef __cplusplus
}
#endif

#endif


