/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file AE_CTRL_ctrl_api.h
 *
 * @brief Include all aaa_ctrl AE's function and parameters
 *
 *****************************************************************************/

#ifndef __AE_CTRL_API_H__
#define __AE_CTRL_API_H__

#include "3ai/aei_api.h"

#include "aaa_ctrl_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct AeCtrlContext_t *AeCtrlHandle_t;
#define AE_CTRL_CONFIG_NUM           8

#define HDR_RATIO_FRAME_NUM          3

#define HDR_RATIO_NUM                2
  
#define HDR_MAX_ESD_NUM              4

#define SENSOR_LONG_FRAME_INDEX      0
#define SENSOR_SHORT_FRAME_INDEX     1
#define SENSOR_VSHORT_FRAME_INDEX    2
#define SENSOR_VVSHORT_FRAME_INDEX   3

#define ISP_DG_GAIN_MAX                255.0f
#define ISP_DG_GAIN_MIN                1.0f

#define AE_CTRL_SETPOINT_MIN           0.0f
#define AE_CTRL_SETPOINT_MAX           255.0f

#define AE_CTRL_TOLERANCE_MIN          0.0f
#define AE_CTRL_TOLERANCE_MAX          100.0f

#define AE_CTRL_DAMP_MIN               0.0f
#define AE_CTRL_DAMP_MAX               1.0f

#define AE_CTRL_DAMP_UNNDER_RATIO_MIN  0.0f
#define AE_CTRL_DAMP_UNNDER_RATIO_MAX  1.0f

#define AE_CTRL_DAMP_UNNDER_GAIN_MIN   0.0f
#define AE_CTRL_DAMP_UNNDER_GAIN_MAX   16.0f

#define AE_CTRL_DAMP_OVER_RATIO_MIN    1.0f
#define AE_CTRL_DAMP_OVER_RATIO_MAX    4.0f

#define AE_CTRL_DAMP_OVER_GAIN_MIN     0.0f
#define AE_CTRL_DAMP_OVER_GAIN_MAX     128.0f

#define AE_CTRL_MOTION_PARAMETHER_MIN  0.0f
#define AE_CTRL_MOTION_PARAMETHER_MAX  1.0f

#define AE_CTRL_LINEAR_REPRESS_MIN     0.0f
#define AE_CTRL_LINEAR_REPRESS_MAX     1.0f

#define AE_CTRL_LINEAR_GAIN_MIN        0.0f
#define AE_CTRL_LINEAR_GAIN_MAX        255.0f

#define AE_CTRL_LINEAR_LEVEL_MIN       0.0f
#define AE_CTRL_LINEAR_LEVEL_MAX       16.0f

#define AE_CTRL_HDR_REPRESS_MIN        0.0f
#define AE_CTRL_HDR_REPRESS_MAX        1.0f

#define AE_CTRL_HDR_GAIN_MIN           0.0f
#define AE_CTRL_HDR_GAIN_MAX           255.0f

#define AE_CTRL_HDR_LEVEL_MIN          0.0f
#define AE_CTRL_HDR_LEVEL_MAX          16.0f

#define AE_CTRL_WDR_CONTRAST_MIN       0.0f
#define AE_CTRL_WDR_CONTRAST_MAX       255.0f

#define AE_CTRL_EXPV2_WINDOW_WEIGHT_MIN  0.0f
#define AE_CTRL_EXPV2_WINDOW_WEIGHT_MAX  255.0f

#define AE_CTRL_METADATA_EX_NUM          4

/*****************************************************************************/
/**
 *          AeCtrlState_t
 *
 * @brief   Config structure to create a new sensor instance
 *
 */
/*****************************************************************************/
typedef enum AeCtrlState_e {
    AE_CTRL_STATE_INVALID          = 0,
    AE_CTRL_STATE_INITIALIZED      = 1,
    AE_CTRL_STATE_STOPPED          = 2,
    AE_CTRL_STATE_RUNNING          = 3,
    AE_CTRL_STATE_RUNNING_SETTING  = 4,
    AE_CTRL_STATE_LOCK             = 5,
    AE_CTRL_STATE_MAX
} AeCtrlState_t;

/*****************************************************************************/
/**
 *          AeLibInstanceConfig_s
 *
 * @brief   Config structure to create a new ae lib instance
 *
 */
/*****************************************************************************/
typedef struct AeCtrInstanceConfig_s {
    uint8_t             ispId;              /* isp id index, if there are two or more isp running */
#ifndef ES_3A
	AeiLib_t            *pAeiLib;           /* Ae Lib struct interface */
#else
	V_ISP_AE_EXP_FUNC_S  *pAeiLib;
#endif

    CamerIcDrvHandle_t  hCamerIc;           /* cameric driver handle */
    IsiSensorHandle_t   hSensor;            /* sensor handle */
    AeCtrlHandle_t      hAeCtrlHandle;      /* Ae ctrl handle which create by AE_CTRL_ctrl_create */
} AeCtrInstanceConfig_t;


/*****************************************************************************/
/**
 *          AeCtrlSemMode_t
 *
 * @brief   AE sem mode parameters
 *
 */
/*****************************************************************************/
typedef enum AeCtrlSemMode_e {  
    AE_CTRL_SCENE_EVALUATION_FIX        = 0,        
    AE_CTRL_SCENE_EVALUATION_ADAPTIVE   = 1,  
    AE_CTRL_SCENE_METADATA              = 2,           
    AE_CTRL_SCENE_EVALUATION_MAX
} AeCtrlSemMode_t;

/*****************************************************************************/
/**
 *          AeCtrlFlickerPeroid_t
 *
 * @brief   Ae flicker mode 
 *
 */
/*****************************************************************************/
typedef enum AeCtrlFlickerPeroid_e {
    AE_CTRL_FLICKER_OFF              = 0,
    AE_CTRL_FLICKER_50HZ             = 1,
    AE_CTRL_FLICKER_60HZ             = 2,
    AE_CTRL_FLICKER_CUSTOMER_DEFINE  = 3,
    AE_CTRL_FLICKER_MAX
} AeCtrlFlickerPeroid_t;


/*****************************************************************************/
/**
 *          AeCtrlFlickerMode_t
 *
 * @brief   Ae flicker mode parameter
 *
 */
/*****************************************************************************/
typedef struct AeCtrlFlickerMode_e {
    AeCtrlFlickerPeroid_t    flickerPeriod;
    float32_t                    userDefinedPeriodus;
} AeCtrlFlickerMode_t;


/*****************************************************************************/
/**
 *          AeCtrlMode_t
 *
 * @brief   AE Mode set and get function's parameters
 *
 */
/*****************************************************************************/
typedef struct AeCtrlMode_e {
    AeCtrlSemMode_t      semMode;
    AeCtrlFlickerMode_t  antiBandingMode;
} AeCtrlMode_t;


/*****************************************************************************/
/**
 *          AeCtrlResult_t
 *
 * @brief   Ae result which customer can get in top API
 *
 */
/*****************************************************************************/
typedef struct AeCtrlResult_s {
    float32_t    meanluma;
    float32_t    totalGain;
    float32_t    sensorIntergramTime;
    float32_t    sensorGain;
} AeCtrlResult_t;


/*****************************************************************************/
/**
 *          AeCtrlConfig_t
 *
 * @brief   AE lib configuration which lib calculation need
 *
 */
/*****************************************************************************/
typedef struct AeCtrlConfig_s {
    float32_t                    setPoint;
    float32_t                    tolerance;
    float32_t                    dampOver;
    float32_t                    dampUnder;
    float32_t                    dampUnderRatio;
    float32_t                    dampUnderGain;
    float32_t                    dampOverRatio;
    float32_t                    dampOverGain;
    float32_t                    motionFilter;
    float32_t                    motionThreshold;
    float32_t                    targetFilter;
    float32_t                    lowlightLinearRepress[AE_CTRL_CONFIG_NUM];
    float32_t                    lowlightLinearGain[AE_CTRL_CONFIG_NUM];
    uint32_t                     lowlightLinearLevel;
    float32_t                    lowlightHdrRepress[AE_CTRL_CONFIG_NUM];
    float32_t                    lowlightHdrGain[AE_CTRL_CONFIG_NUM];
    uint32_t                     lowlightHdrLevel;
    float32_t                    wdrContrastMin;
    float32_t                    wdrContrastMax;
    
#if defined(ISP_AEV2_V4) || defined(ISP_AEV2_V2) || defined(ISP_AEV2)
    float32_t                    expv2WindowWeight[AE_CTRL_EXPV2_GRID_SIZE * AE_CTRL_EXPV2_GRID_SIZE];    
#endif
    bool_t                       frameCalEnable;
    bool_t                       autoHdrEnable;

    //performance optimization
    CtrlPerformanceOptiMode_t    performanceOptiMode;   
} AeCtrlConfig_t;


/*****************************************************************************/
/**
 *          AeCtrlMetaDataConfig_t
 *
 * @brief   AE meta data value input data
 *
 */
/*****************************************************************************/
typedef struct AeCtrlMetaDataConfig_s {
    bool_t       isSensorMode;
    float32_t    maxMetadataExposureTime;
    float32_t    metadataExposureTime[AE_CTRL_METADATA_EX_NUM];
    float32_t    metadataGain[AE_CTRL_METADATA_EX_NUM];
} AeCtrlMetaDataConfig_t;


/*****************************************************************************/
/**
 *          AeCtrlMetaDataCalResult_t
 *
 * @brief   AE meta data calculating value
 *
 */
/*****************************************************************************/
typedef struct AeCtrlMetaDataCalResult_s {
    float32_t    ExposureTotalGain;
} AeCtrlMetaDataCalResult_t;


/*****************************************************************************/
/**
 *          AeCtrlRegister
 *
 * @brief   register AE lib and get the AE lib's handle
 *
 * @param   pAeCtrConfig      Ctrl AE function's parameters
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT AeCtrlRegister
( 
    AeCtrInstanceConfig_t  *pAeCtrConfig
);

// #ifndef VENDOR_3A
// RESULT AeCtrlInitEs
// (
//     AeCtrlHandle_t  handle,
//     AeCtrInstanceConfig_t  *pAeCtrConfig
// );

// RESULT AeCtrlExitEs
// (
//     AeCtrlHandle_t  handle
// );
// #endif

/*****************************************************************************/
/**
 *          AeCtrlUnRegister
 *
 * @brief   unregister AE lib and release the AE lib's handle
 *
 * @param   handle       Handle to the AE lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlUnRegister
( 
    AeCtrlHandle_t  handle,
    uint8_t         ispId
);


/*****************************************************************************/
/**
 *          AeCtrlSetRoi
 *
 * @brief   Set the Ae calculation's ROI area
 *
 * @param   handle         Handle to the AE lib intance
 * @param   pAeCtrlRoi     Pointer AE regions of interest
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlSetRoi
( 
    AeCtrlHandle_t      handle,
    const AaaCtrlRoi_t  *pAeCtrlRoi 
);


/*****************************************************************************/
/**
 *          AeCtrlGetRoi
 *
 * @brief   Get the Ae calculation's ROI area
 *
 * @param   handle         Handle to the AE lib intance
 * @param   pAeCtrlRoi     Pointer AE regions of interest
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlGetRoi
( 
    AeCtrlHandle_t  handle,
    AaaCtrlRoi_t    *pAeCtrlRoi
);


/*****************************************************************************/
/**
 *          AeCtrlSetMode
 *
 * @brief   Set AE's working mode
 *
 * @param   handle         Handle to the AE lib intance
 * @param   hCtrlModel     Working mode type
 * @param   mode           Working mode to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlSetMode
( 
    AeCtrlHandle_t      handle,
    const AeCtrlMode_t  *pAeCtrlMode
);


/*****************************************************************************/
/**
 *          AeCtrlGetMode
 *
 * @brief   Get AE's working mode
 *
 * @param   handle         Handle to the AE lib intance
 * @param   hCtrlModel      Working mode type
 * @param   mode        The pointer to current Working mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlGetMode
( 
    AeCtrlHandle_t  handle,
    AeCtrlMode_t    *pAeCtrlMode
);


/*****************************************************************************/
/**
 *          AeCtrlSetConfig
 *
 * @brief   Set AE's calibration or tunning parameters
 *
 * @param   handle             Handle to the AE lib intance
 * @param   pAeCtrlConfig      The pointer to tunning or calibraration parameters to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlSetConfig
( 
    AeCtrlHandle_t        handle,
    const AeCtrlConfig_t  *pAeCtrlConfig
);


/*****************************************************************************/
/**
 *          AeCtrlGetConfig
 *
 * @brief   Get AE's calibration or tunning parameters
 *  
 * @param   handle              Handle to the AE lib intance
 * @param   hAeCtrlConfig       The pointer to tunning or calibraration parameters to set
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlGetConfig
( 
    AeCtrlHandle_t  handle,
    AeCtrlConfig_t  *pAeCtrlConfig
);


/*****************************************************************************/
/**
 *          AeCtrlGetStatus
 *
 * @brief   Get AE's status
 *  
 * @param   handle              Handle to the AE lib intance
 * @param   hAeCtrlConfig       The config type to set
 * @param   pAeCtrlSettingsGet  The pointer to tunning or calibraration parameters to set
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlGetStatus
( 
    AeCtrlHandle_t  handle,
    AeCtrlState_t   *pAeCtrlState
);


/*****************************************************************************/
/**
 *          AeCtrlStart
 *
 * @brief   modify the Ctrl ae's state to begin AE module
 *
 * @param   handle           Handle to the AE lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlStart
( 
    AeCtrlHandle_t  handle
);


/*****************************************************************************/
/**
 *          AeCtrlStop
 *
 * @brief   modify the Ctrl ae's state to stop AE module
 *
 * @param   handle                   Handle to the AE lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlStop
( 
    AeCtrlHandle_t  handle
);


/*****************************************************************************/
/**
 *          AeCtrlGetStatisticExpV2
 *
 * @brief   Getting expv2 statistic value from callback function
 *
 * @param   handle               Handle to the AE lib intance
 * @param   statisticExpV2       The pointer to expv2 type's statistic value
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
#if defined ISP_AEV2_V4 || defined ISP_AEV2_V2 || defined ISP_AEV2
RESULT AeCtrlGetStatisticExpV2
( 
    AeCtrlHandle_t      handle,
    CamericAev2Static_t *statisticExpV2
);
#endif


/*****************************************************************************/
/**
 *          AeCtrlExcuteMetaData
 *
 * @brief   ae metadata excute
 *
 * @param   handle                 Handle to the AE lib intance
 * @param   pAeCtrlMetaData        ae metadata input value
 * @param   pAeCtrlMetaDataResult  ae metadata result value

 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlExcuteMetaData
( 
    AeCtrlHandle_t            handle,
    AeCtrlMetaDataConfig_t    *pAeCtrlMetaData,
    AeCtrlMetaDataCalResult_t *pAeCtrlMetaDataResult
);


/*****************************************************************************/
/**
 *          AeCtrlGetResult
 *
 * @brief   AE get result 
 *
 * @param   handle             Handle to the AE lib intance
 * @param   pAeCtrlResult      The pointer to ae calculation result

 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlGetResult
( 
    AeCtrlHandle_t  handle,
    AeCtrlResult_t  *pAeCtrlResult 
);


/*****************************************************************************/
/**
 *          AeCtrlExcute
 *
 * @brief   running AE lib module to get the result
 *
 * @param   handle      Handle to the AE lib intance
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AeCtrlExcute
( 
    AeCtrlHandle_t handle
);

#ifdef __cplusplus
}
#endif

#endif
