/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond AWB_V3
 * 
 * @defgroup cam_device_awb_v3 CamDevice AWB V3 Definitions
 * @{
 *
 */

#ifndef CAMDEV_AWB_API_H
#define CAMDEV_AWB_API_H

//TODO #include "vsi_3alib_interface.h"
#include "cam_device_common.h"
#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAMDEV_AWB_COLOR_TEM_WEIGHT_NUM 7

/*****************************************************************************/
/**
 * @brief    AWB color temperature weight of every lighting
 *
 *****************************************************************************/
typedef float32_t CamDeviceAwbTemWeight_t[CAMDEV_AWB_COLOR_TEM_WEIGHT_NUM];

/*****************************************************************************/
/**
 *
 * @brief   CamDevice AWB status.
 *
 *****************************************************************************/
typedef enum CamDeviceAwbState_e {
    CAMDEV_AWB_STATE_INVALID       = 0,    /**< Invalid status */
    CAMDEV_AWB_STATE_INITIALIZED   = 1,    /**< AWB initialized */
    CAMDEV_AWB_STATE_STOPPED       = 2,    /**< AWB stopped */
    CAMDEV_AWB_STATE_RUNNING       = 3,    /**< AWB running */
    CAMDEV_AWB_STATE_LOCKED        = 4,    /**< AWB locked */
    CAMDEV_AWB_STATE_MAX
} CamDeviceAwbState_t;


/******************************************************************************/
/**
 * @brief   CamDevice AWB mode parameters.
 *
 *****************************************************************************/
typedef enum CamDeviceAwbMode_e {
    CAMDEV_AWB_MODE             = 0,    /**< AWB mode */
    CAMDEV_AWB_METEDATA_MODE    = 1,    /**< AWB METADATA MODE */
} CamDeviceAwbMode_t;

/******************************************************************************/
/**
 * @brief   CamDevice AWB control configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceAwbConfig_s {
    bool_t useDamping;    /**< Use damping */
} CamDeviceAwbConfig_t;


/*****************************************************************************/
/**
 * @brief   This function registers the AWB library.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pAwbLibHandle       Handle to the AWB library
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRegisterAwbLib
(
    CamDeviceHandle_t   hCamDevice,
// #ifndef VENDOR_3A
// 		int 				ispID,
// #endif
    void                *pAwbLibHandle    
);

// #ifndef VENDOR_3A
// RESULT VsiCamDeviceAwbInitEs
// (
//     CamDeviceHandle_t   hCamDevice
// );

// RESULT VsiCamDeviceAwbExitEs
// (
//     CamDeviceHandle_t   hCamDevice
// );
// #endif

/*****************************************************************************/
/**
 * @brief   This function unregisters the AWB library.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceUnRegisterAwbLib
(
    CamDeviceHandle_t  hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function sets the AWB configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to the AWB configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbSetConfig
(
    CamDeviceHandle_t              hCamDevice,
    const CamDeviceAwbConfig_t    *pConfig
);


/*****************************************************************************/
/**
 * @brief   This function gets the AWB configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to the AWB configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbGetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceAwbConfig_t   *pConfig
);

/******************************************************************************/
/**
 *
 * @brief   This function sets the AWB mode.
 *
 * @param   hCamDevice   Handle to the CamDevice instance
 * @param   pAwbMode     Pointer to the mode of AWB
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbSetMode
(
    CamDeviceHandle_t              hCamDevice,
    const CamDeviceAwbMode_t      *pAwbMode
);

/*****************************************************************************/
/**
 *
 * @brief   This function gets the AWB mode.
 *
 * @param   hCamDevice   Handle to the CamDevice instance
 * @param   pAwbMode     Pointer to the mode of AWB
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbGetMode
(
    CamDeviceHandle_t   hCamDevice,
    CamDeviceAwbMode_t  *pAwbMode
);


/*****************************************************************************/
/**
 * @brief   This function sets the AWB ROI.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pRoi                Pointer to the AWB ROI configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbSetRoi
(
    CamDeviceHandle_t       hCamDevice,
    const CamDeviceRoi_t   *pRoi
);

/*****************************************************************************/
/**
 * @brief   This function gets the AWB ROI.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pRoi                Pointer to the AWB ROI configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbGetRoi
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceRoi_t          *pRoi
);


/*****************************************************************************/
/**
 * @brief   This function gets the AWB temperature weight of every lighting
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pAwbTemWeight       Pointer to the AWB temperature weight
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbGetColorTempWeight
(
    CamDeviceHandle_t        hCamDevice,
    CamDeviceAwbTemWeight_t  *pAwbTemWeight
);

/*****************************************************************************/
/**
 * @brief   This function enables AWB.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_INVALID_PARM    Invalid configuration
 * @retval  RET_OUTOFRANGE      A configuration parameter is out of range
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbEnable
(
    CamDeviceHandle_t           hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables AWB.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function resets AWB.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbReset
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets the AWB status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 * @param   pAwbStatus          Pointer to AWB status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbGetStatus
(
    CamDeviceHandle_t           hCamDevice,
    CamDeviceAwbState_t         *pAwbStatus
);

/*****************************************************************************/
/**
 * @brief   This function gets the AWB version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to the AWB version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAwbGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/* @} cam_device_awb_v3 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_AWB_API_H
