/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond EXP_DUMMY
 *
 * @defgroup cam_device_exp_v3 CamDevice EXP V3 Definitions
 * @{
 *
 */

#ifndef CAMDEV_EXP_V3_API_H
#define CAMDEV_EXP_V3_API_H

#include "cam_device_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define CAMDEV_EXP_WINDOW 25
#define CAMDEV_EXPV3_GRID_ITEMS 25
#define CAMDEV_EXPV3_PIXEL_CHANNEL 2

typedef uint32_t CamDeviceExpV3Statistics_t[CAMDEV_EXPV3_GRID_ITEMS * CAMDEV_EXPV3_PIXEL_CHANNEL];

/******************************************************************************/
/**
 * @brief   CamDevice ISP EXP input select.
 *
 *****************************************************************************/
typedef enum CamDeviceExpV3InputSelect_e
{
    CAMDEV_ISP_EXPV3_INPUT_SELECT_INVALID = 0,
    CAMDEV_ISP_EXPV3_INPUT_SELECT_MAX,
} CamDeviceExpV3InputSelect_t;

/******************************************************************************/
/**
 * @brief   CamDevice EXP windows configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceExpV3WindowConfig_s
{
    uint8_t nop;
} CamDeviceExpV3WindowConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice EXP configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceExpV3Config_s
{
    uint8_t nop;
} CamDeviceExpV3Config_t;

/******************************************************************************/
/**
 * @brief   CamDevice EXP status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceExpV3Status_s
{
    uint8_t nop;
}CamDeviceExpV3Status_t;


/*****************************************************************************/
/**
 * @brief   This function sets EXP configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to EXP configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3SetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceExpV3Config_t    *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function gets EXP configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to EXP configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3GetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceExpV3Config_t    *pConfig
);


/*****************************************************************************/
/**
 * @brief   This function sets EXP measure window.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   window              Pointer to EXP measure window
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3SetMeasureWindow
(
    CamDeviceHandle_t              hCamDevice,
    CamDeviceExpV3WindowConfig_t     *pWindow
);


/*****************************************************************************/
/**
 * @brief   This function gets EXP measure window.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pWindow             Pointer to EXP measure window
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3GetMeasureWindow
(
    CamDeviceHandle_t               hCamDevice,
    CamDeviceExpV3WindowConfig_t     *pWindow
);

/*****************************************************************************/
/**
 * @brief   This function gets EXP statistic parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @Param   pExpStatic          Pointer to the EXP statistics
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3GetStatistics
(
    CamDeviceHandle_t          hCamDevice,
    CamDeviceExpV3Statistics_t  *pExpStatic
);


/*****************************************************************************/
/**
 * @brief   This function enables EXP.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3Enable
(
    CamDeviceHandle_t   hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables EXP.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3Disable
(
    CamDeviceHandle_t   hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets EXP status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pStatus             Pointer to EXP status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3GetStatus
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceExpV3Status_t    *pStatus
);

/*****************************************************************************/
/**
 * @brief   This function gets EXP version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to EXP version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3GetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/*****************************************************************************/
/**
 * @brief   This function resets EXP
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceExpV3Reset
(
    CamDeviceHandle_t        hCamDevice
);


/* @} cam_device_exp_v3 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif /* CAMDEV_EXP_V3_API_H */

