/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_FLEXA_API_H
#define CAMDEV_FLEXA_API_H

#include "cam_device_common.h"

/**
 * @cond FLEXA
 *
 * @defgroup cam_device_flexa CamDevice FLEXA Definitions
 * @{
 *
 */

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************/
/**
 * @brief   CamDevice FLEXA SBI configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceEzsbiMiConfig_s {
    uint32_t available;    /**< Available value */
    uint32_t slice_per_frame;    /**< Slice  of per frame */
    uint32_t Y_segment_entry_size;    /**< Y segment entry size */
    uint32_t Y_segment_entry_count;    /**< Y segment entry count */
    uint32_t CB_segment_entry_size;    /**< CB segment entry size */
    uint32_t CB_segment_entry_count;    /**< CB segment entry count */
    uint32_t CR_segment_entry_size;    /**< CR segment entry size */
    uint32_t CR_segment_entry_count;    /**< CR segment entry count */
    int valid_width;    /**< Valid width */
    uint32_t g_sizeY;    /**< Y size */
    uint32_t g_sizeCb;    /**< CB size */
}CamDeviceEzsbiMiConfig_t;


/*****************************************************************************/
/**
 * @brief   This function sets FLEXA configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to FLEXA configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFlexaSbiMiSetConfig
(
    CamDeviceHandle_t                hCamDevice,
    const CamDeviceEzsbiMiConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function gets FLEXA configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to FLEXA configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFlexaSbiMiGetConfig
(
    CamDeviceHandle_t         hCamDevice,
    CamDeviceEzsbiMiConfig_t *pConfig
);

/* @} cam_device_flexa */
/* @endcond */

#ifdef __cplusplus
}
#endif


#endif

