/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_FUSA_API_H
#define CAMDEV_FUSA_API_H

#include "cam_device_common.h"

/**
 * @cond FUSA_DUMMY
 *
 * @defgroup cam_device_fusa CamDevice FuSa Definitions
 * @{
 *
 */

#ifdef __cplusplus
extern "C"
{
#endif


/*****************************************************************************/
/**
 * @brief   CamDevice FuSa fault injection configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceFaultInjectConfig_s
{
    uint8_t nop;
} CamDeviceFaultInjectConfig_t;


/*****************************************************************************/
/**
 * @brief   CamDevice FuSa CRC configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceFusaCrcConfig_s {
    uint8_t nop;
} CamDeviceFusaCrcConfig_t;

/*****************************************************************************/
/**
 * @brief   CamDevice FuSa pixel count configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceFusaPixelCountConfig_s {
    uint8_t nop;
} CamDeviceFusaPixelCountConfig_t;

/*****************************************************************************/
/**
 * @brief   CamDevice FuSa BIST configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceFusaBistConfig_s {
    uint8_t nop;
} CamDeviceFusaBistConfig_t;


/****************************************************************************/
/**
 * @brief   This function enables FuSa fault injection.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to FuSa fault injection configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaFaultInjectEnable
(
    CamDeviceHandle_t             hCamDevice,
    CamDeviceFaultInjectConfig_t  *pConfig
);

/****************************************************************************/
/**
 * @brief   This function enables FuSa ECC.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaEccEnable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function disables FuSa ECC.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaEccDisable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function enables FuSa pixel count.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaPixelCountEnable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function disables FuSa pixel count.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaPixelCountDisable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function sets FuSa pixel count configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaPixelCountSetConfig
(
    CamDeviceHandle_t   hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function gets FuSa pixel count configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to FuSa pixel count configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaPixelCountGetConfig
(
    CamDeviceHandle_t                hCamDevice,
    CamDeviceFusaPixelCountConfig_t  *pConfig
);

/****************************************************************************/
/**
 * @brief   This function enables FuSa timeout.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaTimeoutEnable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function disables FuSa timeout.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaTimeoutDisable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function enables FuSa BIST.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaBistEnable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function disables FuSa BIST.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaBistDisable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function enables FuSa BISTPowerUp.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaBistPowerUpEnable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function disables FuSa BISTPowerUp.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaBistPowerUpDisable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function enables FuSa CRC.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaCrcEnable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function disables FuSa CRC.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaCrcDisable
(
    CamDeviceHandle_t  hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function sets FuSa CRC configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to FuSa CRC configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaCrcSetConfig
(
    CamDeviceHandle_t         hCamDevice,
    CamDeviceFusaCrcConfig_t  *pConfig
);

/****************************************************************************/
/**
 * @brief   This function gets FuSa CRC configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to FuSa CRC configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaCrcGetConfig
(
    CamDeviceHandle_t         hCamDevice,
    CamDeviceFusaCrcConfig_t  *pConfig
);

/****************************************************************************/
/**
 * @brief   This function gets FuSa BIST configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to FuSa BIST configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaBistGetConfig
(
    CamDeviceHandle_t          hCamDevice,
    CamDeviceFusaBistConfig_t  *pConfig
);

/****************************************************************************/
/**
 * @brief   This function resets FuSa.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaReset
(
    CamDeviceHandle_t hCamDevice
);

/****************************************************************************/
/**
 * @brief   This function gets the FuSa version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to FuSa version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceFusaGetVersion
(
    CamDeviceHandle_t    hCamDevice,
    uint32_t             *pVersion
);

/* @} cam_device_fusa */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif

