/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 *
 * @cond GTM_DUMMY
 * @defgroup cam_device_gtm_api CamDevice GTM Dummy API
 * @{
 *
 */
#ifndef CAMDEV_GTM_API_H
#define CAMDEV_GTM_API_H

typedef struct CamDeviceGtmConfig_s {
    uint8_t nop;
} CamDeviceGtmConfig_t;

typedef struct CamDeviceGtmStatus_s {
    bool_t  gtmRunning;
} CamDeviceGtmStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets the GTM configuration
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             configuration of GTM
 *
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_CONFIG    image effects isn't configured
 * @retval  RET_NOTAVAILABLE    module not available by driver or hardware
 *
 *****************************************************************************/
RESULT VsiCamDeviceGtmSetConfig
(
    CamDeviceHandle_t   hCamDevice,
    CamDeviceGtmConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function gets the GTM configuration
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             configuration of GTM
 *
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_CONFIG    image effects isn't configured
 * @retval  RET_NOTAVAILABLE    module not available by driver or hardware
 *
 *****************************************************************************/
RESULT VsiCamDeviceGtmGetConfig
(
    CamDeviceHandle_t   hCamDevice,
    CamDeviceGtmConfig_t *pConfig
);


/*****************************************************************************/
/**
 * @brief   This function enable the GTM
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceGtmEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disable the GTM
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceGtmDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function get the status of GTM
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   status            running status


 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceGtmGetStatus
(
    CamDeviceHandle_t    hCamDevice,
    CamDeviceGtmStatus_t *pStatus
);


/*****************************************************************************/
/**
 * @brief   This function resets GTM.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGtmReset
(
    CamDeviceHandle_t             hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets GTM version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            This pointer of GTM version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGtmGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t         *pVersion
);


/* @} cam_device_gtm_api */
/* @endcond */

#endif
