/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond GWDR_DUMMY
 * @defgroup cam_device_gwdr CamDevice GWDR Dummy Definitions
 * @{
 *
 */

#ifndef CAMDEV_GWDR_API_H
#define CAMDEV_GWDR_API_H

#include "cam_device_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************/
/**
 * @brief   CamDevice GWDR Dummy configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceGWdrConfig_s {
    uint8_t nop;
}CamDeviceGWdrConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice GWDR Dummy status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceGWdrStatus_s {
    uint8_t nop;
}CamDeviceGWdrStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets GWDR Dummy configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pGWdrCfg             Pointer to GWDR Dummy configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGWdrSetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceGWdrConfig_t *pGWdrCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets GWDR Dummy configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pGWdrCfg             Pointer to GWDR Dummy configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGWdrGetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceGWdrConfig_t *pGWdrCfg
);

/*****************************************************************************/
/**
 * @brief   This function enables GWDR Dummy.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGWdrEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables GWDR Dummy.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGWdrDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets GWDR Dummy status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pGWdrStatus          Pointer to GWDR status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGWdrGetStatus
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceGWdrStatus_t *pGWdrStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets GWDR to Dummy
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGWdrReset
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets GWDR Dummy version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to GWDR version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceGWdrGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/* @} cam_device_gwdr dummy */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif /* CAMDEV_GWDR_API_H */
