/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_WB_API_H
#define CAMDEV_WB_API_H

#include "cam_device_common.h"

#ifdef __cplusplus
extern "C"
{
#endif


/**
 * @cond WB_V1_1
 * 
 * @defgroup cam_device_wb_v1_1 CamDevice WB V1.1 Definitions
 * @{
 *
 */

/******************************************************************************/
/**
 * @brief   CamDevice WB manual configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceManualWbConfig_s {
    float32_t gain[CAMDEV_RAW_CHANNEL_NUM];       /**< WB gain */
}CamDeviceManualWbConfig_t;

 /******************************************************************************/
/**
 * @brief   CamDevice WB configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceWbConfig_s {
    CamDeviceManualWbConfig_t manualCfg;  /**< WB manual configuration */
}CamDeviceWbConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice WB status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceWbStatus_s {
    bool_t enable;              /**< WB enable status */
    CamDeviceConfigMode_t currentMode;        /**< The run mode: 0--manual, 1--auto */
    CamDeviceManualWbConfig_t currentCfg;  /**< WB current configuration */
}CamDeviceWbStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets WB configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pWbCfg              Pointer to WB configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWbSetConfig
(
    CamDeviceHandle_t hCamDevice,
    const CamDeviceWbConfig_t *pWbCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets WB configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pWbCfg              Pointer to WB configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWbGetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceWbConfig_t *pWbCfg
);

/*****************************************************************************/
/**
 * @brief   This function enables WB.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWbEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables WB.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWbDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets WB status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pWbStatus           Pointer to WB status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWbGetStatus
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceWbStatus_t *pWbStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets WB.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWbReset
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets WB version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to WB version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWbGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/* @} cam_device_wb_v1_1 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_WB_API_H
