/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    cam_engine_aaa
 *
 * @file    cam_engine_aaa_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine Auto Algorithms.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_aaa_api CamEngine AAA Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_AAA_API_H
#define CAM_ENGINE_AAA_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/ispcore_defs.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/**
 * @brief   This functions get otp data
 *
 * @param   hCamEngine          A handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGetOtpData
(
    CamEngineHandle_t        hCamEngine      /**< handle CamEngine */
);


#define SENSOR_ID_LEN 11
#define MODULE_SN_LEN 12
typedef struct CamEngineOtpModuleInfo_s
{
  uint16_t HWVersion;
  uint16_t EEPROMRevision;
  uint16_t SensorRevision;
  uint16_t TLensRevision;
  uint16_t IRCFRevision;
  uint16_t LensRevision;
  uint16_t CARevision;
  uint16_t ModuleInteID;
  uint16_t FactoryID;
  uint16_t MirrorFlip;
  uint16_t TLensSlaveID;
  uint16_t EEPROMSlaveID;
  uint16_t SensorSlaveID;
  uint8_t  SensorID[SENSOR_ID_LEN];
  uint16_t ManuDateYear;
  uint16_t ManuDateMonth;
  uint16_t ManuDateDay;
  uint8_t  BarcodeModuleSN[MODULE_SN_LEN];
  uint16_t MapTotalSize;
} CamEngineOtpModuleInfo_t;


/*****************************************************************************/
/**
 * @brief   This functions get otp module info
 *
 * @param   hCamEngine          A handle to the CamEngine instance.
 * @param   OtpModuleInfo       Otp module info struct
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGetOtpModuleInfo
(
    CamEngineHandle_t        hCamEngine,      /**< handle CamEngine */
    CamEngineOtpModuleInfo_t *OtpModuleInfo
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_aaa_api */


#endif /* CAM_ENGINE_AAA_API_H */

