/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    CSM
 * 
 * @file    cam_engine_csm_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine CSM.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_csm_api CamEngine CSM API
 * @{
 *
 */

#ifndef CAM_ENGINE_CSM_API_H
#define CAM_ENGINE_CSM_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif


/******************************************************************************/
/**
 * @brief   A structure to represent a 3x3 matrix.
 *
 *          The nine values are laid out as follows (zero based index):
 *
 *               | 0 1 2 | \n
 *               | 3 4 5 | \n
 *               | 6 7 8 | \n
 *
 * @note    The values are represented as float numbers.
 *
 *****************************************************************************/
typedef struct CamEngineCsm3x3FloatMatrix_s
{
    float32_t fCoeff[9];               /**< array of 3x3 float values */
} CamEngineCsm3x3FloatMatrix_t;

/*****************************************************************************/
/**
 * @brief   This function enables the csm.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCsmEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the csm.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCsmDisable
(
    CamEngineHandle_t       hCamEngine
);


/*****************************************************************************/
/**
 * @brief   This function config the csm.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of csm
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCsmSetConfig
(
    CamEngineHandle_t     hCamEngine,
    const CamEngineCsm3x3FloatMatrix_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function returns the csm matrix.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of csm matrix
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCsmGetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineCsm3x3FloatMatrix_t   *pConfig
);

#ifdef __cplusplus
}
#endif

/* @} cam_engine_csm_api */

#endif /* CAM_ENGINE_CSM_API_H */

