/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    cam_engine_drv.h
 *
 * @file    cam_engine_drv_api.h
 *
 * @brief
 *
 *   Interface function to the CamerIc Driver.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_drv_api CamEngine DRV Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_ISP_DRV_H
#define CAM_ENGINE_ISP_DRV_H

#include <ebase/types.h>

#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif


/*****************************************************************************/
/**
 * @brief   This function returns the CamerIc (Master) Revision ID.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   revision            reference to CamerIc Master Revision ID
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_INVALID_PARM    invalid parameter (revision is a NULL pointer)
 *
 *****************************************************************************/
RESULT CamEngineCamerIcMasterId
(
    CamEngineHandle_t   hCamEngine,
    uint32_t            *revision
);


/*****************************************************************************/
/**
 * @brief   This function returns the CamerIc (Slave) Revision ID.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   revision            reference to CamerIc Slave Revision ID
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_INVALID_PARM    invalid parameter (revision is a NULL pointer)
 *
 *****************************************************************************/
RESULT CamEngineCamerIcSlaveId
(
    CamEngineHandle_t   hCamEngine,
    uint32_t            *revision
);


#ifdef __cplusplus
}
#endif


/* @} cam_engine_isp_api */


#endif /* CAM_ENGINE_ISP_DRV_H */

