/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    cam_engine_mi_api.h
 *
 * @file    cam_engine_mi_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine MI.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_mi_api CamEngine MI Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_MI_API_H
#define CAM_ENGINE_MI_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#include <cameric_drv/cameric_mi_drv_api.h>
#include <cameric_drv/cameric_isp_sbi_mi_api.h>

#ifdef ISP_MI_HDR
#include <cameric_drv/cameric_mi_hdr_drv_api.h>
#endif

/*****************************************************************************/
/**
 * @brief   Enumeration type to identify Hdr RET STR
 *
 *****************************************************************************/
typedef enum CamEngineMiStitchingHdrRdStr_e
{
    CAM_ENGINE_MI_STITCHING_HDR_RD_STR_INVALID	    = 0,        /**< lower border (only for an internal evaluation) */
    CAM_ENGINE_MI_STITCHING_HDR_RD_STR_L		    = 1,
    CAM_ENGINE_MI_STITCHING_HDR_RD_STR_L_S		    = 2,
    CAM_ENGINE_MI_STITCHING_HDR_RD_STR_L_S_VS       = 3,
    CAM_ENGINE_MI_STITCHING_HDR_RD_STR_L_VS	        = 4,
    CAM_ENGINE_MI_STITCHING_HDR_RD_STR_L_S_VS_E3	= 5,
    CAM_ENGINE_MI_STITCHING_HDR_RD_STR_LS_VS	    = 6,
    CAM_ENGINE_MI_STITCHING_HDR_RD_STR_MAX					/**< upper border (only for an internal evaluation) */
} CamEngineMiStitchingHdrRdStr_t;

/*****************************************************************************/
/**
 * @brief   Enumeration type to identify Hdr RET PPC
 *
 *****************************************************************************/
typedef enum CamEngineMiHdrPpc_e
{
    CAM_ENGINE_MI_HDR_1PPC		    = 0,
    CAM_ENGINE_MI_HDR_2PPC		    = 1,
    CAM_ENGINE_MI_HDR_PPC_MAX					/**< upper border (only for an internal evaluation) */
} CamEngineMiHdrPpc_t;


/*****************************************************************************/
/**
 * @brief   Flexa Sbi memory interface config structure
 *
 *****************************************************************************/
typedef struct  CamEngineEzsbiMiConfig_s {
    uint32_t available;
    uint32_t slice_per_frame;
    uint32_t Y_segment_entry_size;
    uint32_t Y_segment_entry_count;
    uint32_t CB_segment_entry_size;
    uint32_t CB_segment_entry_count;
    uint32_t CR_segment_entry_size;
    uint32_t CR_segment_entry_count;
    uint32_t valid_width;
    uint32_t g_sizeY;
    uint32_t g_sizeCb;
}CamEngineEzsbiMiConfig_t;


/*****************************************************************************/
/**
 * @brief   This function enables/disables horizontal flip.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineOriginal
(
    CamEngineHandle_t hCamEngine
);



/*****************************************************************************/
/**
 * @brief   This function enables/disables horizontal flip.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHorizontalFlip
(
    CamEngineHandle_t hCamEngine
);



/*****************************************************************************/
/**
 * @brief   This function enables/disables vertical flip.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineVerticalFlip
(
    CamEngineHandle_t hCamEngine
);



/*****************************************************************************/
/**
 * @brief   This function enables/disables rotation.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineRotateLeft
(
    CamEngineHandle_t hCamEngine
);



/*****************************************************************************/
/**
 * @brief   This function enables/disables rotation.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineRotateRight
(
    CamEngineHandle_t hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Dump configure function works for MI MP media buffer dump.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   dump_name           dump file name
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NULL_POINTER    handle is NULL
 * @retval  RET_FAILURE         module not available by driver or hardware
 *
 *****************************************************************************/
RESULT CamEngineMiMpDumpBufCfg
(
    CamEngineHandle_t       hCamEngine,
    const char *            mp_dump_name
);

/*****************************************************************************/
/**
 * @brief   Dump configure function works for MI SP media buffer dump.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   dump_name           dump file name
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NULL_POINTER    handle is NULL
 * @retval  RET_FAILURE         module not available by driver or hardware
 *
 *****************************************************************************/
RESULT CamEngineMiSpDumpBufCfg
(
    CamEngineHandle_t       hCamEngine,
    const char *            sp_dump_name
);

/*****************************************************************************/
/**
 * @brief   Dump function works for MI MP media buffer dump to file.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   dump_name           dump file name
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NULL_POINTER    handle is NULL
 * @retval  RET_FAILURE         module not available by driver or hardware
 *
 *****************************************************************************/
RESULT CamEngineMiMpDumpToFile
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Dump function works for MI SP media buffer dump to file.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   dump_name           dump file name
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NULL_POINTER    handle is NULL
 * @retval  RET_FAILURE         module not available by driver or hardware
 *
 *****************************************************************************/
RESULT CamEngineMiSpDumpToFile
(
    CamEngineHandle_t       hCamEngine
);

RESULT CamEngineSetMiPpwLineNum
(
    CamEngineHandle_t       hCamEngine,
    uint16_t            num
);

RESULT CamEngineGetMiPpwLineCnt
(
    CamEngineHandle_t       hCamEngine,
    uint16_t            *pCnt
);

RESULT CamEngineCfgMiPpDamLineEntry
(
    CamEngineHandle_t       hCamEngine,
    uint16_t            bufNumber,
    uint16_t            lineNumber
);

RESULT CamEngineSetSbiMiCfg
(
    CamEngineHandle_t       hCamEngine,
    CamEngineEzsbiMiConfig_t * Ezsbi
);

RESULT CamEngineGetSbiMiCfg
(
    CamEngineHandle_t       hCamEngine,
    CamEngineEzsbiMiConfig_t * Ezsbi
);

RESULT CamEngineSetHdrRdWrStr
(
    CamEngineHandle_t                hCamEngine,
    CamEngineMiStitchingHdrRdStr_t   rdWrStr
);

RESULT CamerEngineSetMiHdrPpc
(
    CamEngineHandle_t    hCamEngine,
    CamEngineMiHdrPpc_t  ppc
);

#ifdef __cplusplus
}
#endif

/* @} cam_engine_mi_api */

#endif /* CAM_ENGINE_MI_API_H */


