/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 *
 * @cond    video_in
 * 
 * @file    cam_engine_video_in_api.h
 *
 * @brief   This file defines the video in API
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_video_in_api.h CamEngine video in API
 * @{
 *
 */

#ifndef CAM_ENGINE_VIDEO_IN_API_H
#define CAM_ENGINE_VIDEO_IN_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#include <common/return_codes.h>
#include "cam_engine_common.h"

/*****************************************************************************/
/**
 * @brief Processing paths of the CamEngine.
 *
 *****************************************************************************/
typedef enum CamEngineVideoInPathType_e
{
    CAM_ENGINE_VIDEO_IN_PATH_INVALID = -1,
    CAM_ENGINE_VIDEO_IN_PATH0 = 0,
    CAM_ENGINE_VIDEO_IN_PATH1,
    CAM_ENGINE_VIDEO_IN_PATH2,
    CAM_ENGINE_VIDEO_IN_PATH3,
    CAM_ENGINE_VIDEO_IN_PATH_ID_MAX
} CamEngineVideoInPathType_t;


/******************************************************************************/
/**
 * @brief   enum video in data align mode
 *
 *****************************************************************************/
typedef enum CamEngineVideoInAlignMode_e{
    CAM_ENGINE_VIDEO_IN_ALIGN_MODE_INVALID = 0,
    CAM_ENGINE_VIDEO_IN_UNALIGN_MODE          ,
    CAM_ENGINE_VIDEO_IN_ALIGN_MODE1           ,
    CAM_ENGINE_VIDEO_IN_ALIGN_MODE2           ,
	CAM_ENGINE_VIDEO_IN_ALIGN_MODE_MAX,
} CamEngineVideoInAlignMode_t;

/******************************************************************************/
/**
 * @brief   enum video in format
 *
 *****************************************************************************/
typedef enum CamEngineVideoInFmt_e{
    CAM_ENGINE_VIDEO_IN_FMT_DIABLED = 0,
	CAM_ENGINE_VIDEO_IN_FMT_RAW8 ,
	CAM_ENGINE_VIDEO_IN_FMT_RAW10,
	CAM_ENGINE_VIDEO_IN_FMT_RAW12,
	CAM_ENGINE_VIDEO_IN_FMT_RAW14,
	CAM_ENGINE_VIDEO_IN_FMT_RAW16,
	CAM_ENGINE_VIDEO_IN_FMT_RAW20_COMPRESS,
	CAM_ENGINE_VIDEO_IN_FMT_RAW24,
	CAM_ENGINE_VIDEO_IN_FMT_RAW24_COMPRESS,
	CAM_ENGINE_VIDEO_IN_FMT_MAX,
} CamEngineVideoInFmt_t;

/*****************************************************************************/
/**
 * @brief   Configuration structure of the CamEngineVideoIn path.
 *
 *****************************************************************************/
typedef struct CamEngineCamEngineVideoInPathConfig_s
{
    CamEngineVideoInPathType_t  path;
    uint16_t                    width;
    uint16_t                    height;
    CamEngineVideoInFmt_t       format;
    CamEngineVideoInAlignMode_t align;

} CamEngineVideoPathConfig_t;

/*****************************************************************************/
/**
 * @brief   This function set vi200  path
 *
 * @param    hCamEngine          handle to the CamEngine instance
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
RESULT CamEngineVideoInSetPath
(
    CamEngineHandle_t hCamEngine,
    CamEngineVideoInPathType_t path
);

/*****************************************************************************/
/**
 * @brief   This function config mcm write format
 *
 * @param   handle             video in handle
 * @param   pCfg               config pointer
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
RESULT CamEngineVideoInSetMcmWrCfg
(
    CamEngineHandle_t hCamEngine,
    const CamEngineVideoPathConfig_t *pCfg
);

/*****************************************************************************/
/**
 * @brief   This function set meta buffer handle
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   bufMgmtHandle       meta buffer handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
RESULT CamEngineSetMetaBufHandle
(
    CamEngineHandle_t hCamEngine,
    BufMgmtHandle_t bufMgmtHandle
);

#ifdef __cplusplus
}
#endif

/* @} cam_engine_video_in_api */

#endif /* CAM_ENGINE_VIDEO_IN_API_H */
