/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    WB_V11
 *
 * @file    cam_engine_wb_v11_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine WB.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_wb_api CamEngine WB V11 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_WB_API_H
#define CAM_ENGINE_WB_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif


#define ISP_AWB_MASK 0x4000
#define ISP_AWB_RATIO 1024.0f

#define ISP_WB_STITCH_MASK 0x400
#define ISP_WB_STITCH_RATIO 256.0f
#define ISP_WB_GAIN_MAX 15.999f
/******************************************************************************/
/**
 * @brief   WB manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineWbConfig_s
{
	float32_t gain[4]; /**< WB gain */
} CamEngineWbConfig_t;

/*****************************************************************************/
/**
 * @brief   Enables the WB.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineWbEnable
(
    CamEngineHandle_t      hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the WB.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineWbDisable
(
    CamEngineHandle_t      hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Sets the gain value of the WB.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Digital gain value
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineWbSetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineWbConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the WB status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Digital gain value
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineWbGetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineWbConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets the status of WB.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The running status
 * @param   pCfg                WB configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineWbGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    CamEngineWbConfig_t *pConfig
);


#ifdef __cplusplus
}
#endif


/* @} cam_engine_wb_api */


#endif /* CAM_ENGINE_WB_API_H */
