/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */
#ifndef CAMERIC_ISP_DG_DRV_API_H
#define CAMERIC_ISP_DG_DRV_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

//#include "cameric_isp_drv_api.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define DIGITAL_GAIN_MAX 65535
#define DIGITAL_GAIN_MIN 0

/******************************************************************************/
/**
 * @brief   Stucture of DigitalGain configuration.
 *
 *****************************************************************************/
typedef struct CamerIcIspDgConfig_s {
    uint16_t gainR;
    uint16_t gainB;
    uint16_t gainGr;
    uint16_t gainGb;

} CamerIcIspDgConfig_t;

/* @endcond */

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function enable digital gain.
 *
 * @param   handle              CamerIc driver handle


 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read gamma-out
 *                              status
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/

extern RESULT CamerIcIspDgEnable
(
    CamerIcDrvHandle_t handle
);
/* @endcond */

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function disable digital gain.
 *
 * @param   handle              CamerIc driver handle


 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read gamma-out
 *                              status
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/

extern RESULT CamerIcIspDgDisable
(
    CamerIcDrvHandle_t handle
);
/* @endcond */

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function get the digital gain module enable status..
 *
 * @param   handle              CamerIc driver handle
 * @param   pIsEnabled          pointer to the digital enable status.


 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read gamma-out
 *                              status
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
extern RESULT CamerIcIspDgIsActivated
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

/*****************************************************************************/
/**
 *
 * @brief   This function set the digital gain module gain.
 *
 * @param   handle              CamerIc driver handle
 * @param   pDigiCfg          pointer to the digital config context.


 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read gamma-out
 *                              status
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
extern RESULT CamerIcIspDgSetConfig
(
    CamerIcDrvHandle_t handle,
    const CamerIcIspDgConfig_t *pDigiCfg
);

/*****************************************************************************/
/**
 *
 * @brief   This function get the digital gain module gain.
 *
 * @param   handle              CamerIc driver handle
 * @param   pDigiCfg          pointer to the digital config context.


 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read gamma-out
 *                              status
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
extern RESULT CamerIcIspDgGetConfig
(
	CamerIcDrvHandle_t handle,
	CamerIcIspDgConfig_t *pDigiCfg
);

#ifdef __cplusplus
}
#endif

/* @} cameric_isp_dg_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_DG_DRV_API_H */
