/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

#ifndef CAMERIC_ISP_GE_DRV_API_H
#define CAMERIC_ISP_GE_DRV_API_H

/**
 * @cond    cameric_isp_green_equilibration
 *
 * @file    cameric_isp_green_equilibration_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP GREEN_EQUILIBRATION (video stabilization) Driver
 *          API definitions
 *
 *****************************************************************************/
/**
 * @defgroup cameric_isp_green_equilibration_drv_api CamerIC ISP GREEN_EQUILIBRATION Driver API definitions
 *
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAMERIC_ISP_GE_THRESHOLD_MIN 0
#define CAMERIC_ISP_GE_THRESHOLD_MAX 65535

/*****************************************************************************/
/**
 * @brief   This functions enables the CamerIC ISP green equilibration
 *           module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspGeEnable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This functions disables the CamerIC ISP green equilibration
 *            module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspGeDisable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   Get CamerIC ISP green equilibration module status.
 *
 * @param   handle          CamerIc driver handle.
 * @param   pIsEnabled
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspGeIsEnabled
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

/*****************************************************************************/
/**
 * @brief
 *
 * @param   handle          CamerIc driver handle.
 * @param
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspGeSetThreshold
(
    CamerIcDrvHandle_t      handle,
    uint16_t                threshold
);

RESULT CamerIcIspGeGetThreshold
(
    CamerIcDrvHandle_t      handle,
    uint32_t                *pThreshold
);

#ifdef __cplusplus
}
#endif

/* @} cameric_isp_ge_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_GE_DRV_API_H */


