/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2023 */

#ifndef CAMERIC_ISP_MPCTL_DRV_API_H
#define CAMERIC_ISP_MPCTL_DRV_API_H

/**
 * @cond    cameric_isp_mpctl
 *
 * @file    cameric_isp_mpctl_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP MPCTL driver API definitions.
 *
 *****************************************************************************/
/**
 * @defgroup cameric_isp_mpctl_drv_api CamerIC ISP MPCTL driver API definitions
 * @{
 *
 *
 * The mpctrl module is used
 */
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct CamericIspMpctlCfg_s {
    bool_t outPixelNum;        /**<"At output port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"*/
    bool_t inPixelNum;        /**<"At input port 1'b0: one pixel in one cycle 1'b1: two pixel in one cycle"*/
    uint16_t cropSize;             /**<    crop size for one line */
    uint16_t cropOffset;         /**<    crop offset for one line */
    // bool_t buf_full;         //    buf full indicator  for line SRAM

} CamericIspMpctlCfg_t;
/*****************************************************************************/
/**
 * @brief   This function enables the CamerIC ISP MPCTL Module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspMpctlEnable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
 * @brief   This function disables the CamerIc ISP MPCTL module (bypass the
 *          mpctrl module)
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspMpctlDisable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This function returns the status of the CamerIC ISP MPCTL module.
 *
 * @param   handle              CamerIc driver handle
 * @param   pIsEnabled          Pointer to value to store current state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    pIsEnabled is a NULL pointer
 *
 *****************************************************************************/
extern RESULT CamerIcIspMpctlIsActivated
(
    CamerIcDrvHandle_t  handle,
    bool_t              *pIsEnabled
);

/*****************************************************************************/
/**
 * @brief   This function set config of the CamerIC ISP MPCTL module.
 *
 * @param   handle              CamerIc driver handle
 * @param   pIsEnabled          Pointer to value to store current state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    pIsEnabled is a NULL pointer
 *
 *****************************************************************************/
extern RESULT CamerIcIspMpctlSetCfg
(
    CamerIcDrvHandle_t  handle,
    const CamericIspMpctlCfg_t  *pMpctlCfg

);

/*****************************************************************************/
/**
 * @brief   This function get config of the CamerIC ISP MPCTL module.
 *
 * @param   handle              CamerIc driver handle
 * @param   pIsEnabled          Pointer to value to store current state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    pIsEnabled is a NULL pointer
 *
 *****************************************************************************/
extern RESULT CamerIcIspMpctlGetCfg
(
    CamerIcDrvHandle_t  handle,
    CamericIspMpctlCfg_t  *pMpctlCfg

);
#ifdef __cplusplus
}
#endif

/* @} cameric_isp_mpctl_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_MPCTL_DRV_API_H */

