/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

#ifndef CAMERIC_ISP_WDR4_DRV_API_H
#define CAMERIC_ISP_WDR4_DRV_API_H

/**
 * @file    cameric_isp_wdr4_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP WDR driver API definitions.
 *
 *****************************************************************************/
/**
 * @cond cameric_isp_wdr4
 *
 * @defgroup cameric_isp_wdr4_drv_api CamerIC ISP WDR4 Driver API definitions
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAMIC_WDR4_CONTRAST_MAX 1023
#define CAMIC_WDR4_CONTRAST_MIN -1023

#define CAMIC_WDR4_ENTROPY_BASE_MAX 1023
#define CAMIC_WDR4_ENTROPY_BASE_MIN 0

#define CAMIC_WDR4_ENTROPY_SLOPE_MAX 1023
#define CAMIC_WDR4_ENTROPY_SLOPE_MIN 0

#define CAMIC_WDR4_FLAT_STRENGHTH_MAX 19
#define CAMIC_WDR4_FLAT_STRENGHTH_MIN 0

#define CAMIC_WDR4_FLAT_THR_MAX 20
#define CAMIC_WDR4_FLAT_THR_MIN 0

#define CAMIC_WDR4_GLOBAL_STRENGHTH_MAX 128
#define CAMIC_WDR4_GLOBAL_STRENGHTH_MIN 0

#define CAMIC_WDR4_HIGH_STRENGHTH_MAX 128
#define CAMIC_WDR4_HIGH_STRENGHTH_MIN 0

#define CAMIC_WDR4_LOW_STRENGHTH_MAX 255
#define CAMIC_WDR4_LOW_STRENGHTH_MIN 0

#define CAMIC_WDR4_STRENGHTH_MAX 128
#define CAMIC_WDR4_STRENGHTH_MIN 0

typedef struct CamerIcIspWdr4Config_s {
    uint8_t               strength;			/**< strength */
    uint8_t               highStrength;		/**< highStrength */
    uint8_t               lowStrength;		/**< lowStrength */
    uint8_t               globalStrength;	/**< globalStrength */
    int32_t                   contrast;         /**< contrast */
} CamerIcIspWdr4Config_t;


/*****************************************************************************/
/**
 *          CamerIcIspWdr4Enable()
 *
 * @brief   Enable WDR4 Module
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspWdr4Enable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 *          CamerIcIspWdr4Disable()
 *
 * @brief   Disable WDR4 Module
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspWdr4Disable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
*          CamerIcIspWdr4Config()
*
* @brief   Config WDR4 Module
*
* @param   handle          CamerIc driver handle
* @param   pWdr4Cfg        set wdr4 parameter
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4Config
(
	CamerIcDrvHandle_t 		handle,
    const CamerIcIspWdr4Config_t  *pWdr4Cfg
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4GetConfig()
*
* @brief   Config WDR4 Module
*
* @param   handle          CamerIc driver handle
* @param   pWdr4Cfg        pointer to wdr4 parameter
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4GetConfig
(
	CamerIcDrvHandle_t 		handle,
    CamerIcIspWdr4Config_t  *pWdr4Cfg
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4SetFlat()
*
* @brief   set wdr4 flat
*
* @param   handle          CamerIc driver handle
* @param   strength		   set wdr4 flat strength
* @param   threshold       set wdr4 flat threshold
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4SetFlat
(
	CamerIcDrvHandle_t handle,
    uint8_t strength,
    uint8_t threshold
);


/*****************************************************************************/
/**
*          CamerIcIspWdr4GetFlat()
*
* @brief   set wdr4 flat
*
* @param   handle          CamerIc driver handle
* @param   strength		   pointer to wdr4 flat strength
* @param   threshold       pointer to wdr4 flat threshold
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4GetFlat
(
	CamerIcDrvHandle_t handle,
    uint8_t *strength,
    uint8_t *threshold
);


/*****************************************************************************/
/**
*          CamerIcIspWdr4IsEnable()
*
* @brief   get wdr4  enable status
*
* @param   handle          CamerIc driver handle
* @param   pEnabled        pointer to the wdr4 enable
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4IsEnable
(
	CamerIcDrvHandle_t handle,
    bool *pEnabled
);

#define WDR4_ENTROY_CURVE_BIN 			20
#define WDR4_GAMMAPRE_CURVE_BIN 		20
#define WDR4_GAMMAUP_CURVE_BIN 			20
#define WDR4_GAMMADOWN_CURVE_BIN 		20

typedef struct CamerIcIspWdr4Entroy_s
{
    uint32_t  curve[WDR4_ENTROY_CURVE_BIN];
} CamerIcIspWdr4Entroy_t;

typedef struct CamerIcIspWdr4GammaPre_s
{
    uint32_t  curve[WDR4_GAMMAPRE_CURVE_BIN];
} CamerIcIspWdr4GammaPre_t;

typedef struct CamerIcIspWdr4GammaUp_s
{
    uint32_t  curve[WDR4_GAMMAUP_CURVE_BIN];
} CamerIcIspWdr4GammaUp_t;

typedef struct CamerIcIspWdr4GammaDown_s
{
    uint32_t  curve[WDR4_GAMMADOWN_CURVE_BIN];
} CamerIcIspWdr4GammaDown_t;


/*****************************************************************************/
/**
*          CamerIcIspWdr4SetEntropy()
*
* @brief   set wdr4  entropy
*
* @param   handle          CamerIc driver handle
* @param   entropy         pointer to entropy
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4SetEntropy
(
	CamerIcDrvHandle_t handle,
    CamerIcIspWdr4Entroy_t *pEntropy
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4GetEntropy()
*
* @brief   get wdr4  entropy
*
* @param   handle          CamerIc driver handle
* @param   entropy         pointer to entropy
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4GetEntropy
(
	CamerIcDrvHandle_t handle,
    CamerIcIspWdr4Entroy_t *pEntropy
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4SetGammaPre()
*
* @brief   set wdr4  gamma pre
*
* @param   handle          CamerIc driver handle
* @param   gammapre        pointer to gammapre
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4SetGammaPre
(
	CamerIcDrvHandle_t handle,
    CamerIcIspWdr4GammaPre_t *gammapre
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4GetGammaPre()
*
* @brief   get wdr4  gamma pre
*
* @param   handle          CamerIc driver handle
* @param   gammapre        pointer to gammapre
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4GetGammaPre
(
	CamerIcDrvHandle_t handle,
    CamerIcIspWdr4GammaPre_t *gammapre
);


/*****************************************************************************/
/**
*          CamerIcIspWdr4SetGammaUp()
*
* @brief   set wdr4  gamma up
*
* @param   handle          CamerIc driver handle
* @param   gammaup         pointer to gammaup
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4SetGammaUp
(
	CamerIcDrvHandle_t handle,
    CamerIcIspWdr4GammaUp_t *gammaup
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4GetGammaUp()
*
* @brief   get wdr4  gamma up
*
* @param   handle          CamerIc driver handle
* @param   gammaup         pointer to gammaup
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4GetGammaUp
(
	CamerIcDrvHandle_t handle,
    CamerIcIspWdr4GammaUp_t *gammaup
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4SetGammaDown()
*
* @brief   set wdr4  gamma down
*
* @param   handle          CamerIc driver handle
* @param   gammadown       pointer to gammadown
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4SetGammaDown
(
	CamerIcDrvHandle_t handle,
    CamerIcIspWdr4GammaDown_t *gammadown
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4GetGammaDown()
*
* @brief   get wdr4  gamma down
*
* @param   handle          CamerIc driver handle
* @param   gammadown       pointer to gammadown
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4GetGammaDown
(
	CamerIcDrvHandle_t handle,
    CamerIcIspWdr4GammaDown_t *gammadown
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4SetEntropySlope()
*
* @brief   set wdr4  entropy slope
*
* @param   handle          CamerIc driver handle
* @param   slope           set wdr4 entropy slope
* @param   base            set wdr4 entropy base
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4SetEntropySlope
(
	CamerIcDrvHandle_t handle,
    uint32_t slope,
    uint32_t base
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4GetEntropySlope()
*
* @brief   get wdr4  entropy slope
*
* @param   handle          CamerIc driver handle
* @param   slope           pointer to wdr4 entropy slope
* @param   base            pointer to wdr4 entropy base
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4GetEntropySlope
(
	CamerIcDrvHandle_t handle,
    uint32_t *slope,
    uint32_t *base
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4SetRatio()
*
* @brief   set wdr4  ratio
*
* @param   handle          CamerIc driver handle
* @param   ratio           set wdr4 ratio
* @param   ratioLSVS       set wdr4 ratioLSVS
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4SetRatio
(
	CamerIcDrvHandle_t 		handle,
    uint32_t ratio,
    uint32_t ratioLSVS
);

/*****************************************************************************/
/**
*          CamerIcIspWdr4GetRatio()
*
* @brief   get wdr4  ratio
*
* @param   handle          CamerIc driver handle
* @param   ratio           pointer to wdr4 ratio
* @param   ratioLSVS       pointer to wdr4 ratioLSVS
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspWdr4GetRatio
(
	CamerIcDrvHandle_t 		handle,
    uint32_t *ratio,
    uint32_t *ratioLSVS
);



#ifdef __cplusplus
}
#endif



#endif /* CAMERIC_ISP_WDR4_DRV_API_H */


