#ifndef __HAL_VICAP_H__
#define __HAL_VICAP_H__

#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef void* HalVicapHandle_t;

typedef struct VicapImageRect_s{
    uint32_t left;
    uint32_t top;
    uint32_t width;
    uint32_t height;
} VicapImageRect_t;

typedef enum _vicap_data_type {
    VICAP_DATA_TYPE_RAW_8BIT = 0,
    VICAP_DATA_TYPE_RAW_10BIT,
    VICAP_DATA_TYPE_RAW_12BIT,
    VICAP_DATA_TYPE_RAW_14BIT,
    VICAP_DATA_TYPE_RAW_16BIT,
    VICAP_DATA_TYPE_BUTT
} VicapDataType_t;

typedef enum VicapLaneId_e {
    VICAP_LANE_0       = 0,
    VICAP_LANE_1,
    VICAP_LANE_2,
    VICAP_LANE_3,
    VICAP_LANE_4,
    VICAP_LANE_5,
    VICAP_LANE_6,
    VICAP_LANE_7,
    VICAP_LANE_MAX
} VicapLaneId_t;

typedef enum VicapComboMode_e {
    MODE_TYPE_HISPI = 0,
    MODE_TYPE_LVDS,
    MODE_TYPE_BUTT,
} VicapComboMode_t;

typedef struct HalVicapConfig_s{
    VicapImageRect_t     imageRect; /* Rx device area */
    VicapDataType_t      dataType;
    VicapComboMode_t     comboMode;
    bool_t  LaneEnable[VICAP_LANE_MAX];
} HalVicapConfig_t;

typedef struct HalVicapContext_s {
    uint32_t            refCount;               //!< internal ref count
    uint8_t             devId;
    int32_t             fd;
    osMutex             refMutex;               //!< common short term mutex; e.g. for read-modify-write accesses
} HalVicapContext_t;


HalVicapHandle_t HalVicapDrvOpen(uint8_t vicapId);
RESULT HalVicapDrvClose(HalVicapHandle_t vicapHalHandle);
RESULT HalVicapDrvSystemInit(HalVicapHandle_t vicapHalHandle);
RESULT HalVicapSetConfig(HalVicapHandle_t vicapHalHandle, HalVicapConfig_t *vicapConfig);
RESULT HalVicapDrvStreamOn(HalVicapHandle_t vicapHalHandle);
RESULT HalVicapDrvStreamOff(HalVicapHandle_t vicapHalHandle);

#ifdef __cplusplus
}
#endif

#endif /* __HAL_VICAP_H__ */
