/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef ISPCMDBUFFERAPI_H
#define ISPCMDBUFFERAPI_H
// #include <binder/binder_hal_api.h>
#include <common/return_codes.h>
// #include <oslayer/oslayer.h>
#include <ebase/types.h>

#define ISP_CMDBUFFER_FULL_BUFFER_BYTE_MAX      (0x8000U) /* non-ctrl + ctrl + special registers */
#define ISP_CMDBUFFER_REG_OFFSET_BYTE_MAX       (0x8000U)
#define ISP_CMDBUFFER_REG_SIZE                  (4U) /* Byte */
#define ISP_CMDBUFFER_FULL_BUFFER_NUM           (ISP_CMDBUFFER_FULL_BUFFER_BYTE_MAX/ISP_CMDBUFFER_REG_SIZE)
#define ISP_CMDBUFFER_REG_NUM                   (ISP_CMDBUFFER_REG_OFFSET_BYTE_MAX/ISP_CMDBUFFER_REG_SIZE)
#define ISP_CMDBUFFER_REG_PART_REFRESH_NUM      (0x200U)
#define ISP_CMDBUFFER_REG_STL_REFRESH_NUM       (0x2000U)
#define ISP_CMDBUFFER_FIXED_OFFSET_READ_NUM     (256U)
#define ISP_CMDBUFFER_STATUS_REGS_MAX           (14U)
#define ISP_CMDBUFFER_TBL_REG_MAX               (24U+18U+3U+2U+8U+1U) // 2 gamma febe curve + 7 cnr curve + 1 yuv curve+ 1 dpcc curve
#define ISP_CMDBUFFER_VIRT_MAXCNT               (16U)
#define ISP_CMDBUFFER_SPECIAL_RGE_NUM           (2U)
#define ISP_CMDBUFFER_SPECIAL_RGE_INDEX         (ISP_CMDBUFFER_FULL_BUFFER_NUM - ISP_CMDBUFFER_SPECIAL_RGE_NUM)

/* Batch mode part */
#define ISP_CMDBUFFER_BATCH_FRAME_COUNT_MAX     (32U)
#define ISP_CMDBUFFER_BATCH_FRAME_COUNT_MIN     (1U)
#define ISP_CMDBUFFER_BATCH_WR_CMD_NUM          (ARRAY_SIZE(batchRegs)) /* write-command number of one frame(2~32) in batch mode */

/* RDCD part */
#define ISP_CMDBUFFER_RDCD_REG_NUM               ((uint32_t)ARRAY_SIZE(ispCmdBufferRdcdDevReg))
#define ISP_CMDBUFFER_RDCD_REG_BYTE              (ISP_CMDBUFFER_RDCD_REG_NUM * ISP_CMDBUFFER_REG_SIZE)
#define ISP_CMDBUFFER_RDCD_REG_NUM_MAX           (ISP_CMDBUFFER_RDCD_REG_NUM + (uint32_t)ARRAY_SIZE(rdcdCtrlRegs))
#define ISP_CMDBUFFER_RDCD_REG_NUM_MAX_BYTE      (ISP_CMDBUFFER_RDCD_REG_NUM_MAX * ISP_CMDBUFFER_REG_SIZE)


/*****************************************************************************/
/**
 * @brief   Handle to a ISP CMD Buffer instance.
 *
 *****************************************************************************/
typedef struct IspCmdBufferContext_s *IspCmdBufferHandle_t;

typedef struct IspCmdBufferReg_s {
    uint32_t isp_ctrl;	/*register offset */
    uint32_t isp_tpg_ctrl;	/*register offset */
    uint32_t isp_mis;	/*register offset */
    uint32_t isp_icr;	/*register offset */

    uint32_t mi_ctrl;	/*register offset */
    uint32_t mi_mis;	/*register offset */
    uint32_t mi_mis1;	/*register offset */
    uint32_t mi_mis2;	/*register offset */
    uint32_t mi_mis3;	/*register offset */
    uint32_t mi_icr;	/*register offset */
    uint32_t mi_icr1;	/*register offset */
    uint32_t mi_icr2;	/*register offset */
    uint32_t mi_icr3;	/*register offset */
    uint32_t mi_imsc3;	/*register offset */
    uint32_t mi_mcm_bus_id;	/*register offset */
    uint32_t mi_init;	/*register offset */
    uint32_t mi_hdr_ctrl;	/*register offset */

    uint32_t isp_hist;	/*register offset */
    uint32_t isp_stitching_ctrl;	/*register offset */
    uint32_t isp_stitching_mis;	/*register offset */
    uint32_t isp_stitching_icr;	/*register offset */
    uint32_t isp_stitching_ctrl1;	/*register offset */
    uint32_t isp_stitching_ctrl2;	/*register offset */
    uint32_t isp_stitching_ctrl3;	/*register offset */
    uint32_t mipi_mis;	/*register offset */
    uint32_t mipi_icr;	/*register offset */

    uint32_t denoise3d_ctrl[3];

    uint32_t fe_ctrl;	/*register offset */
    uint32_t fe_dma_start;	/*register offset */
    uint32_t fe_dma_ad;	/*register offset */
    uint32_t fe_imsc;	/*register offset */
    uint32_t fe_mis;	/*register offset */
    uint32_t fe_icr;	/*register offset */

    uint32_t mi_init_mask;
    uint32_t mi_ctrl_mask;
    uint32_t mi_imsc3_mask;
    uint32_t isp_ctrl_mask[4];	/*register mask bit */
    uint32_t stitching_ctrl_mask[3];
    uint32_t isp_tpg_ctrl_mask;	/*register mask bit */
    uint32_t mi_mcm_bus_id_mask[4];
    uint32_t mi_mcm_bus_id_shift[3];
    uint32_t mi_hdr_ctrl_mask[2];
    uint32_t denoise3d_ctrl_mask[4];
    uint32_t isp_wo_bits_mask[4];
}IspCmdBufferReg_t;

typedef struct IspCmdBufferTbl_s {
    uint32_t tblRegOffset;
    uint16_t paramsNum;
}IspCmdBufferTbl_t;

typedef enum IspCmdBufferWorkMode_e{
    ISP_CMDBUFFER_RDMA_MODE,
    ISP_CMDBUFFER_STREAM_MODE,
    ISP_CMDBUFFER_BATCH_MODE,
    ISP_CMDBUFFER_RDCD_MODE
}IspCmdBufferWorkMode_t;

typedef struct IspCmdBufferStatusRegs_s {
    uint32_t regBase;
    uint32_t regNum;
    uint32_t regStep;  //4/8/12/...
}IspCmdBufferStatusRegs_t;

typedef struct IspCmdBufferParams_s {
    //CMD Buffer Init the table
    IspCmdBufferReg_t generalCtrl;
    uint8_t tblRegAddrNum;   /*number of register address for LUT */
    uint16_t tblTotalParamsNum;  /*the total number of params for LUT */
    IspCmdBufferTbl_t tbl[ISP_CMDBUFFER_TBL_REG_MAX];
    uint8_t statusRegsNum;
    IspCmdBufferStatusRegs_t statusRegs[ISP_CMDBUFFER_STATUS_REGS_MAX];
    uint8_t batchFrameCount;      /*frame count of a batch in batch mode */
}IspCmdBufferParams_t;

typedef struct IspCmdBufferConfig_s{
    IspCmdBufferHandle_t     hIspCmdBuffer;    /**< Handle to CmdBuffer context, set by @ref SwitchInit if successful, undefined otherwise. */
    void *                   hHalHandle;        /**< Handle to Hal context. */
    void *                   hIspHalHandle;
    uint8_t                  VID;               /**< Viture Pipeline ID. */
    IspCmdBufferWorkMode_t   ispCmdBufWorkMode;    /**< CMD Buffer Work Mode.*/
    IspCmdBufferParams_t     IspCmdBufferParams;
} IspCmdBufferConfig_t;

typedef struct IspCmdRunBuffer_s{
    uint32_t cmdNumMax;
    uint32_t cmdDmaAddr;
    void *regCmdBuffer;
} IspCmdRunBuffer_t;

RESULT IspCmdBufferInit(IspCmdBufferConfig_t *pConfig);
RESULT IspCmdBufferDeinit(IspCmdBufferHandle_t hIspCmdBuffer);
RESULT IspCmdBufferReadReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t *val);
RESULT IspCmdBufferWriteReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t val);
RESULT IspCmdBufferReadTbl(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t *val);
RESULT IspCmdBufferWriteTbl(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t val);

RESULT IspCmdBufferGetRunBuffer(IspCmdBufferHandle_t hIspCmdBuffer,IspCmdRunBuffer_t *pCmdRunBuffer, bool pingpongFlag);
RESULT IspCmdBufferReadRunReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t *val);
RESULT IspCmdBufferWriteRunReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t val);

RESULT IspCmdBufferWriteBackStatus(IspCmdBufferHandle_t hIspCmdBuffer);

//debug used
void IspCmdBufferRegDump(IspCmdBufferHandle_t hIspCmdBuffer);
void IspCmdBufferRunRegDump(IspCmdBufferHandle_t hIspCmdBuffer);

RESULT IspCmdBufferBatchWriteReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t *pVal);
RESULT IspCmdBufferBatchWriteRunReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t *pVal);
RESULT IspCmdBufferBatchSyncCtrlReg(IspCmdBufferHandle_t hIspCmdBuffer);
/* RDCD part */
RESULT IspCmdBufferRdcdWriteReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t val);
RESULT IspCmdBufferRdcdReadReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t *val);
RESULT IspCmdBufferRdcdWriteRunReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t val);
RESULT IspCmdBufferRdcdReadRunReg(IspCmdBufferHandle_t hIspCmdBuffer, uint32_t offset, uint32_t *val);
RESULT IspCmdBufferRdcdSyncCtrlReg(IspCmdBufferHandle_t hIspCmdBuffer);

// //RESULT IspCmdBufferSetParams(IspCmdBufferHandle_t hIspCmdBuffer, IspCmdBufferParams_t IspCmdBufferParams);
// RESULT IspCmdBufferReset(IspCmdBufferHandle_t hIspCmdBuffer);
#endif /*_ISPCMDBUFFERDEV_H_*/
