/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

#ifndef ISP_DRV_CMD_H
#define ISP_DRV_CMD_H
#include "isp_ic_dev.h"

#define ISP_DRV_OPT_SHIFT    0U
#define ISP_DRV_OPT_MASK     0xffU

#define ISP_DRV_MODULE_SHIFT 8U
#define ISP_DRV_MODULE_MASK  0xff00U

#define ISP_DRV_OPT_COUNT(module, num) (((u32)module << ISP_DRV_MODULE_SHIFT) | (u32)num)

enum {
    ISP_DRV_INVALID = -1,
    ISP_DRV_BASE,
    ISP_DRV_TPG,
    ISP_DRV_2DNR,
    ISP_DRV_3DNR,
    ISP_DRV_AFMV1,
    ISP_DRV_AFMV3,
    ISP_DRV_CNR,
    ISP_DRV_CPD,
    ISP_DRV_COLORING,
    ISP_DRV_DEMOSAIC,
    ISP_DRV_DG,
    ISP_DRV_DPF,
    ISP_DRV_EE,
    ISP_DRV_EXPV1,
    ISP_DRV_EXPV2,
    ISP_DRV_EXPV3,
    ISP_DRV_FEBE,
    ISP_DRV_GC,
    ISP_DRV_GTM,
    ISP_DRV_GWDR,
    ISP_DRV_HDR,
    ISP_DRV_LSC,
    ISP_DRV_3DLUT,
    ISP_DRV_MI,
    ISP_DRV_MPCTL,
    ISP_DRV_PDAF,
    ISP_DRV_RAW_CROP,
    ISP_DRV_RDCD,
    ISP_DRV_RGBIR,
    ISP_DRV_AWB,
    ISP_DRV_WB,
    ISP_DRV_WDR,
    ISP_DRV_YNR,
    ISP_DRV_BLS,
    ISP_DRV_IS,
    ISP_DRV_CSM,
    ISP_DRV_CCM,
    ISP_DRV_HDREXP,
    ISP_DRV_HIST256,
    ISP_DRV_HDRHIST,
    ISP_DRV_OPT_COUNT,
    ISP_DRV_HIST64,
    ISP_DRV_DPCC,
    ISP_DRV_DMSC,
    ISP_DRV_DEGAMMA,
    ISP_DRV_VSM,
    ISP_DRV_CPROC,
    ISP_DRV_GCMONO,
    ISP_DRV_GE,
    ISP_DRV_CROP,
    ISP_DRV_STA,
    ISP_DRV_SBIMI,
    ISP_DRV_FUSA,
    ISP_DRV_MSS,
    ISP_DRV_MI_HDR,
    ISP_DRV_MI_RDMA,
    ISP_DRV_MODULE_MAX
};

enum {
    ISPDRV_RESET                = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 0),
    ISPDRV_WRITE_REG            = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 1),
    ISPDRV_READ_REG             = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 2),
    ISPDRV_S_INPUT              = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 3),
    ISPDRV_ENABLE               = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 4),
    ISPDRV_DISABLE              = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 5),
    ISPDRV_ISP_STATUS           = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 6),
    ISPDRV_ISP_STOP             = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 7),
    ISPDRV_START_CAPTURE        = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 8),
    ISPDRV_DISABLE_ISP_OFF      = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 9),
    ISPDRV_S_MUX                = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 10),
    ISPDRV_S_SIMP               = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 11),
    ISPDRV_G_MIS_IRQADDR        = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 12),
    ISPDRV_S_REG_SIZE           = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 13),
    ISPDRV_S_NR_RELOC           = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 14),
    ISPDRV_S_DPF_RAW_OUTPUT     = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 15),
    ISPDRV_3A_STATISTIC_SEL     = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 16),
    ISPDRV_FAST_START           = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 17),
    ISPDRV_SYSTEM_INIT          = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 18),
    ISPDRV_STREAM_PATH_ENABLE   = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 19),
    ISPDRV_STREAM_PATH_DISABLE  = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 20),
    ISPDRV_S_CLOCK_GATING       = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 21),
    ISPDRV_MI_SAFETY_RESET      = ISP_DRV_OPT_COUNT(ISP_DRV_BASE, 22),
};

enum {
    ISPDRV_SET_BUFFER           = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 0),
    ISPDRV_SET_BP_BUFFER        = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 1),
    ISPDRV_START_DMA_READ       = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 2),
    ISPDRV_MI_START             = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 3),
    ISPDRV_MI_STOP              = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 4),
    ISPDRV_CFG_DMA              = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 5),
    ISPDRV_SET_PPW_LINE_NUM     = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 6),
    ISPDRV_GET_PPW_LINE_CNT     = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 7),
    ISPDRV_MI_CFG               = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 8),
    ISPDRV_S_PP_PATH            = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 9),
    ISPDRV_MI_PATH_ENABLE       = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 10),
    ISPDRV_MI_PATH_DISABLE      = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 11),
    ISPDRV_MI_CFG_UPDATE        = ISP_DRV_OPT_COUNT(ISP_DRV_MI, 12),
};

enum {
    ISPDRV_I_MCM                = ISP_DRV_OPT_COUNT(ISP_DRV_MI_RDMA, 0),  //init mcm
    ISPDRV_CFG_MCM_RDMA         = ISP_DRV_OPT_COUNT(ISP_DRV_MI_RDMA, 1),
    ISPDRV_START_MCM_RDMA       = ISP_DRV_OPT_COUNT(ISP_DRV_MI_RDMA, 2),
};

enum {
    ISPDRV_ENABLE_SENSOR0_FRAME_CNT = ISP_DRV_OPT_COUNT(ISP_DRV_MI_HDR, 0),
    ISPDRV_S_HDR_PPC                = ISP_DRV_OPT_COUNT(ISP_DRV_MI_HDR, 1),
};

enum {
    ISPDRV_ENABLE_TPG           = ISP_DRV_OPT_COUNT(ISP_DRV_TPG, 0),
    ISPDRV_DISABLE_TPG          = ISP_DRV_OPT_COUNT(ISP_DRV_TPG, 1),
    ISPDRV_S_TPG                = ISP_DRV_OPT_COUNT(ISP_DRV_TPG, 2),
};

enum {
    ISPDRV_ENABLE_BLS           = ISP_DRV_OPT_COUNT(ISP_DRV_BLS, 0),
    ISPDRV_DISABLE_BLS          = ISP_DRV_OPT_COUNT(ISP_DRV_BLS, 1),
    ISPDRV_S_BLS                = ISP_DRV_OPT_COUNT(ISP_DRV_BLS, 2),
};

enum {
    ISPDRV_ENABLE_AWB           = ISP_DRV_OPT_COUNT(ISP_DRV_AWB, 0),
    ISPDRV_DISABLE_AWB          = ISP_DRV_OPT_COUNT(ISP_DRV_AWB, 1),
    ISPDRV_S_AWB                = ISP_DRV_OPT_COUNT(ISP_DRV_AWB, 2),
    ISPDRV_G_AWBMEAN            = ISP_DRV_OPT_COUNT(ISP_DRV_AWB, 3),
    ISPDRV_S_ELAWB              = ISP_DRV_OPT_COUNT(ISP_DRV_AWB, 4),
    ISPDRV_ENABLE_WB            = ISP_DRV_OPT_COUNT(ISP_DRV_AWB, 5),
    ISPDRV_DISABLE_WB           = ISP_DRV_OPT_COUNT(ISP_DRV_AWB, 6),
};

enum {
    ISPDRV_S_IS                 = ISP_DRV_OPT_COUNT(ISP_DRV_IS, 0),
    ISPDRV_S_RAW_IS             = ISP_DRV_OPT_COUNT(ISP_DRV_IS, 1),
};

enum {
    ISPDRV_S_CC                 = ISP_DRV_OPT_COUNT(ISP_DRV_CSM, 0),
};

enum {
    ISPDRV_S_XTALK              = ISP_DRV_OPT_COUNT(ISP_DRV_CCM, 0),
};

enum {
    ISPDRV_S_GAMMA_OUT_CURVE      = ISP_DRV_OPT_COUNT(ISP_DRV_GC, 0),
    ISPDRV_ENABLE_GAMMA_OUT       = ISP_DRV_OPT_COUNT(ISP_DRV_GC, 1),
    ISPDRV_S_GAMMA_OUT_CURVE_MODE = ISP_DRV_OPT_COUNT(ISP_DRV_GC, 2),
    ISPDRV_ENABLE_RGBGAMMA        = ISP_DRV_OPT_COUNT(ISP_DRV_GC, 3),
    ISPDRV_DISABLE_RGBGAMMA       = ISP_DRV_OPT_COUNT(ISP_DRV_GC, 4),
    ISPDRV_S_RGBGAMMA             = ISP_DRV_OPT_COUNT(ISP_DRV_GC, 5),
};

enum {
    ISPDRV_ENABLE_LSC           = ISP_DRV_OPT_COUNT(ISP_DRV_LSC, 0),
    ISPDRV_DISABLE_LSC          = ISP_DRV_OPT_COUNT(ISP_DRV_LSC, 1),
    ISPDRV_S_LSC_TBL            = ISP_DRV_OPT_COUNT(ISP_DRV_LSC, 2),
    ISPDRV_S_LSC_SEC            = ISP_DRV_OPT_COUNT(ISP_DRV_LSC, 3),
};

enum {
    ISPDRV_S_DPF                = ISP_DRV_OPT_COUNT(ISP_DRV_DPF, 0),
};

enum {
    ISPDRV_S_EE                 = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 0),
    ISPDRV_S_CURVE              = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 1),
    ISPDRV_S_EE3                = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 2),
    ISPDRV_S_EE3_EDGE_DETAIL    = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 3),
    ISPDRV_S_EE3_SKIN           = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 4),
    ISPDRV_S_EE3_MERGE_CURVE    = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 5),
    ISPDRV_S_EE3_NOISE          = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 6),
    ISPDRV_S_EE3_ENHANCE        = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 7),
    ISPDRV_S_EE3_DEPURPLE       = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 8),
    ISPDRV_S_CA_CURVE           = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 9),  
    ISPDRV_S_DCI_CURVE          = ISP_DRV_OPT_COUNT(ISP_DRV_EE, 10),
};


enum {
    ISPDRV_S_EXP                = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV1, 0),
    ISPDRV_G_EXPMEAN            = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV1, 1),
};

enum {
    ISPDRV_S_EXP2               = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV2, 0),
    ISPDRV_S_EXP2_INPUTSEL      = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV2, 1),
    ISPDRV_S_EXP2_SIZERATIO     = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV2, 2),
    ISPDRV_G_AEV2_STA           = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV2, 3), //only get aev2 statistics in binder mode.
};

#ifdef ISP_EXP_V3
enum {

    ISPDRV_ENABLE_EXPV3         = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV3, 0),
    ISPDRV_EXPV3_INPUT_SELECT   = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV3, 1),
    ISPDRV_EXPV3_MEASURE_MODE   = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV3, 2),
    ISPDRV_EXPV3_SET_WINDOWS    = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV3, 3),
    ISPDRV_GET_EXPV3MEAN        = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV3, 4),
    ISPDRV_EXPV3_HW_INIT        = ISP_DRV_OPT_COUNT(ISP_DRV_EXPV3, 5),
};
#endif

enum {
    ISPDRV_S_HDREXP             = ISP_DRV_OPT_COUNT(ISP_DRV_HDREXP, 0),
    ISPDRV_G_HDREXPMEAN         = ISP_DRV_OPT_COUNT(ISP_DRV_HDREXP, 1),
};

enum {
    ISPDRV_S_HIST               = ISP_DRV_OPT_COUNT(ISP_DRV_HIST256, 0),
    ISPDRV_G_HISTMEAN           = ISP_DRV_OPT_COUNT(ISP_DRV_HIST256, 1),
};

enum {
    ISPDRV_S_HDRHIST            = ISP_DRV_OPT_COUNT(ISP_DRV_HDRHIST, 0),
    ISPDRV_G_HDRHISTMEAN        = ISP_DRV_OPT_COUNT(ISP_DRV_HDRHIST, 1),
};

enum {
    ISPDRV_S_HIST64             = ISP_DRV_OPT_COUNT(ISP_DRV_HIST64, 0),
    ISPDRV_G_HIST64MEAN         = ISP_DRV_OPT_COUNT(ISP_DRV_HIST64, 1),
    ISPDRV_G_HIST64VSTART_STATUS= ISP_DRV_OPT_COUNT(ISP_DRV_HIST64, 2),
    ISPDRV_U_HIST64             = ISP_DRV_OPT_COUNT(ISP_DRV_HIST64, 3),
};

enum {
    ISPDRV_S_DPCC               = ISP_DRV_OPT_COUNT(ISP_DRV_DPCC, 0),
    ISPDRV_C_DPCC_BPT           = ISP_DRV_OPT_COUNT(ISP_DRV_DPCC, 1),
};

enum {
    ISPDRV_S_DMSC_FLT           = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 0),
    ISPDRV_S_DMSC               = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 1),
    ISPDRV_S_DMSC_INTP          = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 2),
    ISPDRV_S_DMSC_DMOI          = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 3),
    ISPDRV_S_DMSC_SKIN          = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 4),
    ISPDRV_S_DMSC_CAC           = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 5),
    ISPDRV_S_DMSC_SHAP          = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 6),
    ISPDRV_S_DMSC_SHAP_LINE     = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 7),
    ISPDRV_S_DMSC_DEPURPLE      = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 8),
    ISPDRV_S_DMSC_GFILTER       = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 9),
    ISPDRV_S_DMSC3_INTP         = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 10),
    ISPDRV_S_DMSC3_INTERPCORNER = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 11),
    ISPDRV_S_DMSC3_DECLINE      = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 12),
    ISPDRV_S_DMSC3_DMOI         = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 13),
    ISPDRV_S_DMSC3_DEFALSE      = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 14),
    ISPDRV_S_DMSC3_CAC          = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 15),
    ISPDRV_S_DMSC3_SHAP         = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 16),
    ISPDRV_S_DMSC3_DEPURPLE     = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 17),
    ISPDRV_S_DMSC3_ENABLE       = ISP_DRV_OPT_COUNT(ISP_DRV_DMSC, 18),
};

enum {
    ISPDRV_S_DEG                = ISP_DRV_OPT_COUNT(ISP_DRV_DEGAMMA, 0),
};

enum {
    ISPDRV_S_AFM                = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV1, 0),
    ISPDRV_G_AFM                = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV1, 1),
};

#ifdef ISP_AFM_V3
enum {

    ISPDRV_AFMV3_ENABLE         = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 0),
    ISPDRV_AFMV3_GAMMA_ENABLE   = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 1),
    ISPDRV_AFMV3_S_HIST_THRES   = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 2),
    ISPDRV_AFMV3_S_BPT          = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 3),
    ISPDRV_AFMV3_S_BLS          = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 4),
    ISPDRV_AFMV3_S_BLOCK        = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 5),
    ISPDRV_AFMV3_S_IIR          = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 6),
    ISPDRV_AFMV3_S_FIR          = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 7),
    ISPDRV_AFMV3_S_GAMMA        = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 8),
    ISPDRV_AFMV3_S_LDG          = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 9),
    ISPDRV_AFMV3_S_CORING       = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 10),
    ISPDRV_AFMV3_G_STATISTIC    = ISP_DRV_OPT_COUNT(ISP_DRV_AFMV3, 11),
};
#endif

enum {
    ISPDRV_S_VSM                = ISP_DRV_OPT_COUNT(ISP_DRV_VSM, 0),
    ISPDRV_G_VSM                = ISP_DRV_OPT_COUNT(ISP_DRV_VSM, 1),
};

enum {
    ISPDRV_ENABLE_WDR3          = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 0),
    ISPDRV_DISABLE_WDR3         = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 1),
    ISPDRV_U_WDR3               = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 2),
    ISPDRV_S_WDR3               = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 3),
    ISPDRV_ENABLE_WDR4          = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 4),
    ISPDRV_DISABLE_WDR4         = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 5),
    ISPDRV_U_WDR4               = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 6),
    ISPDRV_S_WDR4               = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 7),
#if defined(ISP_WDR_V5) || defined(ISP_WDR_V5_1) || defined(ISP_WDR_V5_2) || defined(ISP_WDR_V5_2_1) || defined(ISP_WDR_V5_3)
    ISPDRV_G_WDR5_AVG_STD       = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 8),
    ISPDRV_ENABLE_WDR5          = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 9),
    ISPDRV_WDR5_STRENGTH        = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 10),
    ISPDRV_WDR5_CONTRAST        = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 11),
    ISPDRV_WDR5_WEIGHT          = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 12),
    ISPDRV_WDR5_BAYER_RATIO     = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 13),
    ISPDRV_WDR5_ENTROPY_SLOPE   = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 14),
    ISPDRV_WDR5_ADJUST_SLOPE    = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 15),
    ISPDRV_WDR5_LOCAL_FLAT_MODE = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 16),
    ISPDRV_WDR5_LOCAL_FLAT_LEVEL= ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 17),
    ISPDRV_WDR5_GLOBAL_CURVE_MODE   = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 18),
    ISPDRV_WDR5_GLOBAL_FLAT_LEVEL   = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 19),
    ISPDRV_WDR5_GLOBAL_FIXED_WEIGHT = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 20),
    ISPDRV_WDR5_WDR5_HISTGRAM   = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 21),
    ISPDRV_WDR5_GRMMAPRE        = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 22),
    ISPDRV_WDR5_GAMMAUP         = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 23),
    ISPDRV_WDR5_MAXGAIN         = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 24),
    ISPDRV_WDR5_MINGAIN         = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 25),
    ISPDRV_WDR5_DEGAMMA         = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 26),
    ISPDRV_WDR5_GAMMADOWN       = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 27),
    ISPDRV_WDR5_WEIGHT_RAW      = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 28),
    ISPDRV_WDR5_ENTROPY         = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 29),
    ISPDRV_WDR5_LUT_SHIFT       = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 30),
    ISPDRV_WDR5_LUT_SHIFT0      = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 31),
    ISPDRV_WDR5_DISTANCE_WEIGHT = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 32),
    ISPDRV_WDR5_DIFFERENCE_WEIGH= ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 33),
    ISPDRV_WDR5_FLAT_FACTOR     = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 34),
    ISPDRV_WDR5_FLAT_LEVEL      = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 35),
    ISPDRV_WDR5_SAT_X           = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 36),
    ISPDRV_WDR5_SAT             = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 37),
    ISPDRV_WDR5_SAT_SHIFT       = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 38),
#if defined(ISP_WDR_V5_2) || defined(ISP_WDR_V5_2_1) || defined(ISP_WDR_V5_3)
    ISPDRV_S_WDR5_LUMA          = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 39),
    ISPDRV_WDR5_LIGHT_ENABLE    = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 40),
    ISPDRV_WDR5_LIGHT_SAT       = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 41),
    ISPDRV_WDR5_HALO_COLOR_THR  = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 42),
    ISPDRV_WDR5_WDR5_HLC        = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 43),
    ISPDRV_S_WDR5_DAMP          = ISP_DRV_OPT_COUNT(ISP_DRV_WDR, 44),
#endif
#endif
};

enum {
    ISPDRV_S_2DNR               = ISP_DRV_OPT_COUNT(ISP_DRV_2DNR, 0),
    ISPDRV_CFG_2DNR_LUMA_CURVE  = ISP_DRV_OPT_COUNT(ISP_DRV_2DNR, 1),
    ISPDRV_CFG_2DNR_BLEND       = ISP_DRV_OPT_COUNT(ISP_DRV_2DNR, 2),
#if defined(ISP_2DNR_V6) 
    ISPDRV_S_2DNR_LUMA_CURVE        = ISP_DRV_OPT_COUNT(ISP_DRV_2DNR, 3),
    ISPDRV_S_2DNR_LSC_COMP_CURVE    = ISP_DRV_OPT_COUNT(ISP_DRV_2DNR, 4),
    ISPDRV_S_2DNR_MOTION_FAC_CURVE  = ISP_DRV_OPT_COUNT(ISP_DRV_2DNR, 5),
#endif
};

enum {
    ISPDRV_S_3DNR                = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 0),
    ISPDRV_G_3DNR                = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 1), /* get last avg */
    ISPDRV_U_3DNR                = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 2), /* update */
    ISPDRV_R_3DNR                = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 3), /* read back 3dnr reference image. */
    ISPDRV_S_3DNR_CMP            = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 4), /*config 3dnr compress */
    ISPDRV_U_3DNR_STRENGTH       = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 5),
    ISPDRV_S_3DNR_MOT            = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 6), /*config 3dnr motion*/
    ISPDRV_S_3DNR_DLT            = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 7), /*config 3dnr delta*/
    ISPDRV_S_3DNR_RST            = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 8),
    ISPDRV_S_3DNR3               = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 9),
    ISPDRV_3DNR3_ENABLE          = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 10),
    ISPDRV_3DNR3_DISABLE        = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 11),
    ISPDRV_3DNR3_ENABLE_TDNR    = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 12),
    ISPDRV_3DNR3_DISABLE_TDNR   = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 13),
    ISPDRV_3DNR3_ENABLE_2DNR    = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 14),
    ISPDRV_3DNR3_DISABLE_2DNR   = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 15),
    ISPDRV_S_3DNR3_STRENGTH     = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 16),
    ISPDRV_U_3DNR3_NOISE        = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 17),
    ISPDRV_U_3DNR3_THR          = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 18),
    ISPDRV_S_3DNR_BUF           = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 19),  // refer and motion
    ISPDRV_R_3DNR_REFER         = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 20),
    ISPDRV_R_3DNR_MOTION        = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 21),
    ISPDRV_S_3DNR3_CURVE        = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 22),

#ifdef ISP_3DNR_V4
    ISPDRV_3DNR4_ENABLE_TDNR            = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 23),
    ISPDRV_3DNR4_DISABLE_TDNR           = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 24),
    ISPDRV_S_3DNR4_FE_CURVE             = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 25),
    ISPDRV_S_3DNR4_STRENGTH             = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 26),
    ISPDRV_U_3DNR4_NOISE                = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 27),
    ISPDRV_U_3DNR4_THR                  = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 28),
    ISPDRV_S_3DNR4_DECOMPOSE            = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 29),
    ISPDRV_S_3DNR4_LUMA_LUT             = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 30),
    ISPDRV_S_3DNR4_BOOST                = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 31),
    ISPDRV_S_3DNR4_MOTION_GAUSS_KERNEL  = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 32),
    ISPDRV_S_3DNR4_CURVE                = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 33),
    ISPDRV_S_3DNR4                      = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 34),
#endif
#if defined(ISP_3DNR_V2_4) 
    ISPDRV_S_3DNR_MOT_ROI_POS           = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 35),
    ISPDRV_S_3DNR_MOT_ROI_EN            = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 36),
    ISPDRV_G_3DNR_MOT_ROI_DIFF          = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 37),
#endif
    ISPDRV_S_3DNR_UPD_TEMP        = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 38),
    ISPDRV_S_3DNR_STR_CUR_TEMP    = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 39),
    ISPDRV_S_3DNR_RANGE_T         = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 40),
    ISPDRV_S_3DNR_TEMP_CURVE      = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 41),
    ISPDRV_S_3DNR_DILATE_EN       = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 42),
    ISPDRV_S_3DNR_TNR             = ISP_DRV_OPT_COUNT(ISP_DRV_3DNR, 43),
};

enum {
    ISPDRV_S_COMP                 = ISP_DRV_OPT_COUNT(ISP_DRV_CPD, 0),
    ISPDRV_S_COMP_BLS             = ISP_DRV_OPT_COUNT(ISP_DRV_CPD, 1),
};

enum {
    ISPDRV_S_CPROC                = ISP_DRV_OPT_COUNT(ISP_DRV_CPROC, 0),
};

enum {
    ISPDRV_ENABLE_GCMONO          = ISP_DRV_OPT_COUNT(ISP_DRV_GCMONO, 0),
    ISPDRV_DISABLE_GCMONO         = ISP_DRV_OPT_COUNT(ISP_DRV_GCMONO, 1),
    ISPDRV_S_GCMONO               = ISP_DRV_OPT_COUNT(ISP_DRV_GCMONO, 2),
};

enum {
    ISPDRV_S_GE                   = ISP_DRV_OPT_COUNT(ISP_DRV_GE, 0),
};

enum {
    ISPDRV_S_DIGITAL_GAIN         = ISP_DRV_OPT_COUNT(ISP_DRV_DG, 0),
    ISPDRV_ENABLE_DIGITAL_GAIN    = ISP_DRV_OPT_COUNT(ISP_DRV_DG, 1),
};

enum {
    ISPDRV_ENABLE_RGBIR          = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 0),
    ISPDRV_S_RGBIR               = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 1),
    ISPDRV_RGBIR_HW_INIT         = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 2),
    ISPDRV_RGBIR_S_IR_DNR        = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 3),
    ISPDRV_RGBIR_S_SHARPEN       = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 4),
    ISPDRV_RGBIR_S_DES           = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 5),
    ISPDRV_RGBIR_S_CC_MATRIX     = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 6),
    ISPDRV_RGBIR_S_DPCC          = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 7),
    ISPDRV_RGBIR_S_GAIN          = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 8),
    ISPDRV_RGBIR_S_BLS           = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 9),
    ISPDRV_RGBIR_S_IR_RAW_OUT    = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 10),
    ISPDRV_ENABLE_RCCC           = ISP_DRV_OPT_COUNT(ISP_DRV_RGBIR, 11),
};

enum {
    ISPDRV_S_CROP                = ISP_DRV_OPT_COUNT(ISP_DRV_CROP, 0),
    ISPDRV_S_CROP_BACKEND        = ISP_DRV_OPT_COUNT(ISP_DRV_CROP, 1),
#ifdef ISP_RAW_CROP
    ISPDRV_S_RAW_CROP_WIN        = ISP_DRV_OPT_COUNT(ISP_DRV_CROP, 2),
    ISPDRV_S_RAW_CROP_ENABLED    = ISP_DRV_OPT_COUNT(ISP_DRV_CROP, 3),
#endif
};

enum {
    ISPDRV_ENABLE_PDAF           = ISP_DRV_OPT_COUNT(ISP_DRV_PDAF, 0),
    ISPDRV_DISABLE_PDAF          = ISP_DRV_OPT_COUNT(ISP_DRV_PDAF, 1),
    ISPDRV_S_PDAF                = ISP_DRV_OPT_COUNT(ISP_DRV_PDAF, 2),
};

enum {
#if defined (ISP_CNR_V2) || defined (ISP_CNR_V2_1) || defined (ISP_CNR_V2_2)
    ISPDRV_CNR2_ENABLE          = ISP_DRV_OPT_COUNT(ISP_DRV_CNR, 0),
    ISPDRV_CNR2_DISABLE         = ISP_DRV_OPT_COUNT(ISP_DRV_CNR, 1),
    ISPDRV_S_CNR2_STRENGTH      = ISP_DRV_OPT_COUNT(ISP_DRV_CNR, 2),
    ISPDRV_S_CNR2_FILTER        = ISP_DRV_OPT_COUNT(ISP_DRV_CNR, 3),
    ISPDRV_S_CNR2_TBL           = ISP_DRV_OPT_COUNT(ISP_DRV_CNR, 4),
#endif
    ISPDRV_ENABLE_CNR           = ISP_DRV_OPT_COUNT(ISP_DRV_CNR, 5),
    ISPDRV_S_CNR                = ISP_DRV_OPT_COUNT(ISP_DRV_CNR, 6),
};

#if defined (ISP_COLORING_V1)
enum {
    ISPDRV_ENABLE_COLORING          = ISP_DRV_OPT_COUNT(ISP_DRV_COLORING, 0),
    ISPDRV_DISABLE_COLORING        = ISP_DRV_OPT_COUNT(ISP_DRV_COLORING, 1),
    ISPDRV_S_COLORING               = ISP_DRV_OPT_COUNT(ISP_DRV_COLORING, 2),
};
#endif

#ifdef ISP_YNR
enum {

    ISPDRV_YNR_ENABLE           = ISP_DRV_OPT_COUNT(ISP_DRV_YNR, 0),
    ISPDRV_YNR_DISABLE          = ISP_DRV_OPT_COUNT(ISP_DRV_YNR, 1),
    ISPDRV_S_YNR_STRENGTH       = ISP_DRV_OPT_COUNT(ISP_DRV_YNR, 2),
    ISPDRV_S_YNR_TBL            = ISP_DRV_OPT_COUNT(ISP_DRV_YNR, 3),
};
#endif

enum {
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V2_1) || (defined ISP_HDR_STITCH_V2_1_2DOL) || (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_2DOL)
    ISPDRV_S_HDR_COMPRESS_LUT      = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 0),
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V3)
    ISPDRV_S_HDR_DPF_L             = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 1),
    ISPDRV_S_HDR_DPF_S             = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 2),
    ISPDRV_S_HDR_DPF_VS            = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 3),
    ISPDRV_ENABLE_HDR_DPF_L        = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 4),
    ISPDRV_ENABLE_HDR_DPF_S        = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 5),
    ISPDRV_ENABLE_HDR_DPF_VS       = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 6),
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_2DOL)
    ISPDRV_S_HDR_DEGHOST           = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 7),
    ISPDRV_ENABLE_HDR_DEGHOST_LS   = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 8),
    ISPDRV_ENABLE_HDR_DEGHOST_LSVS = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 9),
#endif
#if (defined ISP_MI_HDR) || (defined ISP_HDR_STITCH_V1_3)
    ISPDRV_S_HDR_DMA_START_MODE         = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 10),
    ISPDRV_ENABLE_HDR                   = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 11),
    ISPDRV_ENABLE_HDR_DG                = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 12),
    ISPDRV_ENABLE_HDR_WB                = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 13),
    ISPDRV_S_HDR_PROPERTIES             = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 14),
    ISPDRV_S_HDR_BLS                    = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 15),
    ISPDRV_S_HDR_WB                     = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 16),
    ISPDRV_S_HDR_DG                     = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 17),
    ISPDRV_S_HDR_TRANSRANGE_LINEAR      = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 18),
    ISPDRV_S_HDR_TRANSRANGE_NONLINEAR   = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 19),
    ISPDRV_S_HDR_LS_RATIO               = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 20),
    ISPDRV_S_HDR_VSS_RATIO              = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 21),
    ISPDRV_S_HDR_EXBIT                  = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 22),
    ISPDRV_S_HDR_VS_PARAMS              = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 23),
    ISPDRV_S_HDR_L_PARAMS               = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 24),
    ISPDRV_S_HDR_COLOR_WEIGHT           = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 25),
    ISPDRV_S_HDR_BASE_FRAME             = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 26),
    ISPDRV_S_HDR_BYPASS_SELECT          = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 27),
    ISPDRV_S_HDR_COMBINE_MODE           = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 28),
    ISPDRV_S_HDR_MODE                   = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 29),
    ISPDRV_S_HDR_IMAGE_SIZE             = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 30),
    ISPDRV_S_HDR_BAYER_PATTERN          = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 31),
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V2_1) || (defined ISP_HDR_STITCH_V2_1_2DOL) || (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_1)  || (defined ISP_HDR_STITCH_V3_2DOL)
    ISPDRV_S_HDR_TRANSRANGE_REF_S        = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 32),
    ISPDRV_S_HDR_TRANSRANGE_REF_VS       = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 33),
#endif

    ISPDRV_S_HDR_TRANSRANGE_REF_LSVS    = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 34),
    ISPDRV_S_HDR_TRANSRANGE_REF_E3      = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 35),
    ISPDRV_S_HDR_VSE3_RATIO             = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 36),
    ISPDRV_S_HDR_BLS_OUT                = ISP_DRV_OPT_COUNT(ISP_DRV_HDR, 37),
};

enum {
    ISPDRV_CFG_MP_CTRL              = ISP_DRV_OPT_COUNT(ISP_DRV_MPCTL, 0),
    ISPDRV_GET_MP_CTRL_STATUS       = ISP_DRV_OPT_COUNT(ISP_DRV_MPCTL, 1),
};

enum {
    ISPDRV_S_STA                    = ISP_DRV_OPT_COUNT(ISP_DRV_STA, 0),
    ISPDRV_G_AFM_NEED               = ISP_DRV_OPT_COUNT(ISP_DRV_STA, 1), //only for tilemode afm parameter pass
};

#ifdef ISP_3D_LUT
enum {

    ISPDRV_LUT3D_ENABLE               = ISP_DRV_OPT_COUNT(ISP_DRV_3DLUT, 0),
    ISPDRV_S_LUT3D_TAL                = ISP_DRV_OPT_COUNT(ISP_DRV_3DLUT, 1),
};
#endif

enum {
    ISPDRV_S_GTM                    = ISP_DRV_OPT_COUNT(ISP_DRV_GTM, 0),
    ISPDRV_G_GTMDATA                = ISP_DRV_OPT_COUNT(ISP_DRV_GTM, 1),
    ISPDRV_S_GTM_BWCOR_BASESLOPE    = ISP_DRV_OPT_COUNT(ISP_DRV_GTM, 2),
    ISPDRV_S_GTM_WEIGHTS            = ISP_DRV_OPT_COUNT(ISP_DRV_GTM, 3),
};

enum {
    ISPDRV_CONFIG_SBI_MI            = ISP_DRV_OPT_COUNT(ISP_DRV_SBIMI, 0),
};

enum {
    ISPDRV_S_GAMMA_BE_BLS           = ISP_DRV_OPT_COUNT(ISP_DRV_FEBE, 0),
    ISPDRV_S_GAMMA_PRE_BLS          = ISP_DRV_OPT_COUNT(ISP_DRV_FEBE, 1),
    ISPDRV_S_GAMMA_FE_BE_CURVE      = ISP_DRV_OPT_COUNT(ISP_DRV_FEBE, 2),
    ISPDRV_S_GAMMA_FE_BE            = ISP_DRV_OPT_COUNT(ISP_DRV_FEBE, 3),  //enable or disable gamma fe ,be and bls
};

#ifdef ISP_FUSA_MECHANISM
enum {
    ISPDRV_ENABLE_FUSA_ECC                  = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 0),
    ISPDRV_ENABLE_FUSA_PIX_COUNT            = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 1),
    ISPDRV_ENABLE_FUSA_TIMOUT               = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 2),
    ISPDRV_ENABLE_FUSA_BIST                 = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 3),
    ISPDRV_ENABLE_FUSA_CRC                  = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 4),
    ISPDRV_ENABLE_FUSA_CRC_IN_REV           = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 5),
    ISPDRV_ENABLE_FUSA_CRC_OUT_REV          = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 6),
    ISPDRV_ENABLE_FUSA_CRC_XOR              = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 7),
    ISPDRV_ENABLE_FUSA_FAULT_INJECTION      = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 8),
    ISPDRV_S_FUSA_CRC_LEVEL                 = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 9),
    ISPDRV_S_FUSA_CRC_MP_ROI                = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 10),
    ISPDRV_S_FUSA_CRC_SP1_ROI               = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 11),
    ISPDRV_S_FUSA_CRC_SP2_ROI               = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 12),
    ISPDRV_S_FUSA_PIX_COUNT_MP              = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 13),
    ISPDRV_S_FUSA_PIX_COUNT_SP1             = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 14),
    ISPDRV_S_FUSA_PIX_COUNT_SP2             = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 15),
    ISPDRV_S_FUSA_PIX_COUNT_FILT            = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 16),
    ISPDRV_S_FUSA_BIST_CRC_REV              = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 17),
    ISPDRV_FUSA_START                       = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 18),
    ISPDRV_FUSA_STOP                        = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 19),
    ISPDRV_ENABLE_FUSA_RDCD                 = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 20),
    ISPDRV_ENABLE_FUSA_FAULT_INJECTION_RDCD = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 21),
    ISPDRV_FUSA_BIST_POWER_UP               = ISP_DRV_OPT_COUNT(ISP_DRV_FUSA, 22),
};
#endif

#ifdef ISP_GWDR
enum {
    ISPDRV_ENABLE_GWDR            = ISP_DRV_OPT_COUNT(ISP_DRV_GWDR, 0),
    ISPDRV_S_GWDR_MODE            = ISP_DRV_OPT_COUNT(ISP_DRV_GWDR, 1),
    ISPDRV_S_GWDR_OFFSET          = ISP_DRV_OPT_COUNT(ISP_DRV_GWDR, 2),
    ISPDRV_S_GWDR_CURVE           = ISP_DRV_OPT_COUNT(ISP_DRV_GWDR, 3),
};
#endif

#ifdef ISP_MSS
enum {
    ISPDRV_S_MSS_DMA             = ISP_DRV_OPT_COUNT(ISP_DRV_MSS, 0),
    ISPDRV_START_MSS_DMA_READ    = ISP_DRV_OPT_COUNT(ISP_DRV_MSS, 1),
};
#endif

enum {
    ISPDRV_I_RDCD            = ISP_DRV_OPT_COUNT(ISP_DRV_RDCD, 0),
    ISPDRV_CFG_RDCD          = ISP_DRV_OPT_COUNT(ISP_DRV_RDCD, 1),
    ISPDRV_START_RDCD        = ISP_DRV_OPT_COUNT(ISP_DRV_RDCD, 2),
};

#define  ISP_GEN_CFG_UPDATE(dev)	{                                \
    uint32_t isp_ctrl = isp_read_reg(dev, REG_ADDR(isp_ctrl));       \
    REG_SET_SLICE(isp_ctrl, MRV_ISP_ISP_GEN_CFG_UPD, 1U);			\
    isp_write_reg(dev, REG_ADDR(isp_ctrl), isp_ctrl);               \
    }

USE_TRACER(ISP_DRV_INFO);
USE_TRACER(ISP_DRV_WARN);
USE_TRACER(ISP_DRV_ERROR);

#ifdef ISP_FUSA_MECHANISM
int32_t isp_fusa_enable_ecc(struct isp_ic_dev *dev);
int32_t isp_fusa_enable_pix_cnt(struct isp_ic_dev *dev);
int32_t isp_fusa_enable_timeout(struct isp_ic_dev *dev);
int32_t isp_fusa_enable_bist(struct isp_ic_dev *dev);
int32_t isp_fusa_bist_power_up(struct isp_ic_dev *dev);
int32_t isp_fusa_enable_crcEn(struct isp_ic_dev *dev);
int32_t isp_fusa_enable_crcInEn(struct isp_ic_dev *dev);
int32_t isp_fusa_enable_crcOutEn(struct isp_ic_dev *dev);
int32_t isp_fusa_enable_crcXorEn(struct isp_ic_dev *dev);
int32_t isp_fusa_enable_faultInjectEn(struct isp_ic_dev *dev);
int32_t isp_fusa_crc_level(struct isp_ic_dev *dev);
int32_t isp_fusa_crc_mp_roi(struct isp_ic_dev *dev);
int32_t isp_fusa_crc_sp1_roi(struct isp_ic_dev *dev);
int32_t isp_fusa_crc_sp2_roi(struct isp_ic_dev *dev);
int32_t isp_fusa_pix_cnt_mp(struct isp_ic_dev *dev);
int32_t isp_fusa_pix_cnt_sp1(struct isp_ic_dev *dev);
int32_t isp_fusa_pix_cnt_sp2(struct isp_ic_dev *dev);
int32_t isp_fusa_pix_filt_out(struct isp_ic_dev *dev);
int32_t isp_fusa_bist_cr_rev(struct isp_ic_dev *dev);
#ifdef ISP_RDCD
int32_t isp_fusa_rdcd_enable(struct isp_ic_dev *dev);
int32_t isp_fusa_rdcd_faultInjectEn(struct isp_ic_dev *dev);
#endif
#endif

#ifdef ISP_EXP_V3
int32_t isp_enable_expv3(struct isp_ic_dev *dev);
int32_t isp_expv3_input_select(struct isp_ic_dev *dev);
int32_t isp_expv3_measure_mode(struct isp_ic_dev *dev);
int32_t isp_expv3_set_windows(struct isp_ic_dev *dev);
int32_t isp_get_expv3mean(struct isp_ic_dev *dev, u32 *mean);
int32_t isp_expv3_hw_init(struct isp_ic_dev *dev);
#endif

#define ISP_REG_GAP 4U
int32_t isp_drv_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_fpga_cmd(HalHandle_t HalHandle,u32 cmd, void *args);
int64_t isp_copy_data(void *dst, const void *src, int32_t size);

#ifdef ISP_3D_LUT
int32_t isp_lut3d_enable(struct isp_ic_dev *dev);
// int32_t isp_lut3d_tbl_sel(struct isp_ic_dev *dev);
int32_t isp_s_lut3d_tbl(struct isp_ic_dev *dev);
#endif

/* internal functions, can called by v4l2 video device and ioctl */
#if defined (ISP_CNR_V2) || defined (ISP_CNR_V2_1) || defined (ISP_CNR_V2_2)
int32_t isp_cnr2_enable(struct isp_ic_dev *dev);
int32_t isp_cnr2_disable(struct isp_ic_dev *dev);
int32_t isp_s_cnr2_strength(struct isp_ic_dev *dev);
int32_t isp_s_cnr2_filter(struct isp_ic_dev *dev);
int32_t isp_s_cnr2_tbl(struct isp_ic_dev *dev);
#endif
#if defined (ISP_COLORING_V1)
int32_t isp_coloring_enable(struct isp_ic_dev *dev);
int32_t isp_coloring_disable(struct isp_ic_dev *dev);
int32_t isp_s_coloring(struct isp_ic_dev *dev);
#endif
#ifdef ISP_YNR
int32_t isp_ynr_enable(struct isp_ic_dev *dev);
int32_t isp_ynr_disable(struct isp_ic_dev *dev);
int32_t isp_s_ynr_strength(struct isp_ic_dev *dev);
int32_t isp_s_ynr_tbl(struct isp_ic_dev *dev);
#endif
int32_t isp_reset(struct isp_ic_dev *dev);
#ifdef ISP_TPG
int32_t isp_enable_tpg(struct isp_ic_dev *dev);
int32_t isp_disable_tpg(struct isp_ic_dev *dev);
int32_t isp_s_tpg(struct isp_ic_dev *dev);
#endif
int32_t isp_enable_bls(struct isp_ic_dev *dev);
int32_t isp_disable_bls(struct isp_ic_dev *dev);
int32_t isp_enable(struct isp_ic_dev *dev);
int32_t isp_disable(struct isp_ic_dev *dev);
bool_t is_isp_enable(struct isp_ic_dev *dev);
int32_t isp_enable_lsc(struct isp_ic_dev *dev);
int32_t isp_disable_lsc(struct isp_ic_dev *dev);
int32_t isp_enable_digital_gain(struct isp_ic_dev *dev);
int32_t isp_s_digital_gain(struct isp_ic_dev *dev);
int32_t isp_s_input(struct isp_ic_dev *dev);
int32_t isp_set_dmsc(struct isp_ic_dev *dev);
int32_t isp_s_mcm_wr(struct isp_ic_dev *dev);
int32_t isp_bypass_mcm(struct isp_ic_dev *dev);
int32_t isp_s_mux(struct isp_ic_dev *dev);
int32_t isp_s_bls(struct isp_ic_dev *dev);
int32_t isp_enable_awb(struct isp_ic_dev *dev);
int32_t isp_disable_awb(struct isp_ic_dev *dev);
int32_t isp_s_awb(struct isp_ic_dev *dev);
int32_t isp_g_awbmean(struct isp_ic_dev *dev, struct isp_awb_mean *mean);
int32_t isp_s_is(struct isp_ic_dev *dev);
int32_t isp_s_raw_is(struct isp_ic_dev *dev);
int32_t isp_cnr_enable(struct isp_ic_dev *dev);
int32_t isp_s_cnr(struct isp_ic_dev *dev);
int32_t isp_start_stream(struct isp_ic_dev *dev, u32 framenum);
int32_t isp_stop_stream(struct isp_ic_dev *dev);
int32_t isp_s_cc(struct isp_ic_dev *dev);
int32_t isp_s_xtalk(struct isp_ic_dev *dev);
int32_t isp_enable_wb(struct isp_ic_dev *dev, bool_t bEnable);
int32_t isp_enable_gamma_out(struct isp_ic_dev *dev, void *args);
int32_t isp_s_gamma_out_curve_mode(struct isp_ic_dev *dev);
int32_t isp_s_gamma_out_curve(struct isp_ic_dev *dev);
int32_t isp_ioc_disable_isp_off(struct isp_ic_dev *dev);
int32_t isp_s_lsc_sec(struct isp_ic_dev *dev);
int32_t isp_s_lsc_tbl(struct isp_ic_dev *dev);
int32_t isp_s_dpf(struct isp_ic_dev *dev);
int32_t isp_enable_dpf_raw_output(struct isp_ic_dev *dev);
int32_t isp_3a_statistic_sel(struct isp_ic_dev *dev);

#if defined(ISP_EEV1)
int32_t isp_s_ee(struct isp_ic_dev *dev);
#endif
#if defined(ISP_EEV2) 
int32_t isp_s_ee(struct isp_ic_dev *dev);
int32_t isp_s_curve(struct isp_ic_dev *dev);
int32_t isp_s_ca_curve(struct isp_ic_dev *dev);
int32_t isp_s_dci_curve(struct isp_ic_dev *dev);
#endif
#if defined(ISP_EEV3)
int32_t isp_s_ee(struct isp_ic_dev *dev);
int32_t isp_s_ca_curve(struct isp_ic_dev *dev);
int32_t isp_s_dci_curve(struct isp_ic_dev *dev);
int32_t isp_s_curve(struct isp_ic_dev *dev);
int32_t isp_set_ee3_edge_detail(struct isp_ic_dev *dev);
int32_t isp_set_ee3_skin(struct isp_ic_dev *dev);
int32_t isp_set_ee3_merge_curve(struct isp_ic_dev *dev);
int32_t isp_set_ee3_noise(struct isp_ic_dev *dev);
int32_t isp_set_ee3_enhance(struct isp_ic_dev *dev);
int32_t isp_set_ee3_depurple(struct isp_ic_dev *dev);
#endif

int32_t isp_s_exp(struct isp_ic_dev *dev);
int32_t isp_s_hdrexp(struct isp_ic_dev *dev);
int32_t isp_g_expmean(struct isp_ic_dev *dev, u8 * mean);
int32_t isp_g_hdrexpmean(struct isp_ic_dev *dev, u8 * mean);
int32_t isp_s_hist(struct isp_ic_dev *dev);
int32_t isp_g_histmean(struct isp_ic_dev *dev, u32 * mean);
int32_t isp_s_hdrhist(struct isp_ic_dev *dev);
int32_t isp_g_hdrhistmean(struct isp_ic_dev *dev, u32 * mean);
#ifdef ISP_HIST64
int32_t isp_s_hist64(struct isp_ic_dev *dev);
int32_t isp_g_hist64mean(struct isp_ic_dev *dev, u32 *mean);
int32_t isp_g_hist64_vstart(struct isp_ic_dev *dev, u32 start_line);
int32_t isp_update_hist64(struct isp_ic_dev *dev);
int32_t isp_g_hist64_vstart_status(struct isp_ic_dev *dev, u32 *status);
#endif
int32_t isp_s_gtm(struct isp_ic_dev *dev);
int32_t isp_g_gtmdata(struct isp_ic_dev *dev, struct isp_gtm_sta *sta);
int32_t isp_s_gtm_bw_cor_base_slope(struct isp_ic_dev *dev);
int32_t isp_s_gtm_weights(struct isp_ic_dev *dev);
int32_t isp_s_dpcc(struct isp_ic_dev *dev);
int32_t isp_cfg_dpcc_bpt(struct isp_ic_dev *dev);
int32_t isp_s_deg(struct isp_ic_dev *dev);
int32_t isp_s_ie(struct isp_ic_dev *dev);
int32_t isp_s_vsm(struct isp_ic_dev *dev);
int32_t isp_g_vsm(struct isp_ic_dev *dev, struct isp_vsm_result *vsm);
int32_t isp_s_afm(struct isp_ic_dev *dev);
int32_t isp_g_afm(struct isp_ic_dev *dev, struct isp_afm_result *afm);
#ifdef ISP_AFM_V3
int32_t isp_afm_enable(struct isp_ic_dev *dev);
int32_t isp_afm_enable_gamma(struct isp_ic_dev *dev);
int32_t isp_afm_s_hist_threshold(struct isp_ic_dev *dev);
int32_t isp_afm_s_bpt(struct isp_ic_dev *dev);
int32_t isp_afm_s_bls(struct isp_ic_dev *dev);
int32_t isp_afm_s_block(struct isp_ic_dev *dev);
int32_t isp_afm_s_iir(struct isp_ic_dev *dev);
int32_t isp_afm_s_fir(struct isp_ic_dev *dev);
int32_t isp_afm_s_gamma(struct isp_ic_dev *dev);
int32_t isp_afm_s_ldg(struct isp_ic_dev *dev);
int32_t isp_afm_s_coring(struct isp_ic_dev *dev);
int32_t isp_afm_g_statistic(struct isp_ic_dev *dev, struct isp_afmv3_result *afm);
#endif
#ifdef ISP_WDR_V3
int32_t isp_enable_wdr3(struct isp_ic_dev *dev);
int32_t isp_disable_wdr3(struct isp_ic_dev *dev);
int32_t isp_u_wdr3(struct isp_ic_dev *dev);
int32_t isp_s_wdr3(struct isp_ic_dev *dev);
#endif
#ifdef ISP_WDR_V4
int32_t isp_enable_wdr4(struct isp_ic_dev *dev);
int32_t isp_disable_wdr4(struct isp_ic_dev *dev);
int32_t isp_u_wdr4(struct isp_ic_dev *dev);
int32_t isp_s_wdr4(struct isp_ic_dev *dev);
#endif
#if defined(ISP_WDR_V5) || defined(ISP_WDR_V5_1) || defined(ISP_WDR_V5_2) || defined(ISP_WDR_V5_2_1) || defined(ISP_WDR_V5_3)
int32_t  isp_enable_wdr5(struct isp_ic_dev *dev);
int32_t  isp_wdr5_strength(struct isp_ic_dev *dev);
int32_t  isp_wdr5_set_contrast(struct isp_ic_dev *dev);
int32_t  isp_wdr5_weight(struct isp_ic_dev *dev);
int32_t  isp_wdr5_bayer_ratio(struct isp_ic_dev *dev);
int32_t  isp_wdr5_entropy_slope(struct isp_ic_dev *dev);
int32_t  isp_wdr5_adjust_slope(struct isp_ic_dev *dev);
int32_t  isp_wdr5_local_flat_mode(struct isp_ic_dev *dev);
int32_t  isp_wdr5_local_flat_level(struct isp_ic_dev *dev);
int32_t  isp_wdr5_global_curve_mode(struct isp_ic_dev *dev);
int32_t  isp_wdr5_global_flat_level(struct isp_ic_dev *dev);
int32_t  isp_wdr5_global_fixed_weight(struct isp_ic_dev *dev);
int32_t  isp_wdr5_histogram(struct isp_ic_dev *dev);
int32_t  isp_wdr5_gammapre(struct isp_ic_dev *dev);
int32_t  isp_wdr5_gammaup(struct isp_ic_dev *dev);
int32_t  isp_wdr5_maxgain(struct isp_ic_dev *dev);
int32_t  isp_wdr5_mingain(struct isp_ic_dev *dev);
int32_t  isp_wdr5_degamma(struct isp_ic_dev *dev);
int32_t  isp_wdr5_gammadown(struct isp_ic_dev *dev);
int32_t  isp_wdr5_weight_raw(struct isp_ic_dev *dev);
int32_t  isp_wdr5_entropy(struct isp_ic_dev *dev);
int32_t  isp_wdr5_lut_shift(struct isp_ic_dev *dev);
int32_t  isp_wdr5_lut_shift0(struct isp_ic_dev *dev);
int32_t  isp_wdr5_distance_weight(struct isp_ic_dev *dev);
int32_t  isp_wdr5_difference_weight(struct isp_ic_dev *dev);
int32_t  isp_wdr5_flat_factor(struct isp_ic_dev *dev);
int32_t  isp_wdr5_flat_level(struct isp_ic_dev *dev);
int32_t  isp_wdr5_sat_x(struct isp_ic_dev *dev);
int32_t  isp_wdr5_sat(struct isp_ic_dev *dev);
int32_t  isp_wdr5_sat_shift(struct isp_ic_dev *dev);
int32_t  isp_g_wdr5_avg_std(struct isp_ic_dev *dev, u32 *avg_std);
#if defined(ISP_WDR_V5_2) || defined(ISP_WDR_V5_2_1) || defined(ISP_WDR_V5_3)
int32_t isp_s_wdr5_luma(struct isp_ic_dev *dev);
int32_t isp_wdr5_light_enable(struct isp_ic_dev *dev);
int32_t isp_wdr5_light_sat(struct isp_ic_dev *dev);
int32_t isp_wdr5_halo_color_thr(struct isp_ic_dev *dev);
int32_t isp_wdr5_hlc(struct isp_ic_dev *dev);
int32_t isp_s_wdr5_damp(struct isp_ic_dev *dev);
#endif
#endif
int32_t isp_s_sta(struct isp_ic_dev *dev);
int32_t isp_s_exp2(struct isp_ic_dev *dev);
int32_t isp_s_exp2_inputsel(struct isp_ic_dev *dev);
int32_t isp_s_exp2_sizeratio(struct isp_ic_dev *dev, u32 ratio);
#ifdef ISP_HDR_STITCH_V3_2DOL
int32_t isp_s_hdr_dma_start_mode(struct isp_ic_dev *dev);
#endif
#if (defined ISP_MI_HDR) || (defined ISP_HDR_STITCH_V1_3)
int32_t isp_enable_hdr(struct isp_ic_dev *dev);
int32_t isp_enable_hdr_dg(struct isp_ic_dev *dev);
int32_t isp_enable_hdr_awb(struct isp_ic_dev *dev);
int32_t isp_s_hdr_properties(struct isp_ic_dev *dev);
int32_t isp_s_hdr_bls(struct isp_ic_dev *dev);
int32_t isp_s_hdr_wb(struct isp_ic_dev *dev);
int32_t isp_s_hdr_dg(struct isp_ic_dev *dev);
int32_t isp_s_transrange_linear(struct isp_ic_dev *dev);
int32_t isp_s_transrange_nonlinear(struct isp_ic_dev *dev);
int32_t isp_s_ls_ratio(struct isp_ic_dev *dev);
int32_t isp_s_vss_ratio(struct isp_ic_dev *dev);
int32_t isp_s_color_weight(struct isp_ic_dev *dev);
int32_t isp_s_bypass_select(struct isp_ic_dev *dev);
int32_t isp_s_hdr_mode(struct isp_ic_dev *dev);
int32_t isp_s_hdr_image(struct isp_ic_dev *dev);
int32_t isp_s_bayer_pattern(struct isp_ic_dev *dev);
int32_t isp_ioc_enable_sensor0_frame_count(struct isp_ic_dev *dev);
#endif
#if (defined ISP_HDR_STITCH_V1_3) || (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V2_1) || (defined ISP_HDR_STITCH_V2_1_2DOL) || (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_2DOL)
int32_t isp_s_extbit(struct isp_ic_dev *dev);
int32_t isp_s_vs_params(struct isp_ic_dev *dev);
int32_t isp_s_l_params(struct isp_ic_dev *dev);
int32_t isp_s_combine_mode(struct isp_ic_dev *dev);
int32_t isp_s_base_frame(struct isp_ic_dev *dev);
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V2_1) || (defined ISP_HDR_STITCH_V2_1_2DOL) || (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_2DOL)
int32_t isp_s_compress_lut(struct isp_ic_dev *dev);
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V3)
int32_t isp_s_hdr_dpf_l(struct isp_ic_dev *dev);
int32_t isp_s_hdr_dpf_s(struct isp_ic_dev *dev);
int32_t isp_s_hdr_dpf_vs(struct isp_ic_dev *dev);
int32_t isp_enable_hdr_dpf_l(struct isp_ic_dev *dev);
int32_t isp_enable_hdr_dpf_s(struct isp_ic_dev *dev);
int32_t isp_enable_hdr_dpf_vs(struct isp_ic_dev *dev);
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_2DOL)
int32_t isp_s_hdr_deghost(struct isp_ic_dev *dev);
int32_t isp_enable_hdr_deghost_ls(struct isp_ic_dev *dev);
int32_t isp_enable_hdr_deghost_lsvs(struct isp_ic_dev *dev);
#endif
#if (defined ISP_HDR_STITCH_V2) || (defined ISP_HDR_STITCH_V2_1) || (defined ISP_HDR_STITCH_V2_1_2DOL) || (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_1) || (defined ISP_HDR_STITCH_V3_2DOL)
int32_t isp_s_transrange_linear_ref_s(struct isp_ic_dev *dev);
int32_t isp_s_transrange_linear_ref_vs(struct isp_ic_dev *dev);
#endif
#ifdef ISP_HDR_STITCH_V3_1
int32_t isp_s_transrange_linear_ref_lsvs(struct isp_ic_dev *dev);
int32_t isp_s_transrange_linear_ref_e3(struct isp_ic_dev *dev);
int32_t isp_s_vse3_ratio(struct isp_ic_dev *dev);
int32_t isp_s_hdr_bls_out(struct isp_ic_dev *dev);
#endif
#if defined(ISP_2DNR_V5_V1) || defined(ISP_2DNR_V5_V2)
int32_t isp_cfg_2dnr_luma_curve(struct isp_ic_dev *dev);
#endif

#if defined (ISP_2DNR_V5_V2)
int32_t isp_cfg_2dnr_blend(struct isp_ic_dev *dev);
#endif

#if defined (ISP_2DNR_V2) || defined (ISP_2DNR_V3) || defined (ISP_2DNR_V4) || \
    defined (ISP_2DNR_V5) || defined (ISP_2DNR_V5_V1) || defined (ISP_2DNR_V5_V2) || \
    defined (ISP_2DNR_V6)
int32_t isp_s_2dnr(struct isp_ic_dev *dev);
#endif

#if defined (ISP_2DNR_V6)
int32_t isp_s_2dnr_lumaCurve(struct isp_ic_dev *dev);
int32_t isp_s_2dnr_lscCompCurve(struct isp_ic_dev *dev);
int32_t isp_s_2dnr_motionFacCurve(struct isp_ic_dev *dev);
#endif

#if defined(ISP_3DNR) || defined(ISP_3DNR_V2) || defined(ISP_3DNR_V2_1) || defined(ISP_3DNR_V2_4) || defined(ISP_3DNR_V1_2)
int32_t isp_s_3dnr(struct isp_ic_dev *dev);
int32_t isp_g_3dnr(struct isp_ic_dev *dev, u32 * avg);
int32_t isp_u_3dnr(struct isp_ic_dev *dev, struct isp_3dnr_update *dnr3_update);
int32_t isp_s_3dnr_motion(struct isp_ic_dev *dev);
int32_t isp_s_3dnr_delta(struct isp_ic_dev *dev);
int32_t isp_denoise3dnV32_update(struct isp_ic_dev *dev);
int32_t isp_3dnr_dilate_en(struct isp_ic_dev *dev);
int32_t isp_s_3dnr_update_temporal(struct isp_ic_dev *dev);
int32_t isp_s_3dnr_strength_curve_temporal(struct isp_ic_dev *dev);
int32_t isp_s_3dnr_range_t(struct isp_ic_dev *dev);
int32_t isp_s_3dnr_temporal_curve(struct isp_ic_dev *dev);

#endif

#if defined(ISP_3DNR_V2_4) 
int32_t isp_s_3dnr_motion_roi_pos(struct isp_ic_dev *dev);
int32_t isp_s_3dnr_motion_roi_en(struct isp_ic_dev *dev);
int32_t isp_g_3dnr_motion_roi_diff(struct isp_ic_dev *dev, struct isp_3dnr_motion_roi_diff *roiDiff);
#endif

#if defined(ISP_3DNR) || defined(ISP_3DNR_V2_1) || defined(ISP_3DNR_V2_4) || defined(ISP_3DNR_V1_2)
int32_t isp_r_3dnr(struct isp_ic_dev *dev);
#endif

#if defined(ISP_3DNR_V2) || defined(ISP_3DNR_V2_1) || defined(ISP_3DNR_V2_4)
int32_t isp_s_3dnr_cmp(struct isp_ic_dev *dev);
#endif

int32_t isp_u_3dnr_strength(struct isp_ic_dev *dev);
int32_t isp_s_comp_bls(struct isp_ic_dev *dev);
int32_t isp_s_comp(struct isp_ic_dev *dev);
int32_t isp_s_simp(struct isp_ic_dev *dev);
int32_t isp_s_cproc(struct isp_ic_dev *dev);
int32_t isp_s_elawb(struct isp_ic_dev *dev);
int32_t isp_ioc_qcap(struct isp_ic_dev *dev, void *args);
int32_t isp_ioc_g_status(struct isp_ic_dev *dev, void *args);

int32_t isp_enable_gcmono(struct isp_ic_dev *dev);
int32_t isp_disable_gcmono(struct isp_ic_dev *dev);
int32_t isp_s_gcmono(struct isp_ic_dev *dev, struct isp_gcmono_data *data);	/* set curve */

#ifdef ISP_GC_V2
int32_t isp_enable_rgbgamma(struct isp_ic_dev *dev);
int32_t isp_disable_rgbgamma(struct isp_ic_dev *dev);
int32_t isp_s_rgbgamma(struct isp_ic_dev *dev, struct isp_rgbgamma_data *data);
#endif

u32 isp_read_mi_irq(struct isp_ic_dev *dev);
void isp_reset_mi_irq(struct isp_ic_dev *dev, u32 icr);

int32_t isp_ioc_init_dma(void *pDev, uint8_t *pPortId);
int32_t isp_ioc_cfg_dma(struct isp_ic_dev *dev, void *args);
int32_t isp_ioc_start_dma_read(struct isp_ic_dev *dev, void *args);
int32_t isp_mi_cfg(struct isp_ic_dev *dev, void *args);
int32_t isp_mi_cfg_update(struct isp_ic_dev *dev, void *args);
int32_t isp_mi_start(struct isp_ic_dev *dev);
int32_t isp_mi_path_enable(struct isp_ic_dev *dev, void *args);
int32_t isp_mi_path_disable(struct isp_ic_dev *dev, void *args);
int32_t isp_stream_path_enable(struct isp_ic_dev *dev, void *args);
int32_t isp_stream_path_disable(struct isp_ic_dev *dev, void *args);
int32_t isp_mi_stop(struct isp_ic_dev *dev);
int32_t isp_set_buffer(struct isp_ic_dev *dev, const struct isp_buffer_context *buf);
int32_t isp_set_bp_buffer(struct isp_ic_dev *dev,
              const struct isp_bp_buffer_context *buf);

int32_t isp_enable_dmsc(struct isp_ic_dev *dev);
int32_t isp_disable_dmsc(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_intp(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_skin(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_gfilter(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_depurple(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_cac(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_sharpen(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_sharpen_line(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_dmoi(struct isp_ic_dev *dev);
int32_t isp_set_dmsc_flt(struct isp_ic_dev *dev);
int32_t isp_s_dmsc(struct isp_ic_dev *dev);
int32_t isp_s_ge(struct isp_ic_dev *dev);
int32_t isp_s_ca(struct isp_ic_dev *dev);
int32_t isp_set_crop_backend(struct isp_ic_dev *dev);
int32_t isp_s_color_adjust(struct isp_ic_dev *dev);
int32_t isp_config_dummy_hblank(struct isp_ic_dev *dev);

int32_t isp_enable_dmsc3(struct isp_ic_dev *dev);
int32_t isp_set_dmsc3_intp(struct isp_ic_dev *dev);
int32_t isp_set_dmsc3_interp_corner(struct isp_ic_dev *dev);
int32_t isp_set_dmsc3_decline(struct isp_ic_dev *dev);
int32_t isp_set_dmsc3_dmoi(struct isp_ic_dev *dev);
int32_t isp_set_dmsc3_defalse(struct isp_ic_dev *dev);
int32_t isp_set_dmsc3_sharpen(struct isp_ic_dev *dev);
int32_t isp_set_dmsc3_cac(struct isp_ic_dev *dev);
int32_t isp_set_dmsc3_depurple(struct isp_ic_dev *dev);

#if defined (ISP_RGBIR_V1) || defined (ISP_RGBIR_V1_1) || defined (ISP_RGBIR_V2) || defined (ISP_RGBIR_V2_1)
int32_t isp_s_rgbir(struct isp_ic_dev *dev);
int32_t isp_enable_rgbir(struct isp_ic_dev *dev);
int32_t isp_rgbir_hw_init(struct isp_ic_dev *dev);
int32_t isp_rgbir_s_ir_dnr(struct isp_ic_dev *dev);
int32_t isp_rgbir_s_sharpen(struct isp_ic_dev *dev);
int32_t isp_rgbir_s_des(struct isp_ic_dev *dev);
int32_t isp_rgbir_s_cc_matrix(struct isp_ic_dev *dev);
int32_t isp_rgbir_s_dpcc(struct isp_ic_dev *dev);
int32_t isp_rgbir_s_gain(struct isp_ic_dev *dev);
int32_t isp_rgbir_s_bls(struct isp_ic_dev *dev);
int32_t isp_rgbir_out_ir_raw(struct isp_ic_dev *dev);
#if defined (ISP_RGBIR_V2) || defined (ISP_RGBIR_V2_1)
int32_t isp_enable_rccc(struct isp_ic_dev *dev);
#endif
#endif
#ifdef ISP_3DNR_V3
int32_t isp_s_tdnr(struct isp_ic_dev *dev);
int32_t isp_3dnr3_set_strength(struct isp_ic_dev *dev);
int32_t isp_3dnr3_set_motion(struct isp_ic_dev *dev);
int32_t isp_3dnr3_enable(struct isp_ic_dev *dev);
int32_t isp_3dnr3_disable(struct isp_ic_dev *dev);
int32_t isp_3dnr3_enable_tdnr(struct isp_ic_dev *dev);
int32_t isp_3dnr3_disable_tdnr(struct isp_ic_dev *dev);
int32_t isp_3dnr3_enable_2dnr(struct isp_ic_dev *dev);
int32_t isp_3dnr3_disable_2dnr(struct isp_ic_dev *dev);
int32_t isp_3dnr_g_stats(struct isp_ic_dev *dev, struct isp_3dnr_stats *avg);
int32_t isp_3dnr3_u_noise(struct isp_ic_dev *dev);
int32_t isp_r_tdnr_refer(struct isp_ic_dev *dev);
int32_t isp_r_tdnr_motion(struct isp_ic_dev *dev);
int32_t isp_3dnr3_u_thr(struct isp_ic_dev *dev);
int32_t isp_3dnr_s_buf(struct isp_ic_dev *dev);
int32_t isp_3dnr_s_rst(struct isp_ic_dev *dev, bool_t reset);   //reference reset
int32_t isp_3dnr3_cfg_gamma(struct isp_ic_dev *dev);
int32_t isp_3dnr_update_tnr(struct isp_ic_dev *dev);
#endif

#ifdef ISP_3DNR_V3_1
int32_t isp_3dnr3_enable_tdnr(struct isp_ic_dev *dev);
int32_t isp_3dnr3_disable_tdnr(struct isp_ic_dev *dev);
int32_t isp_s_tdnr(struct isp_ic_dev *dev);
int32_t isp_3dnr_g_stats(struct isp_ic_dev *dev, struct isp_3dnr_stats *avg);
int32_t isp_3dnr3_set_strength(struct isp_ic_dev *dev);
int32_t isp_3dnr3_u_noise(struct isp_ic_dev *dev);
int32_t isp_3dnr3_u_thr(struct isp_ic_dev *dev);
int32_t isp_r_tdnr_refer(struct isp_ic_dev *dev);
int32_t isp_r_tdnr_motion(struct isp_ic_dev *dev);
int32_t isp_3dnr_s_buf(struct isp_ic_dev *dev);
int32_t isp_3dnr_s_rst(struct isp_ic_dev *dev, bool_t reset);
int32_t isp_3dnr3_cfg_gamma(struct isp_ic_dev *dev);
int32_t isp_3dnr_update_tnr(struct isp_ic_dev *dev);
#endif

#ifdef ISP_3DNR_V4
int32_t isp_3dnr4_enable_tdnr(struct isp_ic_dev *dev);
int32_t isp_3dnr4_disable_tdnr(struct isp_ic_dev *dev);
int32_t isp_s_3dnr4(struct isp_ic_dev *dev);
int32_t isp_3dnr4_g_stats(struct isp_ic_dev *dev, struct isp_3dnr_stats *avg);
int32_t isp_3dnr4_set_strength(struct isp_ic_dev *dev);
int32_t isp_3dnr4_u_noise(struct isp_ic_dev *dev);
int32_t isp_3dnr4_u_thr(struct isp_ic_dev *dev);
int32_t isp_r_3dnr4_refer(struct isp_ic_dev *dev);
int32_t isp_r_3dnr4_motion(struct isp_ic_dev *dev);
int32_t isp_3dnr_s_buf(struct isp_ic_dev *dev);
int32_t isp_3dnr_s_rst(struct isp_ic_dev *dev, bool_t reset);
int32_t isp_s_3dnr4_decompose(struct isp_ic_dev *dev);
int32_t isp_s_3dnr4_luma_lut(struct isp_ic_dev *dev);
int32_t isp_s_3dnr4_boost(struct isp_ic_dev *dev);
int32_t isp_s_3dnr4_motion_gauss_kernel(struct isp_ic_dev *dev);
int32_t isp_3dnr_update_tnr(struct isp_ic_dev *dev);
#endif

#ifdef ISP_RAW_CROP
int32_t isp_s_raw_crop_win(struct isp_ic_dev *dev);
int32_t isp_s_raw_crop_enabled(struct isp_ic_dev *dev);
#endif

#ifdef ISP_GAMMA_FEBE
int32_t isp_s_gamma_fe_be_curve(struct isp_ic_dev *dev);
int32_t isp_s_gamma_be_pre_bls(struct isp_ic_dev *dev);
int32_t isp_s_gamma_be_bls(struct isp_ic_dev *dev);
int32_t isp_s_gamma_fe_be(struct isp_ic_dev *dev);
#endif

#ifdef ISP_MI_PP_WRITE
int32_t  isp_set_ppw_line_num(struct isp_ic_dev *dev, u16 line_num);
int32_t  isp_get_ppw_pic_cnt(struct isp_ic_dev *dev, u16* pic_cnt);
#endif

#ifdef __KERNEL__
int32_t clean_dma_buffer(struct isp_ic_dev *dev);
irqreturn_t isp_hw_isr(int32_t irq, void *data);
void isp_clear_interrupts(struct isp_ic_dev *dev);
#endif
/*get irq mis value from store array*/
u32 isp_read_mis(struct isp_ic_dev *dev, u32 irq_src);
// int32_t isp_ioc_read_mis(struct isp_ic_dev *dev, void *__user args);

/*set scaler*/
int32_t isp_set_scaling(MiPathId_t id, struct isp_ic_dev *dev, bool_t stabilization, bool_t crop);
int32_t isp_set_crop(struct isp_ic_dev *dev);

int32_t isp_ioc_g_feature(struct isp_ic_dev *dev, void *args);
int32_t isp_ioc_g_feature_veresion(struct isp_ic_dev *dev, void *args);

#ifdef ISP_PDAF
int32_t isp_s_pdaf(struct isp_ic_dev *dev);
int32_t isp_enable_pdaf(struct isp_ic_dev *dev);
int32_t isp_disable_pdaf(struct isp_ic_dev *dev);
#endif

int32_t isp_s_pp_path(struct isp_ic_dev *dev);

int32_t isp_set_nr_reloc(struct isp_ic_dev *dev);

#ifdef ISP_MI_HDR
int32_t isp_ioc_set_mi_hdr_path_fmt(struct isp_ic_dev *dev, u8 rd_wr_str, u32 fmt_addr);
int32_t isp_ioc_start_dma_read_hdr_manual(struct isp_ic_dev *dev);
int32_t isp_ioc_start_dma_read_hdr_auto(struct isp_ic_dev *dev);
int32_t isp_ioc_set_mi_hdr_retiming(struct isp_ic_dev *dev);
#if (defined ISP_HDR_STITCH_V3) || (defined ISP_HDR_STITCH_V3_1) || (defined ISP_HDR_STITCH_V3_2DOL) || (defined ISP_HDR_RT_V3)
int32_t isp_ioc_set_mi_hdr_dma_size_init(struct isp_ic_dev *dev, u32 id);
int32_t isp_ioc_set_mi_hdr_ppc(struct isp_ic_dev *dev);
#endif
#endif

#ifdef ISP_MSS
int32_t isp_mss_dma_cfg(struct isp_ic_dev *dev, void *args);
int32_t isp_mss_start_read(struct isp_ic_dev *dev, void *args);
#endif

#if defined(ISP_MPCTL) || defined(ISP_MPCTL_V1_1)
int32_t isp_cfg_mp_ctrl(struct isp_ic_dev *dev);
int32_t isp_get_mp_ctrl_status(struct isp_ic_dev *dev);

#endif
int32_t isp_mi_safety_reset(struct isp_ic_dev *dev);
int32_t isp_fast_start(struct isp_ic_dev *dev);
int32_t isp_system_init(struct isp_ic_dev *dev);
int32_t isp_set_clock_gating(struct isp_ic_dev *dev);

int32_t isp_drv_tpg_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_mss_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_dg_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_awb_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_ee_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_2dnr_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_3dnr_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_gc_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_mi_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_mi_rdma_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_dmsc_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_mi_hdr_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_is_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_csm_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_bls_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_expv1_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_expv2_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_expv3_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_3dlut_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_lsc_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_cnr_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_coloring_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_ynr_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_dpf_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_degamma_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_vsm_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_afmv1_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_afmv3_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_hdr_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_hdr_exp_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_hdr_hist_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_hist256_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_hist64_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_gtm_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_dpcc_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_gwdr_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_wdr_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_sta_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_cpd_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_cproc_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_ccm_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_gcmono_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_ge_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_rgbir_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_crop_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_pdaf_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_febe_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_mpctl_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_sbi_mi_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_fusa_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
int32_t isp_drv_rdcd_priv_cmd(struct isp_ic_dev *dev, u32 cmd, void *args);
#endif /* _ISP_DRV_H_ */
