/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef ISPSTARTENGINEAPI_H
#define ISPSTARTENGINEAPI_H
// #include <binder/binder_hal_api.h>
#include <common/return_codes.h>
// #include <oslayer/oslayer.h>
#include <ebase/types.h>

#define ISP_STARTENG_VIRT_MAXCNT               (16U)
#define ISP_DEV_RATIO_MAXCNT                   (16U)

/*****************************************************************************/
/**
 * @brief   Handle to a ISP Start Engine instance.
 *
 *****************************************************************************/

typedef enum MCMModeOption_e
{
    ISP_START_ENGINE_SW_MCM_MODE    = 0x0000,  /**< MCM SW MODE */
    ISP_START_ENGINE_HW_MCM_MODE    = 0x0001  /**< MCM HW MODE */
}MCMModeOption_t;


typedef enum HWMCMWorkMode_e
{
    ISP_START_ENGINE_EVENT_ROUNDRB     = 0x0000, /**< HW MCM Mode Round-Robin */
    ISP_START_ENGINE_EVENT_FIXSEQ      = 0x0001, /**< HW MCM Mode Fixed-Sequence */
    ISP_START_ENGINE_EVENT_FIFO        = 0x0002  /**< HW MCM Mode First Input First Output */
}HWMCMWorkMode_t;

typedef enum StartEngWorkMode_e
{
    ISP_START_ENGINE_MODE_INVALID = 0,
    ISP_START_ENGINE_MODE_NORMAL,        /**< normal mode */
    ISP_START_ENGINE_MODE_BATCH,         /**< batch mode */
    ISP_START_ENGINE_MODE_MAX
}StartEngWorkMode_t;

typedef struct IspStartEngineConfig_s{
    void  *hIspStartEngine;    /**< Handle to StartEngine context, set by @ref IspStartEngineInit if successful, undefined otherwise. */
    uint8_t                 HID;                /**< Hardware Pipeline ID. */ 
    void*                   hHalLayer;
    MCMModeOption_t         mcmModeOption;
    HWMCMWorkMode_t         hwMCMWorkMode;
    StartEngWorkMode_t      workMode;
    uint8_t                 batchFrameCount;
} IspStartEngineConfig_t;


typedef enum IspStartEngineEventId_e
{
    ISP_START_ENGINE_EVENT_INVALID           = 0x0000, /**< invalid event (only for an internal evaluation) */
    ISP_START_ENGINE_EVENT_DATA_LOSS         = 0x0001,
    ISP_START_ENGINE_EVENT_FRAME_DONE        = 0x0002
}IspStartEngineEventId_t;

/******************************************************************************/
/**
 * @brief   Event callback
 *
 * @note    This callback is used to signal something to the application
 *          software, e.g. an error or an information.
 *
 * @return  void
 *
 *****************************************************************************/
typedef void (* IspStartEngineEventFunc_t)
(
    const IspStartEngineEventId_t  evtId,
    void                           *pUserContext
);

typedef struct IspStartEngineBuffer_s{
    //TODO:need confirm the buffer register need the Max value or Curr value.
    uint8_t  devID;
    uint32_t cmdNumMax;
    uint32_t cmdDmaAddr;
    uint32_t mcmBusId;
    bool pingpongFlag;
} IspStartEngineBuffer_t;


typedef struct IspStartEngineApiOps_s {

void* (*StartEngineInit)
(
	IspStartEngineConfig_t *pConfig
);

RESULT (*StartEngineDeinit)
(
    void* hIspStartEngine
);

RESULT (*StartEngineRegisterDev)
(
    void* hIspStartEngine,
    uint8_t  devID,
    uint8_t ratio
);

RESULT (*StartEngineDeregisterDev)
(
    void* hIspStartEngine,
    uint8_t  devID
);

RESULT (*StartEngineRegisterEventCb)
(
    void* hIspStartEngine,
    IspStartEngineEventFunc_t func,
    void *pUserContext
);

RESULT (*StartEngineDeregisterEventCb)
(
    void* hIspStartEngine
);

RESULT (*StartEngineStart)
(
    void* hIspStartEngine,
    IspStartEngineBuffer_t startEngineBuffer
);

}IspStartEngineApiOps_t;

typedef struct IspStartEngineCtx_s {
    IspStartEngineApiOps_t* pStartEngineApiOps;    //pointer to the start engine api ops.
    void*   pPrivateCtx;      //pointer to the lower level context.It determines by the lower level.
}IspStartEngineCtx_t;

typedef IspStartEngineCtx_t* IspStartEngineHandle_t;

RESULT IspStartEngineInit(IspStartEngineConfig_t *pConfig);
RESULT IspStartEngineDeinit(IspStartEngineHandle_t hIspStartEngine);
RESULT IspStartEngineRegisterDev(IspStartEngineHandle_t hIspStartEngine, uint8_t  devID, uint8_t ratio);
RESULT IspStartEngineDeregisterDev(IspStartEngineHandle_t hIspStartEngine, uint8_t  devID);
RESULT IspStartEngineRegisterEventCb(IspStartEngineHandle_t hIspStartEngine, IspStartEngineEventFunc_t func, void *pUserContext);
RESULT IspStartEngineDeregisterEventCb(IspStartEngineHandle_t hIspStartEngine);
RESULT IspStartEngineStart(IspStartEngineHandle_t hIspStartEngine,IspStartEngineBuffer_t startEngineBuffer);

#endif /*_ISPStartEngineDEV_H_*/
