/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/******************************************************************************\
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets of    *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2021 */

/**
 * @file virtual_hal_api.h
 *
 * @brief   virtual api *
 *          virtual with abstracts services from different work mode.
 *
 *****************************************************************************/

#ifndef VIRTUAL_HAL_API_H
#define VIRTUAL_HAL_API_H


#include<hal/hal_api.h>
#include<hal/hal_common.h>
#include <ebase/builtins.h>
#include "virtual_hal_ops.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct VirtualCtx_s {
    uint32_t ispId;
    VirtualHalApiOps_t* pHalApiOps;    //pointer to the hal api ops.
    void*   pPrivateCtx;               //pointer to the lower level context.It determines by the lower level.
    HalHandle_t*   hHal;
    VirtualHalBindOpsType_t    virtualMode;        //indictate work mode. 
}VirtualHalCtx_t;
typedef void* VirtualHandle_t;


/*****************************************************************************/
/**
 * @brief   call the virtualed low level opem api.
 * @param   ispId         isp ID
 * @param   VirtualHalBindOpsType   work mode
 * @return  lower level context pointer. NULL on failure.
 *
 *****************************************************************************/
VirtualHandle_t VirtualHalOpen
(
    uint32_t ispId,
    VirtualHalBindOpsType_t VirtualHalBindOpsType,
    const VirtualWorkModeCfg_t *pCfg
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level close api.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalClose
(
    VirtualHandle_t virtualHandle
);


/*****************************************************************************/
/**
 * @brief   call the virtualed low level write register api to write register.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalWriteReg
(
    VirtualHandle_t virtualHandle,
    uint32_t addr,
    uint32_t val
);


/*****************************************************************************/
/**
 * @brief   call the virtualed low level read register api to read register.
 * @return  register value
 *
 *****************************************************************************/
uint32_t VirtualHalReadReg
(
    VirtualHandle_t virtualHandle,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level write register api to write register.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalWriteBatchRegs
(
    VirtualHandle_t virtualHandle,
    uint32_t addr,
    uint32_t *pVal
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level write register api to write table register.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalWriteTbl
(
    VirtualHandle_t virtualHandle,
    uint32_t addr,
    uint32_t val
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level read register api to read table register.
 * @return  register value
 *
 *****************************************************************************/
uint32_t VirtualHalReadTbl
(
    VirtualHandle_t virtualHandle,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level write register api to write RDC register.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalRdcdWriteReg
(
    VirtualHandle_t virtualHandle,
    uint32_t addr,
    uint32_t val
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level read register api to read RDC register.
 * @return  register value
 *
 *****************************************************************************/
uint32_t VirtualHalRdcdReadReg
(
    VirtualHandle_t virtualHandle,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level connect irq api to create irq handler 
 * thread.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalConnectIrq
(
    VirtualHandle_t virtualHandle,
    VirtualHalIrqCtx_t *pIrqCtx,
    uint32_t    int_src,
    osIsrFunc   IsrFunction,
    osDpcFunc   DpcFunction,
    void*       pContext
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level disconnect irq api to destroy irq handler 
 * thread.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalDisconnectIrq
(
    VirtualHandle_t virtualHandle,
    VirtualHalIrqCtx_t *pIrqCtx
);

/*****************************************************************************/
/**
 * @brief   call the virtualed low level disconnect irq api to destroy irq handler 
 * thread.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalDisConnect
(
    VirtualHandle_t virtualHandle,
    VirtualHalIrqCtx_t *pIrqCtx
);


/*****************************************************************************/
/**
 * @brief   call the virtualed low level 
 *           Cmodel specific interface: Hal ISR and Event generator
 * thread.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT VirtualHalEventGenerator
(
    VirtualHandle_t virtualHandle,
    HalIrqCtx_t *pIrqCtx,
    HalIsrSrc_t isr_src,
    uint32_t misValue
);
/*****************************************************************************/
/**
 * @brief   allocates the given amount of hardware memory
 * @param   virtualHandle       Handle to HAL session as returned by @ref VirtualHalOpen.
 * @param   byte_size       Amount of memory to allocate.
 * @return  mem_address     Memory block start address in hardware memory; 0 on failure.
 *
 * @note    Chunks of n*4K byte with addresses aligned to 4K are used internally.
 *          The allocator behind the scenes may be very simple, but still will recombine
 *          free'd adjacent blocks. It may perform reasonably well only for at most a
 *          few dozen allocs active at any time. Allocating/freeing video/audio buffers in
 *          realtime at framerate should nevertheless be posssible without noticeable
 *          performance penalties then.
 *****************************************************************************/
uint32_t VirtualHalAllocMemory(
     VirtualHandle_t virtualHandle,
     uint32_t byte_size,
    void **pIpl_alignment
);


/*****************************************************************************/
/**
 * @brief   frees the given block of hardware memory
 * @param   virtualHandle       Handle to HAL session as returned by @ref VirtualHalOpen.
 * @param   mem_address     Start address of memory block in hardware memory.
 * @return  Result of operation.
 *
 * @note    Chunks of n*4K byte with addresses aligned to 4K are used internally.
 *****************************************************************************/
RESULT VirtualHalFreeMemory
( 
    VirtualHandle_t virtualHandle,
    uint32_t mem_address 
);


/*****************************************************************************/
/**
 * @brief   reads a number of data from the memory to a buffer starting a the given address
 * @param   virtualHandle       Handle to HAL session as returned by @ref VirtualHalOpen.
 * @param   mem_address     Source start address in hardware memory.
 * @param   p_read_buffer   Pointer to local memory holding the data being read.
 * @param   byte_size       Amount of data to read.
 * @return  Result of operation.
 *
 * @note    Certain implementation dependent limitations regarding alignment of
 *          both addresses and transfer size exist!
 *****************************************************************************/
RESULT VirtualHalReadMemory
(
    VirtualHandle_t virtualHandle,
    uint32_t mem_address,
    uint8_t* p_read_buffer,
    uint32_t byte_size
);

/*****************************************************************************/
/**
 * @brief   writes a number of data from a buffer to the memory starting a the given address
 * @param   virtualHandle       Handle to HAL session as returned by @ref VirtualHalOpen.
 * @param   mem_address     Target start address in hardware memory.
 * @param   p_write_buffer  Pointer to local memory holding the data to be written.
 *                          Undefined on failure.
 * @param   byte_size       Amount of data to write.
 * @return  Result of operation.
 *
 * @note    Certain implementation dependent limitations regarding alignment of
 *          both addresses and transfer size exist!
 *****************************************************************************/
RESULT VirtualHalWriteMemory
(
     VirtualHandle_t virtualHandle,
    uint32_t mem_address,
    uint8_t* p_write_buffer,
    uint32_t byte_size
);


/*****************************************************************************/
/**
 * @brief   maps a number of data from the memory into local memory starting at the given address
 * @param   virtualHandle       Handle to HAL session as returned by @ref VirtualHalOpen.
 * @param   mem_address     Source start address in hardware memory.
 * @param   byte_size       Amount of data to map.
 * @param   mapping_type    The way the mapping is performed.
 * @param   pp_mapped_buf   Reference to pointer to the mapped local memory.
 * @return  Result of operation.
 *
 * @note    Certain implementation dependent limitations regarding alignment of
 *          both addresses and transfer size exist!
 *****************************************************************************/
RESULT VirtualHalMapMemory
(
    VirtualHandle_t virtualHandle,
    uint32_t mem_address,
    uint32_t byte_size,
    HalMapMemType_t mapping_type,
    void **pp_mapped_buf
);


/*****************************************************************************/
/**
 * @brief   unmaps previously mapped memory from local memory
 * @param   virtualHandle       Handle to HAL session as returned by @ref VirtualHalOpen.
 * @param   p_mapped_buf    Pointer to local memory to unmap as returned by
 *                          @ref VirtualHalMapMemory().
 * @return  Result of operation.
 *
 * @note    Certain implementation dependent limitations regarding alignment of
 *          both addresses and transfer size exist!
 *****************************************************************************/
RESULT VirtualHalUnMapMemory(
     VirtualHandle_t virtualHandle,
     void* p_mapped_buf
);

/*****************************************************************************/
/**
 * @brief   dynamatic open file
 * @param   virtualHandle       Handle to HAL session as returned by @ref VirtualHalOpen.
 * @param   regDynDumpName      register dynamatic dump file name.
 * @return  Result of operation.
 *
 * @note    Certain implementation dependent limitations regarding alignment of
 *          both addresses and transfer size exist!
 *****************************************************************************/
RESULT VirtualHalDynRegDumpOpen
(
    VirtualHandle_t virtualHandle,
    char* regDynDumpName
);

/*****************************************************************************/
/**
 * @brief   dynamatic close1 file
 * @param   virtualHandle       Handle to HAL session as returned by @ref VirtualHalOpen.
 * @return  Result of operation.
 *
 * @note    Certain implementation dependent limitations regarding alignment of
 *          both addresses and transfer size exist!
 *****************************************************************************/
RESULT VirtualHalDynRegDumpClose
(
     VirtualHandle_t virtualHandle
);

RESULT VirtualHalAddRef(
     VirtualHandle_t virtualHandle
);

RESULT VirtualHalDelRef(
     VirtualHandle_t virtualHandle
);

HalHandle_t VirtualHalGetHalHandler(
     VirtualHandle_t virtualHandle
);

RESULT VirtualHalGetVirtualMode(
     VirtualHandle_t virtualHandle,
     VirtualHalBindOpsType_t *pVirtualMode
);

RESULT VirtualHalSetIcdev(
     VirtualHandle_t virtualHandle,
     void*  pIcdev
);

RESULT VirtualHalSystemInit(
     VirtualHandle_t virtualHandle
);

RESULT VirtualHalFastReset(
     VirtualHandle_t virtualHandle
);

#ifdef HAL_CMODEL
RESULT VirtualHalBufTransfer(
     VirtualHandle_t virtualHandle,
     uint32_t trans_ctrl
);

RESULT VirtualHalTileModeBufTransfer(
     VirtualHandle_t virtualHandle,
     uint32_t trans_ctrl
);

RESULT VirtualHalShdRegisterUpdate(
     VirtualHandle_t virtualHandle,
     uint8_t path
);

RESULT VirtualHalCfgDma(
     VirtualHandle_t virtualHandle,
    HalCmodelBufConfig_t* pBufCfg,
    uint32_t dmaLval
);

RESULT VirtualHalCfgMi(
     VirtualHandle_t virtualHandle,
     uint32_t path,
    HalCmodelBufConfig_t* pBufCfg
);


RESULT VirtualHalIspPipline(
     VirtualHandle_t virtualHandle
);


#endif

#ifdef __cplusplus
}
#endif
#endif
