/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/******************************************************************************\
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets of    *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2023 */

/**
 * @file virtual_hal_ops.h
 *
 * @brief   virtual Layer
 *
 *          Encapsulates and abstracts services from different hardware platforms.
 *
 *****************************************************************************/

#ifndef VIRTUAL_HAL_OPS_H
#define VIRTUAL_HAL_OPS_H

#include <ebase/types.h>
#include <ebase/dct_assert.h>
//#include <ebase/trace.h>

#include <common/return_codes.h>
#include <oslayer/oslayer.h>
#include <hal/hal_common.h>

/******************************************************************************
 * inline implementations of API
 *****************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum VirtualHalBindOpsType_e {
    BIND_VIRTUAL_HAL_OPS_NORMAL_MODE = 0,        //normal hal mode
    BIND_VIRTUAL_HAL_OPS_MCM_MODE,               //MCM mode.
    BIND_VIRTUAL_HAL_OPS_BATCH_MODE,             //Batch mode.
    BIND_VIRTUAL_HAL_OPS_TYPE_MAX
} VirtualHalBindOpsType_t;

/*****************************************************************************/
/**
 * @brief   Device priority
 *
 *****************************************************************************/
typedef enum VirtualWorkModePriority_e {
    VIRTUAL_DEVICE_PRIORITY_INVALID = 0,
    VIRTUAL_DEVICE_PRIORITY_0  = 1, /**<the lower number, the highest priority. */
    VIRTUAL_DEVICE_PRIORITY_1  = 2,
    VIRTUAL_DEVICE_PRIORITY_2  = 3,
    VIRTUAL_DEVICE_PRIORITY_3  = 4,
    VIRTUAL_DEVICE_PRIORITY_4  = 5,
    VIRTUAL_DEVICE_PRIORITY_5  = 6, /**<the lower number, the highest priority. */
    VIRTUAL_DEVICE_PRIORITY_6  = 7,
    VIRTUAL_DEVICE_PRIORITY_7  = 8,
    VIRTUAL_DEVICE_PRIORITY_8  = 9,
    VIRTUAL_DEVICE_PRIORITY_9  = 10,
    VIRTUAL_DEVICE_PRIORITY_10 = 11, /**<the lower number, the highest priority. */
    VIRTUAL_DEVICE_PRIORITY_11 = 12,
    VIRTUAL_DEVICE_PRIORITY_12 = 13,
    VIRTUAL_DEVICE_PRIORITY_13 = 14,
    VIRTUAL_DEVICE_PRIORITY_14 = 15,
    VIRTUAL_DEVICE_PRIORITY_15 = 16,
    VIRTUAL_DEVICE_PRIORITY_MAX
} VirtualWorkModePriority_t;

typedef enum VirtualMcmOpt_e {
    VIRTUAL_MCM_OPT_INVALID     = -1,
    VIRTUAL_MCM_OPT_SW          = 0,
    VIRTUAL_MCM_OPT_HW          = 1,
    VIRTUAL_MCM_OPT_MAX
} VirtualMcmOpt_t;

/*****************************************************************************/
/**
 * @brief   Select the input data entry.
 *
 *****************************************************************************/
typedef enum VirtualMcmSelection_e {
    VIRTUAL_MCM_SEL_INVALID     = -1,
    VIRTUAL_MCM_SEL_RDMA        = 0,
    VIRTUAL_MCM_SEL_RDCD        = 1,
    VIRTUAL_MCM_SEL_MAX
} VirtualMcmSelection_t;

/*****************************************************************************/
/**
 * @brief   workmode cfg
 *
 *****************************************************************************/
typedef struct VirtualWorkModeCfg_e {
    VirtualWorkModePriority_t priority;
    HalHandle_t             memHalHandle;                  /**<memory hal handle*/
    void                    *hCascade;                     /**<Cascade ctx handle*/
    VirtualMcmOpt_t         mcmOpt;
    VirtualMcmSelection_t   mcmSel;                        /**input data entry<*/
    uint8_t                 batchFrameCount;               /**<frame count of a batch in batch mode*/
} VirtualWorkModeCfg_t;

typedef struct HalIrqCtx_s VirtualHalIrqCtx_t;

typedef struct VirtualHalApiOps_s {
/*****************************************************************************/
/**
 * @brief   open
 * @param  ispId        isp instance index
 * @param  VirtualHalBindOpsType  ops type
 * @return  handle of low level context; NULL on failure
 *
 *****************************************************************************/
void *(*open)(
    uint32_t ispId,
    VirtualHalBindOpsType_t VirtualHalBindOpsType,
    const VirtualWorkModeCfg_t *pCfg
);

/*****************************************************************************/
/**
 * @brief   close the low level hal
 * @param   Handle   Handle to session as returned by @ref Open.
 * @return  Result of operation.
 *
 *****************************************************************************/
RESULT (*close)
(
    void* Handle
);

/*****************************************************************************/
/**
 * @brief   writes a value to the given address
 * @param   Handle   Handle to virtual session as returned by @ref Open.
 * @param   reg_address Address of register to write.
 * @param   value       Value to write into register.
 * @return  none
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
void (*writeRegister)
(
    void* Handle,
    uint32_t addr,
    uint32_t val
);


/*****************************************************************************/
/**
 * @brief   writes values to the given address
 * @param   Handle      Handle to virtual session as returned by @ref Open.
 * @param   reg_address Address of register to write.
 * @param   pVal        Values to write into register.
 * @return  none
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
void (*writeBatchRegisters)
(
    void* Handle,
    uint32_t addr,
    uint32_t *pVal
);


/*****************************************************************************/
/**
 * @brief   reads a value from a given address
 * @param   Handle   Handle to Virtual session as returned by @ref Open.
 * @param   reg_address Address of register to read.
 * @return  Register value.
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
uint32_t (*readRegister)
(
    void* Handle,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   writes a value to the given table address
 * @param   Handle   Handle to virtual session as returned by @ref Open.
 * @param   reg_address Address of register to write.
 * @param   value       Value to write into register.
 * @return  none
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
void (*writeTableRegister)
(
    void* Handle,
    uint32_t addr,
    uint32_t val
);

/*****************************************************************************/
/**
 * @brief   reads a value from a given table address
 * @param   Handle   Handle to Virtual session as returned by @ref Open.
 * @param   reg_address Address of register to read.
 * @return  Register value.
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
uint32_t (*readTableRegister)
(
    void* Handle,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   writes a value to the given address
 * @param   Handle   Handle to virtual session as returned by @ref Open.
 * @param   reg_address Address of register to write.
 * @param   value       Value to write into register.
 * @return  none
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
void (*rdcWriteRegister)
(
    void* Handle,
    uint32_t addr,
    uint32_t val
);

/*****************************************************************************/
/**
 * @brief   reads a value from a given address
 * @param   Handle   Handle to Virtual session as returned by @ref Open.
 * @param   reg_address Address of register to read.
 * @return  Register value.
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
uint32_t (*rdcReadRegister)
(
    void* Handle,
    uint32_t addr
);

/*****************************************************************************/
/**
 *          connectIrq()
 *
 *  @brief  Register interrupt service routine with system software.
 *
 *  @param  Handle to virtual session as returned by @ref Open
 *  @param  Reference of Virtual irq context structure that represent this connection
 *  @param  Interrupt object @ref osInterrupt
 *  @param  Number of the interrupt source, set to 0 if not needed
 *  @param  First interrupt routine, (first level handler) set to NULL if not needed
 *  @param  Second interrupt routine (second level handler)
 *  @param  Context provided when the interrupt routines are called
 *
 *  @warning Add platform specific code to connect to your local interrupt source
 *
 *  @return                     Status of operation
 *  @retval RET_SUCCESS         ISR registered successfully
 *  @retval RET_FAILURE et al.  ISR not registered
 *
 *****************************************************************************/
RESULT (*connectIrq)
(
    void* Handle,
    VirtualHalIrqCtx_t *pIrqCtx,
    uint32_t    int_src,
    osIsrFunc   IsrFunction,
    osDpcFunc   DpcFunction,
    void*       pContext
);

/*****************************************************************************/
/**
 *          disconnectIrq()
 *
 *  @brief  Deregister interrupt service routine from system software.
 *  @param  Reference of virtual irq context structure that represent this connection
 *
 *  @return                     Status of operation
 *  @retval RET_SUCCESS         ISR deregistered successfully
 *  @retval RET_FAILURE et al.  ISR not properly deregistered
 *
 *****************************************************************************/
RESULT (*disconnectIrq)
(
    VirtualHalIrqCtx_t *pIrqCtx
);

/*****************************************************************************/
/**
 *          getHalHandler()
 *
 *  @brief get hal level handler.
 *  @param  Handle to virtual session as returned by @ref Open
 *
 *  @return      hanlHandler. NULL on failure
 *
 *****************************************************************************/
HalHandle_t (*getHalHandler)
(
    void* Handle
);

/*****************************************************************************/
/**
 * @brief   tell HAL about another user of low level driver
 *  @param  Handle to virtual session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note
 *****************************************************************************/
RESULT (*addRef)(
    void* Handle
);
/*****************************************************************************/
/**
 * @brief   tell HAL about gone user of low level driver
 * @param  Handle to virtual session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*delRef)(
    void* Handle
);

/*****************************************************************************/
/**
 *          *eventGenerator()
 *
 *  @brief  Cmodel specific interface: Hal ISR and Event generator
 *
 *  @param
 *
 *  @return                     Status of operation
 *  @retval RET_SUCCESS         Input set successfully
 *  @retval RET_FAILURE et al.  Input set fails
 *
 *****************************************************************************/
RESULT (*eventGenerator)
(
    HalIrqCtx_t *pIrqCtx,
    HalIsrSrc_t isr_src,
    uint32_t misValue
);

/*****************************************************************************/
/**
 * @brief   set the ic dev pointer member.It is used for cmodel.
 * @param  Handle to virtual session as returned by @ref Open
 * @param  pIcdev  pointer to ic_dev object.
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*setIcdev)(
    void* Handle,
    void* pIcdev
);

RESULT (*systemInit)(
    void* Handle
);

RESULT (*fastReset)(
    void* Handle
);

#ifdef HAL_CMODEL
/*****************************************************************************/
/**
 * @brief   cmodel Buffer transfer handler
 * @param  Handle to virtual session as returned by @ref Open
 * @param  trans_ctrl contrl value.
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*bufTransfer)(
    void* Handle,
     uint32_t trans_ctrl
);

/*****************************************************************************/
/**
 * @brief   cmodel Buffer transfer handler in tilemode
 * @param  Handle to virtual session as returned by @ref Open
 * @param  trans_ctrl contrl value.
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*tileModeBufTransfer)(
    void* Handle,
     uint32_t trans_ctrl
);

/*****************************************************************************/
/**
 * @brief   Cmodel Shd register update process
 * @param  Handle to virtual session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*shdRegisterUpdate)(
    void* Handle,
    uint8_t path
);

/*****************************************************************************/
/**
 * @brief   cfg dma buffer addr ,format and size
 * @param  Handle to virtual session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*cfgDma)(
    void* Handle,
    HalCmodelBufConfig_t *pPicBuffer,
    uint32_t dmaLval
);

/*****************************************************************************/
/**
 * @brief   cfg mi buffer addr ,format and size
 * @param  Handle to virtual session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*cfgMi)(
    void* Handle,
     uint32_t path,
    HalCmodelBufConfig_t *pPicBuffer
);


/*****************************************************************************/
/**
 * @brief   run cmodel piplien to process
 * @param  Handle to virtual session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*ispPipeline)(
    void* Handle
);

/*****************************************************************************/
/**
 * @brief   run cmodel piplien to process in tilemode
 * @param  Handle to virtual session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*tileModeIspPipeline)(
    void* isp0Handle,
    void* isp1Handle
);
#endif
} VirtualHalApiOps_t;


// #ifdef __cplusplus
// extern "C"
// }
// #endif
#endif
